#! /usr/bin/env python
# -*- coding: utf-8 -*-

from gzip import GzipFile
import os, xml.dom.minidom
import ginv

# assur44.xml.gz       ducos7_3.xml.gz   hf855.xml.gz           quadfor2.xml.gz
# aubry2.xml.gz        ducos7_5.xml.gz   hietarinta1.xml.gz     quadgrid.xml.gz
# augot.xml.gz         ducos8.xml.gz     hunecke.xml.gz         rabmo.xml.gz
# benchmark_D1.xml.gz  eco10.xml.gz      i1.xml.gz              rbpl24.xml.gz
# benchmark_i1.xml.gz  eco11.xml.gz      ilias12.xml.gz         rbpl.xml.gz
# boon.xml.gz          eco12.xml.gz      ilias13.xml.gz         redcyc5.xml.gz
# butcher8.xml.gz      eco7.xml.gz       ilias_k_2.xml.gz       redcyc6.xml.gz
# butcher.xml.gz       eco8.xml.gz       ilias_k_3.xml.gz       redcyc7.xml.gz
# camera1s.xml.gz      eco9.xml.gz       issac97.xml.gz         redcyc8.xml.gz
# caprasse.xml.gz      el44.xml.gz       jcf26.xml.gz           redeco10.xml.gz
# cassou.xml.gz        el50.xml.gz       katsura10.xml.gz       redeco11.xml.gz
# chandra4.xml.gz      extcyc4.xml.gz    katsura6.xml.gz        redeco12.xml.gz
# chandra5.xml.gz      extcyc5.xml.gz    katsura7.xml.gz        redeco7.xml.gz
# chandra6.xml.gz      extcyc6.xml.gz    katsura8.xml.gz        redeco8.xml.gz
# chemequs.xml.gz      extcyc7.xml.gz    katsura9.xml.gz        redeco9.xml.gz
# chemequ.xml.gz       extcyc8.xml.gz    kin1.xml.gz            reif.xml.gz
# chemkin.xml.gz       f633.xml.gz       kinema.xml.gz          reimer4.xml.gz
# cohn2.xml.gz         f744.xml.gz       kotsireas.xml.gz       reimer5.xml.gz
# cohn3.xml.gz         f855.xml.gz       ku10.xml.gz            reimer6.xml.gz
# comb3000s.xml.gz     f966.xml.gz       lanconelli.xml.gz      reimer7.xml.gz
# comb3000.xml.gz      fabrice24.xml.gz  lichtblau.xml.gz       reimer8.xml.gz
# conform1.xml.gz      filter9.xml.gz    liu.xml.gz             rose.xml.gz
# cpdm5.xml.gz         geneig.xml.gz     lorentz.xml.gz         s9_1.xml.gz
# cyclic5.xml.gz       hairer1.xml.gz    matrix.xml.gz          solotarev.xml.gz
# cyclic6.xml.gz       hairer2.xml.gz    mckay.gls50mod.xml.gz  sparse5.xml.gz
# cyclic7.xml.gz       hairer3.xml.gz    mckay.xml.gz           speer.xml.gz
# cyclic8.xml.gz       hairer4.xml.gz    mickey.xml.gz          tangents.xml.gz
# d1.xml.gz            hawes4.xml.gz     morgenstern.xml.gz     test.xml.gz
# des18_3.xml.gz       hcyclic5.xml.gz   noon5.xml.gz           uteshev_bikker.xml.gz
# des22_24.xml.gz      hcyclic6.xml.gz   noon6.xml.gz           vermeer.xml.gz
# dessin1.xml.gz       hcyclic7.xml.gz   noon7.xml.gz           vernov1.xml.gz
# dessin2.xml.gz       hcyclic8.xml.gz   noon8.xml.gz           virasoro.xml.gz
# discret3.xml.gz      heart.xml.gz      noon9.xml.gz           wang16.xml.gz
# dl.xml.gz            hemmecke.xml.gz   pinchon1.xml.gz        wright.xml.gz
# ducos10.xml.gz       hf744.xml.gz      puma.xml.gz

folder = os.path.join(".", "polynomial")
fname = "cyclic7.xml.gz"

def readExample(fname):
  try:
    print fname
    file = GzipFile(fname, 'rb')
    doc = xml.dom.minidom.parse(file)
    root = doc.documentElement
    description = root.getElementsByTagName('Description').item(0).firstChild.nodeValue.strip()
    systemType = root.getElementsByTagName('SystemType').item(0).getAttribute("System")
    independ = map(lambda var: var.firstChild.nodeValue.strip(), root.getElementsByTagName('Variables').item(0).\
                                                                      getElementsByTagName('Independent').item(0).\
                                                                      getElementsByTagName('Variable'))
    equations = map(lambda eq: eq.firstChild.nodeValue.strip(), root.getElementsByTagName('Equations').item(0).\
                                                                     getElementsByTagName('Equation'))
    return (description, systemType, independ, equations)
  except Exception, val:
    print val
    return None

(description, systemType, independ, equations) = readExample(os.path.join(folder, fname))
print description

equations = map(lambda eq: eq.encode('latin1'), equations)

st = ginv.SystemType(systemType)
im = ginv.MonomInterface("DegRevLexByte", st, independ)
ic = ginv.CoeffInterface("GmpZ", st)
# ic = ginv.CoeffInterface("ModularShort", st, modularShort=31013)
ip = ginv.PolyInterface("PolyList", st, im, ic)
# ip = ginv.PolyInterface("PolyArray", st, im, ic)
iw = ginv.WrapInterface("CritPartially", ip)
# iw = ginv.WrapInterface("Without", ip)
id = ginv.DivisionInterface("Janet", iw)

# basis = ginv.basisBuild("TQ", id, equations)
# basis = ginv.basisBuild("TQDegree", id, equations)
# basis = ginv.basisBuild("TQBlockHigh", id, equations)
basis = ginv.basisBuild("TQBlockLow", id, equations)
# basis = ginv.basisBuild("TQBlockS1", id, equations)

print "         lengthIB = ", basis.lengthIB()
print "         lengthGB = ", basis.lengthGB()
print "     numCriterion = ", basis.numCriterion()
print "         numSpoly = ", basis.numSpoly()
print "     numReduction = ", basis.numReduction()
print "         userTime = ", basis.userTime()
print "          sysTime = ", basis.sysTime()
print "         realTime = ", basis.realTime()
print "hilbertPolynomial = ", basis.hilbertPolynomial()

try:
  import Gnuplot

  s = 0
  step, stepTime, Tlength, Qlength, EqLmLength = [], [], [], [], []
  for (sT, lenT, lenQ, lenEqLm) in basis.iterStatistics():
    s += 1
    step.append(s)
    stepTime.append(sT)
    Tlength.append(lenT)
    Qlength.append(lenQ)
    EqLmLength.append(lenEqLm)

  g = Gnuplot.Gnuplot()
  g.xlabel('time')
  g.ylabel('size')
  g.title(fname)
  g.plot(Gnuplot.Data(step, Tlength, title='T', with='lines'),\
         Gnuplot.Data(step, Qlength,  title='Q', with='lines'),\
         Gnuplot.Data(step, EqLmLength, title='eqQ', with='lines'))
  raw_input('Press return to show results...\n')

except Exception, val:
    print val
    print "not module 'gnuplot-py' or 'gnuplot'"

# for p in basis.iterGB():
#   print p
