/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IALGORITHM_H
#define IALGORITHM_H

#include "../util/istatistics.h"
#include "../division/idivision.h"
#include "../algorithm/isett.h"

class IAlgorithm {
protected:
  IStatistics  mStatistics;
  ISetT       *mSetT;
  IDivision   *mDivision;

  unsigned     mQlengthBound;
  unsigned     mMovedBound;
  unsigned     mDegreeBound;
  bool         mQuiet;

  bool isMinimalBasis(const IMonom& m, const ICoeff &c) const;
  void buildStatistics();

public:
  IAlgorithm(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  virtual ~IAlgorithm();

  virtual void build() = 0;

  const IStatistics& statistics() const { return mStatistics; }
  const ISetT& setT() const { return *mSetT; }
  const IDivision* division() const { return mDivision; }

  void read(std::istream &in);
  void write(std::ostream &out) const {
    mSetT->write(out);
    mStatistics.write(out);
  }
};

class IAlgorithmTQ: public IAlgorithm {
protected:
  int mNumEq;

  virtual bool reduce(IWrap &wrap);
  virtual bool reduceQ();

public:
  IAlgorithmTQ(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  ~IAlgorithmTQ();

  void build();
};

class IAlgorithmTQDegree: public IAlgorithmTQ {
protected:
  bool reduceQ();

public:
  IAlgorithmTQDegree(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  ~IAlgorithmTQDegree();
};

class IAlgorithmTQBlock: public IAlgorithmTQ {
protected:
  bool reduceQBlock(ISetWrap &block);

public:
  IAlgorithmTQBlock(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  ~IAlgorithmTQBlock();

  void build();
};

class IAlgorithmTQBlockHigh: public IAlgorithmTQ {
protected:
  virtual bool reduceQBlock(ISetWrap &block);

public:
  IAlgorithmTQBlockHigh(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned  QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  ~IAlgorithmTQBlockHigh();

  void build();
};

class IAlgorithmTQBlockLow: public IAlgorithmTQBlockHigh {
protected:
  bool reduceQBlock(ISetWrap &block);

public:
  IAlgorithmTQBlockLow(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  ~IAlgorithmTQBlockLow();
};

class IAlgorithmTQBlockNorm: public IAlgorithmTQBlockHigh {
protected:
  bool reduceQBlock(ISetWrap &block);

public:
  IAlgorithmTQBlockNorm(ISetQ *setQ, IDivision *division, long time_bound, bool quiet, unsigned QlengthBound, unsigned MovedBound, unsigned DegreeBound);
  ~IAlgorithmTQBlockNorm();
};

#endif // IALGORITHM_H
