/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./isetq.h"


int ISetQ::insert(IPoly *poly) {
  IASSERT(poly != NULL);
  IASSERT(!poly->isZero());
  Iterator i(begin());
  i.insert();
  *i = mWrapInterface->create(poly);
  return insert(i.get());
}

void ISetQ::insert(IWrap *wrap, int var, unsigned d) {
  IASSERT(d > 0);
  if (wrap->notBuild(var, d)) {
    Iterator i(begin());
    i.insert();
    *i = mWrapInterface->create(*wrap, var, d);
    if (mDegreeBound == 0 || (*i)->poly().degree() <= mDegreeBound)
      insert(i.get());
    else
      i.del();
  }
  IASSERTVALID(*this);
}

/*
static int polyCompare(const IPoly &a, const IPoly &b) {
  IPoly::ConstIterator &ia(*a.begin()),
                       &ib(*b.begin());
  int cmp = 0;
  int coeff_equal_or_minus;
  int cmp_coeff = 0;
  if (ia.coeff().compare(ib.coeff()) == 0)
    coeff_equal_or_minus = 1;
  else if (ia.coeff().cmpNorm(ib.coeff()) == 0)
    coeff_equal_or_minus = -1;
  else
    coeff_equal_or_minus = 0;
  while(ia && ib) {
    cmp = ia.monom().compare(ib.monom());
    if (cmp != 0) {
      return cmp;
    }
    if (coeff_equal_or_minus != 0) {
      cmp = ia.coeff().compare(ib.coeff());
      if (cmp_coeff == 0)
	cmp_coeff = cmp;
      if (cmp == 0) {
	if (coeff_equal_or_minus != 1)
	  coeff_equal_or_minus = 0;
      }
      else if (ia.coeff().cmpNorm(ib.coeff()) == 0) {
	if (coeff_equal_or_minus != -1)
	  coeff_equal_or_minus = 0;
      }
    }
    ++ia;
    ++ib;
  }
  if (ia)
    cmp = 1;
  else if (ib)
    cmp = -1;
  else if (coeff_equal_or_minus != 0)
    cmp = 0;
  else
    cmp = cmp_coeff;
  return cmp;
}
*/

int ISetQ::insert(ConstIterator j) {
  Iterator i(begin());
  int pos = 0;
  while(i) {
    if ((*i)->isZero()) {
      i.del();
      continue;
    }

    int cmp = (*i)->lm().setQcompare((*j)->lm());
    if (cmp > 0)
      break;
    if (cmp == 0 && (coeffInterface()->type() == ICoeffInterface::GmpZZ ||
      coeffInterface()->type() == ICoeffInterface::GmpZ)) {
      cmp = (*i)->poly().lc().cmpNorm((*j)->poly().lc());
      if (cmp >= 0)
        break;
/*
      if (cmp == 0) {
	if (polyCompare((*i)->poly(), (*j)->poly()) == 0) {
	  IASSERTVALID(*this);
	  return pos+1;
	}
	break;
      }
*/
    }

    ++i;
    ++pos;
  }
  i.insert(j);

  // --- ??? ---
  if (coeffInterface()->type() == ICoeffInterface::GmpZZ ||
      (monomInterface()->order() & IMonomInterface::DegRevLexDeg) != 0) {
    ++i;
    while(i) {
      if ((*i)->isZero())
	i.del();
      else
	++i;
    }
  }
  // --- ??? ---

  IASSERTVALID(*this);
  return pos;
}

/*
static int compare(const IPoly &a, const IPoly &b) {
  IPoly::ConstIterator &ia(*a.begin()),
                       &ib(*b.begin());
  int cmp = 0;
  while(ia && ib) {
    cmp = ia.monom().compare(ib.monom());
    if (cmp != 0)
      return cmp;
    ++ia;
    ++ib;
  }
  if (ia)
    cmp = 1;
  else
    cmp = -1;
  return cmp;
}
*/

ISetQ::ConstIterator ISetQ::getMin(int &numEq) {
  IASSERTVALID(*this);
  Iterator j(begin());
  Iterator r = j;
  if (coeffInterface()->type() == ICoeffInterface::GmpZZ) {
    ++j;
    while(j) {
      int cmp = (*j)->lm().setQcompare((*r)->lm());
      IASSERT(cmp >= 0);
      if (cmp > 0)
        break;
      else if (cmp == 0) {
	if ((*j)->isZero()) {
	  j.del();
	  continue;
	}
        if ((*r)->poly().lc().cmpNorm((*j)->poly().lc()) > 0)
          r = j;
      }
      ++j;
    }
  }
  else {
    int normr = (*j)->poly().norm();
    int lenr = (*j)->poly().length();
    ++j;
    numEq = 1;
    while(j) {
      int cmp = (*j)->lm().setQcompare((*r)->lm());
  //     int cmp = compare((*j)->poly(), (*r)->poly());
      IASSERT(cmp >= 0);
      if (cmp > 0)
        break;
      else if (cmp == 0) {
        ++numEq;
        int normj = (*j)->poly().norm();
        if (normj < normr) {
          normr = normj;
          lenr = (*j)->poly().length();
          r = j;
        }
        else if (normj == normr) {
          int lenj = (*j)->poly().length();
          if (lenj < lenr) {
            lenr = lenj;
            normr = (*j)->poly().norm();
            r = j;
          }
        }
      }
      ++j;
    }
  }
  return r.get();
}

void ISetQ::assertValid(const char* fileName, int fileLine) const {
  ConstIterator i(begin());
  while(i) {
    ConstIterator j = i;
    ++i;
    if (i) {
      int cmp = (*i)->lm().setQcompare((*j)->lm());
      IASSERT2(fileName, fileLine, cmp >= 0);
    }
  }
}

template <>
IAllocator IList<IWrap*>::Node::sAllocator(sizeof(IList<IWrap*>::Node));
