/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ISETT_H
#define ISETT_H

#include "./isetq.h"

class ISetT: protected ISetWrap {
protected:
  IWrapInterface *mWrapInterface;
  ISetQ  *mSetQ;

public:
  typedef ISetWrap::Iterator Iterator;
  typedef ISetWrap::ConstIterator ConstIterator;

  ISetT(IWrapInterface* wrapInterface):
      ISetWrap(),
      mWrapInterface(wrapInterface),
      mSetQ(NULL) {
    IASSERT(mWrapInterface);
    IPoly *poly = mWrapInterface->polyInterface()->create();
    ICoeff *c = mWrapInterface->coeffInterface()->create();
    c->setOne();
    poly->setCoeff(c);
    ISetWrap::begin().insert(mWrapInterface->create(poly));
  }
  ISetT(ISetQ* setQ):
      ISetWrap(),
      mWrapInterface(setQ->wrapInterface()),
      mSetQ(setQ) {
    IASSERT(mWrapInterface);
  }
  ~ISetT() { delete mSetQ; }

  const ISystemType* systemType() const { return mWrapInterface->systemType(); }
  IMonomInterface* monomInterface() const { return mWrapInterface->monomInterface(); }
  ICoeffInterface* coeffInterface() const { return mWrapInterface->coeffInterface(); }
  IPolyInterface* polyInterface() const { return mWrapInterface->polyInterface(); }
  IWrapInterface* wrapInterface() const { return mWrapInterface; }

  ISetQ* setQ() const { return mSetQ; }
  bool isEmpty() const { return ISetWrap::isEmpty(); }
  int length() const { return ISetWrap::length(); }

  ConstIterator begin() const { return ISetWrap::begin(); }
  Iterator begin() { return ISetWrap::begin(); }

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // ISETT_H
