/***************************************************************************
 *   Copyright (C) 2006 by Daniel Robertz                                  *
 *   daniel@momo.math.rwth-aachen.de                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./ialgextgmpqaux.h"

void IAlgExtGmpQAux::add(IAlgExtGmpQAux& a) {
  Iterator  i1(mHead.begin());
  Iterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert(i2.get());
      ++i1;
    }
    else {
      mpq_add(&(*i1).mCoeff, &(*i1).mCoeff, &(*i2).mCoeff);
      if (mpq_sgn(&(*i1).mCoeff) == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert(i2.get());
    ++i1;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::sub(IAlgExtGmpQAux& a) {
  Iterator  i1(mHead.begin());
  Iterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert(i2.get());
      mpq_neg(&(*i1).mCoeff, &(*i1).mCoeff);
      ++i1;
    }
    else {
      mpq_sub(&(*i1).mCoeff, &(*i1).mCoeff, &(*i2).mCoeff);
      if (mpq_sgn(&(*i1).mCoeff) == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert(i2.get());
    mpq_neg(&(*i1).mCoeff, &(*i1).mCoeff);
    ++i1;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::mult(const MP_RAT& coeff) {
  IASSERT(mpq_sgn(&coeff) != 0);
  ConstIterator i(mHead.begin());
  while(i) {
    mpq_mul(&(*i).mCoeff, &(*i).mCoeff, &coeff);
    ++i;
  }
  IASSERTVALID(*this);
}


void IAlgExtGmpQAux::mult(unsigned degree, const MP_RAT& coeff) {
  IASSERT(mpq_sgn(&coeff) != 0);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mDegree += degree;
    mpq_mul(&(*i).mCoeff, &(*i).mCoeff, &coeff);
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::div(const MP_RAT& coeff) {
  IASSERT(mpq_sgn(&coeff) != 0);
  ConstIterator i(mHead.begin());
  while(i) {
    mpq_div(&(*i).mCoeff, &(*i).mCoeff, &coeff);
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::div(unsigned degree, const MP_RAT& coeff) {
  IASSERT(mpq_sgn(&coeff) != 0);

  ConstIterator i(mHead.begin());
  while(i) {
   IASSERT((*i).mDegree >= degree);
   (*i).mDegree += degree;
   mpq_div(&(*i).mCoeff, &(*i).mCoeff, &coeff);
   ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::rem(const IAlgExtGmpQAux& a) {
  IASSERT(!a.isZero());

  MP_RAT k2, tmp;
  mpq_init(&k2);
  mpq_init(&tmp);

  while(!isZero() && degree() >= a.degree()) {
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg = degree() - a.degree();
    mpq_div(&k2, &(*i1).mCoeff, &(*i2).mCoeff);
    mpq_neg(&k2, &k2);

    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
        mpq_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        ++i1;
        ++i2;
      }
      else {
        mpq_mul(&tmp, &(*i2).mCoeff, &k2);
        mpq_add(&(*i1).mCoeff, &(*i1).mCoeff, &tmp);
        if (mpq_sgn(&(*i1).mCoeff) == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      mpq_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
  mpq_clear(&k2);
  mpq_clear(&tmp);
}

void IAlgExtGmpQAux::rem_ext(const IAlgExtGmpQAux& a, IAlgExtGmpQAux& c1, IAlgExtGmpQAux& c2) {
  IASSERT(!a.isZero());

  MP_RAT k2, tmp;
  mpq_init(&k2);
  mpq_init(&tmp);

  while(!isZero() && degree() >= a.degree()) {
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());
    Iterator  j1(c1.mHead.begin());
    ConstIterator  j2(c2.mHead.begin());

    unsigned deg = degree() - a.degree();
    mpq_div(&k2, &(*i1).mCoeff, &(*i2).mCoeff);
    mpq_neg(&k2, &k2);

    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
        mpq_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        ++i1;
        ++i2;
      }
      else {
        mpq_mul(&tmp, &(*i2).mCoeff, &k2);
        mpq_add(&(*i1).mCoeff, &(*i1).mCoeff, &tmp);
        if (mpq_sgn(&(*i1).mCoeff) == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      mpq_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);

    while(j1 && j2) {
      unsigned degree2 = (*j2).mDegree + deg;
      if ((*j1).mDegree > degree2) {
        ++j1;
      }
      else if ((*j1).mDegree < degree2) {
        j1.insert();
        (*j1).mDegree = degree2;
        mpq_mul(&(*j1).mCoeff, &(*j2).mCoeff, &k2);
        ++j1;
        ++j2;
      }
      else {
        mpq_mul(&tmp, &(*j2).mCoeff, &k2);
        mpq_add(&(*j1).mCoeff, &(*j1).mCoeff, &tmp);
        if (mpq_sgn(&(*j1).mCoeff) == 0)
          j1.del();
        else
          ++j1;
        ++j2;
      }
    }
    while(j2) {
      j1.insert();
      (*j1).mDegree = (*j2).mDegree + deg;
      mpq_mul(&(*j1).mCoeff, &(*j2).mCoeff, &k2);
      ++j1;
      ++j2;
    }
  }
  mpq_clear(&k2);
  mpq_clear(&tmp);
}

void IAlgExtGmpQAux::minus() {
  ConstIterator i(mHead.begin());
  while(i) {
    mpq_neg(&(*i).mCoeff, &(*i).mCoeff);
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::add(const IAlgExtGmpQAux& a) {
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree;
      mpq_set(&(*i1).mCoeff, &(*i2).mCoeff);
      ++i1;
      ++i2;
    }
    else {
      mpq_add(&(*i1).mCoeff, &(*i1).mCoeff, &(*i2).mCoeff);
      if (mpq_sgn(&(*i1).mCoeff) == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
    mpq_set(&(*i1).mCoeff, &(*i2).mCoeff);
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::sub(const IAlgExtGmpQAux& a) {
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree;
      mpq_set(&(*i1).mCoeff, &(*i2).mCoeff);
      mpq_neg(&(*i1).mCoeff, &(*i1).mCoeff);
      ++i1;
      ++i2;
    }
    else {
      mpq_sub(&(*i1).mCoeff, &(*i1).mCoeff, &(*i2).mCoeff);
      if (mpq_sgn(&(*i1).mCoeff) == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
    mpq_set(&(*i1).mCoeff, &(*i2).mCoeff);
    mpq_neg(&(*i1).mCoeff, &(*i1).mCoeff);
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::mult(const IAlgExtGmpQAux& a) {
  if (isZero())
    return;

  IAlgExtGmpQAux prod;
  ConstIterator ia(a.mHead.begin());
  Iterator begin(prod.mHead.begin());
  MP_RAT mp;
  mpq_init(&mp);

  while(ia) {
    ConstIterator i(mHead.begin());
    Iterator ip = begin;
    unsigned degree2 = (*i).mDegree + (*ia).mDegree;

    // look for the starting point
    while(ip && (*ip).mDegree > degree2)
      ++ip;

    begin = ip;

    while(ip && i) {
      degree2 = (*i).mDegree + (*ia).mDegree;
      if ((*ip).mDegree > degree2)
        ++ip;
      else if ((*ip).mDegree < degree2) {
        ip.insert();
        (*ip).mDegree = degree2;
        mpq_mul(&(*ip).mCoeff, &(*i).mCoeff, &(*ia).mCoeff);
        IASSERT(mpq_sgn(&(*ip).mCoeff) != 0);
        //if (mpq_sgn(&(*ip).mCoeff) == 0)
	  //ip.del();
        //else
          ++ip;
        ++i;
      }
      else {
	mpq_mul(&mp, &(*i).mCoeff, &(*ia).mCoeff);
	mpq_add(&(*ip).mCoeff, &(*ip).mCoeff, &mp);
        if (mpq_sgn(&(*ip).mCoeff) == 0)
          ip.del();
        else
          ++ip;
        ++i;
      }
    }

    while(i) {
      ip.insert();
      (*ip).mDegree = (*i).mDegree + (*ia).mDegree;
      mpq_mul(&(*ip).mCoeff, &(*i).mCoeff, &(*ia).mCoeff);
      IASSERT(mpq_sgn(&(*ip).mCoeff) != 0);
      //if (mpq_sgn(&(*ip).mCoeff) == 0)
        //ip.del();
      //else
        ++ip;
      ++i;
    }

    ++ia;
  }
  mpq_clear(&mp);
  mHead.swap(prod.mHead);
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::mult(const IAlgExtGmpQAux& a, const IAlgExtGmpQAux& b) {
  setZero();
  if (a.isZero() || b.isZero())
    return;

  ConstIterator ib(b.mHead.begin());
  Iterator begin(mHead.begin());
  MP_RAT mp;
  mpq_init(&mp);

  while(ib) {
    ConstIterator ia(a.mHead.begin());
    Iterator ip = begin;
    unsigned degree2 = (*ia).mDegree + (*ib).mDegree;

    // look for the starting point
    while(ip && (*ip).mDegree > degree2)
      ++ip;

    begin = ip;

    while(ip && ia) {
      degree2 = (*ia).mDegree + (*ib).mDegree;
      if ((*ip).mDegree > degree2)
        ++ip;
      else if ((*ip).mDegree < degree2) {
        ip.insert();
        (*ip).mDegree = degree2;
        mpq_mul(&(*ip).mCoeff, &(*ia).mCoeff, &(*ib).mCoeff);
        IASSERT(mpq_sgn(&(*ip).mCoeff) != 0);
        //if (mpq_sgn(&(*ip).mCoeff) == 0)
	  //ip.del();
        //else
          ++ip;
        ++ia;
      }
      else {
	mpq_mul(&mp, &(*ia).mCoeff, &(*ib).mCoeff);
	mpq_add(&(*ip).mCoeff, &(*ip).mCoeff, &mp);
        if (mpq_sgn(&(*ip).mCoeff) == 0)
          ip.del();
        else
          ++ip;
        ++ia;
      }
    }

    while(ia) {
      ip.insert();
      (*ip).mDegree = (*ia).mDegree + (*ib).mDegree;
      mpq_mul(&(*ip).mCoeff, &(*ia).mCoeff, &(*ib).mCoeff);
      IASSERT(mpq_sgn(&(*ip).mCoeff) != 0);
      //if (mpq_sgn(&(*ip).mCoeff) == 0)
        //ip.del();
      //else
        ++ip;
      ++ia;
    }

    ++ib;
  }
  mpq_clear(&mp);
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::pow(unsigned deg) {
  IASSERT(deg > 0);
  if (deg > 1) {
    IAlgExtGmpQAux p(*this);
    while(--deg > 0) {
      p.mult(*this);
    }
    mHead.swap(p.mHead);
  }
  IASSERTVALID(*this);
}

void IAlgExtGmpQAux::div(const IAlgExtGmpQAux& a) {
  IASSERT(degree() >= a.degree());

  MP_RAT k1, k2, tmp;
  mpq_init(&k1);
  mpq_init(&k2);
  mpq_init(&tmp);

  IAlgExtGmpQAux  quo;
  Iterator  i(quo.mHead.begin());

  while(!isZero()) {
    IASSERT(degree() >= a.degree());
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg = degree() - a.degree();
    mpq_set(&k1, &(*i2).mCoeff);
    mpq_set(&k2, &(*i1).mCoeff);

    i.insert();
    (*i).mDegree = deg;
    mpq_set(&(*i).mCoeff, &k2);
    ++i;
    IASSERTVALID(quo);

    mpq_neg(&k2, &k2);

    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
        mpq_mul(&(*i1).mCoeff, &(*i1).mCoeff, &k1);
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
        mpq_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        ++i1;
        ++i2;
      }
      else {
        mpq_mul(&(*i1).mCoeff, &(*i1).mCoeff, &k1);
        mpq_mul(&tmp, &(*i2).mCoeff, &k2);
        mpq_add(&(*i1).mCoeff, &(*i1).mCoeff, &tmp);
        if (mpq_sgn(&(*i1).mCoeff) == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      mpq_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
  IASSERT(isZero());
  mHead.swap(quo.mHead);
  mpq_clear(&k1);
  mpq_clear(&k2);
  mpq_clear(&tmp);
}

void IAlgExtGmpQAux::gcd(const IAlgExtGmpQAux& a) {
  IASSERT(!isZero());
  IASSERT(!a.isZero());

  if (degree() == 0 || a.degree() == 0) {
    ConstIterator  i(mHead.begin());
    mpq_set_ui(&(*i).mCoeff, 1ul, 1ul);
  }
  else {
    IAlgExtGmpQAux tmp(a);

    if (tmp.degree() > degree())
      mHead.swap(tmp.mHead);

    do {
      mHead.swap(tmp.mHead);
      tmp.rem(*this);
    } while (!tmp.isZero() && tmp.degree() > 0);

    if (!tmp.isZero()) {
      mHead.swap(tmp.mHead);
      mpq_set_ui(&(*mHead.begin()).mCoeff, 1ul, 1ul);
    }
  }
}

void IAlgExtGmpQAux::assertValid(const char* fileName, int fileLine) const {
  if (!mHead.isEmpty()) {
    ConstIterator i(mHead.begin());
    IASSERT2(fileName, fileLine, mpq_sgn(&(*i).mCoeff) != 0);
    ConstIterator prev(i);
    ++i;
    while(i) {
      IASSERT2(fileName, fileLine, mpq_sgn(&(*i).mCoeff) != 0);
      IASSERT2(fileName, fileLine, (*prev).mDegree > (*i).mDegree);
      prev = i;
      ++i;
    }
  }
}

void IAlgExtGmpQAux::input(std::istream& in, const IVariables &parameters) {
  additive(in, parameters);
}

void IAlgExtGmpQAux::additive(std::istream& in, const IVariables &parameters) {
  multiplicative(in, parameters);
  int op = (in >> std::ws).peek();
  IAlgExtGmpQAux tmp;
  while(op == '+' || op == '-') {
    in.get();
    tmp.multiplicative(in, parameters);
    if (op == '+')
      add(tmp);
    else {
      sub(tmp);
    }
    op = in.peek();
  }
}

void IAlgExtGmpQAux::multiplicative(std::istream& in, const IVariables &parameters) {
  unary(in, parameters);
  int op = (in >> std::ws).peek();
  IAlgExtGmpQAux tmp;
  while(op == '*') {
    in.get();
    tmp.unary(in, parameters);
    mult(tmp);
    op = in.peek();
  }
}

void IAlgExtGmpQAux::unary(std::istream& in, const IVariables &parameters) {
  int ch = (in >> std::ws).peek();
  if (ch != '+' && ch != '-')
    power(in, parameters);
  else {
    int k = 1;
    do {
      ch = in.get();
      if (ch == '-')
        k *= -1;
      ch = (in >> std::ws).peek();
    } while (ch == '+' || ch == '-');
    power(in, parameters);
    if (k < 0)
      minus();
  }
}

void IAlgExtGmpQAux::power(std::istream& in, const IVariables &parameters) {
  bracket(in, parameters);
  int op = (in >> std::ws).peek();
  if (op == '^') {
    in.get();
    int deg=0;
    in >> std::ws >> deg;
    if (in.fail() || deg <= 0)
      IMESSAGE("expected 'degree > 0'");
    pow(deg);
  }
}

void IAlgExtGmpQAux::bracket(std::istream& in, const IVariables &parameters) {
  int op = (in >> std::ws).peek();
  if (op == '(') {
    in.get();
    additive(in, parameters);
    if (in.get() != ')') {
      IMESSAGE("expected ')'");
    }
  }
  else {
    std::streampos posbeg = in.tellg();
    int var = parameters.read(in);
    if (var == 0) {
      setOne();
      (*begin()).mDegree = 1;
      IASSERTVALID(*this);
    }
    else {
      MP_RAT mp;
      mpq_init(&mp);
      in >> (&mp);
      mpq_canonicalize(&mp);
      if (in.fail()) {
        in.clear();
        in.setstate(std::ios::failbit);
        in.seekg(posbeg);
      }
      else {
        IAlgExtGmpQAux::Iterator i(begin());
        i.clear();
        if (mpq_sgn(&mp) != 0) {
          i.insert();
          mpq_set(&(*i).mCoeff, &mp);
        }
        IASSERTVALID(*this);
      }
      mpq_clear(&mp);
    }
  }
}

void IAlgExtGmpQAux::read(std::istream &in) {
  mHead.clear();
  int l;
  in.read((char*)(&l), sizeof(l));
  Iterator i(mHead.begin());
  while(l) {
    i.insert();
    in.read((char*)(&(*i).mDegree), sizeof((*i).mDegree));
    in >> (&(*i).mCoeff);
    ++i;
    --l;
  }
}

void IAlgExtGmpQAux::write(std::ostream &out) const {
  int l = mHead.length();
  out.write((char*)(&l), sizeof(l));
  ConstIterator i(mHead.begin());
  while(i) {
    out.write((char*)(&(*i).mDegree), sizeof((*i).mDegree));
    out << (&(*i).mCoeff);
    ++i;
  }
}

std::ostream& operator<<(std::ostream& out, const IAlgExtGmpQAux& a) {
  IAlgExtGmpQAux::ConstIterator i(a.begin());
  if (!i)
    out << '0';
  else {
    out << (&(*i).mCoeff) << "*t^" <<  (*i).mDegree;
    ++i;
    while(i) {
      out << " + " << (&(*i).mCoeff) << "*t^" <<  (*i).mDegree;
      ++i;
    }
  }
  return out;
}

template <>
IAllocator IList<IAlgExtGmpQAux::Term>::Node::sAllocator(sizeof(IList<IAlgExtGmpQAux::Term>::Node));
