/***************************************************************************
 *   Copyright (C) 2006 by Daniel Robertz                                  *
 *   daniel@momo.math.rwth-aachen.de                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./ialgextmodularshortaux.h"
#include "../util/imodular.h"

void IAlgExtModularShortAux::mult(const int coeff) {
  IASSERT(coeff > 0 && coeff < mPrime);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mCoeff *= coeff;
	(*i).mCoeff %= mPrime;
    ++i;
  }
  IASSERTVALID(*this);
}


void IAlgExtModularShortAux::mult(unsigned degree, const int coeff) {
  IASSERT(coeff > 0 && coeff < mPrime);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mDegree += degree;
    (*i).mCoeff *= coeff;
	(*i).mCoeff %= mPrime;
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::div(const int coeff) {
  IASSERT(coeff > 0 && coeff < mPrime);
  ConstIterator i(mHead.begin());
  int inv = iInverse(coeff, mPrime);
  while(i) {
    (*i).mCoeff *= inv;
	(*i).mCoeff %= mPrime;
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::div(unsigned degree, const int coeff) {
  IASSERT(coeff > 0 && coeff < mPrime);

  ConstIterator i(mHead.begin());
  int inv = iInverse(coeff, mPrime);
  while(i) {
    IASSERT((*i).mDegree >= degree);
    (*i).mDegree += degree;
    (*i).mCoeff *= inv;
	(*i).mCoeff %= mPrime;
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::rem(const IAlgExtModularShortAux& a) {
  IASSERT(!a.isZero());
  IASSERT(a.mPrime == mPrime);

  int k2, tmp;

  while(!isZero() && degree() >= a.degree()) {
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg = degree() - a.degree();
	k2 = (*i1).mCoeff * iInverse((*i2).mCoeff, mPrime) % mPrime;
	k2 = mPrime - k2;

    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
		(*i1).mCoeff = (*i2).mCoeff * k2 % mPrime;
        ++i1;
        ++i2;
      }
      else {
	    tmp = (*i2).mCoeff * k2 % mPrime;
		(*i1).mCoeff = ((*i1).mCoeff + tmp) % mPrime;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
	  (*i1).mCoeff = (*i2).mCoeff * k2 % mPrime;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
}

void IAlgExtModularShortAux::quo(const IAlgExtModularShortAux& a,
    const IAlgExtModularShortAux& b) {
  
  IASSERT(!b.isZero());
  IASSERT(a.mPrime == mPrime);
  IASSERT(b.mPrime == mPrime);

  IAlgExtModularShortAux tmp(a);
  
  setZero();
  
  Iterator it = mHead.begin();

  while(!tmp.isZero() && tmp.degree() >= b.degree()) {
    it.insert();
    (*it).mDegree = tmp.degree() - b.degree();
    (*it).mCoeff = tmp.lc() * iInverse(b.lc(), mPrime) % mPrime;
    tmp.sub(b, (*it).mDegree, (*it).mCoeff);
    
    ++it;
  }
  
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::rem_ext(const IAlgExtModularShortAux& a, IAlgExtModularShortAux& c1, IAlgExtModularShortAux& c2) {
  IASSERT(!a.isZero());
  IASSERT(a.mPrime == mPrime);
  IASSERT(c1.mPrime == mPrime);
  IASSERT(c2.mPrime == mPrime);

  int k2, tmp;

  while(!isZero() && degree() >= a.degree()) {
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());
    Iterator  j1(c1.mHead.begin());
    ConstIterator  j2(c2.mHead.begin());

    unsigned deg = degree() - a.degree();
	k2 = (*i1).mCoeff * iInverse((*i2).mCoeff, mPrime) % mPrime;
	k2 = mPrime - k2;

    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
		(*i1).mCoeff = (*i2).mCoeff * k2 % mPrime;
        ++i1;
        ++i2;
      }
      else {
		tmp = (*i2).mCoeff * k2 % mPrime;
		(*i1).mCoeff = ((*i1).mCoeff + tmp) % mPrime;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
	  (*i1).mCoeff = (*i2).mCoeff * k2 % mPrime;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);

    while(j1 && j2) {
      unsigned degree2 = (*j2).mDegree + deg;
      if ((*j1).mDegree > degree2) {
        ++j1;
      }
      else if ((*j1).mDegree < degree2) {
        j1.insert();
        (*j1).mDegree = degree2;
		(*j1).mCoeff = (*j2).mCoeff * k2 % mPrime;
        ++j1;
        ++j2;
      }
      else {
		tmp = (*j2).mCoeff * k2 % mPrime;
		(*j1).mCoeff = ((*j1).mCoeff + tmp) % mPrime;
        if ((*j1).mCoeff == 0)
          j1.del();
        else
          ++j1;
        ++j2;
      }
    }
    while(j2) {
      j1.insert();
      (*j1).mDegree = (*j2).mDegree + deg;
	  (*j1).mCoeff = (*j2).mCoeff * k2 % mPrime;
      ++j1;
      ++j2;
    }
  }
}

void IAlgExtModularShortAux::minus() {
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mCoeff = mPrime - (*i).mCoeff;
    ++i;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::add(const IAlgExtModularShortAux& a) {
  IASSERT(a.mPrime == mPrime);

  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree;
	  (*i1).mCoeff = (*i2).mCoeff;
      ++i1;
      ++i2;
    }
    else {
	  (*i1).mCoeff = ((*i1).mCoeff + (*i2).mCoeff) % mPrime;
      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
	(*i1).mCoeff = (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::sub(const IAlgExtModularShortAux& a) {
  IASSERT(a.mPrime == mPrime);

  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree;
      (*i1).mCoeff = mPrime - (*i2).mCoeff;
      ++i1;
      ++i2;
    }
    else {
      (*i1).mCoeff += mPrime - (*i2).mCoeff;
      (*i1).mCoeff %= mPrime;
      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
    (*i1).mCoeff = mPrime - (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::sub(const IAlgExtModularShortAux& a,
    unsigned deg,
    int coeff) {
  
  IASSERT(a.mPrime == mPrime);

  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree + deg) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree + deg) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      int tmp = (*i2).mCoeff*coeff; 
      tmp %= mPrime;
      (*i1).mCoeff = mPrime - tmp;
      ++i1;
      ++i2;
    }
    else {
      int tmp = (*i2).mCoeff*coeff;
      tmp %= mPrime;
      (*i1).mCoeff += mPrime - tmp;
      (*i1).mCoeff %= mPrime;
      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree + deg;
    int tmp = (*i2).mCoeff*coeff;
    tmp %= mPrime;
    (*i1).mCoeff = mPrime - tmp;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::mult(const IAlgExtModularShortAux& a) {
  IASSERT(a.mPrime == mPrime);
  if (isZero())
    return;

  IAlgExtModularShortAux prod(mPrime);
  ConstIterator ia(a.mHead.begin());
  Iterator begin(prod.mHead.begin());
  int mp;

  while(ia) {
    ConstIterator i(mHead.begin());
    Iterator ip = begin;
    unsigned degree2 = (*i).mDegree + (*ia).mDegree;

    // look for the starting point
    while(ip && (*ip).mDegree > degree2)
      ++ip;

    begin = ip;

    while(ip && i) {
      degree2 = (*i).mDegree + (*ia).mDegree;
      if ((*ip).mDegree > degree2)
        ++ip;
      else if ((*ip).mDegree < degree2) {
        ip.insert();
        (*ip).mDegree = degree2;
		(*ip).mCoeff = (*i).mCoeff * (*ia).mCoeff % mPrime;
        IASSERT((*ip).mCoeff != 0);
        ++ip;
        ++i;
      }
      else {
	  	mp = (*i).mCoeff * (*ia).mCoeff % mPrime;
		(*ip).mCoeff = ((*ip).mCoeff + mp) % mPrime;
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++i;
      }
    }

    while(i) {
      ip.insert();
      (*ip).mDegree = (*i).mDegree + (*ia).mDegree;
	  (*ip).mCoeff = (*i).mCoeff * (*ia).mCoeff % mPrime;
      IASSERT((*ip).mCoeff != 0);
      ++ip;
      ++i;
    }

    ++ia;
  }
  mHead.swap(prod.mHead);
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::mult(const IAlgExtModularShortAux& a, const IAlgExtModularShortAux& b) {
  IASSERT(a.mPrime == mPrime);
  IASSERT(b.mPrime == mPrime);
  setZero();
  if (a.isZero() || b.isZero())
    return;

  ConstIterator ib(b.mHead.begin());
  Iterator begin(mHead.begin());
  int mp;

  while(ib) {
    ConstIterator ia(a.mHead.begin());
    Iterator ip = begin;
    unsigned degree2 = (*ia).mDegree + (*ib).mDegree;

    // look for the starting point
    while(ip && (*ip).mDegree > degree2)
      ++ip;

    begin = ip;

    while(ip && ia) {
      degree2 = (*ia).mDegree + (*ib).mDegree;
      if ((*ip).mDegree > degree2)
        ++ip;
      else if ((*ip).mDegree < degree2) {
        ip.insert();
        (*ip).mDegree = degree2;
		(*ip).mCoeff = (*ia).mCoeff * (*ib).mCoeff % mPrime;
        IASSERT((*ip).mCoeff != 0);
        ++ip;
        ++ia;
      }
      else {
	  	mp = (*ia).mCoeff * (*ib).mCoeff % mPrime;
		(*ip).mCoeff = ((*ip).mCoeff + mp) % mPrime;
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++ia;
      }
    }

    while(ia) {
      ip.insert();
      (*ip).mDegree = (*ia).mDegree + (*ib).mDegree;
	  (*ip).mCoeff = (*ia).mCoeff * (*ib).mCoeff % mPrime;
      IASSERT((*ip).mCoeff != 0);
      ++ip;
      ++ia;
    }

    ++ib;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::diff() {
  Iterator i(mHead.begin());
  while (i) {
    if ((*i).mDegree == 0) {
      i.del();
      break;
    }
    (*i).mCoeff *= (*i).mDegree;
    (*i).mCoeff %= mPrime;
    if ((*i).mCoeff == 0)
      i.del();
    else { 
      --(*i).mDegree;
      ++i;
    }
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::pow(unsigned deg) {
  IASSERT(deg > 0);
  if (deg > 1) {
    IAlgExtModularShortAux p(*this);
    while(--deg > 0) {
      p.mult(*this);
    }
    mHead.swap(p.mHead);
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::div(const IAlgExtModularShortAux& a) {
  IASSERT(degree() >= a.degree());
  IASSERT(a.mPrime == mPrime);

  int k1, k2, tmp;

  IAlgExtModularShortAux  quo(mPrime);
  Iterator  i(quo.mHead.begin());

  while(!isZero()) {
    IASSERT(degree() >= a.degree());
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg = degree() - a.degree();
    k1 = (*i2).mCoeff;
    k2 = (*i1).mCoeff;

    i.insert();
    (*i).mDegree = deg;
    (*i).mCoeff = k2;
    ++i;
    IASSERTVALID(quo);

    k2 = mPrime - k2;

    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
        (*i1).mCoeff = (*i1).mCoeff * k1 % mPrime;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
        (*i1).mCoeff = (*i2).mCoeff * k2 % mPrime;
        ++i1;
        ++i2;
      }
      else {
        (*i1).mCoeff = (*i1).mCoeff * k1 % mPrime;
        tmp = (*i2).mCoeff * k2 % mPrime;
        (*i1).mCoeff = ((*i1).mCoeff + tmp) % mPrime;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      (*i1).mCoeff = (*i2).mCoeff * k2 % mPrime;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
  IASSERT(isZero());
  mHead.swap(quo.mHead);
}

void IAlgExtModularShortAux::gcd(const IAlgExtModularShortAux& a) {
  IASSERT(a.mPrime == mPrime);
  
  if (a.isZero()) 
    return;
  
  if (isZero())
    *this = a;

  if (degree() == 0 || a.degree() == 0) {
    ConstIterator  i(mHead.begin());
    (*i).mCoeff = 1;
  }
  else {
    IAlgExtModularShortAux tmp(a);

    if (tmp.degree() > degree())
      mHead.swap(tmp.mHead);

    do {
      mHead.swap(tmp.mHead);
      tmp.rem(*this);
    } while (!tmp.isZero() && tmp.degree() > 0);

    if (!tmp.isZero()) {
      mHead.swap(tmp.mHead);
      (*mHead.begin()).mCoeff = 1;
    }
  }
}

void IAlgExtModularShortAux::mod(const IOneParameterGmpZAux& a) {
  mHead.clear();
  
  Iterator i1(mHead.begin());
  IOneParameterGmpZAux::ConstIterator i2(a.begin());
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
    (*i1).mCoeff = mpz_fdiv_ui(&(*i2).mCoeff, mPrime);
    if ((*i1).mCoeff == 0) 
      i1.del();
    else 
      ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::convert(IOneParameterGmpZAux& a) const {
  a.setZero();
  
  ConstIterator i1(mHead.begin());
  IOneParameterGmpZAux::Iterator i2(a.begin());
  while(i1) {
    i2.insert();
    (*i2).mDegree = (*i1).mDegree;
    mpz_set_ui(&(*i2).mCoeff, (*i1).mCoeff);
    ++i2;
    ++i1;
  }
  IASSERTVALID(*this);
}

void IAlgExtModularShortAux::assertValid(const char* fileName, int fileLine) const {
  if (!mHead.isEmpty()) {
    ConstIterator i(mHead.begin());
    IASSERT2(fileName, fileLine, (*i).mCoeff != 0 && (*i).mCoeff < mPrime);
    ConstIterator prev(i);
    ++i;
    while(i) {
      IASSERT2(fileName, fileLine, (*i).mCoeff > 0 && (*i).mCoeff < mPrime);
      IASSERT2(fileName, fileLine, (*prev).mDegree > (*i).mDegree);
      prev = i;
      ++i;
    }
  }
}

void IAlgExtModularShortAux::input(std::istream& in, const IVariables &parameters) {
  additive(in, parameters);
}

void IAlgExtModularShortAux::additive(std::istream& in, const IVariables &parameters) {
  multiplicative(in, parameters);
  int op = (in >> std::ws).peek();
  IAlgExtModularShortAux tmp(mPrime);
  while(op == '+' || op == '-') {
    in.get();
    tmp.multiplicative(in, parameters);
    if (op == '+')
      add(tmp);
    else {
      sub(tmp);
    }
    op = in.peek();
  }
}

void IAlgExtModularShortAux::multiplicative(std::istream& in, const IVariables &parameters) {
  unary(in, parameters);
  int op = (in >> std::ws).peek();
  IAlgExtModularShortAux tmp(mPrime);
  while(op == '*') {
    in.get();
    tmp.unary(in, parameters);
    mult(tmp);
    op = in.peek();
  }
}

void IAlgExtModularShortAux::unary(std::istream& in, const IVariables &parameters) {
  int ch = (in >> std::ws).peek();
  if (ch != '+' && ch != '-')
    power(in, parameters);
  else {
    int k = 1;
    do {
      ch = in.get();
      if (ch == '-')
        k *= -1;
      ch = (in >> std::ws).peek();
    } while (ch == '+' || ch == '-');
    power(in, parameters);
    if (k < 0)
      minus();
  }
}

void IAlgExtModularShortAux::power(std::istream& in, const IVariables &parameters) {
  bracket(in, parameters);
  int op = (in >> std::ws).peek();
  if (op == '^') {
    in.get();
    int deg=0;
    in >> std::ws >> deg;
    if (in.fail() || deg <= 0)
      IMESSAGE("expected 'degree > 0'");
    pow(deg);
  }
}

void IAlgExtModularShortAux::bracket(std::istream& in, const IVariables &parameters) {
  int op = (in >> std::ws).peek();
  if (op == '(') {
    in.get();
    additive(in, parameters);
    if (in.get() != ')') {
      IMESSAGE("expected ')'");
    }
  }
  else {
    std::streampos posbeg = in.tellg();
    int var = parameters.read(in);
    if (var == 0) {
      setOne();
      (*begin()).mDegree = 1;
      IASSERTVALID(*this);
    }
    else {
      int mp;
      in >> mp;
      if (mp < 0) {
        int factor = -(mp / mPrime) + 1;
        mp += factor * mPrime;
      }
      mp %= mPrime;
      if (in.fail()) {
        in.clear();
        in.setstate(std::ios::failbit);
        in.seekg(posbeg);
      }
      else {
        IAlgExtModularShortAux::Iterator i(begin());
        i.clear();
        if (mp != 0) {
          i.insert();
          (*i).mCoeff = mp;
        }
        IASSERTVALID(*this);
      }
    }
  }
}

void IAlgExtModularShortAux::operator=(const IAlgExtModularShortAux& a) {
  mPrime = a.mPrime;
  mHead.clear();

  Iterator i = mHead.begin();
  ConstIterator ia = a.mHead.begin();
  while (ia) {
    i.insert();
    (*i).mDegree = (*ia).mDegree;
    (*i).mCoeff = (*ia).mCoeff;
    ++i;
    ++ia;
  }
}

void IAlgExtModularShortAux::read(std::istream &in) {
  mHead.clear();
  int l;
  in.read((char*)(&l), sizeof(l));
  Iterator i(mHead.begin());
  while(l) {
    i.insert();
    in.read((char*)(&(*i).mDegree), sizeof((*i).mDegree));
    in >> (*i).mCoeff;
    ++i;
    --l;
  }
}

void IAlgExtModularShortAux::write(std::ostream &out) const {
  int l = mHead.length();
  out.write((char*)(&l), sizeof(l));
  ConstIterator i(mHead.begin());
  while(i) {
    out.write((char*)(&(*i).mDegree), sizeof((*i).mDegree));
    out << (*i).mCoeff;
    ++i;
  }
}

std::ostream& operator<<(std::ostream& out, const IAlgExtModularShortAux& a) {
  IAlgExtModularShortAux::ConstIterator i(a.begin());
  if (!i)
    out << '0';
  else {
    out << ((*i).mCoeff) << "*t^" <<  (*i).mDegree;
    ++i;
    while(i) {
      out << " + " << ((*i).mCoeff) << "*t^" <<  (*i).mDegree;
      ++i;
    }
  }
  return out;
}

template <>
IAllocator IList<IAlgExtModularShortAux::Term>::Node::sAllocator(sizeof(IList<IAlgExtModularShortAux::Term>::Node));
