/***************************************************************************
 *   Copyright (C) 2008 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IFACTORIZATIONZ_H
#define IFACTORIZATIONZ_H

#include "ialgextmodularshortaux.h"
#include "ioneparametergmpzaux.h"

#include "../util/imodular.h"

#include <vector>

class IFactorizationZ {

protected:
  void getPoly(IAlgExtModularShortAux& f,
      const IOneParameterGmpZAux& a,
      const IOneParameterGmpZAux& b,
      const IOneParameterGmpZAux& c,
      MP_INT& q);

  void henselLift(IOneParameterGmpZAux& a,
      IOneParameterGmpZAux& b,
      const IOneParameterGmpZAux& c,
      const IAlgExtModularShortAux& u,
      const IAlgExtModularShortAux& v,
      ModularShort p,
      unsigned e);

  void mignotte(MP_INT& b, const IOneParameterGmpZAux& a);

  unsigned getExponent(const IOneParameterGmpZAux& u, ModularShort p);

  void lift(std::vector<IOneParameterGmpZAux*>& result,
      IList<IAlgExtModularShortAux*>& factors,
      IOneParameterGmpZAux& u,
      ModularShort p,
      unsigned e);

  void liftPoly(IOneParameterGmpZAux& x,
      IOneParameterGmpZAux& y,
      IOneParameterGmpZAux& c,
      IAlgExtModularShortAux& a,
      IAlgExtModularShortAux& b,
      ModularShort p,
      unsigned e);

  void polydiv(const IAlgExtModularShortAux& a,
      const IAlgExtModularShortAux& b,
      IAlgExtModularShortAux& q,
      IAlgExtModularShortAux& r);

  void xea(const IAlgExtModularShortAux& a,
      const IAlgExtModularShortAux& b,
      IAlgExtModularShortAux& s,
      IAlgExtModularShortAux& t);

  int testFactor(std::vector<IOneParameterGmpZAux*> lifts, 
      IOneParameterGmpZAux& f, 
      IOneParameterGmpZAux& u, 
      IOneParameterGmpZAux& a, 
      unsigned elements[], 
      int k,
      MP_INT& q);
  
  void setRange(IOneParameterGmpZAux& a, MP_INT& q);

  unsigned* initCombination(int k, int n);
  void updateCombination(unsigned* comb, int k);
  void nextCombination(unsigned* comb, int k, int n);
  bool isValidCombination(unsigned* comb, int k, int n);

  bool* createBinaryString(IList<unsigned>& degrees, int n);
  void createBinaryString(bool* str1, bool* str2, int n);

  
public:
  void factor(const IOneParameterGmpZAux& f,
      IList<IOneParameterGmpZAux*>& factors,
      IList<unsigned>& multiplicities);
};

#endif // IFACTORIZATIONZ_H
