/***************************************************************************
 *   Copyright (C) 2008 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./igcdonevar.h" 
#include "./inparameterutil.h" 

void IGcdOnevar::gcd(INParameterGmpZAux& a,
    INParameterGmpZAux& b,
    INParameterGmpZAux& g) {
  if (a.isZero()) {
    g.set(b);
    return;
  }
  if (b.isZero()) {
    g.set(a);
    return;
  }

  if ((*a.begin()).mMonom[0] == 0) {
    g.setOne();
    if ((*b.begin()).mMonom[0] == 0)
      mpz_gcd(&(*g.mHead.begin()).mCoeff, 
          &(*a.mHead.begin()).mCoeff, 
          &(*b.mHead.begin()).mCoeff);
    else {
      MP_INT cont2;
      mpz_init(&cont2);
      b.cont(cont2);
      mpz_gcd(&(*g.mHead.begin()).mCoeff, &(*a.mHead.begin()).mCoeff, &cont2);
      mpz_clear(&cont2);
    }
  } else if ((*b.begin()).mMonom[0] == 0) {
    g.setOne();
    MP_INT cont1;
    mpz_init(&cont1);
    a.cont(cont1);
    mpz_gcd(&(*g.mHead.begin()).mCoeff, &cont1, &(*b.mHead.begin()).mCoeff);
    mpz_clear(&cont1);
  } else {
    g.set(a);
    INParameterGmpZAux tmp(b);
    MP_INT cont1, cont2, g12;
    mpz_init(&cont1);
    mpz_init(&cont2);
    mpz_init(&g12);

    g.cont(cont1);
    tmp.cont(cont2);

    if (mpz_cmp_ui(&cont1, 1ul) != 0) {
      g.div(cont1);
      if (mpz_cmp_ui(&cont2, 1ul) == 0)
        mpz_set_ui(&g12, 1ul);
      else {
        tmp.div(cont2);
        mpz_gcd(&g12, &cont1, &cont2);
      }
    } else {
      if (mpz_cmp_ui(&cont2, 1ul) != 0)
        tmp.div(cont2);
      mpz_set_ui(&g12, 1ul);
    }

    if ((*tmp.begin()).mMonom[0] > (*g.begin()).mMonom[0])
      g.mHead.swap(tmp.mHead);

    do {
      g.mHead.swap(tmp.mHead);
      tmp.rem(g);
    } while (!tmp.isZero() && (*tmp.begin()).mMonom[0] > 0);

    if (!tmp.isZero()) {
      g.mHead.swap(tmp.mHead);
      mpz_set(&(*g.mHead.begin()).mCoeff, &g12);
    } else {
      g.cont(cont1);
      if (mpz_cmp_ui(&cont1, 1ul) != 0)
        g.div(cont1);
      g.mult(g12);
    }
    mpz_clear(&cont1);
    mpz_clear(&cont2);
    mpz_clear(&g12);
  }
}
