/***************************************************************************
 *   Copyright (C) 2007 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IGCDZIPPEL_H
#define IGCDZIPPEL_H

#include <vector>

#include "igcdalgorithm.h"
#include "imonomaux.h"
#include "../monom/imonom.h"
#include "../gauss/izippelgauss.h"

#ifdef TEST
class IGcdZippelTest;
#endif

class IUnluckyError {

};

class IBadFormError {

};

class IGcdZippel: public IGcdAlgorithm {
#ifdef TEST
  friend class IGcdZippelTest;
#endif

private:
  void zippelP(
      INParameterModularShortAux& a,
      INParameterModularShortAux& b,
      INParameterModularShortAux& g,
      IMonomAuxInterface::MonomIterator d,
      bool deterministic = false);

  void zippelM(INParameterGmpZAux& a, INParameterGmpZAux& b, INParameterGmpZAux& g, IMonomAuxInterface::MonomIterator d);

  int checkForMonomial(INParameterModularShortAux& a);
  int numberOfTerms(INParameterModularShortAux& a);

  void addToList(
      INParameterModularShortAux* g, 
      ModularShort v, 
      IOneParameterModularShortAux& gamma, 
      std::vector<INParameterModularShortAux*>& gcds, 
      std::vector<ModularShort>& alpha);

  INParameterModularShortAux* getGcdByEvaluation(
      INParameterModularShortAux& form, 
      INParameterModularShortAux& a_p, 
      INParameterModularShortAux& b_p, 
      IMonomAuxInterface::MonomIterator d);

  void getCoeffs(const INParameterGmpZAux& a, IList<INParameterGmpZAux*>& coeffs);
  void removeContent(INParameterGmpZAux& a, INParameterGmpZAux& b, INParameterGmpZAux& content);
  void getMinimum(IList<INParameterGmpZAux*>& list, INParameterGmpZAux& minimum);
  void getRandomLinearCombination(IList<INParameterGmpZAux*>& list, INParameterGmpZAux& linearcombination, gmp_randstate_t state);
  void convert(const INParameterGmpZAux& a, INParameterGmpZAux& result);
public:
  void gcd(INParameterGmpZAux& a, INParameterGmpZAux& b, INParameterGmpZAux& g);
};

#endif // IGCDZIPPEL_H
