/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IGMPZ_H
#define IGMPZ_H

#include <gmp.h>

#include "../util/iallocator.h"
#include "./icoeff.h"


class IGmpZ;
class IGmpZInterface: public ICoeffInterface {
  friend class IGmpZ;

public:
  IGmpZInterface();
  ~IGmpZInterface() {}

  ICoeff* create();
  ICoeff* copy(const ICoeff& a);
};

class IGmpZ: public ICoeff {
  friend class IGmpZInterface;

protected:
  MP_INT            mMP_INT;

  static IAllocator sAllocator;

  IGmpZ(IGmpZInterface* r);
  IGmpZ(const ICoeff& a);

public:
  void* operator new(size_t) { return sAllocator.allocate(); }
  void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

  ~IGmpZ();

  void set(const ICoeff& a);
  void setZero();
  void setOne();

  bool isNegative() const;
  bool isZero() const;
  bool isOne() const;
  bool isComposite() const;
  bool isUnit() const;
  int norm() const;

  void minus();
  void abs();
  void add(const ICoeff& a);
  void sub(const ICoeff& a);
  void mult(const ICoeff& a);
  void diff(int var, unsigned deg);
  void shift(int var, unsigned deg);
  void pow(unsigned deg);
  void inverse();
  bool divisibility(const ICoeff& a) const;
  void div(const ICoeff& a);
  void div(const ICoeff& a, const ICoeff& b);
  void gcd(const ICoeff& a);
  void gcd(const ICoeff& a, const ICoeff& b);
  void addmul(const ICoeff& a, const ICoeff& b);
  void addmul(const ICoeff& a, unsigned long b);

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void input(std::istream& in, bool expanded=false);
  void output(std::ostream& out) const;
};

#endif // IGMPZ_H
