/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <cstdio>

#include <gmp.h>

#include "../util/iutil.h"
#include "../util/imodular.h"
#include "./imodularshort.h"


IModularShortInterface::IModularShortInterface(int primeShort):
    ICoeffInterface(),
    mPrimeShort(primeShort) {

  if ((1 << (sizeof(int)*4)-1) < mPrimeShort)
    IERROR("primeShort^2 > (sizeof(int)*4-1)^2");

  if (!iIsPrime(primeShort))
    IERROR("primeShort is not primer");

  mType = ICoeffInterface::ModularShort;
}

ICoeff* IModularShortInterface::create() {
  return new IModularShort(this);
}

ICoeff* IModularShortInterface::copy(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  return new IModularShort(a);
}

IModularShort::IModularShort(IModularShortInterface* r):
    ICoeff(r) {
  mModularShort = 0;
}

IModularShort::IModularShort(const ICoeff &a):
    ICoeff(a.coeffInterface()) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  mModularShort = ((const IModularShort&)a).mModularShort;
  IASSERTVALID(*this);
}

IModularShort::~IModularShort() {
}

void IModularShort::set(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  mModularShort = ((const IModularShort&)a).mModularShort;
  IASSERTVALID(*this);
}

void IModularShort::setZero() {
  mModularShort = 0;
}

void IModularShort::setOne() {
  mModularShort = 1;
}

bool IModularShort::isNegative() const {
  return false;
}

bool IModularShort::isZero() const {
  return mModularShort == 0;
}

bool IModularShort::isOne() const {
  return mModularShort == 1;
}

bool IModularShort::isComposite() const {
  return false;
}

bool IModularShort::isUnit() const {
  return !isZero();
}

int IModularShort::norm() const {
  return 1;
}

void IModularShort::minus() {
  if (mModularShort)
    mModularShort = primeShort() - mModularShort;
  IASSERTVALID(*this);
}

void IModularShort::abs() {
}

void IModularShort::add(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  mModularShort += ((const IModularShort&)a).mModularShort;
  if (mModularShort >= primeShort())
    mModularShort -= primeShort();
  IASSERTVALID(*this);
}

void IModularShort::sub(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  mModularShort -= ((const IModularShort&)a).mModularShort;
  if (mModularShort < 0)
    mModularShort += primeShort();
  IASSERTVALID(*this);
}

void IModularShort::mult(const ICoeff& a){
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  mModularShort = mModularShort * ((const IModularShort&)a).mModularShort % primeShort();
  IASSERTVALID(*this);
}

void IModularShort::diff(int , unsigned ) {
  mModularShort = 0;
}

void IModularShort::shift(int , unsigned ) {
}

void IModularShort::pow(unsigned deg) {
  mModularShort = iPow1(mModularShort, deg, primeShort());
  IASSERTVALID(*this);
}

void IModularShort::inverse() {
  IASSERT(mModularShort != 0);
  mModularShort = iInverse(mModularShort, primeShort());
  IASSERTVALID(*this);
}

void IModularShort::div(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  IASSERT(((const IModularShort&)a).mModularShort != 0);
  mModularShort = (mModularShort * iInverse(((const IModularShort&)a).mModularShort, primeShort())) % primeShort();
  IASSERTVALID(*this);
}

void IModularShort::div(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  IASSERT(b.type() == ICoeffInterface::ModularShort);
  IASSERT(((const IModularShort&)b).mModularShort != 0);
  mModularShort = (((const IModularShort&)a).mModularShort *
      iInverse(((const IModularShort&)b).mModularShort, primeShort())) % primeShort();
  IASSERTVALID(*this);
}

void IModularShort::gcd(const ICoeff& ) {
  IERROR("IModularShort::gcd");
}

void IModularShort::gcd(const ICoeff& , const ICoeff& ) {
  IERROR("IModularShort::gcd");
}

void IModularShort::addmul(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::ModularShort);
  IASSERT(b.type() == ICoeffInterface::ModularShort);
  mModularShort = (mModularShort + ((const IModularShort&)a).mModularShort * ((const IModularShort&)b).mModularShort) % primeShort();
  IASSERTVALID(*this);
}

void IModularShort::read(std::istream &in) {
  in >> mModularShort;
  IASSERTVALID(*this);
}

void IModularShort::write(std::ostream &out) const {
  out << mModularShort;
}

void IModularShort::input(std::istream& in, bool expanded) {
  MP_INT mp;
  mpz_init(&mp);
  in >> (&mp);
  if (!in.fail()) {
    int sign = mpz_sgn(&mp);
    mpz_mod_ui(&mp, &mp, (unsigned long)primeShort());
    mModularShort = (int)mpz_get_ui(&mp);
    if (sign < 0)
      mModularShort = primeShort() - mModularShort;
    IASSERTVALID(*this);
  }
  mpz_clear(&mp);
}

void IModularShort::output(std::ostream& out) const {
  out << mModularShort;
}

void IModularShort::assertValid(const char* fileName, int fileLine) const {
  IASSERT2(fileName, fileLine, 0 <= mModularShort);
  IASSERT2(fileName, fileLine, mModularShort < primeShort());
}

IAllocator IModularShort::sAllocator(sizeof(IModularShort));
