/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./imonomaux.h"

IMonomAuxInterface::MonomIterator IMonomAuxInterface::copy(IMonomAuxInterface::MonomConstIterator ia) const {
  MonomIterator i = create();
  for (int j = 0; j < mEndVar; j++) {
    i[j] = ia[j];
  }
  for (int j = mEndVar; j < mN; j++) {
    i[j] = 0;
  }
  //IASSERT(assertValid(i));
  return i;
}

IMonomAuxInterface::MonomIterator IMonomAuxInterface::copy(IMonomAuxInterface::MonomConstIterator ia, unsigned* permutation) const {
  MonomIterator i = create();
  for (int j = 0; j < mEndVar; j++) {
    i[j] = ia[permutation[j]];
  }
  for (int j = mEndVar; j < mN; j++) {
    i[j] = 0;
  }
  //IASSERT(assertValid(i));
  return i;
}


IMonomAuxInterface::MonomIterator IMonomAuxInterface::shiftdown(
    MonomConstIterator ia) const {
  
  MonomIterator result = new Word[mN];
  for (int j = 0; j < mEndVar; j++) {
    result[j] = ia[j+1];
  }
  for (int j = mEndVar; j < mN; j++) {
    result[j] = 0;
  }
  return result;
}


IMonomAuxInterface::MonomIterator IMonomAuxInterface::shiftdown(
    MonomConstIterator ia,
    int var) const {
  
  MonomIterator result = new Word[mN];
  for (int j = 0; j < mEndVar; j++) {
    result[j] = ia[j+var];
  }
  for (int j = mEndVar; j < mN; j++) {
    result[j] = 0;
  }
  return result;
}

IMonomAuxInterface::MonomIterator IMonomAuxInterface::shiftup(
    MonomConstIterator ia) const {
  
  MonomIterator result = new Word[mN];
  result[0] = 0;
  for (int j = 1; j < mN; j++) {
    result[j] = ia[j - 1];
  }
  return result;
}


IMonomAuxInterface::MonomIterator IMonomAuxInterface::shiftup(
    MonomConstIterator ia,
    int var) const {
  
  MonomIterator result = new Word[mN];
  for (int i = 0; i < var; i++) {
    result[i] = 0;
  }
  for (int j = var; j < mN; j++) {
    result[j] = ia[j - var];
  }
  return result;
}


void IMonomAuxInterface::shiftupInplace(MonomIterator ia,int var) const {
  IASSERT(var >= 0 && var < mN);
  
  for (int j = mEndVar - 1; j >= var; j--) {
    ia[j] = ia[j - var];
  }
  for (int j = 0; j < var; j++) {
    ia[j] = 0;
  }
}


unsigned IMonomAuxInterface::degree(MonomConstIterator i) const {
  
  unsigned d = 0;
  MonomConstIterator end = i + mEndVar;
  do {
    d += *i++;
  } while(i < end);
  return d;
}


void IMonomAuxInterface::set(MonomIterator i, IMonom& m) const {
  for (int j = 0; j < mEndVar; j++) {
    i[j] = m[j];
  }
  for (int j = mEndVar; j < mN; j++) {
    i[j] = 0;
  }
}

void IMonomAuxInterface::set(IMonom& m, MonomConstIterator i) const {
  m.setZero();
  for (int j = 0; j < mN; j++) {
    m.prolong(j, i[j]);
  }
}

void IMonomAuxInterface::set(IMonomAuxInterface::MonomIterator i,
                         IMonomAuxInterface::MonomConstIterator ia) const {
  MonomConstIterator end = i + mEndVar;
  do {
    *i++ = *ia++;
  } while(i < end);
  IASSERT(assertValid(end - mEndVar));
}

void IMonomAuxInterface::setZero(IMonomAuxInterface::MonomIterator i) const {
  MonomConstIterator end = i + mN;
  do {
    *i++ = 0;
  } while(i < end);
  IASSERT(assertValid(end - mN));
}

bool IMonomAuxInterface::isZero(MonomConstIterator i) const {
  MonomConstIterator end = i + mN;
  for (int j = 0; j < mEndVar; j++) {
    if (i[j] != 0)
      return false;
  }

  return true;
}

void IMonomAuxInterface::mult(IMonomAuxInterface::MonomIterator i,
                          IMonomAuxInterface::MonomConstIterator ia) const {
  MonomConstIterator end = i + mEndVar;
  do {
    *i++ += *ia++;
  } while(i < end);
  IASSERT(assertValid(end - mEndVar));
}

void IMonomAuxInterface::mult(IMonomAuxInterface::MonomIterator i,
                          IMonomAuxInterface::MonomConstIterator ia,
                          IMonomAuxInterface::MonomConstIterator ib) const {
  MonomConstIterator end = i + mEndVar;
  do {
    *i++ = *ia++ + *ib++;
  } while(i < end);
  IASSERT(assertValid(end - mEndVar));
}

bool IMonomAuxInterface::divisibility(IMonomAuxInterface::MonomConstIterator i,
                                  IMonomAuxInterface::MonomConstIterator ia) const {
  MonomConstIterator end = i + mEndVar;
  do {
    if (*i++ < *ia++)
      return false;
  } while(i < end);
  return true;
}

void IMonomAuxInterface::divide(IMonomAuxInterface::MonomIterator i,
                            IMonomAuxInterface::MonomConstIterator ia,
                            IMonomAuxInterface::MonomConstIterator ib) const {
  IASSERT(divisibility(ia, ib));
  for (int j = 0; j < mEndVar; j++) {
    i[j] = ia[j] - ib[j];
  }
  for (int j = mEndVar; j < mN; j++) {
    i[j] = 0;
  }
  IASSERT(assertValid(i));
}


void IMonomAuxInterface::divide(IMonomAuxInterface::MonomIterator i,
                            IMonomAuxInterface::MonomConstIterator ia) const {
  IASSERT(divisibility(i, ia));
  for (int j = 0; j < mEndVar; j++) {
    i[j] -= ia[j];
  }
  for (int j = mEndVar; j < mN; j++) {
    i[j] = 0;
  }
  IASSERT(assertValid(i));
}


void IMonomAuxInterface::divide(MonomIterator i, MonomConstIterator ia, int deg) const {
  IASSERT(ia[mEndVar - 1] >= deg);
  MonomConstIterator end = i + mEndVar - 1;
  do {
    *i++ = *ia++;
  } while(i < end);
  *i = *ia - deg;
  IASSERT(assertValid(end - mEndVar));
}


void IMonomAuxInterface::gcd(MonomIterator i, MonomConstIterator ia) const {
  for (int j = 0; j < mEndVar; j++) {
    if (ia[j] < i[j])
      i[j] = ia[j];
  }
  IASSERT(assertValid(i));
}


void IMonomAuxInterface::lcm(MonomIterator i, MonomConstIterator ia) const {
  for (int j = 0; j < mEndVar; j++) {
    if (ia[j] > i[j])
      i[j] = ia[j];
  }
  IASSERT(assertValid(i));
}


int IMonomAuxInterface::compare(IMonomAuxInterface::MonomConstIterator i,
                            IMonomAuxInterface::MonomConstIterator ia) const {
  MonomConstIterator end = i + mEndVar;
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i == end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
}

int IMonomAuxInterface::compare(IMonomAuxInterface::MonomConstIterator i,
                            IMonomAuxInterface::MonomConstIterator ia,
                            IMonomAuxInterface::MonomConstIterator ib) const {
  MonomConstIterator end = i + mEndVar;
  while(i < end && *i == *ia + *ib) {
    ++i;
    ++ia;
    ++ib;
  }
  if (i == end)
    return 0;
  else if (*i > *ia + *ib)
    return 1;
  else
    return -1;
}

int IMonomAuxInterface::compare(MonomConstIterator i, MonomConstIterator ia, int deg) const {
  //TODO: check, if -1 is really ok
  MonomConstIterator end = i + mEndVar - 1;
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i == end) {
    if (*i > *ia + deg)
      return 1;
    else if (*i == *ia + deg)
      return 0;
    else
      return -1;
  } 
  else if (*i > *ia)
    return 1;
  else
    return -1;

}

int IMonomAuxInterface::compare1(IMonomAuxInterface::MonomConstIterator i,
                             IMonomAuxInterface::MonomConstIterator ia) const {
  MonomConstIterator end = i + mEndVar - 1;
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i == end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
}

int IMonomAuxInterface::compareToVar(MonomConstIterator i, 
    MonomConstIterator ia, 
    int var) const {
  
  MonomConstIterator end = i + var;
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i == end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
}

unsigned* IMonomAuxInterface::sortDecreasing(MonomIterator i) const {
  unsigned* result = new unsigned[mEndVar];

  int* tmp = new int[mEndVar];
  for (int j = 0; j < mEndVar; j++) {
    tmp[j] = i[j];
  }

  int max;

  for (int j = 0; j < mEndVar; j++) {
    result[j] = 0;
    max = 0;

    for (int k = 1; k < mEndVar; k++) {
      if (tmp[k] > tmp[result[j]]) {
        result[j] = k;
        max = k;
      }
    }

    tmp[max] = -1;
  }

  delete[] tmp;

  return result;
}

unsigned* IMonomAuxInterface::sortZeroesThenDecreasing(MonomIterator i) const {
  unsigned* result = new unsigned[mEndVar];

  int* tmp = new int[mEndVar];
  for (int j = 0; j < mEndVar; j++) {
    tmp[j] = i[j];
  }
  
  int zeroes = 0;
  for (int j = 0; j < mEndVar; j++) {
    if (tmp[j] == 0) {
      result[zeroes] = j;
      tmp[j] = -1;
      zeroes++;
    }
  }

  int max;

  for (int j = zeroes; j < mEndVar; j++) {
    result[j] = 0;
    max = 0;

    for (int k = 1; k < mEndVar; k++) {
      if (tmp[k] > tmp[result[j]]) {
        result[j] = k;
        max = k;
      }
    }

    tmp[max] = -1;
  }

  delete[] tmp;

  return result;
}

unsigned* IMonomAuxInterface::inversePermutation(
    const unsigned* permutation) const {
  
  unsigned* result = new unsigned[mEndVar];

  for (int i = 0; i < mEndVar; i++) {
    result[permutation[i]] = i;
  }

  return result;
}


void IMonomAuxInterface::read(IMonomAuxInterface::MonomIterator i, std::istream &in) const {
  in.read((char*)i, sizeof(IMonomAuxInterface::Word)*mN);
  IASSERT(assertValid(i));
}

void IMonomAuxInterface::write(IMonomAuxInterface::MonomConstIterator i, std::ostream &out) const {
  out.write((const char*)i, sizeof(IMonomAuxInterface::Word)*mN);
}

void IMonomAuxInterface::output(MonomConstIterator i, std::ostream &out) const {
  if (degree(i) == 0) {
    out << '1';
    return;
  }

  int j = 0;
  while (deg(i, j) == 0)
    j++;
    
  while (j < mEndVar) {
    if (deg(i,j) > 0) {
      out << "x" << j+1 << "^" << deg(i, j);
    }
    if (j < mEndVar - 1 && deg(i, j+1) > 0)
      out << "*";

    j++;
  }
}

bool IMonomAuxInterface::assertValid(IMonomAuxInterface::MonomConstIterator i) const {
  MonomConstIterator end1 = i + mEndVar;
  //MonomConstIterator end = i + mN;
  do {
    if (*i++ < 0)
      return false;
  } while(i < end1);
//  while(i < end) {
//    if (*i++ != 0)
//      return false;
//  }
  return true;
}
