/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IMONOMAUX_H
#define IMONOMAUX_H

#include <iostream>

#include "../monom/imonom.h"

class IMonomAuxInterface {
public:
  typedef unsigned         Word;
  typedef Word*            MonomIterator;
  typedef const Word*      MonomConstIterator;

  int mN;
  int mEndVar;

  IMonomAuxInterface(int n): mN(n), mEndVar(n) { }
  IMonomAuxInterface(const IMonomAuxInterface& interface): mN(interface.mN), mEndVar(interface.mEndVar) { }

  IMonomAuxInterface& operator=(const IMonomAuxInterface& interface) {
    mN = interface.mN;
    mEndVar = interface.mEndVar;
    return *this;
  }

  bool operator==(const IMonomAuxInterface& interface) const {
    return (mN == interface.mN) && (mEndVar == interface.mEndVar);
  }

  MonomIterator create() const {
    return new Word[mN];
  }

  MonomIterator copy(MonomConstIterator ia) const ;

  MonomIterator copy(MonomConstIterator ia, unsigned* permutation) const ;

  MonomIterator shiftdown(MonomConstIterator ia) const ;
  MonomIterator shiftdown(MonomConstIterator ia, int var) const ;

  MonomIterator shiftup(MonomConstIterator ia) const ;
  MonomIterator shiftup(MonomConstIterator ia, int var) const ;
  void shiftupInplace(MonomIterator ia,int var) const;

  void setEndVar(int endVar) {
    IASSERT(0 < endVar && endVar <= mN);
    mEndVar = endVar;
  }

  void incEndVar() {
    IASSERT(mEndVar < mN);
    mEndVar++;
  }

  int getEndVar() const {
    return mEndVar;
  }

  unsigned deg(MonomConstIterator i, int var) const {
    IASSERT(0 <= var && var < mEndVar);
    return i[var];
  }
  unsigned degree(MonomConstIterator i) const;

  void set(MonomIterator i, MonomConstIterator ia) const;
  void set(MonomIterator i, IMonom& m) const;
  void set(IMonom& m, MonomConstIterator i) const;
  void setZero(MonomIterator i) const;

  bool isZero(MonomConstIterator i) const;

  void swap(MonomIterator i, MonomIterator ia) const {
    MonomIterator tmp = i;
    i = ia;
    ia = tmp;
  }

  void prolong(MonomIterator i, int var, int deg) const {
    IASSERT(0 <= var && var < mEndVar);
    IASSERT(i[var] + deg >= 0);
    i[var] += deg;
    IASSERT(assertValid(i));
  }

  void prolong(MonomIterator i, int deg) const {
    IASSERT(i[mEndVar-1] + deg >= 0);
    i[mEndVar-1] += deg;
    IASSERT(assertValid(i));
  }

  void zeroEndVar(MonomIterator i) const {
    i[mEndVar-1] = 0;
    IASSERT(assertValid(i));
  }
  void mult(MonomIterator i, MonomConstIterator ia) const;
  void mult(MonomIterator i, MonomConstIterator ia, MonomConstIterator ib) const;

  bool divisibility(MonomConstIterator i, MonomConstIterator ia) const;
  void divide(MonomIterator i, MonomConstIterator ia) const;
  void divide(MonomIterator i, MonomConstIterator ia, MonomConstIterator ib) const;
  void divide(MonomIterator i, MonomConstIterator ia, int deg) const;

  void gcd(MonomIterator i, MonomConstIterator ia) const;
  void lcm(MonomIterator i, MonomConstIterator ia) const;

  int compare(MonomConstIterator i, MonomConstIterator ia) const;
  int compare(MonomConstIterator i, MonomConstIterator ia, MonomConstIterator ib) const;
  int compare(MonomConstIterator i, MonomConstIterator ia, int deg) const;
  int compare1(MonomConstIterator i, MonomConstIterator ia) const;
  int compareToVar(MonomConstIterator i, MonomConstIterator ia, int var) const;

  unsigned* sortDecreasing(MonomIterator i) const;
  unsigned* sortZeroesThenDecreasing(MonomIterator i) const;
  unsigned* inversePermutation(const unsigned* permutation) const;


  void read(MonomIterator i, std::istream &in) const;
  void write(MonomConstIterator i, std::ostream &out) const;

  void output(MonomConstIterator i, std::ostream &out) const;

  void output(MonomConstIterator i, IMonomInterface* mi, std::ostream &out) const {
    IMonom* tmp = mi->create();
    set(*tmp, i);
    out << *tmp;
    delete tmp;
  }

  bool assertValid(MonomConstIterator i) const;
};

#endif // IMONOMAUX_H
