/***************************************************************************
 *   Copyright (C) 2008 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INPARAMETERFIELD_H
#define INPARAMETERFIELD_H

#include "../monom/imonom.h"
#include "../util/iallocator.h"
#include "icoeff.h"
#include "inparametergmpzaux.h"

class INParameterField;

class INParameterFieldInterface : public ICoeffInterface {
  friend class INParameterField;

  IMonomInterface* mi;

public:
  INParameterFieldInterface(IMonomInterface *mi);
  
  ~INParameterFieldInterface() {
  }

  ICoeff* create();
  ICoeff* copy(const ICoeff& a);
};

class INParameterField : public ICoeff {
  friend class INParameterFieldInterface;

protected:
  INParameterGmpZAux mNumer;
  INParameterGmpZAux mDenom;
  IMonomInterface* mi;

  static IAllocator sAllocator;

  INParameterField(INParameterFieldInterface* r, IMonomInterface* mi);
  INParameterField(const ICoeff& a);
  
  IMonomInterface* monomInterface() const {
      return mi;
  }

public:
  void* operator new(size_t) {
    return sAllocator.allocate();
  }
  void operator delete(void *ptr) {
    sAllocator.deallocate(ptr);
  }

  ~INParameterField();

  void set(const ICoeff& a);
  void setZero();
  void setOne();

  bool isNegative() const;
  bool isZero() const;
  bool isOne() const;
  bool isComposite() const;
  bool isUnit() const;
  int norm() const;

  void minus();
  void abs();
  void add(const ICoeff& a);
  void sub(const ICoeff& a);
  void mult(const ICoeff& a);
  void diff(int var, unsigned deg);
  void shift(int var, unsigned deg);
  void pow(unsigned deg);
  void inverse();
  void div(const ICoeff& a);
  void div(const ICoeff& a, const ICoeff& b);
  void gcd(const ICoeff& a);
  void gcd(const ICoeff& a, const ICoeff& b);
  void addmul(const ICoeff& a, const ICoeff& b);

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void input(std::istream& in, bool expanded = false);
  void output(std::ostream& out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // INPARAMETERFIELD_H
