/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <vector>
#include "inparametergmpz.h"

INParameterGmpZInterface::INParameterGmpZInterface(IMonomInterface* mi):
    ICoeffInterface(),
    mi(mi) {

  mType = ICoeffInterface::NParameterGmpZ;
}

ICoeff* INParameterGmpZInterface::create() {
  return new INParameterGmpZ(this, mi);
}

ICoeff* INParameterGmpZInterface::copy(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  return new INParameterGmpZ(a);
}

INParameterGmpZ::INParameterGmpZ(INParameterGmpZInterface* r, IMonomInterface* mi):
    ICoeff(r),
    mi(mi) {

  mPoly = new INParameterGmpZAux(mi->dimIndepend());
}

INParameterGmpZ::INParameterGmpZ(const ICoeff &a):
    ICoeff(a.coeffInterface()),
    mi(((const INParameterGmpZ&)a).mi) {

  mPoly = new INParameterGmpZAux(*((const INParameterGmpZ&)a).mPoly);
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  IASSERTVALID(*this);
}
 
INParameterGmpZ::~INParameterGmpZ() {
  delete mPoly;
}

void INParameterGmpZ::set(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly = ((const INParameterGmpZ&)a).mPoly;
  IASSERTVALID(*this);
}

void INParameterGmpZ::setZero() {
  mPoly->setZero();
}

void INParameterGmpZ::setOne() {
  mPoly->setOne();
}

bool INParameterGmpZ::isNegative() const {
  return mPoly->isNegative();
}

bool INParameterGmpZ::isZero() const {
  return mPoly->isZero();
}

bool INParameterGmpZ::isOne() const {
  return mPoly->isOne();
}

bool INParameterGmpZ::isComposite() const {
  return mPoly->isComposite();
}

bool INParameterGmpZ::isUnit() const {
  return (!mPoly->isZero() && mPoly->degree() == 0);
}

int INParameterGmpZ::norm() const {
  return mPoly->norm();
}

void INParameterGmpZ::minus() {
  mPoly->minus();
}

void INParameterGmpZ::abs() {
}

void INParameterGmpZ::add(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->add(*((const INParameterGmpZ& )a).mPoly);
  IASSERTVALID(*this);
}

void INParameterGmpZ::sub(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->sub(*((const INParameterGmpZ&)a).mPoly);
  IASSERTVALID(*this);
}

void INParameterGmpZ::mult(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->mult(*((const INParameterGmpZ&)a).mPoly);
  IASSERTVALID(*this);
}

void INParameterGmpZ::diff(int , unsigned ) {
}

void INParameterGmpZ::shift(int , unsigned ) {
}

void INParameterGmpZ::pow(unsigned deg) {
  mPoly->pow(deg);
  IASSERTVALID(*this);
}

void INParameterGmpZ::inverse() {
  IERROR("inverse");
}

void INParameterGmpZ::div(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->div(*((INParameterGmpZ &)a).mPoly);
  IASSERTVALID(*this);
}

void INParameterGmpZ::divtry(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->divtry(*((INParameterGmpZ &)a).mPoly);
  IASSERTVALID(*this);
}

void INParameterGmpZ::div(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  IASSERT(b.type() == ICoeffInterface::NParameterGmpZ);
  *mPoly = *((INParameterGmpZ &)a).mPoly;
  mPoly->div(*((INParameterGmpZ &)b).mPoly);
  // TODO: something more efficient?
  IASSERTVALID(*this);
}

void INParameterGmpZ::gcd(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  
  INParameterGmpZAux g(mi->dimIndepend());
  g.gcd(*mPoly, *((INParameterGmpZ &)a).mPoly);
  mPoly->swap(g);

  IASSERTVALID(*this);
}

void INParameterGmpZ::gcd(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  IASSERT(b.type() == ICoeffInterface::NParameterGmpZ);

  mPoly->gcd(*((INParameterGmpZ &)a).mPoly, *((INParameterGmpZ &)b).mPoly);

  IASSERTVALID(*this);
}

void INParameterGmpZ::addmul(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  IASSERT(b.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->addmul(*((const INParameterGmpZ&)a).mPoly, *((const INParameterGmpZ&)b).mPoly);
  IASSERTVALID(*this);
}

void INParameterGmpZ::addmul(const ICoeff& a, const MP_INT& b) {
  IASSERT(a.type() == ICoeffInterface::NParameterGmpZ);
  mPoly->addmul(*((const INParameterGmpZ&)a).mPoly, b);
  IASSERTVALID(*this);
}

inline void INParameterGmpZ::read(std::istream &in) {
  mPoly->read(in, mi);
  IASSERTVALID(*this);
}

inline void INParameterGmpZ::write(std::ostream &out) const {
  mPoly->write(out, mi);
}

void INParameterGmpZ::input(std::istream& in, bool expanded) {
  mPoly->input(in, expanded, mi);

  IASSERTVALID(*this);
}

void INParameterGmpZ::output(std::ostream& out) const {
  if (isZero())
    out << '0';
  else {
    MP_INT mp;
    mpz_init(&mp);
    INParameterGmpZAux::ConstIterator i(mPoly->begin());
    if (mpz_cmp_ui(&(*i).mCoeff, 1ul) == 0) {
      if (mPoly->interface().degree((*i).mMonom))
        mPoly->interface().output((*i).mMonom, mi, out);
      else
        out << '1';
    }
    else {
      // INParameterGmpZ::INParameterGmpZAux::isComposite
      // cares for minus sign!
      out << (&(*i).mCoeff);
      if (mPoly->interface().degree((*i).mMonom)) {
        out << '*';
        mPoly->interface().output((*i).mMonom, mi, out);
      }
    }

    ++i;
    while(i) {
      if (mpz_sgn(&(*i).mCoeff) > 0) {
        out << " + ";
        if (mpz_cmp_ui(&(*i).mCoeff, 1ul) == 0) {
          if (mPoly->interface().degree((*i).mMonom))
            mPoly->interface().output((*i).mMonom, mi, out);
          else
            out << '1';
        }
        else {
          out << &(*i).mCoeff;
          if (mPoly->interface().degree((*i).mMonom)) {
            out << '*';
            mPoly->interface().output((*i).mMonom, mi, out);
	  }
        }
      }
      else {
        out << " - ";
        if (mpz_cmp_si(&(*i).mCoeff, -1) == 0) {
          if (mPoly->interface().degree((*i).mMonom))
            mPoly->interface().output((*i).mMonom, mi, out);
          else
            out << '1';
        }
        else {
          mpz_neg(&mp, &(*i).mCoeff);
          out << (&mp);
          if (mPoly->interface().degree((*i).mMonom)) {
            out << '*';
            mPoly->interface().output((*i).mMonom, mi, out);
	  }
        }
      }
      ++i;
    }
    mpz_clear(&mp);
  }
}

void INParameterGmpZ::assertValid(const char* fileName, int fileLine) const {
  mPoly->assertValid(fileName, fileLine);
}

IAllocator INParameterGmpZ::sAllocator(sizeof(INParameterGmpZ));
