/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./inparametermodularshortaux.h"
#include <gmp.h>

INParameterModularShortAux::INParameterModularShortAux(const INParameterModularShortAux& a):
    mInterface(a.interface()),
    mPrimeShort(a.mPrimeShort),
    mHead() {

  Iterator      i1(mHead.begin());
  ConstIterator i2(a.mHead.begin());
  while(i2) {
    i1.insert();
    (*i1).mMonom = interface().copy((*i2).mMonom);
    (*i1).mCoeff = (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

INParameterModularShortAux::INParameterModularShortAux(const INParameterModularShortAux& a, MonomConstIterator m, const ModularShort c):
    mInterface(a.interface()),
    mPrimeShort(a.mPrimeShort),
    mHead() {

  IASSERTVALID(a);
  Iterator i(mHead.begin());
  ConstIterator ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = interface().copy((*ia).mMonom);
    interface().mult((*i).mMonom, m);
    (*i).mCoeff = (*ia).mCoeff * c;
    (*i).mCoeff %= mPrimeShort;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

INParameterModularShortAux::INParameterModularShortAux(IMonomAuxInterface interf, const IOneParameterModularShortAux& a, MonomConstIterator m, const ModularShort c):
    mInterface(interf),
    mPrimeShort(a.mPrimeShort),
    mHead() {
  Iterator i(mHead.begin());
  IOneParameterModularShortAux::ConstIterator ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = interface().copy(m);
    interface().prolong((*i).mMonom, (*ia).mDegree);
    (*i).mCoeff = (*ia).mCoeff * c;
    (*i).mCoeff %= mPrimeShort;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}


INParameterModularShortAux::INParameterModularShortAux(const INParameterGmpZAux& a, ModularShort primeShort):
    mInterface(a.interface()),
    mPrimeShort(primeShort),
    mHead() {

  Iterator i1(mHead.begin());
  INParameterGmpZAux::ConstIterator i2(a.mHead.begin());
  while(i2) {
    i1.insert();
    (*i1).mMonom = interface().copy((*i2).mMonom);
    (*i1).mCoeff = mpz_fdiv_ui(&(*i2).mCoeff, mPrimeShort);
    if ((*i1).mCoeff == 0) 
      i1.del();
    else 
      ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void INParameterModularShortAux::operator=(const INParameterModularShortAux& a) {
  IASSERT(mInterface == a.mInterface);
  IASSERT(mPrimeShort == a.mPrimeShort);
  setZero();
  Iterator      i1(mHead.begin());
  ConstIterator i2(a.mHead.begin());
  while(i2) {
    i1.insert();
    (*i1).mMonom = interface().copy((*i2).mMonom);
    (*i1).mCoeff = (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}


void INParameterModularShortAux::eval(INParameterModularShortAux& a, ModularShort t) const {
  IASSERT(mPrimeShort == a.primeShort());

  int var = interface().getEndVar() - 1;
  a.setZero();
  a.interface().setEndVar(var);

  if (isZero())
    return;

  ConstIterator i(mHead.begin());
  Iterator ia(a.mHead.begin());
  ia.insert();
  (*ia).mMonom = interface().copy((*i).mMonom);
  interface().zeroEndVar((*ia).mMonom); 
  (*ia).mCoeff = (*i).mCoeff * iPow0(t, interface().deg((*i).mMonom, var), mPrimeShort);
  (*ia).mCoeff %= mPrimeShort;
  ++i;
  while(i) {
    if (interface().compare1((*i).mMonom, (*ia).mMonom) == 0) {
      (*ia).mCoeff += (*i).mCoeff * iPow0(t, interface().deg((*i).mMonom, var), mPrimeShort);
      (*ia).mCoeff %= mPrimeShort;
    } else {
      if ((*ia).mCoeff == 0)
        ia.del();
      else
        ++ia;
      ia.insert();
      (*ia).mMonom = interface().copy((*i).mMonom);
      interface().zeroEndVar((*ia).mMonom); 
      (*ia).mCoeff = (*i).mCoeff * iPow0(t, interface().deg((*i).mMonom, var), mPrimeShort);
      (*ia).mCoeff %= mPrimeShort;
    }
    ++i;
  }
  if((*ia).mCoeff == 0)
    ia.del();
  IASSERTVALID(a);
}

void INParameterModularShortAux::eval(INParameterModularShortAux& a, ModularShort t[]) const {
  IASSERT(mPrimeShort == a.primeShort());

  int n = interface().getEndVar();
  a.setZero();
  a.interface().setEndVar(1);

  if (isZero())
    return;

  ConstIterator i(mHead.begin());
  Iterator ia(a.mHead.begin());
  ia.insert();
  (*ia).mMonom = interface().create();
  interface().setZero((*ia).mMonom);
  interface().prolong((*ia).mMonom, 0, interface().deg((*i).mMonom, 0));
  (*ia).mCoeff = (*i).mCoeff * iPow0(t[0], interface().deg((*i).mMonom, 1), mPrimeShort);
  (*ia).mCoeff %= mPrimeShort;
  for (int j = 2; j < n; j++) {
    (*ia).mCoeff *= iPow0(t[j-1], interface().deg((*i).mMonom, j), mPrimeShort);
    (*ia).mCoeff %= mPrimeShort;
  }
  ++i;
  while(i) {
    if (interface().deg((*i).mMonom, 0) == interface().deg((*ia).mMonom, 0)) {
      ModularShort tmp = (*i).mCoeff * iPow0(t[0], interface().deg((*i).mMonom, i), mPrimeShort);;
      tmp %= mPrimeShort;
      for (int j = 2; j < n; j++) {
        tmp *= iPow0(t[j-1], interface().deg((*i).mMonom, j), mPrimeShort);
        tmp %= mPrimeShort;
      }
      (*ia).mCoeff += tmp;
      (*ia).mCoeff %= mPrimeShort;
    } else {
      if ((*ia).mCoeff == 0)
        ia.del();
      else
        ++ia;
      ia.insert();
      (*ia).mMonom = interface().create();
      interface().setZero((*ia).mMonom);
      interface().prolong((*ia).mMonom, 0, interface().deg((*i).mMonom, 0));
      (*ia).mCoeff = (*i).mCoeff * iPow0(t[0], interface().deg((*i).mMonom, 1), mPrimeShort);
      (*ia).mCoeff %= mPrimeShort;
      for (int j = 2; j < n; j++) {
        (*ia).mCoeff *= iPow0(t[j-1], interface().deg((*i).mMonom, j), mPrimeShort);
        (*ia).mCoeff %= mPrimeShort;
      }
    }
    ++i;
  }
  if((*ia).mCoeff == 0)
    ia.del();
  IASSERTVALID(a);
}

void INParameterModularShortAux::eval(IOneParameterModularShortAux& result, ModularShort t[], int var) const {
  IASSERT(mPrimeShort == result.primeShort());

  int n = interface().getEndVar();
  result.setZero();

  if (isZero())
    return;

  ConstIterator i(mHead.begin());
  while (i) {
    ModularShort coeff = (*i).mCoeff;
    for (int j = 0; j < var; j++) {
      coeff *= iPow0(t[j], interface().deg((*i).mMonom, j), mPrimeShort);
      coeff %= mPrimeShort;
    }
    for (int j = var + 1; j < n; j++) {
      coeff *= iPow0(t[j-1], interface().deg((*i).mMonom, j), mPrimeShort);
      coeff %= mPrimeShort;
    }

    result.add(interface().deg((*i).mMonom, var), coeff);

    ++i;
  }

  IASSERTVALID(result);
}

void INParameterModularShortAux::set(const INParameterModularShortAux& a) {
  mHead.clear();
  Iterator i(mHead.begin());
  ConstIterator ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = interface().copy((*ia).mMonom);
    (*i).mCoeff = (*ia).mCoeff;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}


void INParameterModularShortAux::set(const IOneParameterModularShortAux& a) {
  mHead.clear();
  Iterator i(mHead.begin());
  IOneParameterModularShortAux::ConstIterator ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = interface().create();
    interface().setZero((*i).mMonom);
    interface().prolong((*i).mMonom, (*ia).mDegree);
    (*i).mCoeff = (*ia).mCoeff;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}


int INParameterModularShortAux::deg() const {
  return deg(interface().getEndVar() - 1);
}

int INParameterModularShortAux::deg(int var) const {
  IASSERT(!isZero());

  int d = 0;
  ConstIterator  i(mHead.begin());
  while(i) {
    if (interface().deg((*i).mMonom, var) > d)
      d = interface().deg((*i).mMonom, var);
    ++i;
  }
  return d;
}

int INParameterModularShortAux::fulldeg() const {
  IASSERT(!isZero());
  ConstIterator i(mHead.begin());
  unsigned result = 0;
  while (i) {
    if (interface().degree((*i).mMonom) > result)
      result = interface().degree((*i).mMonom);
    ++i;
  }
  return result;
}

void INParameterModularShortAux::lc(IOneParameterModularShortAux &a) const {
  IASSERT(mPrimeShort == a.primeShort());

  a.setZero();
  if (isZero())
    return;

  int var = interface().getEndVar() - 1;
  IOneParameterModularShortAux::Iterator ia(a.begin());
  ConstIterator  i(mHead.begin());
  ConstIterator  prev=i;
  ia.insert();
  (*ia).mDegree = interface().deg((*i).mMonom, var);
  (*ia).mCoeff = (*i).mCoeff;
  ++ia;
  ++i;
  while(i && interface().compare1((*i).mMonom, (*prev).mMonom) == 0) {
    prev=i;
    ia.insert();
    (*ia).mDegree = interface().deg((*i).mMonom, var);
    (*ia).mCoeff = (*i).mCoeff;
    ++ia;
    ++i;
  }
  IASSERTVALID(a);
}

void INParameterModularShortAux::lp(IOneParameterModularShortAux& a) const {
  IASSERT(mPrimeShort == a.primeShort());

  a.setZero();

  if (isZero()) 
    return;
  
  ConstIterator i = mHead.begin();

  MonomIterator m = interface().copy((*i).mMonom);
  
  IOneParameterModularShortAux::Iterator ia = a.mHead.begin();
  while (i && interface().compare1(m, (*i).mMonom) == 0) {
    ia.insert();
    (*ia).mDegree = interface().deg((*i).mMonom, interface().getEndVar() - 1);
    (*ia).mCoeff = (*i).mCoeff;
    ++i;
    ++ia;
  }

  delete[] m;
}


void INParameterModularShortAux::cont(IOneParameterModularShortAux &cont) const {
  IASSERT(mPrimeShort == cont.primeShort());

  if (isZero()) {
    cont.setZero();
    return;
  }

  ConstIterator it = mHead.begin();
  nextcoeff(it, cont);

  IOneParameterModularShortAux tmp(primeShort());
  bool success = false;
  while (!success) {
		IOneParameterModularShortAux lincomb(primeShort());
		lincomb.setZero();

    ModularShort rand;
		// the content usually is 1, so start with a linear combination of one
		// coefficient, and if the content is != 1, double the boundary
		unsigned int coeffs = 0;
		unsigned int max_coeffs = 1;
		while (it) {
		  while (it && coeffs < max_coeffs) {
        rand = iRandom(primeShort());
			  nextcoeff(it, tmp);
			  lincomb.addmul(tmp, rand);
			  ++coeffs;
		  }
		  IASSERT(max_coeffs <= INT_MAX/2);
		  max_coeffs <<= 1;
		  cont.gcd(lincomb);
		  if (cont.isOne())
		  	break;
		}

		if (!cont.isOne()) {
      success = isdivisible(cont);
		} else {
			success = true;
		}
		
		it = mHead.begin();
	}

	IASSERTVALID(cont);
}

void INParameterModularShortAux::nextcoeff(ConstIterator& i, IOneParameterModularShortAux& coeff) const {
	MonomIterator m = interface().copy((*i).mMonom);
	coeff.setZero();
  IOneParameterModularShortAux::Iterator ic = coeff.begin();

	while (i && interface().compare1(m, (*i).mMonom) == 0) {
		ic.insert();
    (*ic).mCoeff = (*i).mCoeff;
    (*ic).mDegree = interface().deg((*i).mMonom, interface().getEndVar() - 1);

		++ic;
		++i;
	}

  delete[] m;
}

void INParameterModularShortAux::pppoly(IOneParameterModularShortAux& cont) {
  ConstIterator it = mHead.begin();
  nextcoeff(it, cont);

  IOneParameterModularShortAux tmp(primeShort());
  bool success = false;
  while (!success) {
		IOneParameterModularShortAux lincomb(primeShort());
		lincomb.setZero();

    ModularShort rand;
		// the content usually is 1, so start with a linear combination of one
		// coefficient, and if the content is != 1, double the boundary
		unsigned int coeffs = 0;
		unsigned int max_coeffs = 1;
		while (it) {
		  while (it && coeffs < max_coeffs) {
        rand = iRandom(primeShort());
			  nextcoeff(it, tmp);
			  lincomb.addmul(tmp, rand);
			  ++coeffs;
		  }
		  IASSERT(max_coeffs <= INT_MAX/2);
		  max_coeffs <<= 1;
		  cont.gcd(lincomb);
		  if (cont.isOne())
		  	break;
		}

		if (!cont.isOne()) {
			try {
				divtry(cont);

				success = true;
			} catch (IDivError) {
      }
		} else {
			success = true;
		}
		
		it = mHead.begin();
	}

	IASSERTVALID(*this);
}

void INParameterModularShortAux::add(const INParameterModularShortAux& a) {
  //IASSERT(mInterface == a.mInterface);
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    switch(interface().compare((*i1).mMonom, (*i2).mMonom)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mCoeff = (*i2).mCoeff;
      (*i1).mMonom = interface().copy((*i2).mMonom);
      ++i1;
      ++i2;
      break;
    default:
      (*i1).mCoeff += (*i2).mCoeff;
      if ((*i1).mCoeff >= mPrimeShort)
        (*i1).mCoeff -= mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mCoeff = (*i2).mCoeff;
    (*i1).mMonom = interface().copy((*i2).mMonom);
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void INParameterModularShortAux::minus() {
  Iterator  i(mHead.begin());
  while(i) {
    (*i).mCoeff = mPrimeShort - (*i).mCoeff;
    ++i;
  }
  IASSERTVALID(*this);
}

void INParameterModularShortAux::sub(const INParameterModularShortAux& a) {
  //IASSERT(mInterface.getEndVar() == a.mInterface.getEndVar());
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    switch(interface().compare((*i1).mMonom, (*i2).mMonom)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
      (*i1).mMonom = interface().copy((*i2).mMonom);
      ++i1;
      ++i2;
      break;
    default:
      (*i1).mCoeff -= (*i2).mCoeff;
      if ((*i1).mCoeff < 0)
        (*i1).mCoeff += mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
    (*i1).mMonom = interface().copy((*i2).mMonom);
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void INParameterModularShortAux::sub(const INParameterModularShortAux& a, MonomConstIterator monom, ModularShort coeff) {
  //IASSERT(mInterface.getEndVar() == a.mInterface.getEndVar());
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    switch(interface().compare((*i1).mMonom, (*i2).mMonom, monom)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
      (*i1).mCoeff *= coeff;
      (*i1).mCoeff %= mPrimeShort;
      (*i1).mMonom = interface().copy((*i2).mMonom);
      interface().mult((*i1).mMonom, monom);
      ++i1;
      ++i2;
      break;
    default:
      ModularShort tmp = (*i2).mCoeff * coeff;
      tmp %= mPrimeShort;
      (*i1).mCoeff -= tmp;
      if ((*i1).mCoeff < 0)
        (*i1).mCoeff += mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
    (*i1).mCoeff *= coeff;
    (*i1).mCoeff %= mPrimeShort;
    (*i1).mMonom = interface().copy((*i2).mMonom);
    interface().mult((*i1).mMonom, monom);
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void INParameterModularShortAux::mult(ModularShort coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mCoeff = ((*i).mCoeff * coeff) % mPrimeShort;
    ++i;
  }
  IASSERTVALID(*this);
}


void INParameterModularShortAux::mult(const INParameterModularShortAux& a) {
  INParameterModularShortAux prod(interface(), primeShort());
  ConstIterator ia(a.mHead.begin());
  Iterator begin(prod.mHead.begin());

  while(ia) {
    ConstIterator i(mHead.begin());
    Iterator ip = begin;

    // look for the starting point
    while(ip && i && interface().compare((*ip).mMonom, (*i).mMonom, (*ia).mMonom) == 1)
      ++ip;

    begin = ip;

    while(ip && i)
      switch(interface().compare((*ip).mMonom, (*i).mMonom, (*ia).mMonom)) {
      case 1:
        ++ip;
        break;
      case -1:
        ip.insert();
        (*ip).mMonom = interface().copy((*i).mMonom);
        interface().mult((*ip).mMonom, (*ia).mMonom);
        (*ip).mCoeff = ((*i).mCoeff * (*ia).mCoeff) % prod.primeShort();
        IASSERT((*ip).mCoeff  != 0);
        ++ip;
        ++i;
        break;
      default:
        (*ip).mCoeff += (*i).mCoeff * (*ia).mCoeff;
        (*ip).mCoeff %= prod.primeShort();
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++i;
      }

    while(i) {
      ip.insert();
      (*ip).mMonom = interface().copy((*i).mMonom);
      interface().mult((*ip).mMonom, (*ia).mMonom);
      (*ip).mCoeff = ((*i).mCoeff * (*ia).mCoeff) % prod.primeShort();
      IASSERT((*ip).mCoeff  != 0);
      ++ip;
      ++i;
    }
    
    ++ia;
  }

  swap(prod);
  IASSERTVALID(*this);
}


void INParameterModularShortAux::mult(const IOneParameterModularShortAux& a) {
  INParameterModularShortAux prod(interface(), primeShort());
  IOneParameterModularShortAux::ConstIterator ia(a.mHead.begin());
  Iterator begin(prod.mHead.begin());

  int var = interface().getEndVar() - 1;

  while(ia) {
    ConstIterator i(mHead.begin());
    Iterator ip = begin;

    // look for the starting point
    while(ip && i && interface().compare((*ip).mMonom, (*i).mMonom, (*ia).mDegree) == 1)
      ++ip;

    begin = ip;

    while(ip && i)
      switch(interface().compare((*ip).mMonom, (*i).mMonom, (*ia).mDegree)) {
      case 1:
        ++ip;
        break;
      case -1:
        ip.insert();
        (*ip).mMonom = interface().copy((*i).mMonom);
        interface().prolong((*ip).mMonom, (*ia).mDegree);
        (*ip).mCoeff = ((*i).mCoeff * (*ia).mCoeff) % prod.primeShort();
        IASSERT((*ip).mCoeff  != 0);
        ++ip;
        ++i;
        break;
      default:
        (*ip).mCoeff += (*i).mCoeff * (*ia).mCoeff;
        (*ip).mCoeff %= prod.primeShort();
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++i;
      }

    while(i) {
      ip.insert();
      (*ip).mMonom = interface().copy((*i).mMonom);
      interface().prolong((*ip).mMonom, (*ia).mDegree);
      (*ip).mCoeff = ((*i).mCoeff * (*ia).mCoeff) % prod.primeShort();
      IASSERT((*ip).mCoeff  != 0);
      ++ip;
      ++i;
    }
    
    ++ia;
  }

  swap(prod);
  IASSERTVALID(*this);
}

bool INParameterModularShortAux::isdivisible(const INParameterModularShortAux& a) const {
  if (a.fulldeg() == 0) {
    return true;
  }
  
  INParameterModularShortAux tmp(*this);

  IMonomAuxInterface::MonomIterator monom = a.interface().create();
  ModularShort coeff;

  while (!tmp.isZero()) {
    if (interface().divisibility(tmp.lm(), a.lm())) {
      interface().divide(monom, tmp.lm(), a.lm());
      coeff = iDiv(tmp.lc(), a.lc(), primeShort());
    } else {
      return false;
    }

    // maybe implementation of a method sub(poly, monom, coeff)
    INParameterModularShortAux temp(a, monom, coeff);
    tmp.sub(temp);
  }

  delete[] monom;
  
  return true;
}


void INParameterModularShortAux::div(const INParameterModularShortAux& a) {
  IASSERT(isdivisible(a));
  if (a.fulldeg() == 0) {
    div(a.lc());
    IASSERTVALID(*this);
    return;
  }
  
  INParameterModularShortAux result(interface(), primeShort()); 
  
  Iterator it = result.mHead.begin();
  
  while (!mHead.isEmpty()) {
    it.insert();
    
    (*it).mMonom = interface().create();
    interface().divide((*it).mMonom, lm(), a.lm());
    (*it).mCoeff = iDiv(lc(), a.lc(), primeShort());
    
    // maybe implementation of a method sub(poly, monom, coeff)
    INParameterModularShortAux temp(a, (*it).mMonom, (*it).mCoeff);
    sub(temp);
    
    ++it;
  }
  
  swap(result);
  IASSERTVALID(*this);
}


void INParameterModularShortAux::rem(const INParameterModularShortAux& a) {
  IASSERT(!a.isZero());
  if (a.fulldeg() == 0) {
    setZero();
    return;
  }
  
  INParameterModularShortAux result(interface(), primeShort()); 
  
  Iterator it = result.mHead.begin();

  MonomIterator m = interface().create();
  interface().setZero(m);
  ModularShort c;
  
  while (!isZero() && interface().compare(lm(), a.lm()) >= 0) {
//    it.insert();
//    
//    (*it).mMonom = interface().create();
//    interface().divide((*it).mMonom, lm(), a.lm());
//    (*it).mCoeff = iDiv(lc(), a.lc(), primeShort());
//    
//    // maybe implementation of a method sub(poly, monom, coeff)
//    INParameterModularShortAux temp(a, (*it).mMonom, (*it).mCoeff);
//    sub(temp);
//    
//    ++it;

    interface().divide(m, lm(), a.lm());
    c = iDiv(lc(), a.lc(), primeShort());
    
    sub(a, m, c);
  }

  delete[] m;
  
//  swap(result);
  IASSERTVALID(*this);
}


bool INParameterModularShortAux::isdivisible(const IOneParameterModularShortAux& a) const {
  if (a.degree() == 0) {
    return true;
  }
  
  INParameterModularShortAux tmp(*this);

  INParameterModularShortAux result(interface(), primeShort());
  
  Iterator it = result.mHead.begin();
  
  while (!tmp.mHead.isEmpty()) {
    it.insert();
    
    (*it).mMonom = interface().create();
    if (interface().deg(tmp.lm(), interface().getEndVar() - 1) >= a.degree()) {
      interface().divide((*it).mMonom, tmp.lm(), a.degree());
      (*it).mCoeff = iDiv(tmp.lc(), a.lc(), primeShort());
    } else {
      return false;
    }
    
    // maybe implementation of a method sub(poly, monom, coeff)
    INParameterModularShortAux temp(interface(), a, (*it).mMonom, (*it).mCoeff);
    tmp.sub(temp);
    
    ++it;
  }
  
  return true;
}


void INParameterModularShortAux::div(const IOneParameterModularShortAux& a) {
  IASSERT(primeShort() == a.primeShort());
  //IASSERT(isdivisible(a));
  if (a.degree() == 0) {
    div(a.lc());
    IASSERTVALID(*this);
    return;
  }
  
  INParameterModularShortAux result(interface(), primeShort()); 
  
  Iterator it = result.mHead.begin();

  while (!mHead.isEmpty()) {
    it.insert();
    
    (*it).mMonom = interface().create();
    interface().divide((*it).mMonom, lm(), a.degree());
    (*it).mCoeff = iDiv(lc(), a.lc(), primeShort());
    
    // maybe implementation of a method sub(poly, monom, coeff)
    INParameterModularShortAux temp(interface(), a, (*it).mMonom, (*it).mCoeff);
    sub(temp);

    ++it;
  }
  
  swap(result);
  IASSERTVALID(*this);
}


void INParameterModularShortAux::divtry(const IOneParameterModularShortAux& a) {
  IASSERT(primeShort() == a.primeShort());
  //IASSERT(isdivisible(a));
  if (a.degree() == 0) {
    div(a.lc());
    IASSERTVALID(*this);
    return;
  }

  INParameterModularShortAux backup(*this);
  
  INParameterModularShortAux result(interface(), primeShort()); 
  
  Iterator it = result.mHead.begin();

  IASSERTVALID(*this);
  
  while (!mHead.isEmpty()) {
    if (!interface().deg(lm(), interface().getEndVar() - 1) >= a.degree()) {
      swap(backup);

      throw IDivError();
    }
    
    it.insert();

    (*it).mMonom = interface().create();
    interface().divide((*it).mMonom, lm(), a.degree());
    (*it).mCoeff = iDiv(lc(), a.lc(), primeShort());
    
    // maybe implementation of a method sub(poly, monom, coeff)
    INParameterModularShortAux temp(interface(), a, (*it).mMonom, (*it).mCoeff);
    sub(temp);

    IASSERTVALID(*this);
    
    ++it;
  }
  
  swap(result);
  IASSERTVALID(*this);
}


void INParameterModularShortAux::convert(INParameterGmpZAux& a) {
  IASSERT(interface() == a.interface());
  a.setZero();
  ConstIterator i1(mHead.begin());
  INParameterGmpZAux::Iterator i2(a.mHead.begin());
  while(i1) {
    i2.insert();
    (*i2).mMonom = interface().copy((*i1).mMonom);
    mpz_set_ui(&(*i2).mCoeff, (*i1).mCoeff);
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void INParameterModularShortAux::output(IMonomInterface* mi, std::ostream& out) const {
  if (isZero())
    out << '0';
  else {
    ConstIterator i(mHead.begin());
    if ((*i).mCoeff == 1) {
      if (interface().degree((*i).mMonom))
        interface().output((*i).mMonom, mi, out);
      else
        out << '1';
    } else {
        out << (*i).mCoeff;
      if (interface().degree((*i).mMonom)) {
        out << '*';
        interface().output((*i).mMonom, mi, out);
      }
    }

    ++i;
    while(i) {
      out << " + ";
      if ((*i).mCoeff == 1) {
        if (interface().degree((*i).mMonom))
          interface().output((*i).mMonom, mi, out);
        else
          out << '1';
      } else {
          out << (*i).mCoeff;
        if (interface().degree((*i).mMonom)) {
          out << '*';
          interface().output((*i).mMonom, mi, out);
        }
      }
      ++i;
    }
  }
}

void INParameterModularShortAux::output(std::ostream& out) const {
  if (isZero())
    out << '0';
  else {
    ConstIterator i(mHead.begin());
    if ((*i).mCoeff == 1) {
      if (interface().degree((*i).mMonom))
        interface().output((*i).mMonom, out);
      else
        out << '1';
    } else {
        out << (*i).mCoeff;
      if (interface().degree((*i).mMonom)) {
        out << '*';
        interface().output((*i).mMonom, out);
      }
    }

    ++i;
    while(i) {
      out << " + ";
      if ((*i).mCoeff == 1) {
        if (interface().degree((*i).mMonom))
          interface().output((*i).mMonom, out);
        else
          out << '1';
      } else {
          out << (*i).mCoeff;
        if (interface().degree((*i).mMonom)) {
          out << '*';
          interface().output((*i).mMonom, out);
        }
      }
      ++i;
    }
  }
}

void INParameterModularShortAux::assertValid(const char* fileName, int fileLine) const {
  ConstIterator i(mHead.begin());
  ConstIterator prev;
  while(i) {
    IASSERT2(fileName, fileLine, interface().assertValid((*i).mMonom));
    IASSERT2(fileName, fileLine, 0 < (*i).mCoeff && (*i).mCoeff < mPrimeShort);
    prev = i;
    ++i;
    if (i) {
      IASSERT2(fileName, fileLine, interface().compare((*prev).mMonom, (*i).mMonom) > 0);
    }
  }
}

template <>
IAllocator IList<INParameterModularShortAux::Term>::Node::sAllocator(sizeof(IList<INParameterModularShortAux::Term>::Node));

template <>
IAllocator IList<IOneParameterModularShortAux*>::Node::sAllocator(sizeof(IList<IOneParameterModularShortAux*>::Node));
