/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INPARAMETERMODULARSHORTAUX_H
#define INPARAMETERMODULARSHORTAUX_H

#include "../util/ilist.h"
#include "../util/imodular.h"
#include "./imonomaux.h"
#include "./inparametergmpzaux.h"
#include "./ioneparametermodularshortaux.h"

class INParameterModularShortAux {
  friend class IGcdAlgorithm;
  friend class IGcdOnevar;
protected:
  struct Term {
    IMonomAuxInterface::MonomIterator  mMonom;
    ModularShort   mCoeff;

    Term(): mMonom(NULL), mCoeff(0) {}
    ~Term() {
      delete[] mMonom;
    }
  };

  IMonomAuxInterface mInterface;
  ModularShort mPrimeShort;
  IList<Term> mHead;

public:
  typedef IList<Term>::Iterator  Iterator;
  typedef IList<Term>::ConstIterator  ConstIterator;
  typedef IMonomAuxInterface::MonomIterator MonomIterator;
  typedef IMonomAuxInterface::MonomConstIterator MonomConstIterator;

  INParameterModularShortAux(IMonomAuxInterface& interface, ModularShort primeShort):
      mInterface(interface),
      mPrimeShort(primeShort),
      mHead() {
    IASSERT(iIsPrime(mPrimeShort));
  }

  INParameterModularShortAux(const INParameterModularShortAux& a);
  INParameterModularShortAux(const INParameterGmpZAux& a, ModularShort primeShort);
  INParameterModularShortAux(const INParameterModularShortAux& a, MonomConstIterator m, const ModularShort c);
  INParameterModularShortAux(IMonomAuxInterface interface, const IOneParameterModularShortAux& a, MonomConstIterator m, const ModularShort c);

  ~INParameterModularShortAux() {}

  void eval(INParameterModularShortAux& a, ModularShort t) const;

  void eval(INParameterModularShortAux& a, ModularShort t[]) const;

  void eval(IOneParameterModularShortAux& result, ModularShort t[], int var) const;

  ModularShort primeShort() const { return mPrimeShort; }

  const IMonomAuxInterface& interface() const { return mInterface; }
  IMonomAuxInterface& interface() { return mInterface; }

  void operator=(const INParameterModularShortAux& a);

  void mult(ModularShort coeff);
  void shiftMult(ModularShort coeff);
  void mult(MonomConstIterator i, ModularShort coeff);

  void div(ModularShort coeff) {
    IASSERT(coeff != 0);
    ModularShort inv = iInverse(coeff, mPrimeShort);
    mult(inv);
  }

  void set(const INParameterModularShortAux& a);
  void set(const IOneParameterModularShortAux& a);

  void setZero() { mHead.clear(); }
  void setOne() {
    mHead.clear();
    Iterator i(mHead.begin());
    i.insert();
    (*i).mMonom = interface().create();
    interface().setZero((*i).mMonom);
    (*i).mCoeff = 1;
  }

  bool isZero() const { return mHead.isEmpty(); }

  bool isUnit() {
    Iterator i(mHead.begin());
    return (interface().degree((*i).mMonom) == 0 && (*i).mCoeff != 0);
  }

  int deg() const;
  int deg(int var) const;
  int fulldeg() const;
  ModularShort lc() const {
    return (*mHead.begin()).mCoeff;
  }

  void lc(IOneParameterModularShortAux &a) const;

  void lp(IOneParameterModularShortAux& a) const;

  MonomIterator lm() const {
    return (*mHead.begin()).mMonom;
  }

  void cont(IOneParameterModularShortAux &a) const;

  void ppcoeff() {
    IASSERT(!isZero());
    if (lc() != 1)
      div(lc());
  }

  void pppoly() {
    IOneParameterModularShortAux cont(mPrimeShort);
    pppoly(cont);
  }

  void nextcoeff(ConstIterator& i, IOneParameterModularShortAux& coeff) const;

  void pppoly(IOneParameterModularShortAux& cont);

  void add(const INParameterModularShortAux& a);

  //XXX not efficient
  void addmul(const INParameterModularShortAux& a, const INParameterModularShortAux& b) {
    INParameterModularShortAux tmp(a);
    tmp.mult(b);
    add(tmp);
  }


  void sub(const INParameterModularShortAux& a);
  void sub(const INParameterModularShortAux& a, MonomConstIterator monom, ModularShort coeff);
  void mult(const INParameterModularShortAux& a);
  void mult(const IOneParameterModularShortAux& a);
  void divtry(const IOneParameterModularShortAux& a);
  void div(const IOneParameterModularShortAux& a);
  void div(const INParameterModularShortAux& a);
  void rem(const INParameterModularShortAux& a);

  void minus();

  bool isdivisible(const INParameterModularShortAux& a) const;
  bool isdivisible(const IOneParameterModularShortAux& a) const;

  void swap(INParameterModularShortAux& a) {
    mHead.swap(a.mHead);
  }

  ConstIterator begin() const { return mHead.begin(); }
  Iterator begin() { return mHead.begin(); }

  void convert(INParameterGmpZAux& a);

  void output(IMonomInterface* mi, std::ostream& out) const;
  void output(std::ostream& out) const;

  friend std::ostream& operator<<(std::ostream& out, const INParameterModularShortAux& a) {
    a.output(out);
    return out;
  }

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // INPARAMETERMODULARSHORTAUX_H
