/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./inparameterutil.h"
#include "./inparametergmpzaux.h"

/* sets s and t such that s*a + t*b = gcd(a, b)
 */
void eea(const IOneParameterModularShortAux& a,
    const IOneParameterModularShortAux& b,
    IOneParameterModularShortAux& s,
    IOneParameterModularShortAux& t) {

  ModularShort p = a.primeShort();

  IOneParameterModularShortAux c(a);
  c.pp();
  IOneParameterModularShortAux d(b);
  d.pp();

  IOneParameterModularShortAux c1(p);
  IOneParameterModularShortAux c2(p);
  IOneParameterModularShortAux d1(p);
  IOneParameterModularShortAux d2(p);
  c1.setOne();
  d2.setOne();

  IOneParameterModularShortAux q(p);
  IOneParameterModularShortAux r(p);
  IOneParameterModularShortAux r1(p);
  IOneParameterModularShortAux r2(p);

  while (!d.isZero()) {
    polydiv(c, d, q, r);
    r1 = q;
    r1.mult(d1);
    r1.minus();
    r1.add(c1);

    r2.swap(q);
    r2.mult(d2);
    r2.minus();
    r2.add(c2);

    c.swap(d);
    c1.swap(d1);
    c2.swap(d2);

    d.swap(r);
    d1.swap(r1);
    d2.swap(r2);
  }

  ModularShort temp = c.lc();
  temp *= a.lc();
  temp %= p;
  temp = iInverse(temp, p);

  s.swap(c1);
  s.mult(temp);

  temp = c.lc();
  temp *= b.lc();
  temp %= p;
  temp = iInverse(temp, p);

  t.swap(c2);
  t.mult(temp);
}

/* computes q and r such that a = qb + r, deg(r) < deg(b);
 */
void polydiv(const IOneParameterModularShortAux& a,
    const IOneParameterModularShortAux& b,
    IOneParameterModularShortAux& q,
    IOneParameterModularShortAux& r) {

  ModularShort p = a.primeShort();

  ModularShort lc = iInverse(b.lc(), p);

  r.set(a);

  q.setZero();

  int m2;
  ModularShort k2;
  //  IMonom *m2 = a.mi->create();
  //  MP_INT k2;
  //  mpz_init(&k2);
  IOneParameterModularShortAux::Iterator iq(q.mHead.begin());

  while (!r.isZero() && r.degree() >= b.degree()) {
    IOneParameterModularShortAux::Iterator i1(r.mHead.begin());
    IOneParameterModularShortAux::ConstIterator i2(b.mHead.begin());

    m2 = (*i1).mDegree - (*i2).mDegree;
    k2 = ((*i1).mCoeff * lc) % p;
    k2 = p - k2;

    //    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    //    mpz_set(&k2, &(*i1).mCoeff);
    //    mpz_mul(&k2, &k2, &lc);
    //    mpz_neg(&k2, &k2);
    //    mpz_mod(&k2, &k2, &p);

    i1.del();
    ++i2;

    while (i1 && i2) {
      if ((*i1).mDegree > (*i2).mDegree + m2) {
        ++i1;
      } else if ((*i1).mDegree < (*i2).mDegree + m2) {
        i1.insert();
        (*i1).mDegree = (*i2).mDegree + m2;
        (*i1).mCoeff = ((*i2).mCoeff * k2) %p;

        //        (*i1).mMonom = a.mi->copy(*(*i2).mMonom);
        //        (*i1).mMonom->mult(*m2);
        //        mpz_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        //        mpz_mod(&(*i1).mCoeff, &(*i1).mCoeff, &p);
        ++i1;
        ++i2;
      } else {
        (*i1).mCoeff += (*i2).mCoeff * k2;
        (*i1).mCoeff %= p;
        //        mpz_addmul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        //        mpz_mod(&(*i1).mCoeff, &(*i1).mCoeff, &p);
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while (i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + m2;
      (*i1).mCoeff = ((*i2).mCoeff * k2) %p;
      //      (*i1).mMonom = a.mi->copy(*(*i2).mMonom);
      //      (*i1).mMonom->mult(*m2);
      //      mpz_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
      //      mpz_mod(&(*i1).mCoeff, &(*i1).mCoeff, &p);
      ++i1;
      ++i2;
    }
    iq.insert();
    (*iq).mDegree = m2;
    (*iq).mCoeff = p - k2;
    //    (*iq).mMonom = a.mi->copy(*m2);
    //    mpz_neg(&k2, &k2);
    //    mpz_mod(&(*iq).mCoeff, &k2, &p);
    ++iq;
  }

  //  delete m2;
  //  mpz_clear(&k2);
  //  mpz_clear(&lc);
}

/* computes q and r such that a = qb + r, deg(r) < deg(b);
 */
void polydiv(const INParameterModularShortAux& a,
    const INParameterModularShortAux& b,
    INParameterModularShortAux& q,
    INParameterModularShortAux& r) {

  ModularShort p = a.primeShort();

  ModularShort lc = iInverse(b.lc(), p);

  r.set(a);

  q.setZero();

  int m2;
  ModularShort k2;
  //  IMonom *m2 = a.mi->create();
  //  MP_INT k2;
  //  mpz_init(&k2);
  INParameterModularShortAux::Iterator iq = q.begin();

  while (!r.isZero() && r.lm()[0] >= b.lm()[0]) {
    INParameterModularShortAux::Iterator i1 = r.begin();
    INParameterModularShortAux::ConstIterator i2 = b.begin();

    m2 = (*i1).mMonom[0] - (*i2).mMonom[0];
    k2 = ((*i1).mCoeff * lc) % p;
    k2 = p - k2;

    //    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    //    mpz_set(&k2, &(*i1).mCoeff);
    //    mpz_mul(&k2, &k2, &lc);
    //    mpz_neg(&k2, &k2);
    //    mpz_mod(&k2, &k2, &p);

    i1.del();
    ++i2;

    while (i1 && i2) {
      if ((*i1).mMonom[0] > (*i2).mMonom[0] + m2) {
        ++i1;
      } else if ((*i1).mMonom[0] < (*i2).mMonom[0] + m2) {
        i1.insert();
        (*i1).mMonom = r.interface().copy((*i2).mMonom);
        (*i1).mMonom[0] += m2;
        (*i1).mCoeff = ((*i2).mCoeff * k2) %p;

        //        (*i1).mMonom = a.mi->copy(*(*i2).mMonom);
        //        (*i1).mMonom->mult(*m2);
        //        mpz_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        //        mpz_mod(&(*i1).mCoeff, &(*i1).mCoeff, &p);
        ++i1;
        ++i2;
      } else {
        (*i1).mCoeff += (*i2).mCoeff * k2;
        (*i1).mCoeff %= p;
        //        mpz_addmul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
        //        mpz_mod(&(*i1).mCoeff, &(*i1).mCoeff, &p);
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while (i2) {
      i1.insert();
      (*i1).mMonom = r.interface().copy((*i2).mMonom);
      (*i1).mMonom[0] += m2;
      (*i1).mCoeff = ((*i2).mCoeff * k2) %p;
      //      (*i1).mMonom = a.mi->copy(*(*i2).mMonom);
      //      (*i1).mMonom->mult(*m2);
      //      mpz_mul(&(*i1).mCoeff, &(*i2).mCoeff, &k2);
      //      mpz_mod(&(*i1).mCoeff, &(*i1).mCoeff, &p);
      ++i1;
      ++i2;
    }
    iq.insert();
    (*iq).mMonom = r.interface().create();
    r.interface().setZero((*iq).mMonom);
    (*iq).mMonom[0] = m2;
    (*iq).mCoeff = p - k2;
    //    (*iq).mMonom = a.mi->copy(*m2);
    //    mpz_neg(&k2, &k2);
    //    mpz_mod(&(*iq).mCoeff, &k2, &p);
    ++iq;
  }

  //  delete m2;
  //  mpz_clear(&k2);
  //  mpz_clear(&lc);
}

/* same as polydiv, but it does not fully compute q and by that saves some
 * computing cycles
 */
void polydiv2(const IOneParameterModularShortAux& a,
    const IOneParameterModularShortAux& b,
    IOneParameterModularShortAux& r) {

  ModularShort p = a.primeShort();
  ModularShort lc = iInverse(b.lc(), p);
  r.set(a);
  int m2;
  ModularShort k2;

  while (!r.isZero() && r.degree() >= b.degree()) {
    IOneParameterModularShortAux::Iterator i1(r.mHead.begin());
    IOneParameterModularShortAux::ConstIterator i2(b.mHead.begin());

    m2 = (*i1).mDegree - (*i2).mDegree;
    k2 = ((*i1).mCoeff * lc) % p;
    k2 = p - k2;

    i1.del();
    ++i2;

    while (i1 && i2) {
      if ((*i1).mDegree > (*i2).mDegree) {
        ++i1;
      } else if ((*i1).mDegree < (*i2).mDegree) {
        i1.insert();
        (*i1).mDegree = (*i2).mDegree + m2;
        (*i1).mCoeff = ((*i2).mCoeff * k2) %p;
        ++i1;
        ++i2;
      } else {
        (*i1).mCoeff += (*i2).mCoeff * k2;
        (*i1).mCoeff %= p;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while (i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + m2;
      (*i1).mCoeff = ((*i2).mCoeff * k2) %p;
      ++i1;
      ++i2;
    }
  }
}
