/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./ioneparametergmpz.h"

IOneParameterGmpZInterface::IOneParameterGmpZInterface(IVariables *parameter):
    ICoeffInterface() {
  IASSERT(parameter);
  mParameters = parameter;
  mType = ICoeffInterface::OneParameterGmpZ;
}

ICoeff* IOneParameterGmpZInterface::create() {
  return new IOneParameterGmpZ(this);
}

ICoeff* IOneParameterGmpZInterface::copy(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  return new IOneParameterGmpZ(a);
}

IOneParameterGmpZ::IOneParameterGmpZ(IOneParameterGmpZInterface* r):
   ICoeff(r),
   mPoly() {
}

IOneParameterGmpZ::IOneParameterGmpZ(const ICoeff &a):
   ICoeff(a.coeffInterface()),
   mPoly(((const IOneParameterGmpZ&)a).mPoly) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  IASSERTVALID(*this);
}

IOneParameterGmpZ::~IOneParameterGmpZ() {
}

void IOneParameterGmpZ::set(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly = ((const IOneParameterGmpZ&)a).mPoly;
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::setZero() {
  mPoly.setZero();
}

void IOneParameterGmpZ::setOne() {
  mPoly.setOne();
}

bool IOneParameterGmpZ::isNegative() const {
  return mPoly.isNegative();
}

bool IOneParameterGmpZ::isZero() const {
  return mPoly.isZero();
}

bool IOneParameterGmpZ::isOne() const {
  return mPoly.isOne();
}

bool IOneParameterGmpZ::isComposite() const {
  return mPoly.isComposite();
}

bool IOneParameterGmpZ::isUnit() const {
  return (!mPoly.isZero() && mPoly.degree() == 0);
}

int IOneParameterGmpZ::norm() const {
  return mPoly.norm();
}

void IOneParameterGmpZ::minus() {
  mPoly.minus();
}

void IOneParameterGmpZ::abs() {
}

void IOneParameterGmpZ::add(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly.add(((const IOneParameterGmpZ&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::sub(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly.sub(((const IOneParameterGmpZ&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::mult(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly.mult(((const IOneParameterGmpZ&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::diff(int , unsigned deg) {
  mPoly.diff(deg);
}

void IOneParameterGmpZ::shift(int , unsigned ) {
}

void IOneParameterGmpZ::pow(unsigned deg) {
  mPoly.pow(deg);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::inverse() {
  IERROR("inverse");
}

void IOneParameterGmpZ::div(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  if (!a.isOne()) {
    mPoly.div(((const IOneParameterGmpZ&)a).mPoly);

    //the following is wrong for IPolyList::div!
    //MP_INT multiplier;
    //mpz_init(&multiplier);
    //mPoly.pseudo_div(multiplier, ((const IOneParameterGmpZ&)a).mPoly);
    //mpz_clear(&multiplier);

    IASSERTVALID(*this);
  }
}

void IOneParameterGmpZ::div(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  IASSERT(b.type() == ICoeffInterface::OneParameterGmpZ);
  if (!b.isOne()) {
    mPoly.div(((const IOneParameterGmpZ&)a).mPoly, ((const IOneParameterGmpZ&)b).mPoly);

    //the following is wrong for IPolyList::div!
    //MP_INT multiplier;
    //mpz_init(&multiplier);
    //mPoly.pseudo_div(multiplier, ((const IOneParameterGmpZ&)a).mPoly, ((const IOneParameterGmpZ&)b).mPoly);
    //mpz_clear(&multiplier);
  }
  else
    mPoly = ((const IOneParameterGmpZ&)a).mPoly;
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::gcd(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly.gcd(((const IOneParameterGmpZ&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::gcd(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  IASSERT(b.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly.gcd(((const IOneParameterGmpZ&)a).mPoly, ((const IOneParameterGmpZ&)b).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::addmul(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::OneParameterGmpZ);
  IASSERT(b.type() == ICoeffInterface::OneParameterGmpZ);
  mPoly.addmul(((const IOneParameterGmpZ&)a).mPoly, ((const IOneParameterGmpZ&)b).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterGmpZ::input(std::istream& in, bool expanded) {
  std::streampos posbeg = in.tellg();
  int var = parameters()->read(in);
  if (var == 0) {
    mPoly.setOne();
    (*mPoly.begin()).mDegree = 1;
    IASSERTVALID(*this);
  }
  else {
    MP_INT mp;
    mpz_init(&mp);
    in >> (&mp);
    if (in.fail()) {
      in.clear();
      in.setstate(std::ios::failbit);
      in.seekg(posbeg);
    }
    else {
      IOneParameterGmpZAux::Iterator i(mPoly.begin());
      i.clear();
      if (mpz_sgn(&mp) != 0) {
        i.insert();
        mpz_set(&(*i).mCoeff, &mp);
      }
      IASSERTVALID(*this);
    }
    mpz_clear(&mp);
  }
}

void IOneParameterGmpZ::output(std::ostream& out) const {
  if (isZero())
    out << '0';
  else {
    MP_INT mp;
    mpz_init(&mp);
    IOneParameterGmpZAux::ConstIterator i(mPoly.begin());
    if (mpz_cmp_ui(&(*i).mCoeff, 1ul) == 0)
      if ((*i).mDegree == 0)
        out << '1';
      else if ((*i).mDegree == 1)
        out << parameters()->variable(0);
      else
        out << parameters()->variable(0) << '^' << (*i).mDegree;
    else
      if ((*i).mDegree == 0)
        out << (&(*i).mCoeff);
      else if ((*i).mDegree == 1)
        out << (&(*i).mCoeff) << '*' << parameters()->variable(0);
      else
        out << (&(*i).mCoeff) << '*' << parameters()->variable(0) << '^' << (*i).mDegree;
    ++i;
    while(i) {
      if (mpz_sgn(&(*i).mCoeff) > 0) {
        out << " + ";
        if (mpz_cmp_ui(&(*i).mCoeff, 1ul) == 0)
          if ((*i).mDegree == 0)
            out << '1';
          else if ((*i).mDegree == 1)
            out << parameters()->variable(0);
          else
            out << parameters()->variable(0) << '^' << (*i).mDegree;
        else {
          if ((*i).mDegree == 0)
            out << (&(*i).mCoeff);
          else if ((*i).mDegree == 1)
            out << (&(*i).mCoeff) << '*' << parameters()->variable(0);
          else
            out << (&(*i).mCoeff) << '*' << parameters()->variable(0) << '^' << (*i).mDegree;
        }
      }
      else {
        out << " - ";
        if (mpz_cmp_si(&(*i).mCoeff, -1) == 0)
          if ((*i).mDegree == 0)
            out << '1';
          else if ((*i).mDegree == 1)
            out << parameters()->variable(0);
          else
            out << parameters()->variable(0) << '^' << (*i).mDegree;
        else {
          mpz_neg(&mp, &(*i).mCoeff);
          if ((*i).mDegree == 0)
            out << (&mp);
          else if ((*i).mDegree == 1)
            out << (&mp) << '*' << parameters()->variable(0);
          else
            out << (&mp) << '*' << parameters()->variable(0) << '^' << (*i).mDegree;
        }
      }
      ++i;
    }
    mpz_clear(&mp);
  }
}

void IOneParameterGmpZ::assertValid(const char* fileName, int fileLine) const {
  mPoly.assertValid(fileName, fileLine);
}

IAllocator IOneParameterGmpZ::sAllocator(sizeof(IOneParameterGmpZ));
