/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <gmp.h>

#include "./ioneparametermodularshort.h"

IOneParameterModularShortInterface::IOneParameterModularShortInterface(IVariables *parameter, int primeShort):
    ICoeffInterface(),
    mPrimeShort(primeShort) {
  IASSERT(parameter);
  mParameters = parameter;

  if ((1 << (sizeof(int)*4)-1) < mPrimeShort)
    IERROR("primeShort^2 > (sizeof(int)*4-1)^2");

  if (!iIsPrime(primeShort))
    IERROR("primeShort is not primer");

  mType = ICoeffInterface::OneParameterModularShort;
}

ICoeff* IOneParameterModularShortInterface::create() {
  return new IOneParameterModularShort(this);
}

ICoeff* IOneParameterModularShortInterface::copy(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  return new IOneParameterModularShort(a);
}

IOneParameterModularShort::IOneParameterModularShort(IOneParameterModularShortInterface* r):
    ICoeff(r),
    mPoly(r->mPrimeShort) {
}

IOneParameterModularShort::IOneParameterModularShort(const ICoeff &a):
    ICoeff(a.coeffInterface()),
    mPoly(((const IOneParameterModularShort&)a).mPoly) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  IASSERTVALID(*this);
}

IOneParameterModularShort::~IOneParameterModularShort() {
}

void IOneParameterModularShort::set(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  mPoly = ((const IOneParameterModularShort&)a).mPoly;
  IASSERTVALID(*this);
}

void IOneParameterModularShort::setZero() {
  mPoly.setZero();
}

void IOneParameterModularShort::setOne() {
  mPoly.setOne();
}

bool IOneParameterModularShort::isNegative() const {
  return false;
}

bool IOneParameterModularShort::isZero() const {
  return mPoly.isZero();
}

bool IOneParameterModularShort::isOne() const {
  return mPoly.isOne();
}

bool IOneParameterModularShort::isComposite() const {
  return mPoly.isComposite();
}

bool IOneParameterModularShort::isUnit() const {
  return (!mPoly.isZero() && mPoly.degree() == 0);
}

int IOneParameterModularShort::norm() const {
  return mPoly.norm();
}

void IOneParameterModularShort::minus() {
  mPoly.minus();
}

void IOneParameterModularShort::abs() {
}

void IOneParameterModularShort::add(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.add(((const IOneParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::sub(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.sub(((const IOneParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::mult(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.mult(((const IOneParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::diff(int , unsigned deg) {
  mPoly.diff();
}

void IOneParameterModularShort::shift(int , unsigned ) {
}

void IOneParameterModularShort::pow(unsigned deg) {
  mPoly.pow(deg);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::inverse() {
  IERROR("inverse");
}

void IOneParameterModularShort::div(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.div(((const IOneParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::div(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  IASSERT(b.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.div(((const IOneParameterModularShort&)a).mPoly, ((const IOneParameterModularShort&)b).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::gcd(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.gcd(((const IOneParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::gcd(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  IASSERT(b.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.gcd(((const IOneParameterModularShort&)a).mPoly, ((const IOneParameterModularShort&)b).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::addmul(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::OneParameterModularShort);
  IASSERT(b.type() == ICoeffInterface::OneParameterModularShort);
  mPoly.addmul(((const IOneParameterModularShort&)a).mPoly, ((const IOneParameterModularShort&)b).mPoly);
  IASSERTVALID(*this);
}

void IOneParameterModularShort::input(std::istream& in, bool expanded) {
  std::streampos posbeg = in.tellg();
  int var = parameters()->read(in);
  if (var == 0) {
    mPoly.setOne();
    (*mPoly.begin()).mDegree = 1;
    IASSERTVALID(*this);
  }
  else {
    MP_INT mp;
    mpz_init(&mp);
    in >> (&mp);
    if (in.fail()) {
      in.clear();
      in.setstate(std::ios::failbit);
      in.seekg(posbeg);
    }
    else {
      int sign = mpz_sgn(&mp);
      mpz_mod_ui(&mp, &mp, (unsigned int)primeShort());
      int modularShort = (int)mpz_get_ui(&mp);
      if (modularShort == 0)
        mPoly.setZero();
      else {
        if (sign < 0)
          modularShort = primeShort() - modularShort;
        mPoly.setOne();
        (*mPoly.begin()).mCoeff = modularShort;
      }
      IASSERTVALID(*this);
    }
    mpz_clear(&mp);
  }
}

void IOneParameterModularShort::output(std::ostream& out) const {
  if (isZero())
    out << '0';
  else {
    IOneParameterModularShortAux::ConstIterator i(mPoly.begin());
    if ((*i).mCoeff == 1)
      if ((*i).mDegree == 0)
        out << '1';
      else if ((*i).mDegree == 1)
        out << parameters()->variable(0);
      else
        out << parameters()->variable(0) << '^' << (*i).mDegree;
    else
      if ((*i).mDegree == 0)
        out << (*i).mCoeff;
      else if ((*i).mDegree == 1)
        out << (*i).mCoeff << '*' << parameters()->variable(0);
      else
        out << (*i).mCoeff << '*' << parameters()->variable(0) << '^' << (*i).mDegree;
    ++i;
    while(i) {
      out << " + ";
      if ((*i).mCoeff == 1)
        if ((*i).mDegree == 0)
          out << '1';
        else if ((*i).mDegree == 1)
          out << parameters()->variable(0);
        else
          out << parameters()->variable(0) << '^' << (*i).mDegree;
      else
        if ((*i).mDegree == 0)
          out << (*i).mCoeff;
        else if ((*i).mDegree == 1)
          out << (*i).mCoeff << '*' << parameters()->variable(0);
        else
          out << (*i).mCoeff << '*' << parameters()->variable(0) << '^' << (*i).mDegree;
      ++i;
    }
  }
}

void IOneParameterModularShort::assertValid(const char* fileName, int fileLine) const {
  mPoly.assertValid(fileName, fileLine);
}

IAllocator IOneParameterModularShort::sAllocator(sizeof(IOneParameterModularShort));
