/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IONEPARAMETERMODULARSHORT_H
#define IONEPARAMETERMODULARSHORT_H

#include "../util/iallocator.h"
#include "./ioneparametermodularshortaux.h"
#include "./icoeff.h"

class IOneParameterModularShort;
class IOneParameterModularShortInterface: public ICoeffInterface {
  friend class IOneParameterModularShort;

protected:
  int mPrimeShort;

public:
  IOneParameterModularShortInterface(IVariables *parameter, int primeShort);
  ~IOneParameterModularShortInterface() {}

  ICoeff* create();
  ICoeff* copy(const ICoeff& a);
};

class IOneParameterModularShort: public ICoeff {
  friend class IOneParameterModularShortInterface;

protected:
  int primeShort() const { return ((const IOneParameterModularShortInterface*)coeffInterface())->mPrimeShort; }

  IOneParameterModularShortAux  mPoly;

  static IAllocator            sAllocator;

  IOneParameterModularShort(IOneParameterModularShortInterface* r);
  IOneParameterModularShort(const ICoeff& a);

public:
  void* operator new(size_t) { return sAllocator.allocate(); }
  void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

  ~IOneParameterModularShort();

  void set(const ICoeff& a);
  void setZero();
  void setOne();

  bool isNegative() const;
  bool isZero() const;
  bool isOne() const;
  bool isComposite() const;
  bool isUnit() const;
  int norm() const;

  void minus();
  void abs();
  void add(const ICoeff& a);
  void sub(const ICoeff& a);
  void mult(const ICoeff& a);
  void diff(int var, unsigned deg);
  void shift(int var, unsigned deg);
  void pow(unsigned deg);
  void inverse();
  void div(const ICoeff& a);
  void div(const ICoeff& a, const ICoeff& b);
  void gcd(const ICoeff& a);
  void gcd(const ICoeff& a, const ICoeff& b);
  void addmul(const ICoeff& a, const ICoeff& b);

  void read(std::istream &in) { mPoly.read(in); }
  void write(std::ostream &out) const { mPoly.write(out); }

  void input(std::istream& in, bool expanded);
  void output(std::ostream& out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // IONEPARAMETERMODULARSHORT_H
