/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./ioneparametermodularshortaux.h"

void IOneParameterModularShortAux::add(IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  Iterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert(i2.get());
      ++i1;
    }
    else {
      (*i1).mCoeff += (*i2).mCoeff;
      if ((*i1).mCoeff >= mPrimeShort)
        (*i1).mCoeff -= mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert(i2.get());
    ++i1;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::addmul(const IOneParameterModularShortAux& a, const ModularShort c) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mCoeff = (*i2).mCoeff * c;
      (*i1).mCoeff %= mPrimeShort;
      (*i1).mDegree = (*i2).mDegree;
      ++i1;
      ++i2;
    }
    else {
      (*i1).mCoeff += (*i2).mCoeff * c;
      (*i1).mCoeff %= mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mCoeff = (*i2).mCoeff * c;
    (*i1).mCoeff %= mPrimeShort;
    (*i1).mDegree = (*i2).mDegree;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::sub(IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  Iterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert(i2.get());
      (*i1).mCoeff = mPrimeShort - (*i1).mCoeff;
      ++i1;
    }
    else {
      (*i1).mCoeff -= (*i2).mCoeff;
      if ((*i1).mCoeff < 0)
        (*i1).mCoeff += mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert(i2.get());
    (*i1).mCoeff = mPrimeShort - (*i1).mCoeff;
    ++i1;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::mult(ModularShort coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);
  IASSERTVALID(*this);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mCoeff = ((*i).mCoeff * coeff) ;
    (*i).mCoeff %= mPrimeShort;
    ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::shiftMult(ModularShort coeff) {
  IASSERT(!isZero());
  IASSERT(0 < coeff && coeff < mPrimeShort);
  IASSERT(eval(coeff) != 0);

  coeff = mPrimeShort - coeff;

  mHead.begin().insert();

  Iterator i(mHead.begin());
  Iterator j(mHead.begin());
  ++i;
  (*j).mDegree = (*i).mDegree + 1;
  (*j).mCoeff = 1;
  ++j;
  do {
    ++i;
    if (i) {
      if ((*i).mDegree != (*j).mDegree - 1) {
        i.insert();
        (*i).mDegree = (*j).mDegree - 1;
      }
      (*j).mCoeff = ((*i).mCoeff + (*j).mCoeff * coeff) % mPrimeShort;
      if ((*j).mCoeff == 0)
        j.del();
      else
        ++j;
    }
    else {
      (*j).mCoeff = ((*j).mCoeff * coeff) % mPrimeShort;
      if ((*j).mCoeff == 0) j.del();
      break;
    }
  } while(true);
  IASSERTVALID(*this);
}

ModularShort IOneParameterModularShortAux::eval(ModularShort t) const {
  IASSERT(0 <= t && t < mPrimeShort);
  ConstIterator i(mHead.begin());
  ModularShort sum = 0;
  if (t == 0) {
    while(i) {
      sum = (*i).mCoeff;
      ++i;
    }
  }
  else {
    while(i) {
      sum = (sum + (*i).mCoeff * iPow0(t, (*i).mDegree, mPrimeShort)) % mPrimeShort;
      ++i;
    }
  }
  return sum;
}

void IOneParameterModularShortAux::mult(unsigned degree, ModularShort coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mDegree += degree;
    (*i).mCoeff = ((*i).mCoeff * coeff) % mPrimeShort;
    ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::div(ModularShort coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);

  ModularShort inverse = iInverse(coeff, mPrimeShort);
  ConstIterator i(mHead.begin());
  while(i) {
   (*i).mCoeff = ((*i).mCoeff * inverse) % mPrimeShort;
   ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::div(unsigned degree, ModularShort coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);

  ModularShort inverse = iInverse(coeff, mPrimeShort);
  ConstIterator i(mHead.begin());
  while(i) {
   IASSERT((*i).mDegree >= degree);
   (*i).mDegree += degree;
   (*i).mCoeff = ((*i).mCoeff * inverse) % mPrimeShort;
   ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::rem(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);

  if (isZero() || a.isZero())
    return;

  ModularShort inverse1 = iInverse((*a.mHead.begin()).mCoeff, mPrimeShort);
  inverse1 = mPrimeShort - inverse1;

  while(!isZero() &&
        degree() >= a.degree()) {
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg = degree() - a.degree();
    ModularShort coeff = ((*i1).mCoeff * inverse1) % mPrimeShort;

    IASSERT((*i1).mDegree == (*i2).mDegree + deg);
    IASSERT(((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort == 0);
    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
        (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
        ++i1;
        ++i2;
      }
      else {
        (*i1).mCoeff = ((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
}

void IOneParameterModularShortAux::minus() {
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mCoeff = mPrimeShort - (*i).mCoeff;
    ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::add(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree;
      (*i1).mCoeff = (*i2).mCoeff;
      ++i1;
      ++i2;
    }
    else {
      (*i1).mCoeff += (*i2).mCoeff;
      if ((*i1).mCoeff >= mPrimeShort)
        (*i1).mCoeff -= mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
    (*i1).mCoeff = (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::add(int deg, ModularShort c) {
  Iterator  i(mHead.begin());
  while(i) {
    if ((*i).mDegree == deg) {
      (*i).mCoeff += c;
      (*i).mCoeff %= mPrimeShort;

      if ((*i).mCoeff == 0)
        i.del();

      IASSERTVALID(*this);
      return;
    }

    if ((*i).mDegree < deg) {
      i.insert();
      (*i).mDegree = deg;
      (*i).mCoeff = c;
      (*i).mCoeff %= mPrimeShort;

      IASSERTVALID(*this);
      return;
    }

    ++i;
  }

  i.insert();
  (*i).mDegree = deg;
  (*i).mCoeff = c;
  (*i).mCoeff %= mPrimeShort;

  IASSERTVALID(*this);
  return;
}

void IOneParameterModularShortAux::sub(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    if ((*i1).mDegree > (*i2).mDegree) {
      ++i1;
    }
    else if ((*i1).mDegree < (*i2).mDegree) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree;
      (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
      ++i1;
      ++i2;
    }
    else {
      (*i1).mCoeff -= (*i2).mCoeff;
      if ((*i1).mCoeff < 0)
        (*i1).mCoeff += mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree = (*i2).mDegree;
    (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::mult(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);

  if (isZero())
    return;

  IOneParameterModularShortAux prod(mPrimeShort);
  ConstIterator ia(a.mHead.begin());
  Iterator begin(prod.mHead.begin());

  while(ia) {
    ConstIterator i(mHead.begin());
    Iterator ip = begin;
    unsigned degree2 = (*i).mDegree + (*ia).mDegree;

    // look for the starting point
    while(ip && (*ip).mDegree > degree2)
      ++ip;

    begin = ip;

    while(ip && i) {
      degree2 = (*i).mDegree + (*ia).mDegree;
      if ((*ip).mDegree > degree2)
        ++ip;
      else if ((*ip).mDegree < degree2) {
        ip.insert();
        (*ip).mDegree = degree2;
        (*ip).mCoeff = (*i).mCoeff * (*ia).mCoeff;
        (*ip).mCoeff %= mPrimeShort;
        IASSERT((*ip).mCoeff != 0);
        ++ip;
        ++i;
      }
      else {
        (*ip).mCoeff += (*i).mCoeff * (*ia).mCoeff;
        (*ip).mCoeff %= mPrimeShort;
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++i;
      }
    }

    while(i) {
      ip.insert();
      (*ip).mDegree = (*i).mDegree + (*ia).mDegree;
      (*ip).mCoeff = (*i).mCoeff * (*ia).mCoeff;
      (*ip).mCoeff %= mPrimeShort;
      IASSERT((*ip).mCoeff != 0);
      ++ip;
      ++i;
    }

    ++ia;
  }
  mHead.swap(prod.mHead);
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::mult(const IOneParameterModularShortAux& a, const IOneParameterModularShortAux& b) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(mPrimeShort == b.mPrimeShort);

  setZero();
  if (a.isZero() || b.isZero())
    return;

  ConstIterator ib(b.mHead.begin());
  Iterator begin(mHead.begin());

  while(ib) {
    ConstIterator ia(a.mHead.begin());
    Iterator ip = begin;
    unsigned degree2 = (*ia).mDegree + (*ib).mDegree;

    // look for the starting point
    while(ip && (*ip).mDegree > degree2)
      ++ip;

    begin = ip;

    while(ip && ia) {
      degree2 = (*ia).mDegree + (*ib).mDegree;
      if ((*ip).mDegree > degree2)
        ++ip;
      else if ((*ip).mDegree < degree2) {
        ip.insert();
        (*ip).mDegree = degree2;
        (*ip).mCoeff = ((*ia).mCoeff * (*ib).mCoeff) % mPrimeShort;
        IASSERT((*ip).mCoeff != 0);
        //if ((*ip).mCoeff == 0)
          //ip.del();
        //else
          ++ip;
        ++ia;
      }
      else {
        (*ip).mCoeff = ((*ip).mCoeff + (*ia).mCoeff * (*ib).mCoeff) % mPrimeShort;
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++ia;
      }
    }

    while(ia) {
      ip.insert();
      (*ip).mDegree = (*ia).mDegree + (*ib).mDegree;
      (*ip).mCoeff = ((*ia).mCoeff * (*ib).mCoeff) % mPrimeShort;
      IASSERT((*ip).mCoeff != 0);
      //if ((*ip).mCoeff == 0)
        //ip.del();
      //else
        ++ip;
      ++ia;
    }

    ++ib;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::diff() {
  Iterator i(mHead.begin());
  while(i) {
    if ((*i).mDegree == 0) {
      i.del();
      break;
    }
    (*i).mCoeff = ((*i).mCoeff * (*i).mDegree) % mPrimeShort;
    --(*i).mDegree;
    ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::diff(unsigned deg) {
  IASSERT(deg > 0);
  Iterator i(mHead.begin());
  while(i) {
    if ((*i).mDegree < deg) {
      i.clear();
      break;
    }
    unsigned n = (*i).mDegree;
    (*i).mDegree -= deg;
    do {
      (*i).mCoeff = ((*i).mCoeff * n) % mPrimeShort;
      --n;
    } while (n != (*i).mDegree);
    ++i;
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::pow(unsigned deg) {
  IASSERT(deg > 0);
  if (deg > 1) {
    IOneParameterModularShortAux p(*this);
    while(--deg > 0) {
      p.mult(*this);
    }
    mHead.swap(p.mHead);
  }
  IASSERTVALID(*this);
}

void IOneParameterModularShortAux::div(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(degree() >= a.degree());

  ModularShort inverse1 = iInverse((*a.mHead.begin()).mCoeff, mPrimeShort);
  inverse1 = mPrimeShort - inverse1;

  IOneParameterModularShortAux  quo(mPrimeShort);
  Iterator  i(quo.mHead.begin());

  while(!isZero()) {
    IASSERT(degree() >= a.degree());
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg = degree() - a.degree();
    ModularShort coeff = ((*i1).mCoeff * inverse1) % mPrimeShort;

    i.insert();
    (*i).mDegree = deg;
    (*i).mCoeff = mPrimeShort - coeff;
    ++i;
    IASSERTVALID(quo);

    IASSERT((*i1).mDegree == (*i2).mDegree + deg);
    IASSERT(((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort == 0);
    i1.del();
    ++i2;

    while(i1 && i2) {
      unsigned degree2 = (*i2).mDegree + deg;
      if ((*i1).mDegree > degree2) {
        ++i1;
      }
      else if ((*i1).mDegree < degree2) {
        i1.insert();
        (*i1).mDegree = degree2;
        (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
        ++i1;
        ++i2;
      }
      else {
        (*i1).mCoeff = ((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree = (*i2).mDegree + deg;
      (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
  IASSERT(isZero());
  mHead.swap(quo.mHead);
}

void IOneParameterModularShortAux::gcd(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);

  if (a.isZero())
    return;
  if (isZero()) {
    set(a);
    return;
  }

  if (degree() == 0 || a.degree() == 0)
    setOne();
  else {
    IOneParameterModularShortAux tmp(a);
    if (tmp.degree() > degree())
      mHead.swap(tmp.mHead);

    do {
      mHead.swap(tmp.mHead);
      tmp.rem(*this);
    } while (!tmp.isZero() && tmp.degree() > 0);

    if (!tmp.isZero())
      setOne();
    else
      pp();
  }
}

void IOneParameterModularShortAux::gcd(const IOneParameterModularShortAux& a, const IOneParameterModularShortAux& b) {
  IASSERT(a.mPrimeShort == b.mPrimeShort);

  if (a.isZero()) {
    set(b);
    return;
  }
  if (b.isZero()) {
    set(a);
    return;
  }

  if (a.degree() == 0 || b.degree() == 0)
    setOne();
  else {
    mHead = a.mHead;
    IOneParameterModularShortAux tmp(b);
    if (tmp.degree() > degree())
      mHead.swap(tmp.mHead);

    do {
      mHead.swap(tmp.mHead);
      tmp.rem(*this);
    } while (!tmp.isZero() && tmp.degree() > 0);

    if (!tmp.isZero())
      setOne();
    else
      pp();
  }
}

void IOneParameterModularShortAux::read(std::istream &in) {
  mHead.clear();
  int l;
  in.read((char*)(&l), sizeof(l));
  Iterator i(mHead.begin());
  while(l) {
    i.insert();
    in.read((char*)(&(*i).mDegree), sizeof((*i).mDegree));
    in >> (*i).mCoeff;
    ++i;
    --l;
  }
}

void IOneParameterModularShortAux::write(std::ostream &out) const {
  int l = mHead.length();
  out.write((char*)(&l), sizeof(l));
  ConstIterator i(mHead.begin());
  while(i) {
    out.write((char*)(&(*i).mDegree), sizeof((*i).mDegree));
    out << (*i).mCoeff;
    ++i;
  }
}

void IOneParameterModularShortAux::output(std::ostream &out) const {
  if (isZero()) {
    out << '0';
    return;
  }

  ConstIterator i(mHead.begin());
  if ((*i).mCoeff != 1)
    out << (*i).mCoeff << '*';
  out << "x^" << (*i).mDegree;
  ++i;
  
  while(i) {
    out << " + ";
    if ((*i).mCoeff != 1)
      out << (*i).mCoeff << '*';
    out << "x^" << (*i).mDegree;
    ++i;
  }
}

void IOneParameterModularShortAux::assertValid(const char* fileName, int fileLine) const {
  if (!mHead.isEmpty()) {
    ConstIterator i(mHead.begin());
    IASSERT2(fileName, fileLine, 0 < (*i).mCoeff && (*i).mCoeff < mPrimeShort);
    ConstIterator prev(i);
    ++i;
    while(i) {
      IASSERT2(fileName, fileLine, 0 < (*i).mCoeff && (*i).mCoeff < mPrimeShort);
      IASSERT2(fileName, fileLine, (*prev).mDegree > (*i).mDegree);
      prev = i;
      ++i;
    }
  }
}

void IOneParameterModularShortAux::set(const IOneParameterModularShortAux& a) {
  mHead.clear();
  Iterator i(mHead.begin());
  ConstIterator ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mDegree = (*ia).mDegree;
    (*i).mCoeff = (*ia).mCoeff;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

std::ostream& operator<<(std::ostream& out, const IOneParameterModularShortAux& a) {
  IOneParameterModularShortAux::ConstIterator i(a.begin());
  if (!i)
    out << '0';
  else {
    out << (*i).mCoeff << "*t^" <<  (*i).mDegree;
    ++i;
    while(i) {
      out << " + " << (*i).mCoeff << "*t^" <<  (*i).mDegree;
      ++i;
    }
  }
  return out;
}

template <>
IAllocator IList<IOneParameterModularShortAux::Term>::Node::sAllocator(sizeof(IList<IOneParameterModularShortAux::Term>::Node));
