/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IONEPARAMETERMODULARSHORTAUX_H
#define IONEPARAMETERMODULARSHORTAUX_H

#include <iostream>

#include "../util/iutil.h"
#include "../util/imodular.h"
#include "../util/ilist.h"

class ITwoParameterModularShortAux;
class INParameterModularShortAux;

class IOneParameterModularShortAux {
  friend class ITwoParameterModularShortAux;
  friend class INParameterModularShortAux;
  friend class IGcdAlgorithm;

public:
  struct Term {
    unsigned     mDegree;
    ModularShort mCoeff;

    Term(): mDegree(0), mCoeff(0) {}
    Term(const Term& a): mDegree(a.mDegree), mCoeff(a.mCoeff) {}
    ~Term() {}

    void operator=(const Term& a) {
      mDegree = a.mDegree;
      mCoeff = a.mCoeff;
    }
  };
  IList<Term>        mHead;

protected:
  const ModularShort mPrimeShort;

public:
  void add(IOneParameterModularShortAux& a);
  void sub(IOneParameterModularShortAux& a);
  void shiftMult(ModularShort coeff);
  ModularShort eval(ModularShort t) const;
  void mult(unsigned degree, ModularShort coeff);
  void div(ModularShort coeff);
  void div(unsigned degree, ModularShort coeff);
  void mult(ModularShort coeff);
  void rem(const IOneParameterModularShortAux& a);

  typedef IList<Term>::Iterator  Iterator;
  typedef IList<Term>::ConstIterator  ConstIterator;
  IOneParameterModularShortAux(ModularShort primeShort):
      mPrimeShort(primeShort),
      mHead() {
    IASSERT(iIsPrime(mPrimeShort));
  }
  IOneParameterModularShortAux(const IOneParameterModularShortAux& a):
      mPrimeShort(a.mPrimeShort),
      mHead(a.mHead) {
  }

  ~IOneParameterModularShortAux() {}

  ModularShort primeShort() const { return mPrimeShort; }

  void operator=(const IOneParameterModularShortAux& a) {
    IASSERT(mPrimeShort == a.mPrimeShort);
    mHead = a.mHead;
  }

  void swap(IOneParameterModularShortAux& a) {
    mHead.swap(a.mHead);
  }

  void set(const IOneParameterModularShortAux& a);

  void setZero() { mHead.clear(); }

  void setOne() {
    mHead.clear();
    Iterator i(mHead.begin());
    i.insert();
    (*i).mCoeff = 1;
  }

  bool isZero() const { return mHead.isEmpty(); }
  bool isOne() const {
    ConstIterator i(mHead.begin());
    return i && (*i).mDegree == 0 && (*i).mCoeff == 1;
  }
  bool isComposite() const { return mHead.length() > 1; }
  int norm() const { return mHead.length(); }
  unsigned degree() const {
    IASSERT(!isZero());
    return (*mHead.begin()).mDegree;
  }

  void minus();
  void pp() {
    IASSERT(!isZero());
    div(0, (*mHead.begin()).mCoeff);
  }
  void add(const IOneParameterModularShortAux& a);
  void add(int deg, ModularShort c);
  void sub(const IOneParameterModularShortAux& a);
  void mult(const IOneParameterModularShortAux& a);
  void mult(const IOneParameterModularShortAux& a, const IOneParameterModularShortAux& b);
  void diff();
  void diff(unsigned deg);
  void pow(unsigned deg);
  void div(const IOneParameterModularShortAux& a);
  void div(const IOneParameterModularShortAux& a, const IOneParameterModularShortAux& b) {
    *this = a;
    div(b);
  }
  void gcd(const IOneParameterModularShortAux& a);
  void gcd(const IOneParameterModularShortAux& a, const IOneParameterModularShortAux& b);
  void addmul(const IOneParameterModularShortAux& a, const IOneParameterModularShortAux& b) {
    IOneParameterModularShortAux tmp(mPrimeShort);
    tmp.mult(a, b);
    add(tmp);
  }

  void addmul(const IOneParameterModularShortAux& a, const ModularShort c);

  ConstIterator begin() const { return mHead.begin(); }
  Iterator begin() { return mHead.begin(); }

  ModularShort lc() const {
    return (*mHead.begin()).mCoeff;
  }

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void output(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;

  friend std::ostream& operator<<(std::ostream& out, const IOneParameterModularShortAux& a);
};

#endif // IONEPARAMETERMODULARSHORTAUX_H
