/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <gmp.h>

#include "./itwoparametermodularshort.h"

ITwoParameterModularShortInterface::ITwoParameterModularShortInterface(IVariables *parameter,
                                                                       int primeShort):
    ICoeffInterface(),
    mPrimeShort(primeShort) {
  IASSERT(parameter);
  mParameters = parameter;

  if ((1 << (sizeof(int)*4)-1) < mPrimeShort)
    IERROR("primeShort^2 > (sizeof(int)*4-1)^2");

  if (!iIsPrime(primeShort))
    IERROR("primeShort is not a prime");

  mType = ICoeffInterface::TwoParameterModularShort;
}

ICoeff* ITwoParameterModularShortInterface::create() {
  return new ITwoParameterModularShort(this);
}

ICoeff* ITwoParameterModularShortInterface::copy(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  return new ITwoParameterModularShort(a);
}

ITwoParameterModularShort::ITwoParameterModularShort(ITwoParameterModularShortInterface* r):
    ICoeff(r),
    mPoly(r->mPrimeShort) {
}

ITwoParameterModularShort::ITwoParameterModularShort(const ICoeff &a):
    ICoeff(a.coeffInterface()),
    mPoly(((const ITwoParameterModularShort&)a).mPoly) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  IASSERTVALID(*this);
}

ITwoParameterModularShort::~ITwoParameterModularShort() {
}

void ITwoParameterModularShort::set(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly = ((const ITwoParameterModularShort&)a).mPoly;
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::setZero() {
  mPoly.setZero();
}

void ITwoParameterModularShort::setOne() {
  mPoly.setOne();
}

bool ITwoParameterModularShort::isNegative() const {
  return false;
}

bool ITwoParameterModularShort::isZero() const {
  return mPoly.isZero();
}

bool ITwoParameterModularShort::isOne() const {
  return mPoly.isOne();
}

bool ITwoParameterModularShort::isComposite() const {
  return mPoly.isComposite();
}

bool ITwoParameterModularShort::isUnit() const {
  return (!mPoly.isZero() && mPoly.degree1() == 0 && mPoly.degree2() == 0);
}

int ITwoParameterModularShort::norm() const {
  return mPoly.norm();
}

void ITwoParameterModularShort::minus() {
  mPoly.minus();
}

void ITwoParameterModularShort::abs() {
}

void ITwoParameterModularShort::add(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.add(((const ITwoParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::sub(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.sub(((const ITwoParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::mult(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.mult(((const ITwoParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::diff(int var, unsigned deg) {
  mPoly.diff(var, deg);
}

void ITwoParameterModularShort::shift(int , unsigned ) {
}

void ITwoParameterModularShort::pow(unsigned deg) {
  mPoly.pow(deg);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::inverse() {
  IERROR("inverse");
}

void ITwoParameterModularShort::div(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.div(((const ITwoParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::div(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  IASSERT(b.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.div(((const ITwoParameterModularShort&)a).mPoly, ((const ITwoParameterModularShort&)b).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::gcd(const ICoeff& a) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.gcd(((const ITwoParameterModularShort&)a).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::gcd(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  IASSERT(b.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.gcd(((const ITwoParameterModularShort&)a).mPoly, ((const ITwoParameterModularShort&)b).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::addmul(const ICoeff& a, const ICoeff& b) {
  IASSERT(a.type() == ICoeffInterface::TwoParameterModularShort);
  IASSERT(b.type() == ICoeffInterface::TwoParameterModularShort);
  mPoly.addmul(((const ITwoParameterModularShort&)a).mPoly, ((const ITwoParameterModularShort&)b).mPoly);
  IASSERTVALID(*this);
}

void ITwoParameterModularShort::input(std::istream& in, bool expanded) {
  std::streampos posbeg = in.tellg();
  int var = parameters()->read(in);
  if (var == 0) {
    mPoly.setOne();
    (*mPoly.begin()).mDegree1 = 1;
    IASSERTVALID(*this);
  }
  else if (var == 1) {
    mPoly.setOne();
    (*mPoly.begin()).mDegree2 = 1;
    IASSERTVALID(*this);
  }
  else {
    MP_INT mp;
    mpz_init(&mp);
    in >> (&mp);
    if (in.fail()) {
      in.clear();
      in.setstate(std::ios::failbit);
      in.seekg(posbeg);
    }
    else {
      int sign = mpz_sgn(&mp);
      mpz_mod_ui(&mp, &mp, (unsigned long)primeShort());
      int modularShort = (int)mpz_get_ui(&mp);
      if (modularShort == 0)
        mPoly.setZero();
      else {
        if (sign < 0)
          modularShort = primeShort() - modularShort;
        mPoly.setOne();
        (*mPoly.begin()).mCoeff = modularShort;
      }
      IASSERTVALID(*this);
    }
    mpz_clear(&mp);
  }
}

void ITwoParameterModularShort::output(std::ostream& out) const {
  if (isZero())
    out << '0';
  else {
    ITwoParameterModularShortAux::ConstIterator i(mPoly.begin());
    if ((*i).mCoeff == 1) {
      if ((*i).mDegree1 + (*i).mDegree2 == 0)
        out << '1';
      else {
        if ((*i).mDegree1 == 1)
          out << parameters()->variable(0);
        else if ((*i).mDegree1 > 1)
          out << parameters()->variable(0) << '^' << (*i).mDegree1;
        if ((*i).mDegree1 && (*i).mDegree2)
          out << '*';
        if ((*i).mDegree2 == 1)
          out << parameters()->variable(1);
        else if ((*i).mDegree2 > 1)
          out << parameters()->variable(1) << '^' << (*i).mDegree2;
      }
    }
    else
      if ((*i).mDegree1 + (*i).mDegree2 == 0)
        out << (*i).mCoeff;
      else {
        out << (*i).mCoeff << '*';
        if ((*i).mDegree1 == 1)
          out << parameters()->variable(0);
        else if ((*i).mDegree1 > 1)
          out << parameters()->variable(0) << '^' << (*i).mDegree1;
        if ((*i).mDegree1 && (*i).mDegree2)
          out << '*';
        if ((*i).mDegree2 == 1)
          out << parameters()->variable(1);
        else if ((*i).mDegree2 > 1)
          out << parameters()->variable(1) << '^' << (*i).mDegree2;
      }
    ++i;
    while(i) {
      out << " + ";
      if ((*i).mCoeff == 1)
        if ((*i).mDegree1 + (*i).mDegree2 == 0)
          out << '1';
        else {
          if ((*i).mDegree1 == 1)
            out << parameters()->variable(0);
          else if ((*i).mDegree1 > 1)
            out << parameters()->variable(0) << '^' << (*i).mDegree1;
          if ((*i).mDegree1 && (*i).mDegree2)
            out << '*';
          if ((*i).mDegree2 == 1)
            out << parameters()->variable(1);
          else if ((*i).mDegree2 > 1)
            out << parameters()->variable(1) << '^' << (*i).mDegree2;
        }
      else
        if ((*i).mDegree1 + (*i).mDegree2 == 0)
          out << (*i).mCoeff;
        else {
          out << (*i).mCoeff << '*';
          if ((*i).mDegree1 == 1)
            out << parameters()->variable(0);
          else if ((*i).mDegree1 > 1)
            out << parameters()->variable(0) << '^' << (*i).mDegree1;
          if ((*i).mDegree1 && (*i).mDegree2)
            out << '*';
          if ((*i).mDegree2 == 1)
            out << parameters()->variable(1);
          else if ((*i).mDegree2 > 1)
            out << parameters()->variable(1) << '^' << (*i).mDegree2;
        }
      ++i;
    }
  }
}

void ITwoParameterModularShort::assertValid(const char* fileName, int fileLine) const {
  mPoly.assertValid(fileName, fileLine);
}

IAllocator ITwoParameterModularShort::sAllocator(sizeof(ITwoParameterModularShort));
