/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./itwoparametermodularshortaux.h"

static inline int compare(unsigned a1, unsigned a2, unsigned b1, unsigned b2) {
  if (a1 > b1)
    return 1;
  else if (a1 < b1)
    return -1;
  else if (a2 > b2)
    return 1;
  else if (a2 < b2)
    return -1;
  else
    return 0;
}

unsigned ITwoParameterModularShortAux::maxDegree() const {
  IASSERT(!isZero());
  ConstIterator i(mHead.begin());
  unsigned degree2 = 0;
  while(i) {
    if ((*i).mDegree2 > degree2)
      degree2 = (*i).mDegree2;
    ++i;
  }
  return degree2;
}

void ITwoParameterModularShortAux::set1(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.primeShort());
  Iterator  i(mHead.begin());
  IOneParameterModularShortAux::ConstIterator ia(a.begin());
  i.clear();
  while(ia) {
    i.insert();
    (*i).mDegree1 = (*ia).mDegree;
    (*i).mCoeff = (*ia).mCoeff;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::set2(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.primeShort());
  Iterator  i(mHead.begin());
  IOneParameterModularShortAux::ConstIterator ia(a.begin());
  i.clear();
  while(ia) {
    i.insert();
    (*i).mDegree2 = (*ia).mDegree;
    (*i).mCoeff = (*ia).mCoeff;
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::lc(IOneParameterModularShortAux& a) const {
  IASSERT(mPrimeShort == a.primeShort());
  IASSERT(!isZero());
  ConstIterator i(mHead.begin());
  IOneParameterModularShortAux::Iterator ia(a.begin());
  ia.clear();
  unsigned degree1 = (*i).mDegree1;
  do {
    ia.insert();
    (*ia).mDegree = (*i).mDegree2;
    (*ia).mCoeff = (*i).mCoeff;
    ++ia;
    ++i;
  } while(i && (*i).mDegree1 == degree1);
  IASSERTVALID(a);
}

void ITwoParameterModularShortAux::eval1(IOneParameterModularShortAux& a, int u) const {
  IASSERT(mPrimeShort == a.primeShort());
  IASSERT(!isZero());
  IASSERT(0 < u && u < mPrimeShort);

  ConstIterator i(mHead.begin());
  IOneParameterModularShortAux::Iterator ia(a.begin());
  ia.clear();
  unsigned degree1=(*i).mDegree1;
  int coeff = iPow0(u, degree1, mPrimeShort);
  do {
    ia.insert();
    (*ia).mDegree = (*i).mDegree2;
    (*ia).mCoeff = ((*i).mCoeff * coeff) % mPrimeShort;
    ++ia;
    ++i;
  } while(i && (*i).mDegree1 == degree1);
  IASSERTVALID(a);

  if (i) {
    IOneParameterModularShortAux b(mPrimeShort);
    do {
      IOneParameterModularShortAux::Iterator ib(b.begin());
      ib.clear();
      degree1=(*i).mDegree1;
      coeff = iPow0(u, degree1, mPrimeShort);
      do {
        ib.insert();
        (*ib).mDegree = (*i).mDegree2;
        (*ib).mCoeff = ((*i).mCoeff * coeff) % mPrimeShort;
        ++ib;
        ++i;
      } while(i && (*i).mDegree1 == degree1);
      IASSERTVALID(b);
      a.add(b);
    } while(i);
  }
}

void ITwoParameterModularShortAux::eval2(IOneParameterModularShortAux& a, int v) const {
  IASSERT(mPrimeShort == a.primeShort());
  IASSERT(!isZero());
  IASSERT(0 < v && v < mPrimeShort);
  ConstIterator i(mHead.begin());
  IOneParameterModularShortAux::Iterator ia(a.begin());
  ia.clear();
  do {
    unsigned degree1=(*i).mDegree1;
    int coeff = ((*i).mCoeff * iPow0(v, (*i).mDegree2, mPrimeShort)) % mPrimeShort;
    ++i;
    while(i && (*i).mDegree1 == degree1) {
      coeff = (coeff + (*i).mCoeff * iPow0(v, (*i).mDegree2, mPrimeShort)) % mPrimeShort;
      ++i;
    }
    if (coeff != 0) {
      ia.insert();
      (*ia).mDegree = degree1;
      (*ia).mCoeff = coeff;
      ++ia;
    }
  } while(i);
  IASSERTVALID(a);
}

void ITwoParameterModularShortAux::mult1(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.primeShort());
  IOneParameterModularShortAux::ConstIterator ia(a.begin());
  if (!ia.isNext())
    mult((*ia).mDegree, 0, (*ia).mCoeff);
  else {
    ITwoParameterModularShortAux p(mPrimeShort);
    while(ia) {
      ITwoParameterModularShortAux tmp(*this);
      tmp.mult((*ia).mDegree, 0, (*ia).mCoeff);
      p.add(tmp);
      ++ia;
    }
    mHead.swap(p.mHead);
    IASSERTVALID(*this);
  }
}

void ITwoParameterModularShortAux::mult2(const IOneParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.primeShort());
  IOneParameterModularShortAux::ConstIterator ia(a.begin());
  if (!ia.isNext())
    mult(0, (*ia).mDegree, (*ia).mCoeff);
  else {
    ITwoParameterModularShortAux p(mPrimeShort);
    while(ia) {
      ITwoParameterModularShortAux tmp(*this);
      tmp.mult(0, (*ia).mDegree, (*ia).mCoeff);
      p.add(tmp);
      ++ia;
    }
    mHead.swap(p.mHead);
    IASSERTVALID(*this);
  }
}

void ITwoParameterModularShortAux::addmult(const IOneParameterModularShortAux& u,
                                          const IOneParameterModularShortAux& v) {
  IASSERT(mPrimeShort == u.primeShort());
  IASSERT(mPrimeShort == v.primeShort());

  ITwoParameterModularShortAux tmp(mPrimeShort);
  Iterator i(tmp.mHead.begin());
  IOneParameterModularShortAux::ConstIterator iu(u.begin());
  while(iu) {
    IOneParameterModularShortAux::ConstIterator iv(v.begin());
    while(iv) {
      i.insert();
      (*i).mDegree1 = (*iu).mDegree;
      (*i).mDegree2 = (*iv).mDegree;
      (*i).mCoeff = ((*iu).mCoeff * (*iv).mCoeff) % mPrimeShort;
      ++i;
      ++iv;
    }
    ++iu;
  }
  IASSERTVALID(tmp);
  add(tmp);
}

void ITwoParameterModularShortAux::cont(IOneParameterModularShortAux& a) const {
  IASSERT(mPrimeShort == a.primeShort());
  IASSERT(!isZero());
  //IASSERT(degree1() > 0);
  ConstIterator i(mHead.begin());
  IOneParameterModularShortAux::Iterator ia(a.begin());
  ia.clear();
  unsigned degree1 = (*i).mDegree1;
  do {
    ia.insert();
    (*ia).mDegree = (*i).mDegree2;
    (*ia).mCoeff = (*i).mCoeff;
    ++ia;
    ++i;
  } while(i && (*i).mDegree1 == degree1);
  IASSERTVALID(a);

  if (i && !a.isOne()) {
    IOneParameterModularShortAux b(mPrimeShort);
    do {
      IOneParameterModularShortAux::Iterator ib(b.begin());
      ib.clear();
      degree1=(*i).mDegree1;
      do {
        ib.insert();
        (*ib).mDegree = (*i).mDegree2;
        (*ib).mCoeff = (*i).mCoeff;
        ++ib;
        ++i;
      } while(i && (*i).mDegree1 == degree1);
      IASSERTVALID(b);
      a.gcd(b);
    } while(i && !a.isOne());
  }
}

bool ITwoParameterModularShortAux::ispp() const {
  IOneParameterModularShortAux c(mPrimeShort);
  cont(c);
  return c.isOne();
}

void ITwoParameterModularShortAux::add(ITwoParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  Iterator  i2(a.mHead.begin());
  while(i1 && i2) {
    switch(compare((*i1).mDegree1, (*i1).mDegree2, (*i2).mDegree1, (*i2).mDegree2)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert(i2.get());
      ++i1;
      break;
    default:
      (*i1).mCoeff += (*i2).mCoeff;
      if ((*i1).mCoeff >= mPrimeShort)
        (*i1).mCoeff -= mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert(i2.get());
    ++i1;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::mult(unsigned degree1, unsigned degree2, int coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mDegree1 += degree1;
    (*i).mDegree2 += degree2;
    (*i).mCoeff = ((*i).mCoeff * coeff) % mPrimeShort;
    ++i;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::div(unsigned degree1, unsigned degree2, int coeff) {
  IASSERT(0 < coeff && coeff < mPrimeShort);

  int inverse = iInverse(coeff, mPrimeShort);

  ConstIterator i(mHead.begin());
  while(i) {
    IASSERT((*i).mDegree1 >= degree1);
    IASSERT((*i).mDegree2 >= degree2);
    (*i).mDegree1 -= degree1;
    (*i).mDegree2 -= degree2;
    (*i).mCoeff = ((*i).mCoeff * inverse) % mPrimeShort;
    ++i;
  }
  IASSERTVALID(*this);
}

bool ITwoParameterModularShortAux::divisibility(const ITwoParameterModularShortAux& a) const {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(!a.isZero());

  int inverse1 = iInverse((*a.mHead.begin()).mCoeff, mPrimeShort);
  inverse1 = mPrimeShort - inverse1;

  ITwoParameterModularShortAux  tmp(*this);

  while(!tmp.isZero() &&
        tmp.degree1() >= a.degree1() &&
        tmp.degree2() >= a.degree2()) {

    Iterator  i1(tmp.mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg1 = tmp.degree1() - a.degree1();
    unsigned deg2 = tmp.degree2() - a.degree2();
    int coeff = ((*i1).mCoeff * inverse1) % mPrimeShort;

    IASSERT((*i1).mDegree1 == (*i2).mDegree1 + deg1);
    IASSERT((*i1).mDegree2 == (*i2).mDegree2 + deg2);
    IASSERT(((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort == 0);
    i1.del();
    ++i2;

    while(i1 && i2) {
      switch(compare((*i1).mDegree1, (*i1).mDegree2, (*i2).mDegree1 + deg1, (*i2).mDegree2 + deg2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mDegree1 = (*i2).mDegree1 + deg1;
        (*i1).mDegree2 = (*i2).mDegree2 + deg2;
        (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff = ((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree1 = (*i2).mDegree1 + deg1;
      (*i1).mDegree2 = (*i2).mDegree2 + deg2;
      (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
  return tmp.isZero();
}

void ITwoParameterModularShortAux::div(const ITwoParameterModularShortAux& a, const IOneParameterModularShortAux& b) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(mPrimeShort == b.primeShort());
  IASSERT(!a.isZero());
  IASSERT(!b.isZero());
  IASSERT(a.degree2() >= b.degree());

  int inverse1 = iInverse((*b.begin()).mCoeff, mPrimeShort);
  inverse1 = mPrimeShort - inverse1;

  ITwoParameterModularShortAux rem(a);
  Iterator i(mHead.begin());
  i.clear();

  while(!rem.isZero()) {
    IASSERT(rem.degree2() >= b.degree());
    Iterator i1(rem.mHead.begin());
    IOneParameterModularShortAux::ConstIterator i2(b.begin());

    unsigned deg1 = rem.degree1();
    unsigned deg2 = rem.degree2() - b.degree();
    int coeff = ((*i1).mCoeff * inverse1) % mPrimeShort;

    i.insert();
    (*i).mDegree1 = deg1;
    (*i).mDegree2 = deg2;
    (*i).mCoeff = mPrimeShort - coeff;
    ++i;
    IASSERTVALID(*this);

    IASSERT((*i1).mDegree1 == deg1);
    IASSERT((*i1).mDegree2 == (*i2).mDegree + deg2);
    IASSERT(((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort == 0);
    i1.del();
    ++i2;

    while(i1 && i2) {
      switch(compare((*i1).mDegree1, (*i1).mDegree2, deg1, (*i2).mDegree + deg2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mDegree1 = deg1;
        (*i1).mDegree2 = (*i2).mDegree + deg2;
        (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff = ((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree1 = deg1;
      (*i1).mDegree2 = (*i2).mDegree + deg2;
      (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
      ++i1;
      ++i2;
    }
    IASSERTVALID(rem);
  }
  IASSERT(rem.isZero());
}

void ITwoParameterModularShortAux::gcd1(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(mPrimeShort == b.mPrimeShort);
  IASSERT(a.degree1() > 0);
  IASSERT(b.degree1() > 0);
  IASSERT(a.ispp());
  IASSERT(b.ispp());

/*
  std::cout << "a = " << a << "\nb = " << b << std::endl;
  std::cout << "(" << a.degree1() << ", " << a.maxDegree() << ", "
            << b.degree1() << ", " << b.maxDegree() << "; " << std::flush;
*/

  int v, g1;
  unsigned deg1, deg2;
  bool gcd_ok = false;

  deg1 = a.maxDegree();
  deg2 = b.maxDegree();
  if (deg1 < deg2) deg2 = deg1;

  IOneParameterModularShortAux au(mPrimeShort),
                              bu(mPrimeShort),
                              m(mPrimeShort),
                              gamma(mPrimeShort);
  ITwoParameterModularShortAux g(mPrimeShort);

  a.lc(gamma);
  b.lc(m);
  gamma.gcd(m);

  do {
    do {
      v = iRandom(mPrimeShort);
      g1 = gamma.eval(v);
      if (g1 == 0)
        continue;
      a.eval2(au, v);
      if (au.isZero() ||
          a.degree1() != au.degree())
        continue;
      b.eval2(bu, v);
      if (bu.isZero() ||
          b.degree1() != bu.degree())
        continue;
      break;
    } while(true);

    au.gcd(bu);
    au.mult(g1);

    m.setOne();
    m.shiftMult(v);

    deg1 = au.degree();
    set1(au);

/*
    std::cout << v << " " << std::flush;
*/

    do {
      int v1,c;
      do {
        v1 = iRandom(mPrimeShort);
        g1 = gamma.eval(v1);
        if (g1 == 0)
          continue;
        a.eval2(au, v1);
        if (au.isZero() ||
            a.degree1() != au.degree())
          continue;
        b.eval2(bu, v1);
        if (bu.isZero() ||
            b.degree1() != bu.degree())
          continue;
        c = m.eval(v1);
        if (c == 0)  // then v1 has already been considered
          continue;
        break;
      } while(true);

/*
      std::cout << v1 << " " << std::flush;
*/

      au.gcd(bu);
      au.mult(g1);
      if (au.degree() == 0) {
        setOne();
        gcd_ok = true;
        break;
      }

      if (au.degree() > deg1)
        continue;

      eval2(bu, v1);
      au.sub(bu);
      if (au.isZero()) {
        cont(bu);
        if (!bu.isOne()) {
          g.div(*this, bu);
          mHead.swap(g.mHead);
        }
        if (!a.divisibility(*this) ||
            !b.divisibility(*this))
          continue;
        gcd_ok = true;
        break;
      }

      if (m.degree() - 1 > deg2) {
        // very first evaluation point led to a degree bound
        // for the first variable which was too small
        break;
      }

      c = iInverse(c, mPrimeShort);
      au.mult(c);
      addmult(au, m);
      v = v1;
      m.shiftMult(v);
    } while(true);
  } while (!gcd_ok);
/*
  std::cout << ")" << std::flush;
*/
}

void ITwoParameterModularShortAux::minus() {
  ConstIterator i(mHead.begin());
  while(i) {
    (*i).mCoeff = mPrimeShort - (*i).mCoeff;
    ++i;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::add(const ITwoParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    switch(compare((*i1).mDegree1, (*i1).mDegree2, (*i2).mDegree1, (*i2).mDegree2)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mDegree1 = (*i2).mDegree1;
      (*i1).mDegree2 = (*i2).mDegree2;
      (*i1).mCoeff = (*i2).mCoeff;
      ++i1;
      ++i2;
      break;
    default:
      (*i1).mCoeff += (*i2).mCoeff;
      if ((*i1).mCoeff >= mPrimeShort)
        (*i1).mCoeff -= mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree1 = (*i2).mDegree1;
    (*i1).mDegree2 = (*i2).mDegree2;
    (*i1).mCoeff = (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::sub(const ITwoParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  Iterator  i1(mHead.begin());
  ConstIterator  i2(a.mHead.begin());
  while(i1 && i2) {
    switch(compare((*i1).mDegree1, (*i1).mDegree2, (*i2).mDegree1, (*i2).mDegree2)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mDegree1 = (*i2).mDegree1;
      (*i1).mDegree2 = (*i2).mDegree2;
      (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
      ++i1;
      ++i2;
      break;
    default:
      (*i1).mCoeff -= (*i2).mCoeff;
      if ((*i1).mCoeff < 0)
        (*i1).mCoeff += mPrimeShort;

      if ((*i1).mCoeff == 0)
        i1.del();
      else
        ++i1;
      ++i2;
    }
  }
  while(i2) {
    i1.insert();
    (*i1).mDegree1 = (*i2).mDegree1;
    (*i1).mDegree2 = (*i2).mDegree2;
    (*i1).mCoeff = mPrimeShort - (*i2).mCoeff;
    ++i1;
    ++i2;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::mult(const ITwoParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);

  if (isZero())
    return;

  ITwoParameterModularShortAux prod(mPrimeShort);
  ConstIterator ia(a.mHead.begin());
  Iterator begin(prod.mHead.begin());

  while(ia) {
    ConstIterator i(mHead.begin());
    Iterator ip = begin;
    unsigned deg1 = (*i).mDegree1 + (*ia).mDegree1;
    unsigned deg2 = (*i).mDegree2 + (*ia).mDegree2;

    // look for the starting point
    while(ip && compare((*ip).mDegree1, (*ip).mDegree2, deg1, deg2) == 1)
      ++ip;

    begin = ip;

    while(ip && i) {
      deg1 = (*i).mDegree1 + (*ia).mDegree1;
      deg2 = (*i).mDegree2 + (*ia).mDegree2;
      switch(compare((*ip).mDegree1, (*ip).mDegree2, deg1, deg2)) {
      case 1:
        ++ip;
        break;
      case -1:
        ip.insert();
        (*ip).mDegree1 = deg1;
        (*ip).mDegree2 = deg2;
        (*ip).mCoeff = ((*i).mCoeff * (*ia).mCoeff) % mPrimeShort;
        IASSERT((*ip).mCoeff != 0);
        //if ((*ip).mCoeff == 0)
          //ip.del();
        //else
          ++ip;
        ++i;
        break;
      default:
        (*ip).mCoeff = ((*ip).mCoeff + (*i).mCoeff * (*ia).mCoeff) % mPrimeShort;
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++i;
      }
    }

    while(i) {
      ip.insert();
      (*ip).mDegree1 = (*i).mDegree1 + (*ia).mDegree1;
      (*ip).mDegree2 = (*i).mDegree2 + (*ia).mDegree2;
      (*ip).mCoeff = ((*i).mCoeff * (*ia).mCoeff) % mPrimeShort;
      IASSERT((*ip).mCoeff != 0);
      //if ((*ip).mCoeff == 0)
        //ip.del();
      //else
        ++ip;
      ++i;
    }

    ++ia;
  }
  mHead.swap(prod.mHead);
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::mult(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(mPrimeShort == b.mPrimeShort);

  setZero();
  if (a.isZero() || b.isZero())
    return;

  ConstIterator ib(b.mHead.begin());
  Iterator begin(mHead.begin());

  while(ib) {
    ConstIterator ia(a.mHead.begin());
    Iterator ip = begin;
    unsigned deg1 = (*ia).mDegree1 + (*ib).mDegree1;
    unsigned deg2 = (*ia).mDegree2 + (*ib).mDegree2;

    // look for the starting point
    while(ip && compare((*ip).mDegree1, (*ip).mDegree2, deg1, deg2) == 1)
      ++ip;

    begin = ip;

    while(ip && ia) {
      deg1 = (*ia).mDegree1 + (*ib).mDegree1;
      deg2 = (*ia).mDegree2 + (*ib).mDegree2;
      switch(compare((*ip).mDegree1, (*ip).mDegree2, deg1, deg2)) {
      case 1:
        ++ip;
        break;
      case -1:
        ip.insert();
        (*ip).mDegree1 = deg1;
        (*ip).mDegree2 = deg2;
        (*ip).mCoeff = ((*ia).mCoeff * (*ib).mCoeff) % mPrimeShort;
        IASSERT((*ip).mCoeff != 0);
        //if ((*ip).mCoeff == 0)
          //ip.del();
        //else
          ++ip;
        ++ia;
        break;
      default:
        (*ip).mCoeff = ((*ip).mCoeff + (*ia).mCoeff * (*ib).mCoeff) % mPrimeShort;
        if ((*ip).mCoeff == 0)
          ip.del();
        else
          ++ip;
        ++ia;
      }
    }

    while(ia) {
      ip.insert();
      (*ip).mDegree1 = (*ia).mDegree1 + (*ib).mDegree1;
      (*ip).mDegree2 = (*ia).mDegree2 + (*ib).mDegree2;
      (*ip).mCoeff = ((*ia).mCoeff * (*ib).mCoeff) % mPrimeShort;
      IASSERT((*ip).mCoeff != 0);
      //if ((*ip).mCoeff == 0)
        //ip.del();
      //else
        ++ip;
      ++ia;
    }

    ++ib;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::diff1() {
  Iterator i(mHead.begin());
  while(i) {
    if ((*i).mDegree1 == 0) {
      i.del();
      break;
    }
    (*i).mCoeff = ((*i).mCoeff * (*i).mDegree1) % mPrimeShort;
    --(*i).mDegree1;
    ++i;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::diff2() {
  Iterator i(mHead.begin());
  while(i) {
    if ((*i).mDegree2 == 0) {
      i.del();
      break;
    }
    (*i).mCoeff = ((*i).mCoeff * (*i).mDegree2) % mPrimeShort;
    --(*i).mDegree2;
    ++i;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::diff(unsigned deg1, unsigned deg2) {
  IASSERT(deg1 > 0);
  IASSERT(deg2 > 0);
  Iterator i(mHead.begin());
  while(i) {
    if ((*i).mDegree1 < deg1 ||
        (*i).mDegree2 < deg2) {
      i.clear();
      break;
    }

    unsigned n = (*i).mDegree1;
    (*i).mDegree1 -= deg1;
    do {
      (*i).mCoeff = ((*i).mCoeff * n) % mPrimeShort;
      --n;
    } while (n != (*i).mDegree1);

    n = (*i).mDegree2;
    (*i).mDegree2 -= deg2;
    do {
      (*i).mCoeff = ((*i).mCoeff * n) % mPrimeShort;
      --n;
    } while (n != (*i).mDegree2);

    ++i;
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::pow(unsigned deg) {
  IASSERT(deg > 0);
  if (deg > 1) {
    ITwoParameterModularShortAux p(*this);
    while(--deg > 0) {
      p.mult(*this);
    }
    mHead.swap(p.mHead);
  }
  IASSERTVALID(*this);
}

void ITwoParameterModularShortAux::div(const ITwoParameterModularShortAux& a) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(!a.isZero());
  IASSERT(degree1() >= a.degree1());
  IASSERT(degree2() >= a.degree2());

  int inverse1 = iInverse((*a.mHead.begin()).mCoeff, mPrimeShort);
  inverse1 = mPrimeShort - inverse1;

  ITwoParameterModularShortAux  quo(mPrimeShort);
  Iterator  i(quo.mHead.begin());

  while(!isZero()) {
    IASSERT(degree1() >= a.degree1());
    IASSERT(degree2() >= a.degree2());
    Iterator  i1(mHead.begin());
    ConstIterator  i2(a.mHead.begin());

    unsigned deg1 = degree1() - a.degree1();
    unsigned deg2 = degree2() - a.degree2();
    int coeff = ((*i1).mCoeff * inverse1) % mPrimeShort;

    i.insert();
    (*i).mDegree1 = deg1;
    (*i).mDegree2 = deg2;
    (*i).mCoeff = mPrimeShort - coeff;
    ++i;
    IASSERTVALID(quo);

    IASSERT((*i1).mDegree1 == (*i2).mDegree1 + deg1);
    IASSERT((*i1).mDegree2 == (*i2).mDegree2 + deg2);
    IASSERT(((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort == 0);
    i1.del();
    ++i2;

    while(i1 && i2) {
      switch(compare((*i1).mDegree1, (*i1).mDegree2, (*i2).mDegree1 + deg1, (*i2).mDegree2 + deg2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mDegree1 = (*i2).mDegree1 + deg1;
        (*i1).mDegree2 = (*i2).mDegree2 + deg2;
        (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff = ((*i1).mCoeff + (*i2).mCoeff * coeff) % mPrimeShort;
        if ((*i1).mCoeff == 0)
          i1.del();
        else
          ++i1;
        ++i2;
      }
    }
    while(i2) {
      i1.insert();
      (*i1).mDegree1 = (*i2).mDegree1 + deg1;
      (*i1).mDegree2 = (*i2).mDegree2 + deg2;
      (*i1).mCoeff = ((*i2).mCoeff * coeff) % mPrimeShort;
      ++i1;
      ++i2;
    }
    IASSERTVALID(*this);
  }
  IASSERT(isZero());
  mHead.swap(quo.mHead);
}

void ITwoParameterModularShortAux::gcd(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b) {
  IASSERT(mPrimeShort == a.mPrimeShort);
  IASSERT(mPrimeShort == b.mPrimeShort);
  IASSERT(!a.isZero());
  IASSERT(!b.isZero());

  if (a.degree1() == 0 || b.degree1() == 0) {
    if (a.degree2() == 0 || b.degree2() == 0)
      setOne();
    else {
      IOneParameterModularShortAux contA(mPrimeShort);
      a.cont(contA);
      if (contA.isOne())
        setOne();
      else {
        IOneParameterModularShortAux contB(mPrimeShort);
        b.cont(contB);
        if (contB.isOne())
          setOne();
        else {
          contA.gcd(contB);
          if (contA.isOne())
            setOne();
          else
            set2(contA);
        }
      }
    }
  }
  else {
    IOneParameterModularShortAux contA(mPrimeShort),
                                contB(mPrimeShort);
    a.cont(contA);
    b.cont(contB);

    ITwoParameterModularShortAux ppA(mPrimeShort),
                                ppB(mPrimeShort);
    if (contA.isOne())
      ppA = a;
    else
      ppA.div(a, contA);

    if (contB.isOne())
      ppB = b;
    else
      ppB.div(b, contB);

    gcd1(ppA, ppB);

    if (!contA.isOne() &&
        !contB.isOne()) {
      contA.gcd(contB);
      if (!contA.isOne()) {
        if (isOne())
          set2(contA);
        else
          mult2(contA);
      }
    }
  }
}

void ITwoParameterModularShortAux::read(std::istream &in) {
  mHead.clear();
  int l;
  in.read((char*)(&l), sizeof(l));
  Iterator i(mHead.begin());
  while(l) {
    i.insert();
    in.read((char*)(&(*i).mDegree1), sizeof((*i).mDegree1));
    in.read((char*)(&(*i).mDegree2), sizeof((*i).mDegree2));
    in >> (*i).mCoeff;
    ++i;
    --l;
  }
}

void ITwoParameterModularShortAux::write(std::ostream &out) const {
  int l = mHead.length();
  out.write((char*)(&l), sizeof(l));
  ConstIterator i(mHead.begin());
  while(i) {
    out.write((char*)(&(*i).mDegree1), sizeof((*i).mDegree1));
    out.write((char*)(&(*i).mDegree2), sizeof((*i).mDegree2));
    out << (*i).mCoeff;
    ++i;
  }
}

void ITwoParameterModularShortAux::assertValid(const char* fileName, int fileLine) const {
  if (!mHead.isEmpty()) {
    ConstIterator i(mHead.begin());
    IASSERT2(fileName, fileLine, 0 < (*i).mCoeff && (*i).mCoeff < mPrimeShort);
    ConstIterator prev(i);
    ++i;
    while(i) {
      IASSERT2(fileName, fileLine, 0 < (*i).mCoeff && (*i).mCoeff < mPrimeShort);
      IASSERT2(fileName, fileLine, (*prev).mDegree1 > (*i).mDegree1 ||
                                   ((*prev).mDegree1 == (*i).mDegree1 && (*prev).mDegree2 > (*i).mDegree2));
      prev = i;
      ++i;
    }
  }
}

std::ostream& operator<<(std::ostream& out, const ITwoParameterModularShortAux& a) {
  ITwoParameterModularShortAux::ConstIterator i(a.begin());
  if (!i)
    out << '0';
  else {
    out << (*i).mCoeff << "*u^" <<  (*i).mDegree1 << "*w^" <<  (*i).mDegree2;
    ++i;
    while(i) {
      out << " + " << (*i).mCoeff << "*u^" <<  (*i).mDegree1 << "*w^" <<  (*i).mDegree2;
      ++i;
    }
  }
  return out;
}

template <>
IAllocator IList<ITwoParameterModularShortAux::Term>::Node::sAllocator(sizeof(IList<ITwoParameterModularShortAux::Term>::Node));
