/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ITWOPARAMETERMODULARSHORTAUX_H
#define ITWOPARAMETERMODULARSHORTAUX_H

#include <iostream>

#include "../util/iutil.h"
#include "../util/imodular.h"
#include "../util/ilist.h"
#include "./ioneparametermodularshortaux.h"

class ITwoParameterModularShortAux {
public:
  struct Term {
    unsigned  mDegree1;
    unsigned  mDegree2;
    int  mCoeff;

    Term():
        mDegree1(0),
        mDegree2(0),
        mCoeff(0) {}
    Term(const Term& a):
        mDegree1(a.mDegree1),
        mDegree2(a.mDegree2),
        mCoeff(a.mCoeff) {}
    ~Term() {}

    void operator=(const Term& a) {
      mDegree1 = a.mDegree1;
      mDegree2 = a.mDegree2;
      mCoeff = a.mCoeff;
    }
  };

protected:
  const int  mPrimeShort;
  IList<Term>  mHead;

  unsigned maxDegree() const;
  void set1(const IOneParameterModularShortAux& a);
  void set2(const IOneParameterModularShortAux& a);
  void lc(IOneParameterModularShortAux& a) const;
  void eval1(IOneParameterModularShortAux& a, int u) const;
  void eval2(IOneParameterModularShortAux& a, int v) const;
  void mult1(const IOneParameterModularShortAux& a);
  void mult2(const IOneParameterModularShortAux& a);
  void addmult(const IOneParameterModularShortAux& u, const IOneParameterModularShortAux& v);
  void cont(IOneParameterModularShortAux& a) const;
  bool ispp() const;
  void add(ITwoParameterModularShortAux& a);
  void mult(unsigned degree1, unsigned degree2, int coeff);
  void div(unsigned degree1, unsigned degree2, int coeff);
  bool divisibility(const ITwoParameterModularShortAux& a) const;
  void div(const ITwoParameterModularShortAux& a, const IOneParameterModularShortAux& b);
  void gcd1(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b);

public:
  typedef IList<Term>::Iterator  Iterator;
  typedef IList<Term>::ConstIterator  ConstIterator;
  ITwoParameterModularShortAux(int primeShort):
      mPrimeShort(primeShort),
      mHead() {
    IASSERT(iIsPrime(mPrimeShort));
  }
  ITwoParameterModularShortAux(const ITwoParameterModularShortAux& a):
      mPrimeShort(a.mPrimeShort),
      mHead(a.mHead) {
  }

  ~ITwoParameterModularShortAux() {}

  int primeShort() const { return mPrimeShort; }

  void operator=(const ITwoParameterModularShortAux& a) {
    IASSERT(mPrimeShort == a.mPrimeShort);
    mHead = a.mHead;
  }
  void setZero() { mHead.clear(); }
  void setOne() {
    mHead.clear();
    Iterator i(mHead.begin());
    i.insert();
    (*i).mCoeff = 1;
  }

  bool isZero() const { return mHead.isEmpty(); }
  bool isOne() const {
    ConstIterator i(mHead.begin());
    return i && (*i).mDegree1 == 0 && (*i).mDegree2 == 0 && (*i).mCoeff == 1;
  }
  bool isComposite() const { return mHead.length() > 1; }
  int norm() const { return mHead.length(); }
  unsigned degree1() const {
    IASSERT(!isZero());
    return (*mHead.begin()).mDegree1;
  }
  unsigned degree2() const {
    IASSERT(!isZero());
    return (*mHead.begin()).mDegree2;
  }

  void minus();
  void pp() {
    IASSERT(!isZero());
    div(0, 0, (*mHead.begin()).mCoeff);
  }
  void add(const ITwoParameterModularShortAux& a);
  void sub(const ITwoParameterModularShortAux& a);
  void mult(const ITwoParameterModularShortAux& a);
  void mult(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b);
  void diff1();
  void diff2();
  void diff(unsigned deg1, unsigned deg2);
  void pow(unsigned deg);
  void div(const ITwoParameterModularShortAux& a);
  void div(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b) {
    *this = a;
    div(b);
  }
  void gcd(const ITwoParameterModularShortAux& a) {
    gcd(*this, a);
  }
  void gcd(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b);
  void addmul(const ITwoParameterModularShortAux& a, const ITwoParameterModularShortAux& b) {
    ITwoParameterModularShortAux tmp(mPrimeShort);
    tmp.mult(a, b);
    add(tmp);
  }

  ConstIterator begin() const { return mHead.begin(); }
  Iterator begin() { return mHead.begin(); }

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;
  friend std::ostream& operator<<(std::ostream& out, const ITwoParameterModularShortAux& a);
};

#endif // ITWOPARAMETERMODULARSHORTAUX_H
