/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../algorithm/isett.h"
#include "./ic1c2c3.h"


IC1C2C3Interface::IC1C2C3Interface():
    IWrapInterface() {
  mType = IWrapInterface::C1C2C3;
}

IWrap* IC1C2C3Interface::create(IPoly* a) {
  return new IC1C2C3(this, a);
}

IWrap* IC1C2C3Interface::create(IWrap& a, int var, unsigned deg) {
  IASSERT(this == a.wrapInterface());
  IASSERT(deg > 0);
  return new IC1C2C3((IC1C2C3&)a, var, deg);
}

IWrap* IC1C2C3Interface::create() {
  return new IC1C2C3(this);
}

IC1C2C3::IC1C2C3(IWrapInterface *r, IPoly *poly):
    IWrap(r, poly) {
}

IC1C2C3::IC1C2C3(IC1C2C3& w, int var, unsigned deg):
    IWrap(w, var, deg) {
}

IC1C2C3::IC1C2C3(IWrapInterface *r):
    IWrap(r) {
}

bool IC1C2C3::criterion(IWrap& divisor, ISetT &setT) {
  bool r = false;
  if (lm().degree() != ancestor().degree())
    if (monomInterface()->depend() == NULL &&
        systemType()->module() < 2 &&
        lm().degree() >= ancestor().degree() + divisor.ancestor().degree())
      r = true;
    else if (lm().degree() > ancestor().lcm(divisor.ancestor()))
      r = true;
    else {
      IMonom *monom1 = monomInterface()->create();
      IMonom *monom2 = monomInterface()->create();
      IMonom *monom3 = monomInterface()->create();
      monom1->lcm(ancestor(), divisor.ancestor());
      ISetT::ConstIterator j = setT.begin();
      while(j) {
	if (systemType()->module() < 2 || (*j)->lm().dependVar() == lm().dependVar()) {
	  monom2->lcm((*j)->lm(), ancestor());
	  monom3->lcm((*j)->lm(), divisor.ancestor());
	  if (monom1->divisibilityTrue(*monom2) &&
	      monom1->divisibilityTrue(*monom3)) {
	    r = true;
	    break;
	  }
	}
        ++j;
      }
      delete monom1;
      delete monom2;
      delete monom3;
    }

  return r;
}
