/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <cstdarg>

#include "./iwrap.h"
#include "./iwithout.h"
#include "./icritpartially.h"
#include "./ic1.h"
#include "./ic1c2c3.h"
#include "./ic1c2c3c4.h"

IWrapInterface::IWrapInterface():
    mType(Indefinitely),
    mPolyInterface(NULL),
    mDim(0),
    mAllocatorBuild(NULL),
    mCurrentIdx(0) {
}

IWrapInterface* IWrapInterface::create(Type type,
                                       IPolyInterface* polyInterface, ...) {
  if (type == IWrapInterface::Indefinitely)
    IERROR("error in IWrapInterface: type == IWrapInterface::Indefinitely");
  if (polyInterface == NULL || polyInterface->type() == IPolyInterface::Indefinitely)
    IERROR("error in IPolyInterface: polyInterface == NULL || polyInterface->type() == polyInterface::Indefinitely");

  va_list marker;
  va_start(marker, polyInterface);

  IWrapInterface* r = NULL;
  switch(type) {
  case Without:
    r = new IWithoutInterface;
    break;
  case CritPartially:
    r = new ICritPartiallyInterface;
    break;
  case C1:
    r = new IC1Interface;
    break;
  case C1C2C3:
    r = new IC1C2C3Interface;
    break;
  case C1C2C3C4:
    r = new IC1C2C3C4Interface;
    break;
  }
  va_end(marker);

  if (r) {
    r->mPolyInterface = polyInterface;
    r->mDim = r->monomInterface()->dimIndepend();
    r->mAllocatorBuild = new IAllocator(sizeof(unsigned)*r->mDim);
  }
  return r;
}

IWrap::IWrap(IWrapInterface *r, IPoly *poly):
    mRealization(r),
    mLeadMonom(r->monomInterface()->copy(poly->lm())),
    mAncestor(r->monomInterface()->copy(poly->lm())),
    mPoly(poly),
    mBuild((unsigned*)r->mAllocatorBuild->allocate()),
    mIsProlong(false),
    mVarProlong(-1),
    mDegProlong(0),
    mRef(new int),
    mIdx(0),
    mIdxAncestor(0) {
  IASSERT(mRealization);
  for(int independ=0; independ < mRealization->mDim; independ++)
    mBuild[independ] = 0;

  *mRef = 0;
  ++r->mCurrentIdx;
  mIdx = r->mCurrentIdx;
  mIdxAncestor = mIdx;

  IASSERTVALID(*this);
}

IWrap::IWrap(IWrap& w, int var, unsigned deg):
    mRealization(w.mRealization),
    mLeadMonom(w.monomInterface()->copy(w.lm())),
    mAncestor(w.monomInterface()->copy(w.ancestor())),
    mPoly(w.mPoly),
    mBuild((unsigned*)w.mRealization->mAllocatorBuild->allocate()),
    mIsProlong(true),
    mVarProlong(-1),
    mDegProlong(0),
    mRef(w.mRef),
    mIdx(0),
    mIdxAncestor(w.mIdx) {
  IASSERT(mRealization);
  mVarProlong = var;
  mDegProlong = deg;
  mLeadMonom->prolong(mVarProlong, mDegProlong);

  for(int independ=0; independ < mRealization->mDim; independ++)
    mBuild[independ] = 0;

  w.mBuild[var] = deg;
  ++(*mRef);
  ++w.mRealization->mCurrentIdx;
  mIdx = w.mRealization->mCurrentIdx;
  mIdxAncestor = w.mIdxAncestor;

  IASSERTVALID(*this);
//   updateProlong();
}

IWrap::IWrap(IWrapInterface *r):
    mRealization(r),
    mLeadMonom(NULL),
    mAncestor(NULL),
    mPoly(NULL),
    mBuild((unsigned*)r->mAllocatorBuild->allocate()),
    mIsProlong(false),
    mVarProlong(-1),
    mDegProlong(0),
    mRef(new int),
    mIdx(0),
    mIdxAncestor(0) {
  IASSERT(mRealization);
}

IWrap::~IWrap() {
  if (mLeadMonom)
    delete mLeadMonom;
  if (mAncestor)
    delete mAncestor;
  mRealization->mAllocatorBuild->deallocate(mBuild);
  if (mVarProlong < 0)
    mPoly->setZero();

  --(*mRef);
  if (*mRef < 0) {
    delete mRef;
    delete mPoly;
  }
}

int IWrap::multi() const {
  int m = mRealization->mDim;
  for(int independ=0; independ < mRealization->mDim; independ++)
    if (mBuild[independ] > 0)
     --m;
  return m;
}

void IWrap::clearBuild() {
  for(int independ=0; independ < mRealization->mDim; independ++)
    mBuild[independ] = 0;
}

void IWrap::updateProlong() {
  if (mVarProlong >= 0) {
    IASSERT(*mRef > 0);
    IASSERT(!mPoly->isZero());
    IASSERT(mDegProlong > 0);

    if (mDegProlong == 1)
      mPoly = polyInterface()->copy(*mPoly, mVarProlong);
    else
      mPoly = polyInterface()->copy(*mPoly, mVarProlong, mDegProlong);

    mVarProlong = -1;
    mDegProlong = 0;
    --(*mRef);
    mRef = new int;
    *mRef = 0;
    IASSERTVALID(*this);
  }
}

bool IWrap::isZero() const {
  return (mVarProlong >= 0  &&
          (mPoly->isZero() ||
           !mLeadMonom->equality(mPoly->lm(), mVarProlong, mDegProlong)));
}

void IWrap::setEquivalent(IWrap &w) {
  IASSERT(w.mLeadMonom->divisibility(*mLeadMonom));
/*  if (mLeadMonom->degree() == w.mLeadMonom->degree() &&
      w.mLeadMonom->degree() > w.mAncestor->degree() &&
      mLeadMonom->degree() == mAncestor->degree()) {*/
  if (mLeadMonom->divisibility(*w.mAncestor) &&
      mLeadMonom->degree() > w.mAncestor->degree() &&
      mLeadMonom->degree() == mAncestor->degree()) {
    *mAncestor = *w.mAncestor;
    mIsProlong = mLeadMonom->degree() > mAncestor->degree();
    mIdx = w.mIdx;
    mIdxAncestor = w.mIdxAncestor;
  }
  IASSERTVALID(*this);
}

void IWrap::setProlong(IWrap &w) {
  IASSERT(!mIsProlong);
  IASSERT(mLeadMonom->divisibility(*w.mLeadMonom));
  IASSERT(mLeadMonom->divisibility(*w.mAncestor));
  *mAncestor = *w.mAncestor;
  IASSERT(mLeadMonom->degree() > mAncestor->degree());
  mIsProlong = true;
}

void IWrap::update() {
  IASSERT(mVarProlong < 0);
//   if (mLeadMonom->compare(mPoly->lm()) != 0) {
    mIsProlong = false;
    *mLeadMonom = mPoly->lm();
    *mAncestor = *mLeadMonom;

    for(int independ=0; independ < mRealization->mDim; independ++)
      mBuild[independ] = 0;

    IASSERT(!isZero());
    IASSERTVALID(*this);
//   }
}

void IWrap::read(std::istream &in) {
  mLeadMonom = monomInterface()->create();
  mLeadMonom->read(in);
  mAncestor = monomInterface()->create();
  mAncestor->read(in);
  mPoly = polyInterface()->create();
  mPoly->read(in);
  in.read((char*)mBuild, sizeof(unsigned)*mRealization->mDim);
  in.read((char*)&mIsProlong, sizeof(mIsProlong));
  in.read((char*)&mVarProlong, sizeof(mVarProlong));
  in.read((char*)&mDegProlong, sizeof(mDegProlong));
  IASSERTVALID(*this);
}

void IWrap::write(std::ostream &out) const {
  IASSERT(mLeadMonom);
  mLeadMonom->write(out);
  IASSERT(mAncestor);
  mAncestor->write(out);
  IASSERT(mPoly);
  mPoly->write(out);
  out.write((char*)mBuild, sizeof(unsigned)*mRealization->mDim);
  out.write((char*)&mIsProlong, sizeof(mIsProlong));
  out.write((char*)&mVarProlong, sizeof(mVarProlong));
  out.write((char*)&mDegProlong, sizeof(mDegProlong));
}

void IWrap::assertValid(const char* fileName, int fileLine) const {
  if (mVarProlong >= 0)
    IASSERT2(fileName, fileLine, mDegProlong > 0);
  IASSERT2(fileName, fileLine, mLeadMonom->divisibility(mPoly->lm()));
  IASSERT2(fileName, fileLine, mLeadMonom->divisibility(*mAncestor));
  IASSERT2(fileName, fileLine, !mIsProlong || mLeadMonom->degree() > mAncestor->degree());
}
