/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IWRAP_H
#define IWRAP_H

#include "../util/iutil.h"
#include "../util/iallocator.h"
#include "../monom/imonom.h"
#include "../poly/ipoly.h"


class IWrap;
class IWrapInterface {
  friend class IWrap;

public:
  enum Type {Indefinitely  = 0x00000000,
             Without       = 0x00000001,
             CritPartially = 0x00000002,
             C1C2          = 0x00000002,
             C1            = 0x10000000,
             C1C2C3        = 0x20000000,
             C1C2C3C4      = 0x40000000,
  };

protected:
  Type            mType;
  IPolyInterface *mPolyInterface;

  int           mDim;
  IAllocator   *mAllocatorBuild;
  unsigned      mCurrentIdx;

  IWrapInterface();

public:
  static IWrapInterface* create(Type type,
                                IPolyInterface* polyInterface, ...);
  virtual ~IWrapInterface() { delete mAllocatorBuild; }

  Type type() const { return mType; }
  const ISystemType* systemType() const { return mPolyInterface->systemType(); }
  IMonomInterface* monomInterface() const { return mPolyInterface->monomInterface(); }
  ICoeffInterface* coeffInterface() const { return mPolyInterface->coeffInterface(); }
  IPolyInterface* polyInterface() const { return mPolyInterface; }

  virtual IWrap* create(IPoly* a) = 0;
  virtual IWrap* create(IWrap& a, int var, unsigned deg) = 0;
  virtual IWrap* create() = 0;
};

class ISetT;
class IWrap {
protected:
  IWrapInterface *mRealization;

  IMonom                *mLeadMonom;
  IMonom                *mAncestor;
  IPoly                 *mPoly;

  unsigned             *mBuild;
  bool                  mIsProlong;
  int                   mVarProlong;
  unsigned              mDegProlong;
  int*                  mRef;
  unsigned              mIdx;
  unsigned              mIdxAncestor;

  IWrap(IWrapInterface *r, IPoly *poly);
  IWrap(IWrap& w, int var, unsigned deg=1);
  IWrap(IWrapInterface *r);

public:
  typedef IWrapInterface::Type Type;

  virtual ~IWrap();

  Type type() const { return mRealization->type(); }
  const ISystemType* systemType() const { return mRealization->systemType(); }
  IMonomInterface* monomInterface() const { return mRealization->monomInterface(); }
  ICoeffInterface* coeffInterface() const { return mRealization->coeffInterface(); }
  IPolyInterface* polyInterface() const { return mRealization->polyInterface(); }
  IWrapInterface* wrapInterface() const { return mRealization; }

  const IMonom& lm() const { return *mLeadMonom; }

  const IMonom& ancestor() const { return *mAncestor; }
  const IPoly& poly() const { return *mPoly; }
  IPoly& poly() { return *mPoly; }

  int multi() const;
  unsigned build(int independ) const { return mBuild[independ]; }
  bool notBuild(int independ, unsigned deg=1) const;
  void clearBuild();
  void clearBuild(int independ);

  int degProlong() const;
  bool isProlong() const { return mIsProlong; }
  void updateProlong();

  bool isZero() const;
  void setEquivalent(IWrap &w);
  void setProlong(IWrap &w);
  void update();

  virtual bool criterion(IWrap& divisor, ISetT& setT) = 0;
  unsigned idx() const { return mIdx; }
  unsigned idxAncestor() const { return mIdxAncestor; }

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

inline bool IWrap::notBuild(int independ, unsigned deg) const {
  IASSERT(0 <= independ && independ < mRealization->mDim);
  return mBuild[independ] != deg;
}

inline void IWrap::clearBuild(int independ) {
  IASSERT(0 <= independ && independ < mRealization->mDim);
  mBuild[independ] = 0;
}

inline int IWrap::degProlong() const {
  return mDegProlong;
}

#endif // IWRAP_H

