/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IDIVISION_H
#define IDIVISION_H

#include "../poly/ioneparameterpoly.h"
#include "../poly/ipoly.h"
#include "../coeff/icoeff.h"
#include "../algorithm/isetq.h"

class IDivision;
class IDivisionInterface {
  friend class IDivision;

public:
  enum Type {Indefinitely,
             Janet,
             JanetLike
  };

protected:
  Type  mType;
  IWrapInterface  *mWrapInterface;

  IDivisionInterface();

public:
  static IDivisionInterface* create(Type type,
                                    IWrapInterface *wrapInterface, ...);

  virtual ~IDivisionInterface() {}

  Type type() const { return mType; }
  IMonomInterface* monomInterface() const { return mWrapInterface->monomInterface(); }
  ICoeffInterface* coeffInterface() const { return mWrapInterface->coeffInterface(); }
  IPolyInterface* polyInterface() const { return mWrapInterface->polyInterface(); }
  IWrapInterface* wrapInterface() const { return mWrapInterface; }

  virtual IDivision* create() = 0;
};

class IDivision {
  friend class IDivisionInterface;

protected:
  IDivisionInterface* mRealization;

  IDivision(IDivisionInterface* divisionInterface):
      mRealization(divisionInterface) {
    IASSERT(mRealization);
  }

private:
  IDivision(const IDivision&): mRealization(NULL) {}
  void operator=(const IDivision&) {}

public:
  typedef IDivisionInterface::Type Type;

  virtual ~IDivision() {}

  Type type() const { return mRealization->mType; }
  IMonomInterface* monomInterface() const { return mRealization->monomInterface(); }
  ICoeffInterface* coeffInterface() const { return mRealization->coeffInterface(); }
  IPolyInterface* polyInterface() const { return mRealization->polyInterface(); }
  IWrapInterface* wrapInterface() const { return mRealization->wrapInterface(); }
  IDivisionInterface* divisionInterface() const { return mRealization; }

  virtual IWrap* find(const IMonom &m) const = 0;
  virtual void insert(IWrap *wrap, ISetQ &setQ) = 0;
  virtual void insert(IWrap *wrap) = 0;
  virtual void update(IWrap *wrap, ISetQ &setQ) = 0;
  virtual void clear() = 0;

  int nf(ICoeff &multiplier, IPoly &poly, const IPoly &red, ISetQ &setQ) const;
  int nfExact(IPoly &poly, IPoly &red) const;
  int nf(ICoeff &multiplier, IPoly &poly) const;
  bool isNf(const IPoly &poly) const;
  int nfTail(ICoeff &multiplier, IPoly &poly) const;
  int nfTailCanonical(ICoeff &multiplier, IPoly &poly) const;
  bool isNfTail(const IPoly &poly) const;

  bool minPoly(const IPoly& poly, IOneParameterPoly& mp, int maxDegree=30) const;
};

#endif // IDIVISION_H

