/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./ijanetlike.h"


IJanetLikeInterface::IJanetLikeInterface():
    IDivisionInterface() {
  mType = IDivisionInterface::JanetLike;
}

IDivision* IJanetLikeInterface::create() {
  return new IJanetLike(this);
}

inline void IJanetLike::Iterator::del() {
  Link tmp = *i;
  IASSERT(tmp);
  *i = tmp->mNextDeg;
  delete tmp;
}

void IJanetLike::Iterator::build(unsigned d, int var, IWrap *wrap) {
  IASSERT(d >= wrap->lm()[var]);
  Link r =  new IJanetLike::Node(wrap->lm()[var]), j = r;
  while(d > wrap->lm()[var]) {
    d -= wrap->lm()[var];
    ++var;
    j->mNextVar = new IJanetLike::Node(wrap->lm()[var]);
    j = j->mNextVar;
  }
  j->mWrap = wrap;

  IASSERT(!*i || r->mDeg < (*i)->mDeg);
  r->mNextDeg = *i;
  *i = r;
}

void IJanetLike::ConstIterator::prolong(ISetQ &q, int var, unsigned degree) {
  while(nextDeg()) {
    nextVar().prolong(q, var, degree);
    deg();
  }
  if (nextVar())
    nextVar().prolong(q, var, degree);
  else {
    if (wrap()->notBuild(var, degree))
      q.insert(wrap(), var, degree);
  }
}

void IJanetLike::ConstIterator::assertValid(const char* fileName, int fileLine) {
  while(nextDeg()) {
    IASSERT2(fileName, fileLine, degree() < nextDeg().degree());
    IASSERT2(fileName, fileLine, nextVar());
    nextVar().assertValid(fileName, fileLine);
    deg();
  }
  if (nextVar())
    nextVar().assertValid(fileName, fileLine);
  else
    IASSERT2(fileName, fileLine, wrap());
}

void IJanetLike::Iterator::clear() {
  while (nextDeg()) {
    IASSERT(nextVar());
    IJanetLike::Iterator((*i)->mNextVar).clear();
    del();
  }
  del();
}

IJanetLike::IJanetLike(IDivisionInterface* divisionInterface):
    IDivision(divisionInterface),
    mRoot(NULL) {
  int dim = monomInterface()->dimDepend();
  if (dim == 0)
    dim = 1;

  mRoot = new Link[dim];
  for(int i=0; i < dim; i++)
    mRoot[i] = NULL;
}

IJanetLike::~IJanetLike() {
  clear();
  delete[] mRoot;
}

IWrap* IJanetLike::find(const IMonom &m, const ICoeff &c) const {
  int depend = 0;
  if (monomInterface()->dimDepend())
    depend = m.dependVar();

  Link root = mRoot[depend];

  IWrap* wrap = NULL;

  if (root) {
    IJanetLike::ConstIterator j(root);
    unsigned d = m.degree();
    int var = 0;
    if (d == 0) {
      if (j.degree() == 0)
        wrap = j.wrap();
    }
    else {
      do {
        ConstIterator prev(j);
        while(j.degree() < m[var] && j.nextDeg()) {
          prev = j;
          j.deg();
        }

        if (j.degree() > m[var]) {
          if (prev.degree() < m[var]) {
            IASSERT(prev.nextVar());
            j = prev;
            d -= m[var];
            ++var;
            j.var();
          }
          else
            break;
        }
        else if (j.nextVar()) {
          d -= m[var];
          if (d == 0)
            break;
          ++var;
          j.var();
        }
        else {
          wrap = j.wrap();
          IASSERT(m.divisibility(wrap->lm()));
          break;
        }
      } while(true);
    }
  }
  if (wrap &&
      wrap->coeffInterface()->type() == ICoeffInterface::GmpZZ &&
      wrap->poly().lc().cmpNorm(c) > 0)
    wrap = NULL;

  return wrap;
}

IWrap* IJanetLike::find(const IMonom &m) const {
  int depend = 0;
  if (monomInterface()->dimDepend())
    depend = m.dependVar();

  Link root = mRoot[depend];

  IWrap* wrap = NULL;

  if (root) {
    IJanetLike::ConstIterator j(root);
    unsigned d = m.degree();
    int var = 0;
    if (d == 0) {
      if (j.degree() == 0)
        wrap = j.wrap();
    }
    else {
      do {
        ConstIterator prev(j);
        while(j.degree() < m[var] && j.nextDeg()) {
          prev = j;
          j.deg();
        }

        if (j.degree() > m[var]) {
          if (prev.degree() < m[var]) {
            IASSERT(prev.nextVar());
            j = prev;
            d -= m[var];
            ++var;
            j.var();
          }
          else
            break;
        }
        else if (j.nextVar()) {
          d -= m[var];
          if (d == 0)
            break;
          ++var;
          j.var();
        }
        else {
          wrap = j.wrap();
          IASSERT(m.divisibility(wrap->lm()));
          break;
        }
      } while(true);
    }
  }

  return wrap;
}

void IJanetLike::insert(IWrap *wrap, ISetQ &setQ) {
  IASSERT(wrap != NULL);
  IASSERT(find(wrap->lm(), wrap->poly().lc()) == NULL);

  int depend = 0;
  if (monomInterface()->dimDepend())
    depend = wrap->lm().dependVar();

  Link &root = mRoot[depend];

  unsigned d = wrap->lm().degree();
  IJanetLike::Iterator j(root);
  if (root == NULL) {
    wrap->clearBuild();
    j.build(d, 0, wrap);
  }
  else {
    int var = 0;
    do {
      ConstIterator prev(j);
      while(j.degree() < wrap->lm()[var] && j.nextDeg()) {
        prev = j;
        j.deg();
      }

      if (j.degree() > wrap->lm()[var]) {
        if (prev.degree() < wrap->lm()[var] && prev.nextVar())
          prev.nextVar().prolong(setQ, var, wrap->lm()[var] - prev.degree());
        setQ.insert(wrap, var, j.degree() - wrap->lm()[var]);
        j.build(d, var, wrap);
        for(++var; var < wrap->lm().dimIndepend(); var++)
          wrap->clearBuild(var);
        break;
      }
      else if (j.degree() == wrap->lm()[var]) {
        if (j.nextDeg())
          setQ.insert(wrap, var, j.nextDeg().degree() - wrap->lm()[var]);
        else
          wrap->clearBuild(var);
        d -= wrap->lm()[var];
        ++var;
        j.var();
      }
      else {
        IJanetLike::ConstIterator(j).nextVar().prolong(setQ, var, wrap->lm()[var] - j.degree());
        j.deg();
        j.build(d, var, wrap);
        for(; var < wrap->lm().dimIndepend(); var++)
          wrap->clearBuild(var);
        break;
      }
    } while(true);
  }
  IASSERT(find(wrap->lm(), wrap->poly().lc()) != NULL);
  IASSERTVALID(IJanetLike::ConstIterator(root));
}

void IJanetLike::insert(IWrap *wrap) {
  IASSERT(wrap != NULL);
  IASSERT(find(wrap->lm(), wrap->poly().lc()) == NULL);

  int depend = 0;
  if (monomInterface()->dimDepend())
    depend = wrap->lm().dependVar();

  Link &root = mRoot[depend];

  unsigned d = wrap->lm().degree();
  IJanetLike::Iterator j(root);
  if (root == NULL)
    j.build(d, 0, wrap);
  else {
    int var = 0;
    do {
      while(j.degree() < wrap->lm()[var] && j.nextDeg())
        j.deg();

      if (j.degree() > wrap->lm()[var]) {
        j.build(d, var, wrap);
        break;
      }
      else if (j.degree() == wrap->lm()[var]) {
        d -= wrap->lm()[var];
        ++var;
        j.var();
      }
      else {
        j.deg();
        j.build(d, var, wrap);
        break;
      }
    } while(true);
  }
  IASSERT(find(wrap->lm(), wrap->poly().lc()) != NULL);
  IASSERTVALID(IJanetLike::ConstIterator(root));
}

void IJanetLike::update(IWrap* wrap, ISetQ &setQ) {
  IASSERT(wrap != NULL);
  IASSERT(find(wrap->lm(), wrap->poly().lc()) != NULL);

  int depend = 0;
  if (monomInterface()->dimDepend())
    depend = wrap->lm().dependVar();

  Link root = mRoot[depend];

  IJanetLike::ConstIterator j(root);
  int var = 0;
  do {
    while(j.degree() < wrap->lm()[var]) {
      IASSERT(j.nextDeg());
      j.deg();
    }
    IASSERT(j.degree() == wrap->lm()[var]);
    if (!j.nextDeg())
      wrap->clearBuild(var);
    else if (wrap->notBuild(var, j.nextDeg().degree() - j.degree()))
      setQ.insert(wrap, var, j.nextDeg().degree() - j.degree());
    ++var;
    if (j.nextVar())
        j.var();
    else {
      IASSERT(wrap == j.wrap());
      for(; var < wrap->lm().dimIndepend(); var++)
        wrap->clearBuild(var);
      break;
    }
  } while(true);
}

void IJanetLike::clear() {
  int dim = monomInterface()->dimDepend();
  if (dim == 0)
    dim = 1;

  for(int i=0; i < dim; i++) {
    if (mRoot[i]) {
      IJanetLike::Iterator j(mRoot[i]);
      j.clear();
    }
    IASSERT(mRoot[i] == NULL);
  }
}

IAllocator IJanetLike::Node::sAllocator(sizeof(IJanetLike::Node));

