/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IJANETLIKE_H
#define IJANETLIKE_H

#include "../util/iallocator.h"
#include "./idivision.h"


class IJanetLike;
class IJanetLikeInterface: public IDivisionInterface {
  friend class IJanetLike;

public:
  IJanetLikeInterface();

  ~IJanetLikeInterface() {}

  IDivision* create();
};

class IJanetLike: public IDivision {
  friend class IJanetLikeInterface;

public:
  struct Node {
    int       mDeg;
    IWrap*    mWrap;
    Node*     mNextDeg;
    Node*     mNextVar;

    static IAllocator sAllocator;
    void* operator new(size_t) { return sAllocator.allocate(); }
    void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

    Node(int deg): mDeg(deg), mWrap(NULL),
      mNextDeg(NULL), mNextVar(NULL) {}
    ~Node() {}
  };

private:
  typedef Node *Link;

  class ConstIterator {
    Link i;

  public:
    ConstIterator(Link n): i(n) {}

    void deg() { assert(i); i = i->mNextDeg; }
    void var() { assert(i); i = i->mNextVar; }
    operator bool() const { return (i != NULL); }
    ConstIterator nextDeg() const { assert(i); return i->mNextDeg; }
    ConstIterator nextVar() const { assert(i); return i->mNextVar; }
    IWrap* wrap() const { assert(i); return i->mWrap; }
    unsigned degree() const { assert(i); return i->mDeg; }

    void prolong(ISetQ &q, int var, unsigned degree);

    void assertValid(const char* fileName, int fileLine);
  };

  class Iterator {
    Link *i;

  public:
    Iterator(Link &n): i(&n) {}

    void deg() { assert(*i); i = &(*i)->mNextDeg; }
    void var() { assert(*i); i = &(*i)->mNextVar; }
    operator bool() const { return (*i != NULL); }
    ConstIterator nextDeg() const {
      assert(*i); return (*i)->mNextDeg;
    }
    ConstIterator nextVar() const {
      assert(*i); return (*i)->mNextVar;
    }
    operator ConstIterator() const { return *i; }
    IWrap*& wrap() const { assert(*i); return (*i)->mWrap; }
    unsigned degree() const { assert(i); return (*i)->mDeg; }
    void build(unsigned d, int var, IWrap *wrap);
    void del();
    void clear();
  };

  Link  *mRoot;

  IJanetLike(IDivisionInterface* r);

public:
  ~IJanetLike();

  IWrap* find(const IMonom &m, const ICoeff &c) const;
  IWrap* find(const IMonom &m) const;
  void insert(IWrap *wrap, ISetQ &setQ);
  void insert(IWrap *wrap);
  void update(IWrap *wrap, ISetQ &setQ);
  void clear();
};

#endif // IJANETLIKE_H

