/***************************************************************************
 *   Copyright (C) 2007 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          * *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IGAUSS_H
#define IGAUSS_H

#include <iostream>

#include "ichainoperation.h"
#include "../coeff/icoeff.h"
#include "../poly/ioneparameterpoly.h"
#include "../util/ilist.h"

class IGauss {
protected:
  IList<IList<ICoeff*> > mVectors;
  IChainOperation mOps;
  int mSize;
  ICoeffInterface* mCoeffInterface;

  typedef IList<IList<ICoeff*> >::Iterator VIterator;
  typedef IList<ICoeff*>::Iterator CIterator;

  ICoeff** createUnitVector(int i) const;

public:
  IGauss(ICoeffInterface* interface): mCoeffInterface(interface), mSize(0) { }

  virtual ~IGauss();

  bool addVector(IList<ICoeff*>& list);

  int size() const {
    return mSize;
  }

  void getMinPoly(IOneParameterPoly& minPoly) const;

  friend std::ostream& operator<<(std::ostream& out, const IGauss& a);
};

#endif // IGAUSS_H
