/***************************************************************************
 *   Copyright (C) 2007 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ipolygauss.h"
#include "iswapoperation.h"
#include "imultoperation.h"
#include "iaddmultoperation.h"

IPolyGauss::~IPolyGauss() {
// delete monomials and coefficients?
}

bool IPolyGauss::add(const ICoeff& multiplier, const IPoly& p) {
  if (mMonomials.isEmpty()) {
    IPoly::ConstIterator* it = p.begin();

    MIterator mit = mMonomials.begin();
    VIterator vit = mVectors.begin();
    vit.insert();

    CIterator cit = (*vit).begin();

    while (*it) {
      mit.insert();
      (*mit) = it->monom().copy();
      cit.insert();
      (*cit) = it->coeff().copy();

      ++mit;
      ++cit;
      it->next();
    }
    ++mSize;

    return false;
  }

  VIterator vit = mVectors.begin();

  // insert zeroes into existing coefficient vectors
  while (vit) {
    MIterator mit = mMonomials.begin();
    CIterator cit = (*vit).begin();
    IPoly::ConstIterator* pit = p.begin();

    while (mit && *pit) {
      switch (pit->monom().compare(*(*mit))) {
      case 1:
        cit.insert();
        (*cit) = mCoeffInterface->create();
        (*cit)->setZero();
        ++cit;

        pit->next();
        break;
      case -1:
        ++mit;
        ++cit;
        break;
      case 0:
        pit->next();
        ++mit;
        ++cit;
        break;
      }
    }

    while (*pit) {
      cit.insert();
      (*cit) = mCoeffInterface->create();
      (*cit)->setZero();
      ++cit;

      pit->next();
    }

    ++vit;
  }

  IList<ICoeff*> tmp;
  MIterator mit = mMonomials.begin();
  IPoly::ConstIterator* pit = p.begin();
  CIterator cit = tmp.begin();

  // create new coefficient vector and update list of monomials
  while (mit && *pit) {
    switch (pit->monom().compare(*(*mit))) {
    case 1:
      cit.insert();
      (*cit) = pit->coeff().copy();
      ++cit;

      mit.insert();
      (*mit) = pit->monom().copy();
      ++mit;

      pit->next();
      break;
    case 0:
      cit.insert();
      (*cit) = pit->coeff().copy();
      ++cit;

      pit->next();
      ++mit;
      break;
    case -1:
      cit.insert();
      (*cit) = mCoeffInterface->create();
      (*cit)->setZero();
      ++cit;

      ++mit;
      break;
    }
  }

  while (mit) {
    cit.insert();
    (*cit) = mCoeffInterface->create();
    (*cit)->setZero();
    ++cit;

    ++mit;
  }

  while (*pit) {
    cit.insert();
    (*cit) = pit->coeff().copy();
    ++cit;

    mit.insert();
    (*mit) = pit->monom().copy();
    ++mit;

    pit->next();
  }

  mOps.add(new IMultOperation(mSize, multiplier.copy()));

  return addVector(tmp);
}

std::ostream& operator<<(std::ostream& out, const IPolyGauss& a) {
  if (a.mMonomials.isEmpty()) {
    out << "[]" << std::endl;
    return out;
  }

  IList<IMonom*>::ConstIterator mit = a.mMonomials.begin();
  out << "[";
  out << *(*mit);
  ++mit;
  while (mit) {
    out << ", " << *(*mit);
    ++mit;
  }
  out << "]" << std::endl;

  return operator<<(out, (const IGauss&) a);
}

template <>
IAllocator IList<IMonom*>::Node::sAllocator(sizeof(IList<IMonom*>::Node));
