/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./idegrevlexbyte.h"


// #define WORDS_LITTLEENDIAN
// #undef WORDS_LITTLEENDIAN
// #define WORDS_BIGENDIAN
// #undef WORDS_BIGENDIAN

IDegRevLexByteInterface::IDegRevLexByteInterface(const ISystemType *systemType, IVariables* independ):
    IMonomInterface(systemType),
    mDimWord((independ->dim()*sizeof(IDegRevLexByte::Byte)+sizeof(IDegRevLexByte::Word)-1)/sizeof(IDegRevLexByte::Word)),
    mLittleEndian(mDimWord - 1),
    mAllocator(sizeof(IDegRevLexByte::Word)*mDimWord) {
  mOrder = IMonomInterface::DegRevLexByte;
  mDimIndepend = independ->dim();
  mIndepend = independ;
}

IMonom* IDegRevLexByteInterface::create() {
  return new IDegRevLexByte(this);
}

IMonom* IDegRevLexByteInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IDegRevLexByte* r;
  if (this == a.monomInterface())
    r = new IDegRevLexByte(a);
  else {
    r = new IDegRevLexByte(this);
    r->IDegRevLexByte::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->IDegRevLexByte::prolong(var, a.deg(var));
  }
  return r;
}

IDegRevLexByte::IDegRevLexByte(IDegRevLexByteInterface* r):
    IMonom(r),
    mBegin((Iterator)r->mAllocator.allocate()) {
}

IDegRevLexByte::IDegRevLexByte(const IMonom& a):
    IMonom((IDegRevLexByteInterface*)a.monomInterface()),
    mBegin((Iterator)((IDegRevLexByteInterface*)a.monomInterface())->mAllocator.allocate()) {
#if defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  It i = (It)mBegin;
  ConstIt end = i + dimWord(), ia = (ConstIt)((const IDegRevLexByte&)a).mBegin;
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend(), ia = ((const IDegRevLexByte&)a).mBegin;
#endif // defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  do {
    *i++ = *ia++;
  } while(i < end);
  IASSERTVALID(*this);
}

IDegRevLexByte::~IDegRevLexByte() {
  ((IDegRevLexByteInterface*)monomInterface())->mAllocator.deallocate(mBegin);
}

unsigned IDegRevLexByte::deg(int var) const {
  IASSERT(0 <= var && var < dimIndepend());
  unsigned d;
#if defined(WORDS_LITTLEENDIAN)
  if (var)
    d = mBegin[var] - mBegin[var-1];
  else
    d = *mBegin;
#else
  ConstIterator end = mBegin + dimIndepend();
  if (var)
    d = end[-var-1] - end[-var];
  else
    d = end[-var-1];
#endif // defined(WORDS_LITTLEENDIAN)
   return d;
}

unsigned IDegRevLexByte::degree() const {
#if defined(WORDS_LITTLEENDIAN)
  return *(mBegin + dimIndepend() - 1);
#else
  return *mBegin;
#endif // defined(WORDS_LITTLEENDIAN)
}

int IDegRevLexByte::dependVar() const {
  return -1;
}

void IDegRevLexByte::set(const IMonom& a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  if (monomInterface() == a.monomInterface()) {
#if defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
    It i = (It)mBegin;
    ConstIt end = i + dimWord(), ia = (ConstIt)((const IDegRevLexByte&)a).mBegin;
#else
    Iterator i = mBegin;
    ConstIterator end = i + dimIndepend(), ia = ((const IDegRevLexByte&)a).mBegin;
#endif // defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
    do {
      *i++ = *ia++;
    } while(i < end);
  }
  else {
    setZero();
    for(int var=0; var < dimDepend(); var++)
      prolong(var, a[var]);
  }
  IASSERTVALID(*this);
}

void IDegRevLexByte::setZero() {
#if defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  It i = (It)mBegin;
  ConstIt end = i + dimWord();
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend();
#endif // defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  do {
    *i++ = 0;
  } while(i < end);
  IASSERTVALID(*this);
}

void IDegRevLexByte::setDependVar(int) {
  IERROR("setDependVar");
}

void IDegRevLexByte::swap(IMonom& a) {
  IASSERT(monomInterface() == a.monomInterface());
  Iterator tmp = mBegin;
  mBegin = ((IDegRevLexByte&)a).mBegin;
  ((IDegRevLexByte&)a).mBegin = tmp;
}

void IDegRevLexByte::prolong(int var) {
  IASSERT(0 <= var && var < dimIndepend());
#if defined(WORDS_LITTLEENDIAN)
  Iterator i = mBegin + var;
  ConstIterator end = mBegin + dimIndepend();
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend() - var;
#endif // defined(WORDS_LITTLEENDIAN)
  do {
    ++(*i++);
  } while(i < end);
  IASSERTVALID(*this);
}

void IDegRevLexByte::prolong(int var, unsigned deg) {
  IASSERT(0 <= var && var < dimIndepend());
#if defined(WORDS_LITTLEENDIAN)
  Iterator i = mBegin + var;
  ConstIterator end = mBegin + dimIndepend();
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend() - var;
#endif // defined(WORDS_LITTLEENDIAN)
  do {
    *i++ += deg;
  } while(i < end);
  IASSERTVALID(*this);
}

void IDegRevLexByte::div(int var) {
  IASSERT(0 <= var && var < dimIndepend());
#if defined(WORDS_LITTLEENDIAN)
  Iterator i = mBegin + var;
  ConstIterator end = mBegin + dimIndepend();
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend() - var;
#endif // defined(WORDS_LITTLEENDIAN)
  do {
    --(*i++);
  } while(i < end);
  IASSERTVALID(*this);
}

void IDegRevLexByte::mult(const IMonom& a) {
  IASSERT(monomInterface() == a.monomInterface());
#if defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  It i = (It)mBegin;
  ConstIt end = i + dimWord(), ia = (ConstIt)((const IDegRevLexByte&)a).mBegin;
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend(), ia = ((const IDegRevLexByte&)a).mBegin;
#endif // defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  do {
    *i++ += *ia++;
  } while(i < end);
  IASSERTVALID(*this);
}

void IDegRevLexByte::mult(const IMonom& a, const IMonom& b) {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
#if defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  It i = (It)mBegin;
  ConstIt end = i + dimWord(),
      ia = (ConstIt)((const IDegRevLexByte&)a).mBegin,
      ib = (ConstIt)((const IDegRevLexByte&)b).mBegin;
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend(),
      ia = ((const IDegRevLexByte&)a).mBegin,
      ib = ((const IDegRevLexByte&)b).mBegin;
#endif // defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  do {
    *i++ = *ia++ + *ib++;
  } while(i < end);
  IASSERTVALID(*this);
}

bool IDegRevLexByte::divisibility(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  int var;
  for(var = 0; var < dimIndepend(); var++)
    if (IDegRevLexByte::deg(var) < ((const IDegRevLexByte&)a).IDegRevLexByte::deg(var))
      return false;
  return true;
}

void IDegRevLexByte::divide(const IMonom& a, const IMonom& b) {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(a.divisibility(b));
#if defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  It i = (It)mBegin;
  ConstIt end = i + dimWord(),
      ia = (ConstIt)((const IDegRevLexByte&)a).mBegin,
      ib = (ConstIt)((const IDegRevLexByte&)b).mBegin;
#else
  Iterator i = mBegin;
  ConstIterator end = i + dimIndepend(),
      ia = ((const IDegRevLexByte&)a).mBegin,
      ib = ((const IDegRevLexByte&)b).mBegin;
#endif // defined(WORDS_LITTLEENDIAN) || defined(WORDS_BIGENDIAN)
  do {
    *i++ = *ia++ - *ib++;
  } while(i < end);
  IASSERTVALID(*this);
}

static inline unsigned min(unsigned a, unsigned b) {
  return (a < b) ? a: b;
}

unsigned IDegRevLexByte::gcd(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  unsigned d = 0;
  for(int var = 0; var < dimIndepend(); var++)
    d += min(IDegRevLexByte::deg(var), ((const IDegRevLexByte&)a).IDegRevLexByte::deg(var));
  return d;
}

static inline unsigned max(unsigned a, unsigned b) {
  return (a > b) ? a: b;
}

unsigned IDegRevLexByte::lcm(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  unsigned d = 0;
  for(int var = 0; var < dimIndepend(); var++)
    d += max(IDegRevLexByte::deg(var), ((const IDegRevLexByte&)a).IDegRevLexByte::deg(var));
  return d;
}

int IDegRevLexByte::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
#if defined(WORDS_BIGENDIAN)
  ConstIt i = (ConstIt)mBegin,
          ia = (ConstIt)((const IDegRevLexByte&)a).mBegin;
  ConstIt end = i + dimWord();
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i == end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
#elif defined(WORDS_LITTLEENDIAN)
  ConstIt i = (ConstIt)mBegin + littleEndian(),
          end = (ConstIt)mBegin,
          ia = (ConstIt)((const IDegRevLexByte&)a).mBegin + littleEndian();
  while(i >= (ConstIt)mBegin && *i == *ia) {
    --i;
    --ia;
  }
  if (i < (ConstIt)mBegin)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
#else
  ConstIterator i = mBegin,
                ia = ((const IDegRevLexByte&)a).mBegin;
  ConstIterator end = i + dimIndepend();
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i >= end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
#endif // defined(WORDS_BIGENDIAN)
}

int IDegRevLexByte::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
#if defined(WORDS_BIGENDIAN)
  ConstIt i = (ConstIt)mBegin,
          ia = (ConstIt)((const IDegRevLexByte&)a).mBegin,
          ib = (ConstIt)((const IDegRevLexByte&)b).mBegin;
  ConstIt end = i + dimWord();
  while(i < end && *i == *ia + *ib) {
    ++i;
    ++ia;
    ++ib;
  }
  if (i == end)
    return 0;
  else if (*i > *ia + *ib)
    return 1;
  else
    return -1;
#elif defined(WORDS_LITTLEENDIAN)
  ConstIt i = (ConstIt)mBegin + littleEndian(),
          end = (ConstIt)mBegin,
          ia = (ConstIt)((const IDegRevLexByte&)a).mBegin + littleEndian(),
          ib = (ConstIt)((const IDegRevLexByte&)b).mBegin + littleEndian();
  while(i >= (ConstIt)mBegin && *i == *ia + *ib) {
    --i;
    --ia;
    --ib;
  }
  if (i < (ConstIt)mBegin)
    return 0;
  else if (*i > *ia + *ib)
    return 1;
  else
    return -1;
#else
  ConstIterator i = mBegin,
                ia = ((const IDegRevLexByte&)a).mBegin,
                ib = ((const IDegRevLexByte&)b).mBegin;
  ConstIterator end = i + dimIndepend();
  while(i < end && *i == *ia + *ib) {
    ++i;
    ++ia;
    ++ib;
  }
  if (i == end)
    return 0;
  else if (*i > *ia + *ib)
    return 1;
  else
    return -1;
#endif // defined(WORDS_BIGENDIAN)
}

int IDegRevLexByte::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
#if defined(WORDS_BIGENDIAN)
  ConstIt i = (ConstIt)mBegin,
          ia = (ConstIt)((const IDegRevLexByte&)a).mBegin;
  ConstIt end = i + dimWord();
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i == end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
#elif defined(WORDS_LITTLEENDIAN)
  ConstIt i = (ConstIt)mBegin + littleEndian(),
          end = (ConstIt)mBegin,
          ia = (ConstIt)((const IDegRevLexByte&)a).mBegin + littleEndian();
  while(i >= (ConstIt)mBegin && *i == *ia) {
    --i;
    --ia;
  }
  if (i < (ConstIt)mBegin)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
#else
  ConstIterator i = mBegin,
                ia = ((const IDegRevLexByte&)a).mBegin;
  ConstIterator end = i + dimIndepend();
  while(i < end && *i == *ia) {
    ++i;
    ++ia;
  }
  if (i >= end)
    return 0;
  else if (*i > *ia)
    return 1;
  else
    return -1;
#endif // defined(WORDS_BIGENDIAN)
}

void IDegRevLexByte::read(std::istream &in) {
  in.read((char*)mBegin, sizeof(IDegRevLexByte::Word)*
    ((IDegRevLexByteInterface *)mRealization)->mDimWord);
  IASSERTVALID(*this);
}

void IDegRevLexByte::write(std::ostream &out) const {
  out.write((char*)mBegin, sizeof(IDegRevLexByte::Word)*
    ((IDegRevLexByteInterface *)mRealization)->mDimWord);
}

void IDegRevLexByte::assertValid(const char* fileName, int fileLine) const {
  unsigned d=0;
  for(int var = 0; var < dimIndepend(); var++) {
    IASSERT2(fileName, fileLine, deg(var) >= 0);
    d += deg(var);
  }
  IASSERT2(fileName, fileLine, d == degree());
}

IAllocator IDegRevLexByte::sAllocator(sizeof(IDegRevLexByte));
