/***************************************************************************
 *   Copyright (C) 2006 by D. Robertz                                      *
 *   daniel@momo.math.rwth-aachen.de                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./idegrevlexdeg.h"


IDegRevLexDegInterface::IDegRevLexDegInterface(const ISystemType *systemType, IVariables* independ, unsigned *degs):
    ILexInterface(systemType, independ) {
  mOrder = IMonomInterface::DegRevLexDeg;
  degrees = new unsigned[dimIndepend()];
  for (int i = 0; i < dimIndepend(); i++)
    degrees[i] = degs[i];
}

IMonom* IDegRevLexDegInterface::create() {
  return new IDegRevLexDeg(this);
}

IMonom* IDegRevLexDegInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IDegRevLexDeg* r;
  if (this == a.monomInterface())
    r = new IDegRevLexDeg(a);
  else {
    r = new IDegRevLexDeg(this);
    r->IDegRevLexDeg::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->IDegRevLexDeg::prolong(var, a.deg(var));
  }
  return r;
}

IDegRevLexDeg::IDegRevLexDeg(IDegRevLexDegInterface* r):
  ILex(r) {
}

IDegRevLexDeg::IDegRevLexDeg(const IMonom& a):
  ILex(a) {
}

IDegRevLexDeg::~IDegRevLexDeg() {
}

unsigned IDegRevLexDeg::weighted_degree() const {
  unsigned d = 0;
  for (int i = 0; i < dimIndepend(); i++)
    d += deg(i) * ((IDegRevLexDegInterface*)mRealization)->degrees[i];
  return d;
}

int IDegRevLexDeg::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  unsigned w = weighted_degree();
  unsigned wa = ((const IDegRevLexDeg&)a).weighted_degree();
  if (w > wa)
    return 1;
  else if (w < wa)
    return -1;

  ConstIterator i = mBegin + dimIndepend() - 1,
                ia = ((const IDegRevLexDeg&)a).mBegin + dimIndepend() - 1;
  ConstIterator end = mBegin;
  while(i >= end && *i == *ia) {
    --i;
    --ia;
  }
  if (i < end)
    return 0;
  else if (*i < *ia)
    return 1;
  else
    return -1;
}

int IDegRevLexDeg::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  unsigned w = weighted_degree();
  unsigned wab = ((const IDegRevLexDeg&)a).weighted_degree() + ((const IDegRevLexDeg&)b).weighted_degree();
  if (w > wab)
    return 1;
  else if (w < wab)
    return -1;

  ConstIterator i = mBegin + dimIndepend() - 1,
                ia = ((const IDegRevLexDeg&)a).mBegin + dimIndepend() - 1,
                ib = ((const IDegRevLexDeg&)b).mBegin + dimIndepend() - 1;
  ConstIterator end = mBegin;
  while(i >= end && *i == *ia + *ib) {
    --i;
    --ia;
    --ib;
  }
  if (i < end)
    return 0;
  else if (*i < *ia + *ib)
    return 1;
  else
    return -1;
}

int IDegRevLexDeg::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  unsigned w = weighted_degree();
  unsigned wa = ((const IDegRevLexDeg&)a).weighted_degree();
  if (w > wa)
    return 1;
  else if (w < wa)
    return -1;

  ConstIterator i = mBegin + dimIndepend() - 1,
                ia = ((const IDegRevLexDeg&)a).mBegin + dimIndepend() - 1;
  ConstIterator end = mBegin;
  while(i >= end && *i == *ia) {
    --i;
    --ia;
  }
  if (i < end)
    return 0;
  else if (*i < *ia)
    return 1;
  else
    return -1;
}

void IDegRevLexDeg::assertValid(const char* fileName, int fileLine) const {
}

IAllocator IDegRevLexDeg::sAllocator(sizeof(IDegRevLexDeg));
