/***************************************************************************
 *   Copyright (C) 2006 by D. Robertz                                      *
 *   daniel@momo.math.rwth-aachen.de                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IDEGREVLEXDEG_H
#define IDEGREVLEXDEG_H

#include "./ilex.h"

class IDegRevLexDeg;
class IDegRevLexDegInterface: public ILexInterface {
  friend class IDegRevLexDeg;

protected:
  unsigned              *degrees;

public:
  IDegRevLexDegInterface(const ISystemType *systemType, IVariables* independ, unsigned *degs);
  ~IDegRevLexDegInterface() { delete degrees; }

  IMonom* create();
  IMonom* copy(const IMonom& a);
};

class IDegRevLexDeg: public ILex {
  friend class IDegRevLexDegInterface;

protected:
  static IAllocator     sAllocator;

  IDegRevLexDeg(IDegRevLexDegInterface* r);
  IDegRevLexDeg(const IMonom& a);

public:
  void* operator new(size_t) { return sAllocator.allocate(); }
  void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

  ~IDegRevLexDeg();

  unsigned weighted_degree() const;

  int compare(const IMonom& a) const;
  int compare(const IMonom& a, const IMonom& b) const;
  int setQcompare(const IMonom& a) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // IDEGREVLEXDEG_H
