/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ILEX_H
#define ILEX_H

#include "../util/iallocator.h"
#include "./imonom.h"


class ILex;
class ILexInterface: public IMonomInterface {
  friend class ILex;
  IAllocator           mAllocator;

public:
  ILexInterface(const ISystemType *systemType, IVariables* independ);
  ~ILexInterface() {}

  IMonom* create();
  IMonom* copy(const IMonom& a);
};

class ILex: public IMonom {
  friend class ILexInterface;

protected:
  typedef unsigned     Word;
  typedef Word*        Iterator;
  typedef const Word*  ConstIterator;

  Iterator             mBegin;

  static IAllocator    sAllocator;

  ILex(ILexInterface* r);
  ILex(const IMonom& a);

public:
  void* operator new(size_t) { return sAllocator.allocate(); }
  void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

  ~ILex();

  unsigned deg(int var) const;
  unsigned degree() const;
  int dependVar() const;

  void set(const IMonom& a);
  void setZero();
  void setDependVar(int depend);

  void swap(IMonom& a);

  void prolong(int var);
  void prolong(int var, unsigned deg);
  void div(int var);

  void mult(const IMonom& a);
  void mult(const IMonom& a, const IMonom& b);

  bool divisibility(const IMonom& a) const;
  void divide(const IMonom& a, const IMonom& b);

  unsigned gcd(const IMonom& a) const;
  unsigned lcm(const IMonom& a) const;

  int compare(const IMonom& a) const;
  int compare(const IMonom& a, const IMonom& b) const;
  int compare(const IMonom& a, int var) const;
  int setQcompare(const IMonom& a) const;

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // ILEX_H
