/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iposelim.h"


IPosElimInterface::IPosElimInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int elim):
    ITopDegRevLexInterface(systemType, independ, depend),
    mTupElim(elim) {
  IASSERT(0 < mTupElim && mTupElim < mDimDepend);
  mOrder = IMonomInterface::PosElim;
}

int IPosElimInterface::elim() const {
  return mTupElim;
}

IMonom* IPosElimInterface::create() {
  return new IPosElim(this);
}

IMonom* IPosElimInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPosElim* r;
  if (this == a.monomInterface())
    r = new IPosElim(a);
  else {
    r = new IPosElim(this);
    r->ITopDegRevLex::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->ITopDegRevLex::prolong(var, a.deg(var));
    r->ITopDegRevLex::setDependVar(a.dependVar());
  }
  return r;
}

IPosElim::IPosElim(IPosElimInterface* r):
    ITopDegRevLex(r) {
}

IPosElim::IPosElim(const IMonom& a):
    ITopDegRevLex(a) {
  IASSERTVALID(*this);
}

IPosElim::~IPosElim() {
}

int IPosElim::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElim&)a).mDependVar)
    return 1;
  else if (((IPosElim&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return ITopDegRevLex::compare(a);
}

int IPosElim::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPosElim&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPosElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElim&)a).mDependVar)
    return 1;
  else if (((IPosElim&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return ITopDegRevLex::compare(a, b);
}

int IPosElim::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElim&)a).mDependVar)
    return 1;
  else if (((IPosElim&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return ITopDegRevLex::compare(a);
}

void IPosElim::assertValid(const char* fileName, int fileLine) const {
  ITopDegRevLex::assertValid(fileName, fileLine);
}

IAllocator IPosElim::sAllocator(sizeof(IPosElim));
