/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iposelimdeglex.h"

IPosElimDegLexInterface::IPosElimDegLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int elim):
    ITopDegLexInterface(systemType, independ, depend),
    mTupElim(elim) {
  IASSERT(0 < mTupElim && mTupElim < mDimDepend);
  mOrder = IMonomInterface::PosElimDegLex;
}

int IPosElimDegLexInterface::elim() const {
  return mTupElim;
}

IMonom* IPosElimDegLexInterface::create() {
  return new IPosElimDegLex(this);
}

IMonom* IPosElimDegLexInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPosElimDegLex* r;
  if (this == a.monomInterface())
    r = new IPosElimDegLex(a);
  else {
    r = new IPosElimDegLex(this);
    r->ITopDegLex::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->ITopDegLex::prolong(var, a.deg(var));
    r->ITopDegLex::setDependVar(a.dependVar());
  }
  return r;
}

IPosElimDegLex::IPosElimDegLex(IPosElimDegLexInterface* r):
    ITopDegLex(r) {
}

IPosElimDegLex::IPosElimDegLex(const IMonom& a):
    ITopDegLex(a) {
  IASSERTVALID(*this);
}

IPosElimDegLex::~IPosElimDegLex() {
}

int IPosElimDegLex::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimDegLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegLex&)a).mDependVar)
    return 1;
  else if (((IPosElimDegLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegLex::compare(a);
}

int IPosElimDegLex::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPosElimDegLex&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPosElimDegLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegLex&)a).mDependVar)
    return 1;
  else if (((IPosElimDegLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegLex::compare(a, b);
}

int IPosElimDegLex::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimDegLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegLex&)a).mDependVar)
    return 1;
  else if (((IPosElimDegLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegLex::compare(a);
}

void IPosElimDegLex::assertValid(const char* fileName, int fileLine) const {
  ITopDegLex::assertValid(fileName, fileLine);
}

IAllocator IPosElimDegLex::sAllocator(sizeof(IPosElimDegLex));
