/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iposelimdegrevlex.h"

IPosElimDegRevLexInterface::IPosElimDegRevLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int elim):
    ITopDegRevLexInterface(systemType, independ, depend),
    mTupElim(elim) {
  IASSERT(0 < mTupElim && mTupElim < mDimDepend);
  mOrder = IMonomInterface::PosElimDegRevLex;
}

int IPosElimDegRevLexInterface::elim() const {
  return mTupElim;
}

IMonom* IPosElimDegRevLexInterface::create() {
  return new IPosElimDegRevLex(this);
}

IMonom* IPosElimDegRevLexInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPosElimDegRevLex* r;
  if (this == a.monomInterface())
    r = new IPosElimDegRevLex(a);
  else {
    r = new IPosElimDegRevLex(this);
    r->ITopDegRevLex::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->ITopDegRevLex::prolong(var, a.deg(var));
    r->ITopDegRevLex::setDependVar(a.dependVar());
  }
  return r;
}

IPosElimDegRevLex::IPosElimDegRevLex(IPosElimDegRevLexInterface* r):
    ITopDegRevLex(r) {
}

IPosElimDegRevLex::IPosElimDegRevLex(const IMonom& a):
    ITopDegRevLex(a) {
  IASSERTVALID(*this);
}

IPosElimDegRevLex::~IPosElimDegRevLex() {
}

int IPosElimDegRevLex::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegRevLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegRevLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegRevLex&)a).mDependVar)
    return 1;
  else if (((IPosElimDegRevLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegRevLex::compare(a);
}

int IPosElimDegRevLex::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPosElimDegRevLex&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPosElimDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegRevLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegRevLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegRevLex&)a).mDependVar)
    return 1;
  else if (((IPosElimDegRevLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegRevLex::compare(a, b);
}

int IPosElimDegRevLex::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegRevLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegRevLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegRevLex&)a).mDependVar)
    return 1;
  else if (((IPosElimDegRevLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegRevLex::compare(a);
}

void IPosElimDegRevLex::assertValid(const char* fileName, int fileLine) const {
  ITopDegRevLex::assertValid(fileName, fileLine);
}

IAllocator IPosElimDegRevLex::sAllocator(sizeof(IPosElimDegRevLex));
