/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iposelimdegrevlexdeg.h"

IPosElimDegRevLexDegInterface::IPosElimDegRevLexDegInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int elim, unsigned *degs):
    ITopDegRevLexDegInterface(systemType, independ, depend, degs),
    mTupElim(elim) {
  IASSERT(0 < mTupElim && mTupElim < mDimDepend);
  mOrder = IMonomInterface::PosElimDegRevLexDeg;
}

int IPosElimDegRevLexDegInterface::elim() const {
  return mTupElim;
}

IMonom* IPosElimDegRevLexDegInterface::create() {
  return new IPosElimDegRevLexDeg(this);
}

IMonom* IPosElimDegRevLexDegInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPosElimDegRevLexDeg* r;
  if (this == a.monomInterface())
    r = new IPosElimDegRevLexDeg(a);
  else {
    r = new IPosElimDegRevLexDeg(this);
    r->ITopDegRevLexDeg::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->ITopDegRevLexDeg::prolong(var, a.deg(var));
    r->ITopDegRevLexDeg::setDependVar(a.dependVar());
  }
  return r;
}

IPosElimDegRevLexDeg::IPosElimDegRevLexDeg(IPosElimDegRevLexDegInterface* r):
    ITopDegRevLexDeg(r) {
}

IPosElimDegRevLexDeg::IPosElimDegRevLexDeg(const IMonom& a):
    ITopDegRevLexDeg(a) {
  IASSERTVALID(*this);
}

IPosElimDegRevLexDeg::~IPosElimDegRevLexDeg() {
}

int IPosElimDegRevLexDeg::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegRevLexDeg&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegRevLexDeg&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegRevLexDeg&)a).mDependVar)
    return 1;
  else if (((IPosElimDegRevLexDeg&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegRevLexDeg::compare(a);
}

int IPosElimDegRevLexDeg::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPosElimDegRevLexDeg&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPosElimDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegRevLexDeg&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegRevLexDeg&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegRevLexDeg&)a).mDependVar)
    return 1;
  else if (((IPosElimDegRevLexDeg&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegRevLexDeg::compare(a, b);
}

int IPosElimDegRevLexDeg::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimDegRevLexDeg&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimDegRevLexDeg&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimDegRevLexDeg&)a).mDependVar)
    return 1;
  else if (((IPosElimDegRevLexDeg&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return IDegRevLexDeg::compare(a);
}

void IPosElimDegRevLexDeg::assertValid(const char* fileName, int fileLine) const {
  ITopDegRevLexDeg::assertValid(fileName, fileLine);
}

IAllocator IPosElimDegRevLexDeg::sAllocator(sizeof(IPosElimDegRevLexDeg));
