/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iposelimelim.h"

IPosElimElimInterface::IPosElimElimInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int varElim, int tupElim):
    ITopElimInterface(systemType, independ, depend, varElim),
    mTupElim(tupElim) {
  IASSERT(0 < mTupElim && mTupElim < mDimDepend);
  mOrder = IMonomInterface::PosElimElim;
}

IMonom* IPosElimElimInterface::create() {
  return new IPosElimElim(this);
}

IMonom* IPosElimElimInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPosElimElim* r;
  if (this == a.monomInterface())
    r = new IPosElimElim(a);
  else {
    r = new IPosElimElim(this);
    r->ITopElim::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->ITopElim::prolong(var, a.deg(var));
    r->ITopElim::setDependVar(a.dependVar());
  }
  return r;
}

IPosElimElim::IPosElimElim(IPosElimElimInterface* r):
    ITopElim(r) {
}

IPosElimElim::IPosElimElim(const IMonom& a):
    ITopElim(a) {
  IASSERTVALID(*this);
}

IPosElimElim::~IPosElimElim() {
}

int IPosElimElim::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimElim&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimElim&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < tupElim() &&
      tupElim() <= ((IPosElimElim&)a).mDependVar)
    return 1;
  else if (((IPosElimElim&)a).mDependVar < tupElim() &&
             tupElim() <= mDependVar)
    return -1;
  else
    return IElim::compare(a);
}

int IPosElimElim::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPosElimElim&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPosElimElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimElim&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimElim&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < varElim() &&
      varElim() <= ((IPosElimElim&)a).mDependVar)
    return 1;
  else if (((IPosElimElim&)a).mDependVar < varElim() &&
             varElim() <= mDependVar)
    return -1;
  else
    return IElim::compare(a, b);
}

int IPosElimElim::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimElim&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimElim&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < varElim() &&
      varElim() <= ((IPosElimElim&)a).mDependVar)
    return 1;
  else if (((IPosElimElim&)a).mDependVar < varElim() &&
             varElim() <= mDependVar)
    return -1;
  else
    return IElim::compare(a);
}

void IPosElimElim::assertValid(const char* fileName, int fileLine) const {
  ITopElim::assertValid(fileName, fileLine);
}

IAllocator IPosElimElim::sAllocator(sizeof(IPosElimElim));
