/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IPOSELIMELIM_H
#define IPOSELIMELIM_H

#include "../util/iallocator.h"
#include "./itopelim.h"


class IPosElimElim;
class IPosElimElimInterface: public ITopElimInterface {
  friend class IPosElimElim;

protected:
  int                  mVarElim;
  int                  mTupElim;

public:
  IPosElimElimInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int varElim, int tupElim);
  ~IPosElimElimInterface() {}

  int varElim() const { return mVarElim; }
  int tupElim() const { return mTupElim; }

  IMonom* create();
  IMonom* copy(const IMonom& a);
};

class IPosElimElim: public ITopElim {
  friend class IPosElimElimInterface;

protected:
  static IAllocator     sAllocator;

  int varElim() const { return ((const IPosElimElimInterface*)mRealization)->mVarElim; }
  int tupElim() const { return ((const IPosElimElimInterface*)mRealization)->mTupElim; }

  IPosElimElim(IPosElimElimInterface* r);
  IPosElimElim(const IMonom& a);

public:
  void* operator new(size_t) { return sAllocator.allocate(); }
  void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

  ~IPosElimElim();

  int compare(const IMonom& a) const;
  int compare(const IMonom& a, const IMonom& b) const;
  int setQcompare(const IMonom& a) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // IPOSELIMELIM_H
