/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iposelimlex.h"

IPosElimLexInterface::IPosElimLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int elim):
    ITopLexInterface(systemType, independ, depend),
    mTupElim(elim) {
  IASSERT(0 < mTupElim && mTupElim < mDimDepend);
  mOrder = IMonomInterface::PosElimLex;
}

int IPosElimLexInterface::elim() const {
  return mTupElim;
}

IMonom* IPosElimLexInterface::create() {
  return new IPosElimLex(this);
}

IMonom* IPosElimLexInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPosElimLex* r;
  if (this == a.monomInterface())
    r = new IPosElimLex(a);
  else {
    r = new IPosElimLex(this);
    r->ITopLex::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->ITopLex::prolong(var, a.deg(var));
    r->ITopLex::setDependVar(a.dependVar());
  }
  return r;
}

IPosElimLex::IPosElimLex(IPosElimLexInterface* r):
    ITopLex(r) {
}

IPosElimLex::IPosElimLex(const IMonom& a):
    ITopLex(a) {
  IASSERTVALID(*this);
}

IPosElimLex::~IPosElimLex() {
}

int IPosElimLex::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimLex&)a).mDependVar)
    return 1;
  else if (((IPosElimLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return ILex::compare(a);
}

int IPosElimLex::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPosElimLex&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPosElimLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimLex&)a).mDependVar)
    return 1;
  else if (((IPosElimLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return ILex::compare(a, b);
}

int IPosElimLex::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPosElimLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPosElimLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((IPosElimLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((IPosElimLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  if (mDependVar < elim() &&
      elim() <= ((IPosElimLex&)a).mDependVar)
    return 1;
  else if (((IPosElimLex&)a).mDependVar < elim() &&
             elim() <= mDependVar)
    return -1;
  else
    return ILex::compare(a);
}

void IPosElimLex::assertValid(const char* fileName, int fileLine) const {
  ITopLex::assertValid(fileName, fileLine);
}

IAllocator IPosElimLex::sAllocator(sizeof(IPosElimLex));
