/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./ipotdeglex.h"

IPotDegLexInterface::IPotDegLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend):
    ITopDegLexInterface(systemType, independ, depend) {
  mOrder = IMonomInterface::PotDegLex;
}

IMonom* IPotDegLexInterface::create() {
  return new IPotDegLex(this);
}

IMonom* IPotDegLexInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPotDegLex* r;
  if (this == a.monomInterface())
    r = new IPotDegLex(a);
  else {
    r = new IPotDegLex(this);
    r->IPotDegLex::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->IPotDegLex::prolong(var, a.deg(var));
    r->IPotDegLex::setDependVar(a.dependVar());
  }
  return r;
}

IPotDegLex::IPotDegLex(IPotDegLexInterface* r):
    ITopDegLex(r) {
}

IPotDegLex::IPotDegLex(const IMonom& a):
    ITopDegLex(a) {
  IASSERTVALID(*this);
}

IPotDegLex::~IPotDegLex() {
}

int IPotDegLex::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotDegLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegLex&)a).mDependVar)
    return 1;
  else
    return IDegLex::compare(a);
}

int IPotDegLex::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPotDegLex&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPotDegLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegLex&)a).mDependVar)
    return 1;
  else
    return IDegLex::compare(a, b);
}

int IPotDegLex::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotDegLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegLex&)a).mDependVar)
    return 1;
  else
    return IDegLex::compare(a);
}

void IPotDegLex::assertValid(const char* fileName, int fileLine) const {
  ITopDegLex::assertValid(fileName, fileLine);
}

IAllocator IPotDegLex::sAllocator(sizeof(IPotDegLex));
