/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./ipotdegrevlex.h"

IPotDegRevLexInterface::IPotDegRevLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend):
    ITopDegRevLexInterface(systemType, independ, depend) {
  mOrder = IMonomInterface::PotDegRevLex;
}

IMonom* IPotDegRevLexInterface::create() {
  return new IPotDegRevLex(this);
}

IMonom* IPotDegRevLexInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPotDegRevLex* r;
  if (this == a.monomInterface())
    r = new IPotDegRevLex(a);
  else {
    r = new IPotDegRevLex(this);
    r->IPotDegRevLex::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->IPotDegRevLex::prolong(var, a.deg(var));
    r->IPotDegRevLex::setDependVar(a.dependVar());
  }
  return r;
}

IPotDegRevLex::IPotDegRevLex(IPotDegRevLexInterface* r):
    ITopDegRevLex(r) {
}

IPotDegRevLex::IPotDegRevLex(const IMonom& a):
    ITopDegRevLex(a) {
  IASSERTVALID(*this);
}

IPotDegRevLex::~IPotDegRevLex() {
}

int IPotDegRevLex::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegRevLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegRevLex&)a).mDependVar)
    return 1;
  else
    return IDegRevLex::compare(a);
}

int IPotDegRevLex::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPotDegRevLex&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPotDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegRevLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegRevLex&)a).mDependVar)
    return 1;
  else
    return IDegRevLex::compare(a, b);
}

int IPotDegRevLex::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegRevLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegRevLex&)a).mDependVar)
    return 1;
  else
    return IDegRevLex::compare(a);
}

void IPotDegRevLex::assertValid(const char* fileName, int fileLine) const {
  ITopDegRevLex::assertValid(fileName, fileLine);
}

IAllocator IPotDegRevLex::sAllocator(sizeof(IPotDegRevLex));
