/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./ipotdegrevlexdeg.h"


IPotDegRevLexDegInterface::IPotDegRevLexDegInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, unsigned* degs):
    ITopDegRevLexDegInterface(systemType, independ, depend, degs) {
  mOrder = IMonomInterface::PotDegRevLexDeg;
}

IMonom* IPotDegRevLexDegInterface::create() {
  return new IPotDegRevLexDeg(this);
}

IMonom* IPotDegRevLexDegInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPotDegRevLexDeg* r;
  if (this == a.monomInterface())
    r = new IPotDegRevLexDeg(a);
  else {
    r = new IPotDegRevLexDeg(this);
    r->IPotDegRevLexDeg::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->IPotDegRevLexDeg::prolong(var, a.deg(var));
    r->IPotDegRevLexDeg::setDependVar(a.dependVar());
  }
  return r;
}

IPotDegRevLexDeg::IPotDegRevLexDeg(IPotDegRevLexDegInterface* r):
    ITopDegRevLexDeg(r) {
}

IPotDegRevLexDeg::IPotDegRevLexDeg(const IMonom& a):
    ITopDegRevLexDeg(a) {
  IASSERTVALID(*this);
}

IPotDegRevLexDeg::~IPotDegRevLexDeg() {
}

int IPotDegRevLexDeg::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegRevLexDeg&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegRevLexDeg&)a).mDependVar)
    return 1;
  else
    return IDegRevLexDeg::compare(a);
}

int IPotDegRevLexDeg::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPotDegRevLexDeg&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPotDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegRevLexDeg&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegRevLexDeg&)a).mDependVar)
    return 1;
  else
    return IDegRevLexDeg::compare(a, b);
}

int IPotDegRevLexDeg::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotDegRevLexDeg&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotDegRevLexDeg&)a).mDependVar)
    return 1;
  else
    return IDegRevLexDeg::setQcompare(a);
}

void IPotDegRevLexDeg::assertValid(const char* fileName, int fileLine) const {
  ITopDegRevLexDeg::assertValid(fileName, fileLine);
}

IAllocator IPotDegRevLexDeg::sAllocator(sizeof(IPotDegRevLexDeg));
