/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./ipotelim.h"


IPotElimInterface::IPotElimInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, int elim):
    ITopElimInterface(systemType, independ, depend, elim) {
  mOrder = IMonomInterface::PotElim;
}

IMonom* IPotElimInterface::create() {
  return new IPotElim(this);
}

IMonom* IPotElimInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IPotElim* r;
  if (this == a.monomInterface())
    r = new IPotElim(a);
  else {
    r = new IPotElim(this);
    r->IPotElim::setZero();
    for(int var=0; var < dimDepend(); var++)
      r->IPotElim::prolong(var, a.deg(var));
    r->IPotElim::setDependVar(a.dependVar());
  }
  return r;
}

IPotElim::IPotElim(IPotElimInterface* r):
    ITopElim(r) {
}

IPotElim::IPotElim(const IMonom& a):
    ITopElim(a) {
  IASSERTVALID(*this);
}

IPotElim::~IPotElim() {
}

int IPotElim::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotElim&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotElim&)a).mDependVar)
    return 1;
  else
    return IElim::compare(a);
}

int IPotElim::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((IPotElim&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((IPotElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotElim&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotElim&)a).mDependVar)
    return 1;
  else
    return IElim::compare(a, b);
}

int IPotElim::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((IPotElim&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((IPotElim&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar > ((IPotElim&)a).mDependVar)
    return -1;
  else if (mDependVar < ((IPotElim&)a).mDependVar)
    return 1;
  else
    return IElim::compare(a);
}

void IPotElim::assertValid(const char* fileName, int fileLine) const {
  ITopElim::assertValid(fileName, fileLine);
}

IAllocator IPotElim::sAllocator(sizeof(IPotElim));
