/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./itopdegrevlex.h"

ITopDegRevLexInterface::ITopDegRevLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend):
    IDegRevLexInterface(systemType, independ) {
  IASSERT(mSystemType->module() > 0 || depend);
  mOrder = IMonomInterface::TopDegRevLex;
  if (depend) {
    IASSERT(mSystemType->module() == 0);
    mDimDepend = depend->dim();
    mDepend = depend;
  }
  else {
    IASSERT(mSystemType->module() > 0);
    mDimDepend = mSystemType->module();
  }
}

IMonom* ITopDegRevLexInterface::create() {
  return new ITopDegRevLex(this);
}

IMonom* ITopDegRevLexInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  ITopDegRevLex* r;
  if (this == a.monomInterface())
    r = new ITopDegRevLex(a);
  else {
    r = new ITopDegRevLex(this);
    r->setZero();
    for(int var=0; var < dimDepend(); var++)
      r->prolong(var, a.deg(var));
    r->setDependVar(a.dependVar());
  }
  return r;
}

ITopDegRevLex::ITopDegRevLex(ITopDegRevLexInterface* r):
    IDegRevLex(r),
    mDependVar(-1) {
}

ITopDegRevLex::ITopDegRevLex(const IMonom& a):
    IDegRevLex(a),
    mDependVar(((const ITopDegRevLex&)a).mDependVar) {
  IASSERTVALID(*this);
}

ITopDegRevLex::~ITopDegRevLex() {
}

int ITopDegRevLex::dependVar() const {
  return mDependVar;
}

void ITopDegRevLex::set(const IMonom& a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IDegRevLex::set(a);
  mDependVar = a.dependVar();
  IASSERTVALID(*this);
}

void ITopDegRevLex::setZero() {
  IDegRevLex::setZero();
  mDependVar = -1;
}

void ITopDegRevLex::setDependVar(int depend) {
  IASSERT(0 <= depend && depend < dimDepend());
  mDependVar = depend;
}

void ITopDegRevLex::swap(IMonom& a) {
  IDegRevLex::swap(a);
  int tmp = mDependVar;
  mDependVar = ((ITopDegRevLex&)a).mDependVar;
  ((ITopDegRevLex&)a).mDependVar = tmp;
}

void ITopDegRevLex::mult(const IMonom& a, const IMonom& b) {
  IDegRevLex::mult(a, b);
  mDependVar = ((ITopDegRevLex&)a).mDependVar;
}

bool ITopDegRevLex::divisibility(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  return mDependVar == ((ITopDegRevLex&)a).mDependVar && IDegRevLex::divisibility(a);
}

unsigned ITopDegRevLex::gcd(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(mDependVar == ((ITopDegRevLex&)a).mDependVar);
  return IDegRevLex::gcd(a);
}

unsigned ITopDegRevLex::lcm(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(mDependVar == ((ITopDegRevLex&)a).mDependVar);
  return IDegRevLex::lcm(a);
}

int ITopDegRevLex::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((ITopDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((ITopDegRevLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((ITopDegRevLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  int cmp = IDegRevLex::compare(a);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLex&)a).mDependVar)
    return 1;
  else
    return 0;
}

int ITopDegRevLex::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((ITopDegRevLex&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((ITopDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((ITopDegRevLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((ITopDegRevLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  int cmp = IDegRevLex::compare(a, b);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLex&)a).mDependVar)
    return 1;
  else
    return 0;
}

int ITopDegRevLex::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((ITopDegRevLex&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLex&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((ITopDegRevLex&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((ITopDegRevLex&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  int cmp = IDegRevLex::compare(a);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLex&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLex&)a).mDependVar)
    return 1;
  else
    return 0;
}

void ITopDegRevLex::read(std::istream &in) {
  IDegRevLex::read(in);
  in.read((char*)&mDependVar, sizeof(mDependVar));
  IASSERTVALID(*this);
}

void ITopDegRevLex::write(std::ostream &out) const {
  IDegRevLex::write(out);
  out.write((char*)&mDependVar, sizeof(mDependVar));
}

void ITopDegRevLex::assertValid(const char* fileName, int fileLine) const {
  IDegRevLex::assertValid(fileName, fileLine);
  IASSERT2(fileName, fileLine, -1 <= mDependVar && mDependVar < dimDepend());
}

IAllocator ITopDegRevLex::sAllocator(sizeof(ITopDegRevLex));
