/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ITOPDEGREVLEX_H
#define ITOPDEGREVLEX_H

#include "../util/iallocator.h"
#include "./idegrevlex.h"


class ITopDegRevLexInterface: public IDegRevLexInterface {
public:
  ITopDegRevLexInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend);
  ~ITopDegRevLexInterface() {}

  IMonom* create();
  IMonom* copy(const IMonom& a);
};

class ITopDegRevLex: public IDegRevLex {
  friend class ITopDegRevLexInterface;

protected:
  int mDependVar;
  static IAllocator     sAllocator;

  ITopDegRevLex(ITopDegRevLexInterface* r);
  ITopDegRevLex(const IMonom& a);

public:
  void* operator new(size_t) { return sAllocator.allocate(); }
  void operator delete(void *ptr) { sAllocator.deallocate(ptr); }

  ~ITopDegRevLex();

  int dependVar() const;

  void set(const IMonom& a);
  void setZero();
  void setDependVar(int depend);

  void swap(IMonom& a);

  void mult(const IMonom& a, const IMonom& b);

  bool divisibility(const IMonom& a) const;

  unsigned gcd(const IMonom& a) const;
  unsigned lcm(const IMonom& a) const;

  int compare(const IMonom& a) const;
  int compare(const IMonom& a, const IMonom& b) const;
  int setQcompare(const IMonom& a) const;

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // ITOPDEGREVLEX_H
