/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./itopdegrevlexbyte.h"

ITopDegRevLexByteInterface::ITopDegRevLexByteInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend):
    IDegRevLexByteInterface(systemType, independ) {
  IASSERT(mSystemType->module() > 0 || depend);
  mOrder = IMonomInterface::TopDegRevLexByte;
  if (depend) {
    IASSERT(mSystemType->module() == 0);
    mDimDepend = depend->dim();
    mDepend = depend;
  }
  else {
    IASSERT(mSystemType->module() > 0);
    mDimDepend = mSystemType->module();
  }
}

IMonom* ITopDegRevLexByteInterface::create() {
  return new ITopDegRevLexByte(this);
}

IMonom* ITopDegRevLexByteInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  ITopDegRevLexByte* r;
  if (this == a.monomInterface())
    r = new ITopDegRevLexByte(a);
  else {
    r = new ITopDegRevLexByte(this);
    r->setZero();
    for(int var=0; var < dimDepend(); var++)
      r->prolong(var, a.deg(var));
    r->setDependVar(a.dependVar());
  }
  return r;
}

ITopDegRevLexByte::ITopDegRevLexByte(ITopDegRevLexByteInterface* r):
    IDegRevLexByte(r),
    mDependVar(-1) {
}

ITopDegRevLexByte::ITopDegRevLexByte(const IMonom& a):
    IDegRevLexByte(a),
    mDependVar(((const ITopDegRevLexByte&)a).mDependVar) {
  IASSERTVALID(*this);
}

ITopDegRevLexByte::~ITopDegRevLexByte() {
}

int ITopDegRevLexByte::dependVar() const {
  return mDependVar;
}

void ITopDegRevLexByte::set(const IMonom& a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IDegRevLexByte::set(a);
  mDependVar = a.dependVar();
  IASSERTVALID(*this);
}

void ITopDegRevLexByte::setZero() {
  IDegRevLexByte::setZero();
  mDependVar = -1;
}

void ITopDegRevLexByte::setDependVar(int depend) {
  IASSERT(0 <= depend && depend < dimDepend());
  mDependVar = depend;
}

void ITopDegRevLexByte::swap(IMonom& a) {
  IDegRevLexByte::swap(a);
  int tmp = mDependVar;
  mDependVar = ((ITopDegRevLexByte&)a).mDependVar;
  ((ITopDegRevLexByte&)a).mDependVar = tmp;
}

void ITopDegRevLexByte::mult(const IMonom& a, const IMonom& b) {
  IDegRevLexByte::mult(a, b);
  mDependVar = ((ITopDegRevLexByte&)a).mDependVar;
}

bool ITopDegRevLexByte::divisibility(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  return mDependVar == ((ITopDegRevLexByte&)a).mDependVar && IDegRevLexByte::divisibility(a);
}

unsigned ITopDegRevLexByte::gcd(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(mDependVar == ((ITopDegRevLexByte&)a).mDependVar);
  return IDegRevLexByte::gcd(a);
}

unsigned ITopDegRevLexByte::lcm(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(mDependVar == ((ITopDegRevLexByte&)a).mDependVar);
  return IDegRevLexByte::lcm(a);
}

int ITopDegRevLexByte::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((ITopDegRevLexByte&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLexByte&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((ITopDegRevLexByte&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((ITopDegRevLexByte&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  int cmp = IDegRevLexByte::compare(a);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLexByte&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLexByte&)a).mDependVar)
    return 1;
  else
    return 0;
}

int ITopDegRevLexByte::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((ITopDegRevLexByte&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((ITopDegRevLexByte&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLexByte&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((ITopDegRevLexByte&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((ITopDegRevLexByte&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  int cmp = IDegRevLexByte::compare(a, b);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLexByte&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLexByte&)a).mDependVar)
    return 1;
  else
    return 0;
}

int ITopDegRevLexByte::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((ITopDegRevLexByte&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLexByte&)a).mDependVar < 0)
      return 1;
  }

  if (mDependVar < monomInterface()->systemType()->rightPart()) {
    if (((ITopDegRevLexByte&)a).mDependVar >= monomInterface()->systemType()->rightPart())
      return 1;
  }
  else {
    if (((ITopDegRevLexByte&)a).mDependVar < monomInterface()->systemType()->rightPart())
      return -1;
  }

  int cmp = IDegRevLexByte::compare(a);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLexByte&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLexByte&)a).mDependVar)
    return 1;
  else
    return 0;
}

void ITopDegRevLexByte::read(std::istream &in) {
  IDegRevLexByte::read(in);
  in.read((char*)&mDependVar, sizeof(mDependVar));
  IASSERTVALID(*this);
}

void ITopDegRevLexByte::write(std::ostream &out) const {
  IDegRevLexByte::write(out);
  out.write((char*)&mDependVar, sizeof(mDependVar));
}

void ITopDegRevLexByte::assertValid(const char* fileName, int fileLine) const {
  IDegRevLexByte::assertValid(fileName, fileLine);
  IASSERT2(fileName, fileLine, -1 <= mDependVar && mDependVar < dimDepend());
}

IAllocator ITopDegRevLexByte::sAllocator(sizeof(ITopDegRevLexByte));
