/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./itopdegrevlexdeg.h"


ITopDegRevLexDegInterface::ITopDegRevLexDegInterface(const ISystemType *systemType, IVariables* independ, IVariables* depend, unsigned* degs):
    IDegRevLexDegInterface(systemType, independ, degs) {
  IASSERT(mSystemType->module() > 0 || depend);
  mOrder = IMonomInterface::TopDegRevLexDeg;
  if (depend) {
    IASSERT(mSystemType->module() == 0);
    mDimDepend = depend->dim();
    mDepend = depend;
  }
  else {
    IASSERT(mSystemType->module() > 0);
    mDimDepend = mSystemType->module();
  }
}

IMonom* ITopDegRevLexDegInterface::create() {
  return new ITopDegRevLexDeg(this);
}

IMonom* ITopDegRevLexDegInterface::copy(const IMonom &a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  ITopDegRevLexDeg* r;
  if (this == a.monomInterface())
    r = new ITopDegRevLexDeg(a);
  else {
    r = new ITopDegRevLexDeg(this);
    r->setZero();
    for(int var=0; var < dimDepend(); var++)
      r->prolong(var, a.deg(var));
    r->setDependVar(a.dependVar());
  }
  return r;
}

ITopDegRevLexDeg::ITopDegRevLexDeg(ITopDegRevLexDegInterface* r):
    IDegRevLexDeg(r),
    mDependVar(-1) {
}

ITopDegRevLexDeg::ITopDegRevLexDeg(const IMonom& a):
    IDegRevLexDeg(a),
    mDependVar(((const ITopDegRevLexDeg&)a).mDependVar) {
  IASSERTVALID(*this);
}

ITopDegRevLexDeg::~ITopDegRevLexDeg() {
}

int ITopDegRevLexDeg::dependVar() const {
  return mDependVar;
}

void ITopDegRevLexDeg::set(const IMonom& a) {
  IASSERT(dimIndepend() == a.dimIndepend());
  IASSERT(independ() == a.independ());
  IASSERT(dimDepend() == a.dimDepend());
  IASSERT(depend() == a.depend());

  IDegRevLexDeg::set(a);
  mDependVar = a.dependVar();
  IASSERTVALID(*this);
}

void ITopDegRevLexDeg::setZero() {
  IDegRevLexDeg::setZero();
  mDependVar = -1;
}

void ITopDegRevLexDeg::setDependVar(int depend) {
  IASSERT(0 <= depend && depend < dimDepend());
  mDependVar = depend;
}

void ITopDegRevLexDeg::swap(IMonom& a) {
  IDegRevLexDeg::swap(a);
  int tmp = mDependVar;
  mDependVar = ((ITopDegRevLexDeg&)a).mDependVar;
  ((ITopDegRevLexDeg&)a).mDependVar = tmp;
}

void ITopDegRevLexDeg::mult(const IMonom& a, const IMonom& b) {
  IDegRevLexDeg::mult(a, b);
  mDependVar = ((ITopDegRevLexDeg&)a).mDependVar;
}

bool ITopDegRevLexDeg::divisibility(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  return mDependVar == ((ITopDegRevLexDeg&)a).mDependVar && IDegRevLexDeg::divisibility(a);
}

unsigned ITopDegRevLexDeg::gcd(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(mDependVar == ((ITopDegRevLexDeg&)a).mDependVar);
  return IDegRevLexDeg::gcd(a);
}

unsigned ITopDegRevLexDeg::lcm(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(mDependVar == ((ITopDegRevLexDeg&)a).mDependVar);
  return IDegRevLexDeg::lcm(a);
}

int ITopDegRevLexDeg::compare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((ITopDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  int cmp = IDegRevLexDeg::compare(a);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLexDeg&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLexDeg&)a).mDependVar)
    return 1;
  else
    return 0;
}

int ITopDegRevLexDeg::compare(const IMonom& a, const IMonom& b) const {
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(monomInterface() == b.monomInterface());
  IASSERT(monomInterface() == a.monomInterface());
  IASSERT(((ITopDegRevLexDeg&)b).mDependVar == -1);
  if (mDependVar < 0) {
    if (((ITopDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  int cmp = IDegRevLexDeg::compare(a, b);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLexDeg&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLexDeg&)a).mDependVar)
    return 1;
  else
    return 0;
}

int ITopDegRevLexDeg::setQcompare(const IMonom& a) const {
  IASSERT(monomInterface() == a.monomInterface());
  if (mDependVar < 0) {
    if (((ITopDegRevLexDeg&)a).mDependVar >= 0)
      return -1;
  }
  else {
    if (((ITopDegRevLexDeg&)a).mDependVar < 0)
      return 1;
  }

  int cmp = IDegRevLexDeg::setQcompare(a);
  if (cmp != 0)
    return cmp;
  else if (mDependVar > ((ITopDegRevLexDeg&)a).mDependVar)
    return -1;
  else if (mDependVar < ((ITopDegRevLexDeg&)a).mDependVar)
    return 1;
  else
    return 0;
}

void ITopDegRevLexDeg::read(std::istream &in) {
  IDegRevLexDeg::read(in);
  in.read((char*)&mDependVar, sizeof(mDependVar));
  IASSERTVALID(*this);
}

void ITopDegRevLexDeg::write(std::ostream &out) const {
  IDegRevLexDeg::write(out);
  out.write((char*)&mDependVar, sizeof(mDependVar));
}

void ITopDegRevLexDeg::assertValid(const char* fileName, int fileLine) const {
  IDegRevLexDeg::assertValid(fileName, fileLine);
  IASSERT2(fileName, fileLine, -1 <= mDependVar && mDependVar < dimDepend());
}

IAllocator ITopDegRevLexDeg::sAllocator(sizeof(ITopDegRevLexDeg));
