/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/iutil.h"
#include "./iexpression.h"


void IExpression::equality(std::istream& in) {
  int module = mNumer->polyInterface()->systemType()->module();
  if (module == 0) {
    if (mExpanded)
      expanded(in);
    else
      additive(in);
  }
  else {
    if ((in >> std::ws).get() != '[')
      IERROR("expected '['");
    for(mDependVar=0; mDependVar < module-1; mDependVar++) {
      IExpression tmp(mNumer->polyInterface(), mDependVar);
      if (mExpanded)
        tmp.expanded(in);
      else
        tmp.additive(in);
      add(tmp);
      in.clear();
      if ((in >> std::ws).get() != ',')
        IERROR("expected ','");
    }
    IExpression tmp(mNumer->polyInterface(), mDependVar);
    if (mExpanded)
      tmp.expanded(in);
    else 
      tmp.additive(in);
    add(tmp);
    in.clear();
    if ((in >> std::ws).get() != ']')
      IERROR("expected ']'");
  }
/*  IExpression tmp(mNumer->polyInterface());
  if ((in >> std::ws).peek() == '=') {
    in.get();
    tmp.additive(in);
    sub(tmp);
  }*/
}

void IExpression::expanded(std::istream& in) {
  mNumer->setZero();
  int op = '+';
  ICoeff *tmp(mNumer->coeffInterface()->create());
  std::streampos pos;
  do {
    if (in.peek() == '+' || in.peek() == '-')
      op = in.get();

    in >> std::ws;

    pos = in.tellg();

    ICoeff *c(mNumer->coeffInterface()->create());
    c->setOne();
    do {
      if ((in >> std::ws).peek() == '*')
        in.get();

      in >> (*tmp);
      if (!in.fail()) {
        c->mult(*tmp);
      }
    } while (!in.fail() && (in >> std::ws).peek() == '*');
    in.clear();
    if ((in >> std::ws).peek() == '*')
      in.get();

    IMonom *m(mNumer->monomInterface()->create());
    in >> (*m);
    if (in.fail()) {
      in.clear();
      m->setZero();
    }

    if (!mNumer->isZero() && mDependVar >= 0)
      mNumer->setDependVar(mDependVar);
    
    if (op == '-')
      c->minus();

    mNumer->addTerm(c, m);

    op = (in >> std::ws).peek();

    if (pos == in.tellg())
      IERROR("expected 'monomial' or 'coefficient'");
  } while (op == '+' || op == '-');

  delete tmp;
  IASSERTVALID(*this);
}

void IExpression::additive(std::istream& in) {
  multiplicative(in);
  int op = (in >> std::ws).peek();
  IExpression tmp(mNumer->polyInterface(), mDependVar);
  while(op == '+' || op == '-') {
    in.get();
    tmp.multiplicative(in);
    if (op == '+')
      add(tmp);
    else
      sub(tmp);
    op = (in >> std::ws).peek();
  }
}

void IExpression::multiplicative(std::istream& in) {
  unary(in);
  int op = (in >> std::ws).peek();
  IExpression tmp(mNumer->polyInterface(), mDependVar);
  while(op == '*' || op == '/') {
    in.get();
    tmp.unary(in);
    if (op == '*') {
      mult(tmp);
    }
    else {
      if (tmp.mNumer->lm().degree() > 0)
        IERROR("expected 'coefficient'");
      else {
        IASSERT(tmp.mDenom->isOne());
        div(tmp.mNumer->lc());
      }
    }
    op = in.peek();
  }
}

void IExpression::unary(std::istream& in) {
  int ch = (in >> std::ws).peek();
  if (ch != '+' && ch != '-')
    power(in);
  else {
    int k = 1;
    do {
      ch = in.get();
      if (ch == '-')
        k *= -1;
      ch = (in >> std::ws).peek();
    } while (ch == '+' || ch == '-');
    power(in);
    if (k < 0)
      minus();
  }
}

void IExpression::power(std::istream& in) {
  bracket(in);
  int op = (in >> std::ws).peek();
  if (op == '^') {
    in.get();
    int deg=0;
    in >> std::ws >> deg;
    if (in.fail() || deg <= 0)
      IERROR("expected 'degree > 0'");
    pow(deg);
  }
}

void IExpression::bracket(std::istream& in) {
  int op = (in >> std::ws).peek();
  if (op == '(') {
    in.get();
    additive(in);
    if (in.get() != ')') {
      IERROR("expected ')'");
    }
  }
  else {
    mDenom->setOne();
    IMonom *m(mNumer->monomInterface()->create());
    in >> (*m);
    if (!in.fail()) {
      mNumer->setMonom(m);
    }
    else {
      delete m;
      in.clear();
      ICoeff *c(mNumer->coeffInterface()->create());
      in >> (*c);
      if (!in.fail()) {
        if (c->isZero())
          mNumer->setZero();
        else
          mNumer->setCoeff(c);
      }
      else {
        delete c;
        std::cerr << *this << std::endl;
        IERROR("expected 'monomial' or 'coefficient'");
      }
    }
    if (!mNumer->isZero() && mDependVar >= 0)
      mNumer->setDependVar(mDependVar);
    
    IASSERTVALID(*this);
  }
}

std::istream& operator>>(std::istream& in, IExpression& a) {
  a.equality(in);
  return in;
}

std::ostream& operator<<(std::ostream& out, IExpression& a) {
  if (a.mDenom->isOne())
    out << (*a.mNumer);
  else
    out << '(' << (*a.mNumer) << ')' << '/' << '(' << (*a.mDenom) << ')';
  return out;
}

void IExpression::assertValid(const char* fileName, int fileLine) const {
  IASSERT2(fileName, fileLine, mNumer);
  IASSERT2(fileName, fileLine, mDenom);
  IASSERT2(fileName, fileLine, !mDenom->isZero());
  IASSERT2(fileName, fileLine, !mDenom->isNegative());
}

