/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IEXPRESSION_H
#define IEXPRESSION_H

#include "./ipoly.h"


class IExpression {
protected:
  bool     mExpanded;
  int      mDependVar;
  IPoly   *mNumer;
  ICoeff  *mDenom;

  IExpression(const IExpression& ):
      mExpanded(false),
      mDependVar(-1),
      mNumer(NULL),
      mDenom(NULL) {}
  IExpression(IPolyInterface *r, int dependVar):
      mExpanded(false),
      mDependVar(dependVar),
      mNumer(NULL),
      mDenom(NULL) {
    IASSERT(r);
    mNumer = r->create();
    mDenom = r->coeffInterface()->create();
    mDenom->setOne();
  }

  void equality(std::istream& in);
  void expanded(std::istream& in);
  void additive(std::istream& in);
  void multiplicative(std::istream& in);
  void unary(std::istream& in);
  void power(std::istream& in);
  void bracket(std::istream& in);

public:
  IExpression(IPolyInterface *r, bool expanded=false):
      mExpanded(expanded),
      mDependVar(-1),
      mNumer(NULL),
      mDenom(NULL) {
    IASSERT(r);
    mNumer = r->create();
    mDenom = r->coeffInterface()->create();
    mDenom->setOne();
  }
  ~IExpression() {
    delete mNumer;
    delete mDenom;
  }

  IPoly  *numer() const { return mNumer; }
  ICoeff *denom() const { return mDenom; }

  IPoly* toPrimitivePoly() {
    IPoly* r = mNumer;
    r->pp();
    mNumer = NULL;
    return r;
  }
  IPoly* toPoly() {
    IPoly* r = mNumer;
    mNumer = NULL;
    return r;
  }
  ICoeff* getDenom() {
    ICoeff* d = mDenom;
    mDenom = NULL;
    return d;
  }
  void minus() { mNumer->minus(); }
  void add(IExpression &a) {
    if (mNumer->coeffInterface()->isField())
      mNumer->add(*a.mNumer);
    else {
      mNumer->mult(*a.mDenom);
      a.mNumer->mult(*mDenom);
      mNumer->add(*a.mNumer);
      mDenom->mult(*a.mDenom);
    }
  }
  void sub(IExpression &a) {
    if (mNumer->coeffInterface()->isField())
      mNumer->sub(*a.mNumer);
    else {
      mNumer->mult(*a.mDenom);
      a.mNumer->mult(*mDenom);
      mNumer->sub(*a.mNumer);
      mDenom->mult(*a.mDenom);
    }
  }
  void mult(const IMonom& m) { mNumer->mult(m); }
  void mult(const ICoeff& a) { mNumer->mult(a); }
  void mult(const IExpression &a) {
    mNumer->mult(*a.mNumer);
    if (!mNumer->coeffInterface()->isField())
      mDenom->mult(*a.mDenom);
  }
  void pow(unsigned deg) {
    mNumer->pow(deg);
    if (!mNumer->coeffInterface()->isField())
      mDenom->pow(deg);
  }
  void div(const ICoeff& c) {
    if (mNumer->coeffInterface()->isField())
      mNumer->div(c);
    else
      mDenom->mult(c);
  }

  void assertValid(const char* fileName, int fileLine) const;

  friend std::istream& operator>>(std::istream& in, IExpression& a);
  friend std::ostream& operator<<(std::ostream& out, IExpression& a);
};

#endif // IEXPRESSION_H

