/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <cstdarg>

#include "./ipoly.h"
#include "./ipolylist.h"


IPolyInterface::IPolyInterface():
    mType(Indefinitely),
    mSystemType(NULL),
    mMonomInterface(NULL),
    mCoeffInterface(NULL) {
}

IPolyInterface* IPolyInterface::create(IPolyInterface::Type type,
                                       ISystemType *systemType,
                                       IMonomInterface* monomInterface,
                                       ICoeffInterface* coeffInterface, ...) {
  if (type == IPolyInterface::Indefinitely)
    IERROR("error in IPolyInterface: type == IPolyInterface::Indefinitely");
  if (systemType == NULL || systemType->type() == ISystemType::Indefinitely)
    IERROR("error in IPolyInterface: systemType == NULL || systemType->type() == ISystemType::Indefinitely");
  if (monomInterface == NULL || monomInterface->order() == IMonomInterface::Indefinitely)
    IERROR("error in IPolyInterface: monomInterface == NULL || monomInterface->order() == IMonomInterface::Indefinitely");
  if (coeffInterface == NULL || coeffInterface->type() == ICoeffInterface::Indefinitely)
    IERROR("error in IPolyInterface: coeffInterface == NULL || coeffInterface->type() == coeffInterface::Indefinitely");

  va_list marker;
  va_start(marker, coeffInterface);

  IPolyInterface* r = NULL;
  switch(type) {
  case PolyList:
    r = new IPolyListInterface;
    break;
  case PolyArray:
    break;
  }
  va_end(marker);

  if (r) {
    r->mSystemType = systemType;
    r->mMonomInterface = monomInterface;
    r->mCoeffInterface = coeffInterface;
  }
  return r;
}

static void output(std::ostream& out, const IPoly& a) {
  if (a.isZero())
    out << '0';
  else {
    IPoly::ConstIterator &i(*a.begin());
    if (i.coeff().isOne()) {
      if (i.monom().degree())
        out << i.monom();
      else
        out << '1';
    }
    else {
      if (i.monom().degree() || i.monom().depend() >= 0)
        if (i.coeff().isComposite())
          out << '(' << i.coeff() << ')' << '*' << i.monom();
        else
          out << i.coeff() << '*' << i.monom();
      else
	out << i.coeff();
    }
    ++i;
    while(i) {
      out << " + ";
      if (i.coeff().isOne()) {
        if (i.monom().degree() || i.monom().depend() >= 0)
          out << i.monom();
        else
          out << '1';
      }
      else {
        if (i.coeff().isComposite())
          out << '(' << i.coeff() << ')';
        else
          out << i.coeff();
        if (i.monom().degree() || i.monom().depend() >= 0)
          out << '*' << i.monom();
      }
      ++i;
    }
  }
}

std::ostream& operator<<(std::ostream& out, const IPoly& a) {
  int module = a.polyInterface()->systemType()->module();
  if (module == 0)
    output(out, a);
  else {
    IPoly *poly = a.polyInterface()->create();
    out << '[';
    int i;
    for(i=0; i < (module-1) ; i++) {
      poly->setPoly(a, i);
      output(out, *poly);
      out << ", ";
    }
    poly->setPoly(a, i);
    output(out, *poly);
    out << ']';
    delete poly;
  }
  return out;
}
