/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../util/imodular.h"
#include "./ipolylist.h"

IPolyListInterface::IPolyListInterface():
    IPolyInterface() {
  mType = IPolyInterface::PolyList;
}

IPoly* IPolyListInterface::create() {
  return new IPolyList(this);
}

IPoly* IPolyListInterface::copy(const IPoly& a) {
  IASSERT(this == a.polyInterface());
  return new IPolyList((const IPolyList&)a);
}

IPoly* IPolyListInterface::copy(const IPoly& a, int var) {
  IASSERT(this == a.polyInterface());
  return new IPolyList((const IPolyList&)a, var);
}

IPoly* IPolyListInterface::copy(const IPoly& a, int var, unsigned deg)  {
  IASSERT(this == a.polyInterface());
  return new IPolyList((const IPolyList&)a, var, deg);
}

IPolyList::IPolyList(IPolyListInterface* r):
    IPoly(r),
    mHead() {
}

IPolyList::IPolyList(const IPolyList& a):
    IPoly(a),
    mHead() {
  It i(mHead.begin());
  ConstIt ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = monomInterface()->copy(*(*ia).mMonom);
    (*i).mCoeff = coeffInterface()->copy(*(*ia).mCoeff);
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

IPolyList::IPolyList(const IPolyList& a, int var):
    IPoly(a),
    mHead() {
  It i(mHead.begin());
  ConstIt ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = monomInterface()->copy(*(*ia).mMonom);
    (*i).mMonom->prolong(var);
    (*i).mCoeff = coeffInterface()->copy(*(*ia).mCoeff);
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

IPolyList::IPolyList(const IPolyList& a, int var, unsigned deg):
    IPoly(a),
    mHead() {
  It i(mHead.begin());
  ConstIt ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = monomInterface()->copy(*(*ia).mMonom);
    (*i).mMonom->prolong(var, deg);
    (*i).mCoeff = coeffInterface()->copy(*(*ia).mCoeff);
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

IPolyList::IPolyList(const IPolyList& a, const IMonom& m, const ICoeff& c):
    IPoly(a),
    mHead() {
  It i(mHead.begin());
  ConstIt ia(a.mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = monomInterface()->copy(*(*ia).mMonom);
    (*i).mMonom->mult(m);
    (*i).mCoeff = coeffInterface()->copy(*(*ia).mCoeff);
    (*i).mCoeff->mult(c);
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

void IPolyList::setZero() {
  mHead.clear();
}

void IPolyList::setOne() {
  mHead.clear();
  It i(mHead.begin());
  i.insert();
  (*i).mMonom = monomInterface()->create();
  (*i).mMonom->setZero();
  (*i).mCoeff = coeffInterface()->create();
  (*i).mCoeff->setOne();
}

void IPolyList::setDependVar(int var) {
  ConstIt i(mHead.begin());
  while(i) {
    IASSERT((*i).mMonom->dependVar() < 0);
    (*i).mMonom->setDependVar(var);
    ++i;
  }
  IASSERTVALID(*this);
}

void IPolyList::setMonom(IMonom *m) {
  mHead.clear();
  It i(mHead.begin());
  i.insert();
  (*i).mMonom = m;
  (*i).mCoeff = coeffInterface()->create();
  (*i).mCoeff->setOne();
}

void IPolyList::setCoeff(ICoeff *c) {
  mHead.clear();
  It i(mHead.begin());
  i.insert();
  (*i).mMonom = monomInterface()->create();
  (*i).mMonom->setZero();
  (*i).mCoeff = c;
}

void IPolyList::setPoly(const IPoly& a, int dependVar) {
  mHead.clear();
  It i(mHead.begin());
  ConstIt ia(((const IPolyList&)a).mHead.begin());
  while(ia) {
    if ((*ia).mMonom->dependVar() == dependVar) {
      i.insert();
      (*i).mMonom = monomInterface()->copy(*(*ia).mMonom);
      (*i).mCoeff = coeffInterface()->copy(*(*ia).mCoeff);
      ++i;
    }
    ++ia;
  }
  IASSERTVALID(*this);
}

void IPolyList::set(const IPoly& a) {
  IASSERT(polyInterface() == a.polyInterface());
  mHead.clear();
  It i(mHead.begin());
  ConstIt ia(((const IPolyList&)a).mHead.begin());
  while(ia) {
    i.insert();
    (*i).mMonom = monomInterface()->copy(*(*ia).mMonom);
    (*i).mCoeff = coeffInterface()->copy(*(*ia).mCoeff);
    ++i;
    ++ia;
  }
  IASSERTVALID(*this);
}

void IPolyList::swap(IPoly& a) {
  IASSERT(polyInterface() == a.polyInterface());
  mHead.swap(((IPolyList&)a).mHead);
}

bool IPolyList::isZero() const {
  return mHead.isEmpty();
}

int IPolyList::length() const {
  return mHead.length();
}

int IPolyList::degree() const {
  ConstIt i(mHead.begin());
  unsigned max=0;

  if (monomInterface()->order() & IMonomInterface::DegRevLex) {
    if (i)
      max = (*i).mMonom->degree();
  }
  else if (monomInterface()->order() == IMonomInterface::DegRevLexDeg) {
    if (i)
      max = (*i).mMonom->weighted_degree();
  }
  else {
    while(i) {
      if ((*i).mMonom->degree() > max)
        max = (*i).mMonom->degree();
      ++i;
    }
  }
  return max;
}

int IPolyList::norm() const {
/*  ConstIt i(mHead.begin());
  int sum = 0;
  while(i) {
    int n = (*i).mCoeff->norm();
    sum += n;
    ++i;
  }
  return sum;*/
/*  ConstIt i(mHead.begin());
  int sum = 0;
  while(i) {
    int n = (*i).mCoeff->norm();
    sum += n*n;
    ++i;
  }
  return sum;*/
/*  ConstIt i(mHead.begin());
  int sum = 0;
  while(i) {
    int n = (*i).mCoeff->norm()*(*i).mMonom->degree();
    sum += n;
    ++i;
  }
  return sum;*/
  ConstIt i(mHead.begin());
  int max = -1;
  while(i) {
    int n = (*i).mCoeff->norm();
    if (max < 0 || n > max)
      max = n;
    ++i;
  }
  return max;
/*  ConstIt i(mHead.begin());
  int max = (*i).mCoeff->norm();
  int deg = (*i).mMonom->degree();
  ++i;
  while(i && deg == (*i).mMonom->degree()) {
    int n = (*i).mCoeff->norm();
    if (n > max)
      max = n;
    ++i;
  }
  return max;*/
//   return lc().norm();
//   return length();
}

const IMonom& IPolyList::lm() const {
  IASSERT(!IPolyList::isZero());
  return *(*mHead.begin()).mMonom;
}

const ICoeff& IPolyList::lc() const {
  IASSERT(!IPolyList::isZero());
  return *(*mHead.begin()).mCoeff;
}

void IPolyList::minus() {
  ConstIt i(mHead.begin());
  while(i) {
    (*i).mCoeff->minus();
    ++i;
  }
  IASSERTVALID(*this);
}

void IPolyList::add(const IPoly &a) {
  IASSERT(polyInterface() == a.polyInterface());
  It i1(mHead.begin());
  ConstIt i2(((IPolyList&)a).mHead.begin());
  while(i1 && i2)
    switch((*i1).mMonom->compare(*(*i2).mMonom)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mCoeff = (*i2).mCoeff->copy();
      (*i1).mMonom = (*i2).mMonom->copy();
      ++i2;
      ++i1;
      break;
    default:
      (*i1).mCoeff->add(*(*i2).mCoeff);
      if ((*i1).mCoeff->isZero())
        i1.del();
      else
        ++i1;
      ++i2;
    }

  while(i2) {
    i1.insert();
    (*i1).mCoeff = (*i2).mCoeff->copy();
    (*i1).mMonom = (*i2).mMonom->copy();
    ++i2;
    ++i1;
  }
  IASSERTVALID(*this);
}

void IPolyList::addTerm(ICoeff *c, IMonom *m) {
  It i(mHead.begin());
  while(i) {
    switch((*i).mMonom->compare(*m)) {
    case 1:
      ++i;
      break;
    case -1:
      i.insert();
      (*i).mMonom = m;
      (*i).mCoeff = c;
      return;
    default:
      (*i).mCoeff->add(*c);
      if ((*i).mCoeff->isZero())
        i.del();

      return;
    }
  }

  i.insert();
  (*i).mMonom = m;
  (*i).mCoeff = c;

  IASSERTVALID(*this);
}

void IPolyList::sub(const IPoly &a) {
  IASSERT(polyInterface() == a.polyInterface());
  It i1(mHead.begin()),
     i2(((IPolyList&)a).mHead.begin());
  while(i1 && i2)
    switch((*i1).mMonom->compare(*(*i2).mMonom)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mCoeff = (*i2).mCoeff->copy();
      (*i1).mMonom = (*i2).mMonom->copy();
      (*i1).mCoeff->minus();
      ++i2;
      ++i1;
      break;
    default:
      (*i1).mCoeff->sub(*(*i2).mCoeff);
      if ((*i1).mCoeff->isZero())
        i1.del();
      else
        ++i1;
      ++i2;
    }

  while(i2) {
    i1.insert();
    (*i1).mCoeff = (*i2).mCoeff->copy();
    (*i1).mMonom = (*i2).mMonom->copy();
    (*i1).mCoeff->minus();
    ++i2;
    ++i1;
  }
  IASSERTVALID(*this);
}

void IPolyList::mult(int var) {
  ConstIt i(mHead.begin());
  while(i) {
    (*i).mMonom->prolong(var);
    ++i;
  }
  IASSERTVALID(*this);
}

void IPolyList::mult(int var, unsigned deg) {
  ConstIt i(mHead.begin());
  while(i) {
    (*i).mMonom->prolong(var, deg);
    ++i;
  }
  IASSERTVALID(*this);
}

void IPolyList::mult(const IMonom& m) {
  ConstIt i(mHead.begin());
  while(i) {
    (*i).mMonom->mult(m);
    ++i;
  }
  IASSERTVALID(*this);
}

void IPolyList::mult(const ICoeff& c) {
  if (c.isZero())
    IPolyList::setZero();
  else {
    ConstIt i(mHead.begin());
    while(i) {
      (*i).mCoeff->mult(c);
      ++i;
    }
  }
  IASSERTVALID(*this);
}

void IPolyList::mult(const IPoly &a) {
  IASSERT(polyInterface() == a.polyInterface());

  IPolyList p((IPolyListInterface*)polyInterface());
  ConstIt ia(((IPolyList&)a).mHead.begin());
  It begin(p.mHead.begin());

  while(ia) {
    ConstIt i(mHead.begin());
    It ip = begin;

    // look for the starting point
    while(ip && i && (*ip).mMonom->compare(*(*i).mMonom, *(*ia).mMonom) == 1)
      ++ip;

    begin = ip;

    while(ip && i)
      switch((*ip).mMonom->compare(*(*i).mMonom, *(*ia).mMonom)) {
      case 1:
        ++ip;
        break;
      case -1:
        ip.insert();
        (*ip).mMonom = monomInterface()->copy(*(*i).mMonom);
        (*ip).mMonom->mult(*(*ia).mMonom);
		(*ip).mCoeff = coeffInterface()->copy(*(*i).mCoeff);
		(*ip).mCoeff->mult(*(*ia).mCoeff);
        if ((*ip).mCoeff->isZero())
          ip.del();
        else
          ++ip;
        ++i;
        break;
      default:
	    (*ip).mCoeff->addmul(*(*i).mCoeff, *(*ia).mCoeff);
        if ((*ip).mCoeff->isZero())
          ip.del();
        else
          ++ip;
        ++i;
      }

    while(i) {
      ip.insert();
      (*ip).mMonom = monomInterface()->copy(*(*i).mMonom);
      (*ip).mMonom->mult(*(*ia).mMonom);
      (*ip).mCoeff = coeffInterface()->copy(*(*i).mCoeff);
      (*ip).mCoeff->mult(*(*ia).mCoeff);
      if ((*ip).mCoeff->isZero())
        ip.del();
      else
        ++ip;
      ++i;
    }
    
    ++ia;
  }

  swap(p);
  IASSERTVALID(*this);
}

void IPolyList::pow(unsigned deg) {
  IASSERT(deg > 0);
  if (deg > 1) {
    IPolyList *p = new IPolyList(*this);
    while(--deg > 0) {
      p->mult(*this);
    }

    swap(*p);
    delete p;
  }
  IASSERTVALID(*this);
}

void IPolyList::div(const ICoeff& c) {
  IASSERT(!c.isZero());
  if (!c.isOne()) {
    coeffInterface()->addDenom(c);
    ConstIt i(mHead.begin());
    while(i) {
      (*i).mCoeff->div(c);
      ++i;
    }
  }
  IASSERTVALID(*this);
}

void IPolyList::content(ICoeff& c) const {
  if (IPolyList::isZero())
    c.setOne();
  else if (coeffInterface()->isField())
    c.set(IPolyList::lc());
/*  else if (coeffInterface()->type() == ICoeffInterface::GmpZ) {
    c.set(IPolyList::lc());
    bool fail;
    do {
      fail = false;
      ConstIt i(mHead.begin());
      ++i;
      ICoeff *sum = coeffInterface()->create();
      while(i) {
        sum->addmul(*(*i).mCoeff, iRandom(1013));
        ++i;
      }
      c.gcd(*sum);
      i = mHead.begin();
      ++i;
      while(i) {
        if (!(*i).mCoeff->divisibility(c)) {
          fail = true;
          break;
        }
        ++i;
      }
    } while(fail);
  }*/
  else {
    c.set(IPolyList::lc());
    ConstIt i(mHead.begin());
    ++i;
    while(i && !c.isOne()) {
      c.gcd(*(*i).mCoeff);
      ++i;
    }
  }
}

#include <cstdlib>

void IPolyList::pp() {
  if (IPolyList::isZero())
    return;
  if (coeffInterface()->type() == ICoeffInterface::GmpZZ)
    ;
  else if (coeffInterface()->isField()) {
    ICoeff *c(coeffInterface()->copy(IPolyList::lc()));
    if (!c->isOne()) {
      c->inverse();
      mult(*c);
    }
    delete c;
    IASSERT(isPp());
    IASSERTVALID(*this);
  }
/*  else if (coeffInterface()->type() == ICoeffInterface::GmpZ) {
    ICoeff *c(coeffInterface()->copy(IPolyList::lc()));
    bool fail;
    int k=0;
    do {
      fail = false;
      ConstIt i(mHead.begin());
      ++i;
      ICoeff *sum = coeffInterface()->create();
      while(i) {
        sum->addmul(*(*i).mCoeff, rand());
        ++i;
      }
      c->gcd(*sum);
      if (c->isOne())
        break;
      i = mHead.begin();
      ++i;
      while(i) {
        if (k > 5)
          std::cerr << k << " ";
        if (!(*i).mCoeff->divisibility(*c)) {
          fail = true;
          break;
        }
        ++i;
      }
      ++k;
    } while(fail);

    if (!c->isOne())
      div(*c);

    IASSERT(isPp());
    IASSERTVALID(*this);
  }*/
/*  else if (false && coeffInterface()->type() == ICoeffInterface::GmpZ) {
    ICoeff *c(coeffInterface()->copy(IPolyList::lc()));
    bool fail;
    int k=0;
    do {
      fail = false;
      ConstIt i(mHead.begin());
      ++i;
      ICoeff *sum = coeffInterface()->create();
      while(i) {
        sum->addmul(*(*i).mCoeff, rand());
        ++i;
      }
      c->gcd(*sum);
      delete sum;
      if (c->isOne())
        break;
      i = mHead.begin();
      ++i;
      if (k > 5)
        std::cerr << k << " ";
      while(i) {
        if (!(*i).mCoeff->divisibility(*c)) {
          fail = true;
          break;
        }
        ++i;
      }
      ++k;
    } while(fail);

    if (!c->isOne())
      div(*c);

    delete c;

    IASSERT(isPp());
    IASSERTVALID(*this);
  }*/
  else {
    ICoeff *c(coeffInterface()->copy(IPolyList::lc()));
    ConstIt i(mHead.begin());
    ++i;
    while(i && !c->isOne()) {
      c->gcd(*(*i).mCoeff);
      ++i;
    }
    if (!c->isOne())
      div(*c);

    IASSERT(isPp());
    IASSERTVALID(*this);
  }
/*  else {
    ConstIt i(mHead.begin());
    ICoeff *c1 = (*i).mCoeff;
    ++i;
    while(i) {
      c1 = (*i).mCoeff;
      ++i;
    }
    if (!c1->isOne()) {
      ICoeff *c(coeffInterface()->copy(*c1));
      i = mHead.begin();
      while(i && !c->isOne()) {
        c->gcd(*(*i).mCoeff);
        ++i;
      }
      if (!c->isOne())
        div(*c);

      IASSERT(isPp());
      IASSERTVALID(*this);
    }
  }*/
}

bool IPolyList::isPp() const {
  IASSERT(!IPolyList::isZero());
  bool r = true;
  if (coeffInterface()->type() == ICoeffInterface::GmpZZ)
    ;
  else if (coeffInterface()->isField())
    r = lc().isOne();
  else {
    ICoeff *c(coeffInterface()->copy(IPolyList::lc()));
    ConstIt i(mHead.begin());
    ++i;
    if (i) {
      while(i && !c->isOne()) {
        c->gcd(*(*i).mCoeff);
        ++i;
      }
      r = c->isOne();
      delete c;
    }
  }
  return r;
}

void IPolyList::reduction(ICoeff& multiplier, const IPoly & a) {
  IASSERT(polyInterface() == a.polyInterface());
  if (coeffInterface()->isField()) {
    IASSERT(IPolyList::lm().divisibility(((const IPolyList&)a).IPolyList::lm()));
    IASSERT(((const IPolyList&)a).IPolyList::lc().isOne());

    It i1(mHead.begin());
    ConstIt i2(((const IPolyList&)a).mHead.begin());

    IMonom *m2(monomInterface()->create());
    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    ICoeff *k2(coeffInterface()->copy(*(*i1).mCoeff));
    k2->minus();

    i1.del();
    ++i2;
    while(i1 && i2)
      switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
        (*i1).mMonom->mult(*m2);
        (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
        (*i1).mCoeff->mult(*k2);
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff->addmul(*(*i2).mCoeff, *k2);
        if ((*i1).mCoeff->isZero())
          i1.del();
        else
          ++i1;
        ++i2;
      }
    while(i2) {
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      (*i1).mCoeff->mult(*k2);
      ++i1;
      ++i2;
    }
    delete m2;
    delete k2;
  }
  else if (coeffInterface()->type() == ICoeffInterface::GmpZZ) {
    IASSERT(IPolyList::lm().divisibility(((const IPolyList&)a).IPolyList::lm()));

    It i1(mHead.begin());
    ConstIt i2(((const IPolyList&)a).mHead.begin());
    IASSERT((*i1).mCoeff->cmpNorm(*(*i2).mCoeff) >= 0);

    IMonom *m2(monomInterface()->create());
    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    ICoeff *k2(coeffInterface()->create()),
           *tmp(coeffInterface()->create());
    k2->quotRem(*(*i1).mCoeff, *(*i2).mCoeff, *tmp);
    k2->minus();
    if (tmp->isZero()) {
      i1.del();
    }
    else {
      *(*i1).mCoeff = *tmp;
      ++i1;
    }
    ++i2;
    while(i1 && i2)
      switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
        (*i1).mMonom->mult(*m2);
        (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
        (*i1).mCoeff->mult(*k2);
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff->addmul(*(*i2).mCoeff, *k2);
        if ((*i1).mCoeff->isZero())
          i1.del();
        else
          ++i1;
        ++i2;
      }
    while(i2) {
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      (*i1).mCoeff->mult(*k2);
      ++i1;
      ++i2;
    }
    delete m2;
    delete k2;
    delete tmp;
  }
  else {
    IASSERT(IPolyList::lm().divisibility(((const IPolyList&)a).IPolyList::lm()));

    It i1(mHead.begin());
    ConstIt i2(((const IPolyList&)a).mHead.begin());

    IMonom *m2(monomInterface()->create());
    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    ICoeff *k1(coeffInterface()->create()),
           *k2(coeffInterface()->create()),
           *tmp(coeffInterface()->create());
    tmp->gcd(*(*i1).mCoeff, *(*i2).mCoeff);
    k1->div(*(*i2).mCoeff, *tmp);
    k2->div(*(*i1).mCoeff, *tmp);
    k2->minus();

    i1.del();
    ++i2;
    while(i1 && i2)
      switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
      case 1:
        (*i1).mCoeff->mult(*k1);
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
        (*i1).mMonom->mult(*m2);
        (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
        (*i1).mCoeff->mult(*k2);
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff->mult(*k1);
        (*i1).mCoeff->addmul(*(*i2).mCoeff, *k2);
        if ((*i1).mCoeff->isZero())
          i1.del();
        else
          ++i1;
        ++i2;
      }
    while(i1) {
      (*i1).mCoeff->mult(*k1);
      ++i1;
    }
    while(i2) {
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      (*i1).mCoeff->mult(*k2);
      ++i1;
      ++i2;
    }
    multiplier.mult(*k1);
    delete m2;
    delete k1;
    delete k2;
    delete tmp;
  }
  IASSERTVALID(*this);
}

void IPolyList::redTail(ICoeff& multiplier, IPoly::Iterator& i, const IPoly& a) {
  IASSERT(polyInterface() == a.polyInterface());
  It i1(((IPolyList::Iterator&)i).mIt);
  if (coeffInterface()->isField()) {
    IASSERT((*i1).mMonom->divisibility(((const IPolyList&)a).IPolyList::lm()));
    IASSERT(((const IPolyList&)a).IPolyList::lc().isOne());

    ConstIt i2(((const IPolyList&)a).mHead.begin());

    IMonom *m2(monomInterface()->create());
    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    ICoeff *k2(coeffInterface()->copy(*(*i1).mCoeff));
    k2->minus();

    i1.del();
    ++i2;
    while(i1 && i2)
      switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
        (*i1).mMonom->mult(*m2);
        (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
        (*i1).mCoeff->mult(*k2);
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff->addmul(*(*i2).mCoeff, *k2);
        if ((*i1).mCoeff->isZero())
          i1.del();
        else
          ++i1;
        ++i2;
      }
    while(i2) {
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      (*i1).mCoeff->mult(*k2);
      ++i1;
      ++i2;
    }
    delete m2;
    delete k2;
  }
  else if (coeffInterface()->type() == ICoeffInterface::GmpZZ) {
    IASSERT((*i1).mMonom->divisibility(((const IPolyList&)a).IPolyList::lm()));

    It i(mHead.begin());
    ConstIt i2(((const IPolyList&)a).mHead.begin());
    IASSERT((*i1).mCoeff->cmpNorm(*(*i2).mCoeff) >= 0);

    IMonom *m2(monomInterface()->create());
    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    ICoeff *k2(coeffInterface()->create()),
           *tmp(coeffInterface()->create());
    k2->quotRem(*(*i1).mCoeff, *(*i2).mCoeff, *tmp);
    k2->minus();
    if (tmp->isZero()) {
      i1.del();
    }
    else {
      *(*i1).mCoeff = *tmp;
      ++i1;
    }
    ++i2;
    while(i1 && i2)
      switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
      case 1:
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
        (*i1).mMonom->mult(*m2);
        (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
        (*i1).mCoeff->mult(*k2);
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff->addmul(*(*i2).mCoeff, *k2);
        if ((*i1).mCoeff->isZero())
          i1.del();
        else
          ++i1;
        ++i2;
      }
    while(i1) {
      ++i1;
    }
    while(i2) {
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      (*i1).mCoeff->mult(*k2);
      ++i1;
      ++i2;
    }
    delete m2;
    delete k2;
    delete tmp;
  }
  else {
    IASSERT((*i1).mMonom->divisibility(((const IPolyList&)a).IPolyList::lm()));

    It i(mHead.begin());
    ConstIt i2(((const IPolyList&)a).mHead.begin());

    IMonom *m2(monomInterface()->create());
    m2->divide(*(*i1).mMonom, *(*i2).mMonom);
    ICoeff *k1(coeffInterface()->create()),
           *k2(coeffInterface()->create()),
           *tmp(coeffInterface()->create());
    tmp->gcd(*(*i1).mCoeff, *(*i2).mCoeff);
    k1->div(*(*i2).mCoeff, *tmp);
    k2->div(*(*i1).mCoeff, *tmp);
    k2->minus();

    while(i1 != i) {
      (*i).mCoeff->mult(*k1);
      ++i;
    }

    i1.del();
    ++i2;
    while(i1 && i2)
      switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
      case 1:
        (*i1).mCoeff->mult(*k1);
        ++i1;
        break;
      case -1:
        i1.insert();
        (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
        (*i1).mMonom->mult(*m2);
        (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
        (*i1).mCoeff->mult(*k2);
        ++i1;
        ++i2;
        break;
      default:
        (*i1).mCoeff->mult(*k1);
        (*i1).mCoeff->addmul(*(*i2).mCoeff, *k2);
        if ((*i1).mCoeff->isZero())
          i1.del();
        else
          ++i1;
        ++i2;
      }
    while(i1) {
      (*i1).mCoeff->mult(*k1);
      ++i1;
    }
    while(i2) {
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      (*i1).mCoeff->mult(*k2);
      ++i1;
      ++i2;
    }
    multiplier.mult(*k1);
    delete m2;
    delete k1;
    delete k2;
    delete tmp;
  }
  IASSERTVALID(*this);
}

void IPolyList::redTailCanonical(IPoly::Iterator& i, const IPoly& a) {
  It i1(((IPolyList::Iterator&)i).mIt);
  IASSERT((*i1).mMonom->divisibility(((const IPolyList&)a).IPolyList::lm()));
  IASSERT(coeffInterface()->type() & ICoeffInterface::Ring);
  //IASSERT((*i1)->coeff().isNegative());
  IASSERT(!a.lc().isNegative());

  ConstIt i2(((const IPolyList&)a).mHead.begin());
  IMonom *m2(monomInterface()->create());
  m2->divide(*(*i1).mMonom, *(*i2).mMonom);

  while(i1 && i2)
    switch((*i1).mMonom->compare(*(*i2).mMonom, *m2)) {
    case 1:
      ++i1;
      break;
    case -1:
      i1.insert();
      (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
      (*i1).mMonom->mult(*m2);
      (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
      ++i1;
      ++i2;
      break;
    default:
      (*i1).mCoeff->add(*(*i2).mCoeff);
      if ((*i1).mCoeff->isZero())
        i1.del();
      else
        ++i1;
      ++i2;
    }
  while(i1) {
    ++i1;
  }
  while(i2) {
    i1.insert();
    (*i1).mMonom = monomInterface()->copy(*(*i2).mMonom);
    (*i1).mMonom->mult(*m2);
    (*i1).mCoeff = coeffInterface()->copy(*(*i2).mCoeff);
    ++i1;
    ++i2;
  }
  delete m2;
  IASSERTVALID(*this);
}

IPoly::ConstIterator* IPolyList::begin() const {
  return new ConstIterator(mHead.begin());
}

IPoly::ConstIterator* IPolyList::beginReverse() const {
  IERROR("beginReverse");
  return NULL;
}

IPoly::Iterator* IPolyList::begin() {
  return new Iterator(mHead.begin());
}

IPoly::Iterator* IPolyList::beginReverse() {
  IERROR("beginReverse");
  return NULL;
}

void IPolyList::read(std::istream &in) {
  mHead.clear();
  int l;
  in.read((char*)(&l), sizeof(l));
  It i(mHead.begin());
  while(l) {
    i.insert();
    (*i).mMonom = monomInterface()->create();
    (*i).mMonom->read(in);
    (*i).mCoeff = coeffInterface()->create();
    (*i).mCoeff->read(in);
    ++i;
    --l;
  }
  IASSERTVALID(*this);
}

void IPolyList::write(std::ostream &out) const {
  int l = mHead.length();
  out.write((char*)(&l), sizeof(l));
  ConstIt i(mHead.begin());
  while(i) {
    (*i).mMonom->write(out);
    (*i).mCoeff->write(out);
    ++i;
  }
}

void IPolyList::assertValid(const char* fileName, int fileLine) const {
  if (!mHead.isEmpty()) {
    ConstIt i(mHead.begin());
    IASSERT2(fileName, fileLine, (*i).mMonom);
    IASSERT2(fileName, fileLine, (*i).mCoeff);
    ConstIt prev(i);
    ++i;
    while(i) {
      IASSERT2(fileName, fileLine, (*i).mMonom);
      IASSERT2(fileName, fileLine, (*i).mCoeff);
      IASSERT2(fileName, fileLine, !(*i).mCoeff->isZero());
/*      if ((*prev).mMonom->compare(*(*i).mMonom) <= 0)
        std::cerr << (*(*prev).mMonom) << " > " << (*(*i).mMonom)
          << " = " << (*prev).mMonom->compare(*(*i).mMonom) << std::endl;*/
      IASSERT2(fileName, fileLine, (*prev).mMonom->compare(*(*i).mMonom) > 0);
      prev = i;
      ++i;
    }
  }
}

template <>
IAllocator IList<IPolyList::Term>::Node::sAllocator(sizeof(IList<IPolyList::Term>::Node));
