/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IPOLYLIST_H
#define IPOLYLIST_H

#include "../util/ilist.h"
#include "./ipoly.h"


class IPolyList;
class IPolyListInterface: public IPolyInterface {
  friend class IPolyList;

public:
  IPolyListInterface();
  ~IPolyListInterface() {}

  IPoly* create();
  IPoly* copy(const IPoly& a);
  IPoly* copy(const IPoly& a, int var);
  IPoly* copy(const IPoly& a, int var, unsigned deg);
};

class IPolyList: public IPoly {
  friend class IPolyListInterface;

public:
  struct Term {
    IMonom    *mMonom;
    ICoeff    *mCoeff;

    Term(): mMonom(NULL), mCoeff(NULL) {}
    ~Term() {
      delete mMonom;
      delete mCoeff;
    }
  };

protected:

  typedef IList<Term>::Iterator      It;
  typedef IList<Term>::ConstIterator ConstIt;

  IList<Term>  mHead;

  IPolyList(IPolyListInterface* r);
  IPolyList(const IPolyList& a);
  IPolyList(const IPolyList& a, int var);
  IPolyList(const IPolyList& a, int var, unsigned deg);
  IPolyList(const IPolyList& a, const IMonom& m, const ICoeff& c);

public:
  class ConstIterator: public IPoly::ConstIterator {
    friend class IPolyList;

  protected:
    IPolyList::ConstIt mConstIt;

    ConstIterator(const IPolyList::ConstIt& constIt):
        IPoly::ConstIterator(),
        mConstIt(constIt) {}

  public:
    ~ConstIterator() {}

    bool isPointTo() const { return mConstIt; }
    const ICoeff& coeff() const { return *(*mConstIt).mCoeff; }
    const IMonom& monom() const { return *(*mConstIt).mMonom; }
    void next() { ++mConstIt; }
    void prev() { IERROR("prev"); }
    bool equal(const IPoly::ConstIterator& a) const {
      return mConstIt == ((const ConstIterator&)a).mConstIt;
    }
  };

  class Iterator: IPoly::Iterator {
    friend class IPolyList;

  protected:
    IPolyList::It mIt;
    Iterator(const IPolyList::It& it):
        IPoly::Iterator(),
        mIt(it) {}

  public:
    ~Iterator() {}

    bool isPointTo() const { return mIt; }
    const ICoeff& coeff() const { return *(*mIt).mCoeff; }
    const IMonom& monom() const { return *(*mIt).mMonom; }
    void next() { ++mIt; }
    void prev() { IERROR("prev"); }
    bool equal(const IPoly::Iterator& a) const {
      return mIt == ((const Iterator&)a).mIt;
    }
  };

  ~IPolyList() {}

  void setZero();
  void setOne();
  void setDependVar(int var);
  void setMonom(IMonom *m);
  void setCoeff(ICoeff *c);
  void setPoly(const IPoly& a, int dependVar);
  void set(const IPoly& a);
  void swap(IPoly& a);

  bool isZero() const;
  int length() const;
  int degree() const;
  int norm() const;

  const IMonom& lm() const;
  const ICoeff& lc() const;

  void minus();
  void add(const IPoly &a);
  void addTerm(ICoeff *c, IMonom *m);
  void sub(const IPoly &a);
  void mult(int var);
  void mult(int var, unsigned deg);
  void mult(const IMonom& m);
  void mult(const ICoeff& c);
  void mult(const IPoly &a);
  void pow(unsigned deg);
  void div(const ICoeff& c);

  void content(ICoeff& c) const;
  void pp();
  bool isPp() const;

  void reduction(ICoeff& multiplier, const IPoly& a);
  void redTail(ICoeff& multiplier, IPoly::Iterator& i1, const IPoly& a);
  void redTailCanonical(IPoly::Iterator& i, const IPoly& a);

  IPoly::ConstIterator* begin() const;
  IPoly::ConstIterator* beginReverse() const;
  IPoly::Iterator* begin();
  IPoly::Iterator* beginReverse();

  void read(std::istream &in);
  void write(std::ostream &out) const;

  void assertValid(const char* fileName, int fileLine) const;
};

#endif // IPOLYLIST_H

