#include <string>
#include <sstream>
#include <iostream>

#include "algorithm/ialgorithm.h"
#include "coeff/icoeff.h"
#include "coeff/inparametergmpz.h"
#include "coeff/igmpq.h"
#include "criteria/iwrap.h"
#include "division/idivision.h"
#include "gauss/igauss.h"
#include "gauss/ipolygauss.h"
#include "poly/iexpression.h"
#include "poly/ioneparameterpoly.h"
#include "poly/ipoly.h"
#include "monom/imonom.h"
#include "system/isystemtype.h"
#include "util/itimer.h"

int instances = 0;
int max_instances = 0;

void iAssert(bool a, const char* file, int line) {
  if (!a) {
  	printf("assert from file: '%s'  line: %d\n", file, line);
  }
}

void iError(const char* msg, const char* file, int line) {
  printf("error %s from file: '%s'  line: %d\n", msg, file, line);
  exit(1);
}

void iMessage(const char* msg, const char* file, int line) {
  // "warning %s from file: '%s'  line: %d\n", msg, file, line
  //PyErr_WarnEx(PyExc_Warning, msg, 1);
//   PyErr_Warn(PyExc_Warning, msg);
}

ICoeff* getCoeff(IPolyInterface* ip, std::string str) {
	IExpression* exp = new IExpression(ip);
	std::istringstream ist(str);
	ist >> *exp;
	IPoly* p = exp->numer();
  if (p->isZero()) {
    ICoeff* c = p->coeffInterface()->create();
    c->setZero();
    return c;
  }
  else 
    return p->coeffInterface()->copy(p->lc());
}

IPoly* getPoly(IPolyInterface* ip, std::string str) {
	IExpression* exp = new IExpression(ip);
	std::istringstream ist(str);
	ist >> *exp;
	return exp->numer();
}

int main() {
	IVariables* coeffvars = new IVariables();
	coeffvars->add("c1");
	coeffvars->add("c2");
	
	IVariables* polyvars = new IVariables();
	polyvars->add("x");
//	polyvars->add("y");
//	polyvars->add("z");
	
	ISystemType* st = new ISystemType(ISystemType::Polynomial, 0, 0);
	IMonomInterface* im = IMonomInterface::create(IMonomInterface::DegRevLex, st, polyvars);
	ICoeffInterface* ic = ICoeffInterface::create(ICoeffInterface::GmpQ, st);
	IPolyInterface* ip = IPolyInterface::create(IPolyInterface::PolyList, st, im, ic);

  IPoly* p1 = getPoly(ip, "1");
  IPoly* p2 = getPoly(ip, "x+1");
  IPoly* p3 = getPoly(ip, "x");
  
  ICoeff* one = p1->coeffInterface()->copy(p1->lc());
	
  IPolyGauss alg(ic);

  bool lindep = false;

  std::cout << "adding p1 (" << *p1 << "): " << std::endl;
  lindep = alg.add(*one, *p1);
  std::cout << alg;
  std::cout << "p1 is ";
  if (lindep)
    std::cout << "linearly dependent";
  else
    std::cout << "linearly independent";
  std::cout << std::endl;

  std::cout << "adding p2 (" << *p2 << "): " << std::endl;
  lindep = alg.add(*one, *p2);
  std::cout << alg;
  std::cout << "p2 is ";
  if (lindep)
    std::cout << "linearly dependent";
  else
    std::cout << "linearly independent";
  std::cout << std::endl;

  std::cout << "adding p3 (" << *p3 << "): " << std::endl;
  lindep = alg.add(*one, *p3);
  std::cout << alg;
  std::cout << "p3 is ";
  if (lindep)
    std::cout << "linearly dependent";
  else
    std::cout << "linearly independent";
  std::cout << std::endl;

  IOneParameterPoly mp(p1->coeffInterface());
  alg.getMinPoly(mp);
  std::cout << mp << std::endl;
}
