/***************************************************************************
 *   Copyright (C) 2007 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "igcdzippeltest.h"

#include "../coeff/igcdbrown.h"
#include "../util/itimer.h"

#include <cppunit/extensions/HelperMacros.h>

CPPUNIT_TEST_SUITE_REGISTRATION(IGcdZippelTest);


void IGcdZippelTest::setUp() {
  IGinvTest::setUp();

  IVariables* deg1vars = new IVariables();
  deg1vars->add("x");
  icdeg1 = ICoeffInterface::create(ICoeffInterface::NParameterGmpZ, st, deg1vars);
  ipdeg1 = IPolyInterface::create(IPolyInterface::PolyList, st, im, icdeg1);

  IVariables* deg2vars = new IVariables();
  deg2vars->add("x");
  deg2vars->add("y");
  icdeg2 = ICoeffInterface::create(ICoeffInterface::NParameterGmpZ, st, deg2vars);
  ipdeg2 = IPolyInterface::create(IPolyInterface::PolyList, st, im, icdeg2);

  IVariables* deg3vars = new IVariables();
  deg3vars->add("x");
  deg3vars->add("y");
  deg3vars->add("z");
  icdeg3 = ICoeffInterface::create(ICoeffInterface::NParameterGmpZ, st, deg3vars);
  ipdeg3 = IPolyInterface::create(IPolyInterface::PolyList, st, im, icdeg3);
}

void IGcdZippelTest::tearDown() {
  IGinvTest::tearDown();
}

void IGcdZippelTest::testZippelPDeg1() {
  ModularShort p = 3119;
  
  INParameterModularShortAux* a = getModularPoly("2237+1523*x+175*x^10+2288*x^9+388*x^7+61*x^6+1377*x^4+1720*x^3+565*x^5+2131*x^8+1694*x^2", p, 1);
  INParameterModularShortAux* b = getModularPoly("2646+20*x+3018*x^10+1181*x^9+709*x^7+1251*x^6+780*x^4+2950*x^3+1023*x^5+1584*x^8+2378*x^2", p, 1);
  INParameterModularShortAux* expected = getModularPoly("x^5+2938*x^4+2672*x^3+978*x^2+1246*x+1870", p, 1);
  INParameterModularShortAux* g = getModularPoly("0", p, 1);

  IMonomAuxInterface::MonomIterator d = a->interface().create();
  d[0] = 5;

  IGcdZippel algorithm;
  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelPDeg2() {
  ModularShort p = 3119;

  INParameterModularShortAux* a = getModularPoly("x^2*y+x^2+y+1", p, 2);
  INParameterModularShortAux* b = getModularPoly("x*y+x+y+1", p, 2);
  INParameterModularShortAux* g = getModularPoly("0", p, 2);

  IMonomAuxInterface::MonomIterator d = a->interface().create();
  d[0] = 0;

  IGcdZippel algorithm;
  algorithm.zippelP(*a, *b, *g, d);

  // content in y
  CPPUNIT_ASSERT(g->isZero());

  delete a;
  delete b;

  a = getModularPoly("313*x^2*y+1623*x^3*y^2+155*x^3*y^3+186*x^2*y^5+1198*x^5*y^3+1292*x^4*y^4+441*x^3*y^5+357*x^2*y^6+584*x^6*y+1818*x^8*y+1105*x^7*y^2+1832*x^6*y^3+1915*x^5*y^4+975*x^4*y^3+2573*x^4*y^5+1743*x^3*y^6+2632*y^7*x+375*y^7*x^2+2615*y^8*x+2090*y^6*x^4+1037*y^7*x^3+2859*y^8*x^2+2196*y^9*x+3070*x*y^4+2034*x*y^3+2865*x^4*y+733*x^2*y^3", p, 2);
  b = getModularPoly("352+1046*x+2815*x*y^2+1644*x^3*y^2+1494*x^9+1744*x^2*y^5+2435*x^2*y^6+625*x^7*y^2+115*x^5*y^4+456*x^4*y^3+225*x^3*y^6+252*y^8*x+2570*y^9*x+1541*x*y^5+1219*x^6*y^2+2859*x^4+1492*y^5+112*y^4+1398*y^3+2916*x^5+2259*x*y^4+1579*y^7+2456*y^6+236*y^8+2337*y+1594*y^5*x^5+1526*x*y^3+2208*x^4*y+1550*x^2*y^3+1465*x^2*y^2", p, 2);
  INParameterModularShortAux* expected = getModularPoly("x^4 + 1919*x^2*y^2 + 529*x*y^2 + 2928*y^5 + 1775*y^4 + 2014", p, 2);
  d[0] = 4;
  d[1] = 5;

  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());

  d[0] = 10;
  d[1] = 10;

  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());

  a = getModularPoly("(x*y + 1)*(x*y - 1)", p, 2);
  b = getModularPoly("(x*y + 1)*(x*y + 1)", p, 2);
  expected = getModularPoly("(x*y + 1)", p, 2);
  d[0] = 1;
  d[1] = 1;

  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelPDeg3() {
  ModularShort p = 3119;

  INParameterModularShortAux* a = getModularPoly("x*(x+y+z)", p, 3);
  INParameterModularShortAux* b = getModularPoly("y*(x+y+z)", p, 3);
  INParameterModularShortAux* g = getModularPoly("0", p, 3);
  INParameterModularShortAux* expected = getModularPoly("(x+y+z)", p, 3);

  IMonomAuxInterface::MonomIterator d = a->interface().create();
  d[0] = 1;
  d[1] = 1;
  d[2] = 1;

  IGcdZippel algorithm;
  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
  
  a = getModularPoly("3118*x*(2804*x*y+450*x*z+837*x*y^2*z+31*x^2*y+1738*x^2*z+987*x^2*y^2*z+2524*z^2*y+850*z^3+1581*y^2*z^3)", p, 3);
  b = getModularPoly("2*z*(261*x+207*x*z+2741*x^4+509*x^2+1049*x^2*z+661*x^5+493*z^2+391*z^3+2405*z^2*x^3)", p, 3);
  g = getModularPoly("0", p, 3);
  expected = getModularPoly("1490*z^2+x^2+2807*x", p, 3);

  d = a->interface().create();
  d[0] = 2;
  d[1] = 1;
  d[2] = 3;

  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
  
  a = getModularPoly("2706*x^7*y^4*z^3+54*x^6*y^3*z^3+581*x^8*y^2*z^6+1324*x^6*y^2*z^8+581*x^4*y^5*z^8+2944*x^11*y^4*z^3+2214*x^7*y^5*z^6+3114*x^4*y^3*z^6+235*x^6*y^2*z^9+455*x^4*y^2*z^11+235*x^2*y^5*z^11+305*x^9*y^4*z^6+2914*x^5*y^5*z^9+99*z^9*x^3*y+1585*z^14*x*y^3+199*z^9*x^8*y^2+940*z^12*x^4*y^3+3058*x^4*y^7*z^3+2867*x^6*y^6*z^6+2432*x^4*y^6*z^8+2867*x^2*y^9*z^8+602*x^9*y^8*z^3+618*x^5*y^9*z^6+2619*y^3*x^5*z^3+2350*y^3*x^3*z^5+2619*y^6*x*z^5+38*y^6*x^4*z^3+17*x^8*y*z^3+232*x^6*y*z^5+77*y^4*x^3+1541*y^5*x^8+66*x^6*y^2+2212*x^11*y^3+1585*z^12*x^5+348*z^14*x^3+17*x^4*y^4*z^5", p, 3);
  b = getModularPoly("1771*y^4*x*z^2+2889*y^7*x*z^3+2388*y^6*x^6*z+538*y^6*x^3*z^4+1518*x^4*y^2*z^2+694*x^4*y^5*z^3+1254*x^3*y^5*z^4+2938*x^9*y^4*z+2689*x^6*y^4*z^4+1594*x^4*y^4*z^3+1242*x^4*y^3*z^5+1702*x^4*y^6*z^6+1026*x^3*y^6*z^7+419*x^9*y^5*z^4+1633*x^6*y^5*z^7+430*x^2*y^4*z^6+3004*x^2*y^3*z^8+420*x^2*y^6*z^9+3024*x*y^6*z^10+250*x^7*y^5*z^7+2679*x^4*y^5*z^10+843*z^9*x*y^2+2277*z^11*x*y+1041*z^12*x*y^4+1288*z^10*x^6*y^3+2474*z^13*x^3*y^3+2127*x^2*y^8*z^3+1716*x^2*y^7*z^5+2005*x^2*y^10*z^6+1960*x*y^10*z^7+3050*x^7*y^9*z^4+870*x^4*y^9*z^7+2735*y^5*x+1463*y^7*z^4+562*x^4*y^3+1881*z^13*y^4", p, 3);
  g = getModularPoly("0", p, 3);
  expected = getModularPoly("x^3*y^2*z^3 + 1734*x^3*y^1*z^0 + 2367*x^1*y^6*z^3 + 2137*x^1*y^2*z^6 + 2023*x^0*y^3*z^0 + 2601*x^0*y^0*z^9", p, 3);

  d = a->interface().create();
  d[0] = 3;
  d[1] = 6;
  d[2] = 9;

  algorithm.zippelP(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelMDeg1() {
  INParameterGmpZAux* a = ((INParameterGmpZ*) getCoeff(ipdeg1, "1972+3338*x-4929*x^5-6396*x^4-7175*x^3-1650*x^10-2648*x^9+5080*x^8+10*x^6+9071*x^7-2103*x^2"))->mPoly;
  INParameterGmpZAux* b = ((INParameterGmpZ*) getCoeff(ipdeg1, "145+683*x-914*x^5-6768*x^4-2309*x^3-425*x^10-2926*x^9-2607*x^8+9593*x^6+6990*x^7+1058*x^2"))->mPoly;
  INParameterGmpZAux* g = ((INParameterGmpZ*) getCoeff(ipdeg1, "0"))->mPoly;
  INParameterGmpZAux* expected = ((INParameterGmpZ*) getCoeff(ipdeg1, "29+25*x^5+28*x^4-61*x^3-60*x^2+9*x"))->mPoly;

  IMonomAuxInterface::MonomIterator d = a->interface().create();
  d[0] = 5;

  IGcdZippel algorithm;
  algorithm.zippelM(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelMDeg2() {
  INParameterGmpZAux* a = ((INParameterGmpZ*) getCoeff(ipdeg2, " -3680*y^3-9016*y^5-4050*x^9-6975*x^8-6825*x^6+5278*x*y^3+6825*x^6*y^2-2106*x^6*y^3+750*x^7*y^3+390*x^4*y^6+5076*x^6*y+8742*x^5*y-8648*x^2*y^3+1911*x*y^4+12274*x^3*y-2989*x^3*y^3-400*x^2*y^4-8918*x*y^5-1134*x^4*y^2-3627*x^5*y^3-940*x^4*y^4+3549*x^3*y^5-980*y^6*x^2+3798*x^2*y^5+2160*x^4*y-1953*x^3*y^2+6900*x^5*y^2+5292*x^4*y^3-1911*x*y^2+1932*y^4+3640*x*y "))->mPoly;
  INParameterGmpZAux* b = ((INParameterGmpZ*) getCoeff(ipdeg2, "-1656*y^5-432*x^5+7901*x^4+3348*x^9+5766*x^8+14477*x^6+5130*x^7+1638*x*y^3-8018*x^6*y^2-620*x^7*y^3+440*x^4*y^5+6072*y^6-6006*x*y^4+728*x^2*y^2+1754*x^3*y^3+4048*x^2*y^4-1638*x*y^5-8645*x^4*y^2-950*x^5*y^3-3564*x^4*y^4-180*y^6*x^2-2134*x^3*y^4+6006*x*y^6+660*x^2*y^7-12744*x^3*y^2-9796*x^5*y^2+972*x^4*y^3+736*x*y^2-728*x^2"))->mPoly;
  INParameterGmpZAux* g = ((INParameterGmpZ*) getCoeff(ipdeg2, "0"))->mPoly;
  INParameterGmpZAux* expected = ((INParameterGmpZ*) getCoeff(ipdeg2, "-92*y^2-91*x*y^2+91*x-10*x^2*y^3+93*x^3+54*x^4"))->mPoly;

  IMonomAuxInterface::MonomIterator d = a->interface().create();
  d[0] = 5;
  d[1] = 4;

  IGcdZippel algorithm;
  algorithm.zippelM(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelMDeg3() {
  INParameterGmpZAux* a = ((INParameterGmpZ*) getCoeff(ipdeg3, "396*x^5+4556*x^8-6633*x^6-272*x^7-3870*y^3*x^2*z+5762*y^2*z^2+4950*x^4*z-2881*y*z^3+268*x^4*z^2-304*x^8*z-3824*x^7*z^2+3350*x^3*z^3-3400*x^6*z-300*x^6*z^3-3350*y^2*z^4-4489*x^5*z^2+5092*x^9*z+402*x^8*z^2-6536*y^2*x^4*z+2884*y^2*x^3*z^2+2924*y*z*x^3+1935*y^2*z^2*x^2+3268*y*z^2*x^4+258*y*z^3*x^3-180*x^6*y*z-2250*x^5*z^2*y+2250*y^3*z^3*x^2+3800*y^2*z^3*x^4+300*y^2*z^4*x^3+3015*x^7*y*z-4257*y*z*x-4950*y^2*z^2*x-5848*x^3*y^2+8514*x*y^2 "))->mPoly;
  INParameterGmpZAux* b = ((INParameterGmpZ*) getCoeff(ipdeg3, "4042*y^3*z*x^3+2752*y^4*z^3-2814*x^10-2914*x^9+184*x^8-1978*y*z*x^4-2924*y*z^3*x^2-1806*y*z*x^5-1376*y^3*z^4+2100*x^8*z-4556*x^7*z^2-2021*y^2*x^3*z^2+5848*y^2*z^2*x^2+128*y^2*z^3*x^4+1600*y^2*z^4*x^3+188*x^7*y*z-3182*y^2*z^5+1591*y*z^6+272*x^6*z^2-148*x^4*z^5+2300*x^7*z+3400*x^5*z^3-1850*x^3*z^6-1600*y^4*z^5+1850*y^2*z^7+2479*x^5*z^5+2350*x^6*z^2*y-2300*y^2*z^2*x^4-3400*y^2*z^4*x^2-2100*y^2*z^2*x^5-2350*y^3*z^3*x^3-3149*x^8*y*z-2144*x^5*y^2*z^3+3956*x^4*y^2+3612*x^5*y^2"))->mPoly;
  INParameterGmpZAux* g = ((INParameterGmpZ*) getCoeff(ipdeg3, "0"))->mPoly;
  INParameterGmpZAux* expected = ((INParameterGmpZ*) getCoeff(ipdeg3, "-86*y^2+43*y*z-4*x^4-50*x^3*z+50*y^2*z^2+67*x^5"))->mPoly;

  IMonomAuxInterface::MonomIterator d = a->interface().create();
  d[0] = 5;
  d[1] = 2;
  d[2] = 2;

  IGcdZippel algorithm;
  algorithm.zippelM(*a, *b, *g, d);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelDeg2() {
//  INParameterGmpZAux* a = ((INParameterGmpZ*) getCoeff(ipdeg2, " -3680*y^3-9016*y^5-4050*x^9-6975*x^8-6825*x^6+5278*x*y^3+6825*x^6*y^2-2106*x^6*y^3+750*x^7*y^3+390*x^4*y^6+5076*x^6*y+8742*x^5*y-8648*x^2*y^3+1911*x*y^4+12274*x^3*y-2989*x^3*y^3-400*x^2*y^4-8918*x*y^5-1134*x^4*y^2-3627*x^5*y^3-940*x^4*y^4+3549*x^3*y^5-980*y^6*x^2+3798*x^2*y^5+2160*x^4*y-1953*x^3*y^2+6900*x^5*y^2+5292*x^4*y^3-1911*x*y^2+1932*y^4+3640*x*y "))->mPoly;
//  INParameterGmpZAux* b = ((INParameterGmpZ*) getCoeff(ipdeg2, "-1656*y^5-432*x^5+7901*x^4+3348*x^9+5766*x^8+14477*x^6+5130*x^7+1638*x*y^3-8018*x^6*y^2-620*x^7*y^3+440*x^4*y^5+6072*y^6-6006*x*y^4+728*x^2*y^2+1754*x^3*y^3+4048*x^2*y^4-1638*x*y^5-8645*x^4*y^2-950*x^5*y^3-3564*x^4*y^4-180*y^6*x^2-2134*x^3*y^4+6006*x*y^6+660*x^2*y^7-12744*x^3*y^2-9796*x^5*y^2+972*x^4*y^3+736*x*y^2-728*x^2"))->mPoly;
  INParameterGmpZAux* a = ((INParameterGmpZ*) getCoeff(ipdeg2, "(x^2 + 2*y + x*y)*(x^3 + 17)*(y + 1)"))->mPoly;
  INParameterGmpZAux* b = ((INParameterGmpZ*) getCoeff(ipdeg2, "(x^2 + 2*y + x*y)*(x^3 + 18)*(y + 1)"))->mPoly;
  INParameterGmpZAux* g = ((INParameterGmpZ*) getCoeff(ipdeg2, "0"))->mPoly;
  INParameterGmpZAux* expected = ((INParameterGmpZ*) getCoeff(ipdeg2, "(x^2 + 2*y + x*y)*(y + 1)"))->mPoly;

  IGcdZippel algorithm;
  algorithm.gcd(*a, *b, *g);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}

void IGcdZippelTest::testZippelDeg3() {
  INParameterGmpZAux* a = ((INParameterGmpZ*) getCoeff(ipdeg3, "(79*y+56*x^2*z^2+49*x*y^2*z+63*y^2*z^2+57*x^3*y^2-59*x^2*y^3)*(77*y*z^2+66*x^3*y+54*x*y*z^2-5*x*z^3+99*y^2*z^2-61*x*z^4)*(-85*y^3-86*y^2*z+30*y^3*z+80*z^4+72*y^2*z^3+66*y*z^4)"))->mPoly;
  INParameterGmpZAux* b = ((INParameterGmpZ*) getCoeff(ipdeg3, "(79*y+56*x^2*z^2+49*x*y^2*z+63*y^2*z^2+57*x^3*y^2-59*x^2*y^3)*(x*z-47*x*y*z-91*x*z^3-47*y*z^3-61*x^4*z+41*x*y^3*z)*(-85*y^3-86*y^2*z+30*y^3*z+80*z^4+72*y^2*z^3+66*y*z^4)"))->mPoly;
  INParameterGmpZAux* g = ((INParameterGmpZ*) getCoeff(ipdeg3, "0"))->mPoly;
  INParameterGmpZAux* expected = ((INParameterGmpZ*) getCoeff(ipdeg3, "(79*y+56*x^2*z^2+49*x*y^2*z+63*y^2*z^2+57*x^3*y^2-59*x^2*y^3)*(-85*y^3-86*y^2*z+30*y^3*z+80*z^4+72*y^2*z^3+66*y*z^4)"))->mPoly;

  IGcdZippel algorithm;
  algorithm.gcd(*a, *b, *g);

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());

  delete a;
  delete b;
  delete g;
  delete expected;

  a = ((INParameterGmpZ*) getCoeff(ipdeg3, "(25*x^21*y^145*z^9+18*y^50*z^111-15*x^79*y^26*z^52+55*x^124*y^5*z^4-34*y^42*z^42-17*x^19*y^12*z^23-40*x^34*y^44*z^36-39*x^24*y^86*z^86+44*x^17*y^134*z^7+65*x^85*y^36*z^19-80*x^105*y^67-57*x^7*y^47*z^126+63*x^135*y^24*z^13-97*x^57*y^17*z^98-94*x^162*y^23*z^13+5*x^46*y^21*z^95+99*x^8*y^74*z^70+32*x^83*y^21*z^58+49*x^57*y^48*z^48+63*x^30*y^10*z^154-65*x^24*y^16*z^103+24*x^75*y^17*z^45-99*x^162*y^17*z^2+86*x^15*y^25*z^144+64*x^69*y^4*z^8+60*x^53*y^9*z^27+99*x^15*y^5*z^72+48*x^47*y^7*z^49-48*x^10*y^155*z^32+8*x^93*y^60*z^37+52*x^11*y^103*z^32+10*x^73*y^71*z^54-20*x^58*y^90*z^51-39*x^89*y^20*z^44-98*x^137*y^24+8*x^21*y^50*z^97+73*x^73*y^35+6*x^17*y^82*z^42+36*x^88*y^6*z^11+49*x^3*y^13*z^106+70*x^49*y^69*z^82-84*x^60*y^65*z^60-76*x^9*y^37*z^109+27*x^6*y^152*z^3-53*x^15*y^40*z^125-10*x^106*y^83*z^2+90*x^121*y^30*z^30-76*x^58*y^9*z^112+36*x^4*y^147*z^10)*(-5*x^72*y^28*z^81-15*x^38*y^80*z^44-80*x^13*y^81*z^83+51*x^80*y^17*z^65+21*x^107*y^58*z^19+23*x^118*y^9*z^21-79*x^64*y^24*z^34-4*x^79*y^21*z^45+63*x^117*z^18+47*x^32*y^41*z^117+93*x^5*y^43*z^99-49*x^9*y^13*z^153+9*x^18*y^29*z^146-21*x^15*y^37*z^128-55*x^77*y^81*z^38-5*x^84*y^70*z^15+83*x^36*y^3*z^113+6*x^12*y^29*z^23-95*x^16*y^41*z^128+53*x^26*y^30*z^74-50*x^74*y^6*z^21-21*x^55*y^4*z^87-8*x^97*y^13*z^54+26*x^67*y^64*z^63+51*x^98*y^14*z^78-98*x^36*y^24*z^108+33*x^73*y^59*z^60-47*x^22*y^46*z^33+53*x^31*y^33*z^103+20*x^21*y^5*z^144-16*x^20*y^50*z^78-72*x^39*y^24*z^111-17*x^86*y^2*z^73+84*x^46*y^87*z^35+10*x^108*y^24*z^43+67*x^69*y^11*z^38-24*x^23*y^100*z^18+75*x^95*y^13*z^5-75*x^26*y^6*z^115-47*x^115*y^51*z^23+5*x^2*y^176*z^12-31*x^172*y^4-44*x^17*y^88*z^75-70*x^24*y^53*z^108-29*x^2*y^37*z^104+33*x^80*y^78*z^19-92*x^51*y^3*z^130+22*x^50*y^12*z^101)"))->mPoly;
  b = ((INParameterGmpZ*) getCoeff(ipdeg3, "(25*x^21*y^145*z^9+18*y^50*z^111-15*x^79*y^26*z^52+55*x^124*y^5*z^4-34*y^42*z^42-17*x^19*y^12*z^23-40*x^34*y^44*z^36-39*x^24*y^86*z^86+44*x^17*y^134*z^7+65*x^85*y^36*z^19-80*x^105*y^67-57*x^7*y^47*z^126+63*x^135*y^24*z^13-97*x^57*y^17*z^98-94*x^162*y^23*z^13+5*x^46*y^21*z^95+99*x^8*y^74*z^70+32*x^83*y^21*z^58+49*x^57*y^48*z^48+63*x^30*y^10*z^154-65*x^24*y^16*z^103+24*x^75*y^17*z^45-99*x^162*y^17*z^2+86*x^15*y^25*z^144+64*x^69*y^4*z^8+60*x^53*y^9*z^27+99*x^15*y^5*z^72+48*x^47*y^7*z^49-48*x^10*y^155*z^32+8*x^93*y^60*z^37+52*x^11*y^103*z^32+10*x^73*y^71*z^54-20*x^58*y^90*z^51-39*x^89*y^20*z^44-98*x^137*y^24+8*x^21*y^50*z^97+73*x^73*y^35+6*x^17*y^82*z^42+36*x^88*y^6*z^11+49*x^3*y^13*z^106+70*x^49*y^69*z^82-84*x^60*y^65*z^60-76*x^9*y^37*z^109+27*x^6*y^152*z^3-53*x^15*y^40*z^125-10*x^106*y^83*z^2+90*x^121*y^30*z^30-76*x^58*y^9*z^112+36*x^4*y^147*z^10)*(-34*x^47*y^90*z^46+85*x*y^145*z^50+70*x^97*y^52*z^28-84*x^16*y^68*z^89-26*x^111*y^79*z^5+54*x^101*y^54*z^2+46*x^9*y^25*z^48+77*x^93*y^96*z^2+39*y^168*z^20+27*x^80*y^24*z^91+39*x^20*y^95*z^12-89*x^137*y^35*z^17-10*x^2*y^97*z^6-69*x^6*y^121*z^22+56*x^69*y^45*z^43+58*x^14*y^31*z^36-27*x^11*y^24*z^45-34*x^53*y^20*z^42+70*x^2*y^82*z^47+83*x^20*y^41*z^59-84*x^5*y^47*z^123+90*x^2*y^175*z^16+74*x^139*y*z^57-32*x^23*y^59*z^114-52*x^71*y^59*z^19-50*x^17*y^43*z^18-86*x^36*y^35*z^56+20*x^56*y^11*z^127-84*x^16*y^66*z^23+93*x^62*y^67*z^34+32*x^42*y^21*z^89-82*x^19*y^148*z^17-67*x^18*y^88*z^60+90*x^96*y^9*z^44+95*x^94*y^8*z^40-22*x^27*y^9*z^155-50*x^7*y^40*z^33+14*x^73*y^5*z^2-79*x^139*y*z^40-3*x^82*y^73*z^25+94*x^66*y^59*z^42-68*x^79*y^29*z^68-84*x^57*y^54-22*x^118*y^36*z^37+65*x^65*y^4*z^64+35*x^2*y^15*z^34+78*x^36*y^144*z^4-66*x^8*y^69*z^40-93*x^12*y^23*z^43+46*x^17*y^6*z^176)"))->mPoly;
  g = ((INParameterGmpZ*) getCoeff(ipdeg3, "0"))->mPoly;
  expected = ((INParameterGmpZ*) getCoeff(ipdeg3, "y^4*(-6*x^17*y^78*z^42-36*x^88*y^2*z^11-49*x^3*y^9*z^106-70*x^49*y^65*z^82+84*x^60*y^61*z^60+76*x^9*y^33*z^109-27*x^6*y^148*z^3+53*x^15*y^36*z^125+10*x^106*y^79*z^2-90*x^121*y^26*z^30+76*x^58*y^5*z^112-36*x^4*y^143*z^10-18*y^46*z^111+34*y^38*z^42+80*x^105*y^63+98*x^137*y^20-73*x^73*y^31-25*x^21*y^141*z^9+15*x^79*y^22*z^52-55*x^124*y*z^4+17*x^19*y^8*z^23+40*x^34*y^40*z^36+39*x^24*y^82*z^86-44*x^17*y^130*z^7-65*x^85*y^32*z^19+57*x^7*y^43*z^126-63*x^135*y^20*z^13+97*x^57*y^13*z^98+94*x^162*y^19*z^13-5*x^46*y^17*z^95-99*x^8*y^70*z^70-32*x^83*y^17*z^58-49*x^57*y^44*z^48-63*x^30*y^6*z^154+65*x^24*y^12*z^103-24*x^75*y^13*z^45+99*x^162*y^13*z^2-86*x^15*y^21*z^144-64*x^69*z^8-60*x^53*y^5*z^27-99*x^15*y*z^72-48*x^47*y^3*z^49+48*x^10*y^151*z^32-8*x^93*y^56*z^37-52*x^11*y^99*z^32-10*x^73*y^67*z^54+20*x^58*y^86*z^51+39*x^89*y^16*z^44-8*x^21*y^46*z^97)"))->mPoly;

ITimer t;
t.start();
  algorithm.gcd(*a, *b, *g);
t.stop();
std::cout << t << std::endl;

  g->sub(*expected);
  CPPUNIT_ASSERT(g->isZero());
}
