/***************************************************************************
 *   Copyright (C) 2007 by Sebastian Jambor                                *
 *   sebastian@momo.math.rwth-aachen.de                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "iginvtest.h"

#include <cppunit/extensions/HelperMacros.h>

ICoeff* IGinvTest::getCoeff(IPolyInterface* ip, std::string str) {
  ICoeff* result;

  IPoly* p = getPoly(ip, str);
  if (p->isZero()) {
    result = p->coeffInterface()->create();
    result->setZero();
  }
  else 
    result = p->lc().copy();

  delete p;
  return result;
}

IPoly* IGinvTest::getPoly(IPolyInterface* ip, std::string str) {
  IExpression expr(ip);
  std::istringstream ist(str);
  ist >> expr;

  return ip->copy(*(expr.numer()));
}

INParameterModularShortAux* IGinvTest::getModularPoly(std::string str, ModularShort p, int endVar) {
  INParameterGmpZ* tmp = (INParameterGmpZ*) getCoeff(ip, str);

  INParameterModularShortAux* result = new INParameterModularShortAux(*tmp->mPoly, p);
  result->interface().setEndVar(endVar);

  delete tmp;

  return result;
}

INParameterGmpZAux* IGinvTest::getGmpZPoly(std::string str, int endVar) {
  INParameterGmpZ* tmp = (INParameterGmpZ*) getCoeff(ip, str);

  INParameterGmpZAux* result = new INParameterGmpZAux(*tmp->mPoly);
//  result->mInterface.setEndVar(endVar);

  delete tmp;

  return result;
}

void IGinvTest::setUp() {
  coeffvars = new IVariables();
  coeffvars->add("x");
  coeffvars->add("y");
  coeffvars->add("z");
  
  polyvars = new IVariables();
  polyvars->add("t");
  
  st = new ISystemType(ISystemType::Polynomial, 0, 0);
  im = IMonomInterface::create(IMonomInterface::DegRevLex, st, polyvars);
  ic = ICoeffInterface::create(ICoeffInterface::NParameterGmpZ, st, coeffvars);
  ip = IPolyInterface::create(IPolyInterface::PolyList, st, im, ic);
}

void IGinvTest::tearDown() {
  delete ip;
  delete ic;
  delete im;
  delete st;
  delete polyvars;
  delete coeffvars;
}
