/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IHILBERTPOLY_H
#define IHILBERTPOLY_H

#include <iostream>

#include <gmp.h>

#include "./iutil.h"

class IHilbertPoly {
  int        mDim;
  MP_RAT*    mCoeff;

public:
  IHilbertPoly():
    mDim(-1),
    mCoeff(NULL) {}
  ~IHilbertPoly();

  void setDim(int dim);
  int dim() const { return mDim; }
  const MP_RAT* operator[](int k) const {
    IASSERT(0 <= k && k < mDim);
    return mCoeff + k;
  }
  MP_RAT* operator[](int k) {
    IASSERT(0 <= k && k < mDim);
    return mCoeff + k;
  }

  void add(const IHilbertPoly& a);
  void sub(const IHilbertPoly& a);
  void mult(int a);
  void binomial(long int a, unsigned long b);

  void read(std::istream &in);
  void write(std::ostream &out) const;

  friend std::ostream& operator<<(std::ostream& out, const IHilbertPoly& a);
  std::ostream& latex(std::ostream& out) const;
};

#endif // IHILBERTPOLY_H

