/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <cstdlib>

#include "./iutil.h"
#include "./imodular.h"

bool iIsPrime(ModularShort p) {
  bool r;

  if (p < 3 || (p & 1) == 0)
    r = p == 2;
  else {
    ModularShort d;
    for(d = 3; d*d <= p ; d += 2) {
      if ((p % d) == 0)
        break;
    }
    r = d*d > p;
  }

  return r;
}

ModularShort iGcd(ModularShort a, ModularShort b, ModularShort& x0, ModularShort& y0) {
  IASSERT(a != 0 || b != 0);
  x0 = 1;
  y0 = 0;
  ModularShort  x1 = 0, y1 = 1;
  while(b) {
    ModularShort q = a / b;

    ModularShort tmp = x1;
    x1 = x0 - q*x1;
    x0 = tmp;

    tmp = y1;
    y1 = y0 - q*y1;
    y0 = tmp;

    tmp = b;
    b = a % b;
    a = tmp;
  }
  return a;
}

ModularShort iPow1(ModularShort a, unsigned deg, ModularShort m) {
  IASSERT(a != 0 || deg > 0);
  IASSERT(0 <= a && a < m);
  ModularShort r = 1;
  while (deg) {   // Invariant a^deg == a^deg*r
    if (deg & 1)
      r = r*a % m;
    a = a*a % m;
    deg >>= 1;
  }
  return r;
}

class ISeedRandom {
public:
  ISeedRandom() {
    srand(31013);
  }
  ~ISeedRandom() {}
};

static ISeedRandom sSeedRandom;

ModularShort iRandom(ModularShort m) {
  ModularShort r = rand() / (RAND_MAX / (m - 1)) + 1;
  return (r >= m) ? m-1: r;
}
