/***************************************************************************
 *   Copyright (C) 2005 by Blinkov Yu. A.                                  *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ISTATISTICS_H
#define ISTATISTICS_H

#include <iostream>

#include "./ilist.h"
#include "./itimer.h"
#include "./ihilbertpoly.h"

struct IStatistics {
  struct Movement {
    double mUserTime;
    int    mTlength;
    int    mQlength;
    int    mEqLmLength;

    Movement():
        mUserTime(0.0),
        mTlength(0),
        mQlength(0),
        mEqLmLength(0) {}
    ~Movement() {}
  };

  IList<Movement> mMovement;

private:
  IList<Movement>::Iterator mMovementEnd;

public:
  ITimer       mTimer;
  unsigned     mCriterion;
  unsigned     mSpoly;
  unsigned     mReduction;

  unsigned     mLengthGB;
  unsigned     mLengthIB;

  IHilbertPoly mHilbertPoly;

  bool         mOutOfTime;
  bool         mComputationInterrupted;

  IStatistics():
      mMovement(),
      mMovementEnd(mMovement.begin()),
      mTimer(),
      mCriterion(0),
      mSpoly(0),
      mReduction(0),
      mLengthGB(0),
      mLengthIB(0),
      mHilbertPoly(),
      mOutOfTime(false),
      mComputationInterrupted(false) {}
  ~IStatistics() {}

  void append(double userTime, int Tlength, int Qlength, int EqLmLength);

  typedef IList<IStatistics::Movement>::ConstIterator ConstIterator;
  ConstIterator begin() const { return mMovement.begin(); }

  void read(std::istream &in);
  void write(std::ostream &out) const;

  friend std::ostream& operator<<(std::ostream& out, const IStatistics& a);
};

#endif // ISTATISTICS_H
