/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ITIMER_H
#define ITIMER_H

#include <iostream>
#include <iomanip>

#ifdef WIN32

#include <ctime>

class ITimer {
  clock_t     mTime;
  clock_t     mTimer;

  clock_t     mAlarmTime;

public:
  ITimer():
      mTime(0),
      mTimer(0),
      mAlarmTime(0) {}

  operator double() const { return double(mTime) / CLOCKS_PER_SEC; }
  void start() { mTime = 0; }
  void cont() { mTimer = clock(); }
  void stop() { mTime += clock() - mTimer; }

  double userTime() const { return operator double(); }
  double sysTime() const { return 0; }
  double realTime() const { return operator double(); }

  void setAlarm(long a) {     // set alarm time to a seconds
    mAlarmTime = (clock_t)(a * CLOCKS_PER_SEC);
  }
  bool earlier_than_alarm() const {
    return (mTime + clock() - mTimer < mAlarmTime);
  }

  friend std::ostream& operator<<(std::ostream& out, const ITimer &a);
};

#else

#include <sys/times.h>

class ITimer {
  static double mHZ;        //  Number of ticks per second
  static tms    mBuffer;

  clock_t       mUser;
  clock_t       mSys;
  clock_t       mTimeUser;
  clock_t       mTimeSys;

  clock_t       mAlarmTime;

public:
  ITimer():
      mUser(0),
      mSys(0),
      mTimeUser(0),
      mTimeSys(0),
      mAlarmTime(0) {}
  ~ITimer() {}

  void start();
  void stop();
  void cont();

  double userTime() const { return mTimeUser / mHZ; }
  double sysTime() const { return mTimeSys / mHZ; }
  double realTime() const { return (mTimeUser + mTimeSys) / mHZ; }

  void operator=(const ITimer &a);

  void setAlarm(long a) {    // set alarm time to a seconds
    mAlarmTime = (clock_t)(a * mHZ);
  }
  bool earlier_than_alarm() const;

  friend std::ostream& operator<<(std::ostream& out, const ITimer &a);
};

#endif // WIN32

#endif // ITIMER_H


