/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./monom.h"
#include "./poly.h"
#include "./oneparameterpoly.h"
#include "./coeff.h"
#include "./wrap.h"
#include "./division.h"
#include "./basis.h"

#include "../ginv/poly/iexpression.h"
#include "../ginv/algorithm/ialgorithm.h"

#include <string.h>
#include <sstream>

void basis_dealloc(Basis *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* basis_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  Basis *self = (Basis*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->algorithm = NULL;
  }

  return (PyObject *)self;
}

PyObject* basis_getattr(Basis *self, char *name) {
  PyObject *r = NULL;
  static PyMethodDef methods[] = {
    {"lengthIB", (PyCFunction)basis_lengthIB, METH_NOARGS , ""},
    {"lengthGB", (PyCFunction)basis_lengthGB, METH_NOARGS , ""},
    {"numCriterion", (PyCFunction)basis_numCriterion, METH_NOARGS , ""},
    {"numSpoly", (PyCFunction)basis_numSpoly, METH_NOARGS , ""},
    {"numReduction", (PyCFunction)basis_numReduction, METH_NOARGS , ""},
    {"userTime", (PyCFunction)basis_userTime, METH_NOARGS , ""},
    {"sysTime", (PyCFunction)basis_sysTime, METH_NOARGS , ""},
    {"realTime", (PyCFunction)basis_realTime, METH_NOARGS , ""},
    {"hilbertPolynomial", (PyCFunction)basis_hilbertPolynomial, METH_NOARGS , ""},
    {"iterStatistics", (PyCFunction)basis_iterStatistics, METH_NOARGS , ""},
    {"iterIB", (PyCFunction)basis_iterIB, METH_NOARGS , ""},
    {"iterGB", (PyCFunction)basis_iterGB, METH_NOARGS , ""},
    {"iterLmIB", (PyCFunction)basis_iterLmIB, METH_NOARGS , ""},
    {"iterLmGB", (PyCFunction)basis_iterLmGB, METH_NOARGS , ""},
    {"iterLcIB", (PyCFunction)basis_iterLcIB, METH_NOARGS , ""},
    {"iterLcGB", (PyCFunction)basis_iterLcGB, METH_NOARGS , ""},
    {"iterAncIB", (PyCFunction)basis_iterAncIB, METH_NOARGS , ""},
    {"iterMultVarIB", (PyCFunction)basis_iterMultVarIB, METH_NOARGS , ""},
    {"find", (PyCFunction)basis_find, METH_VARARGS , ""},
    {"minPoly", (PyCFunction)basis_minPoly, METH_VARARGS | METH_KEYWORDS, ""},
    {"nf", (PyCFunction)basis_nf, METH_VARARGS , ""},
    {"isNf", (PyCFunction)basis_isNf, METH_VARARGS , ""},
    {"nfHead", (PyCFunction)basis_nfHead, METH_VARARGS , ""},
    {"isNfHead", (PyCFunction)basis_isNfHead, METH_VARARGS , ""},
    {"nfTail", (PyCFunction)basis_nfTail, METH_VARARGS , ""},
    {"isNfTail", (PyCFunction)basis_isNfTail, METH_VARARGS , ""},
    {"outOfTime", (PyCFunction)basis_outOfTime, METH_NOARGS , ""},
    {"computationInterrupted", (PyCFunction)basis_computationInterrupted, METH_NOARGS , ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);

  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'Basis'");

  return r;
}

PyObject* basis_getiter(Basis *self) {
  BasisIterator* iter = PyObject_New(BasisIterator, &basisIterator_type);
  iter->iter = self->algorithm->setT().begin();
  return (PyObject*)iter;
}

PyObject* basis_lengthIB(Basis *self) {
  return PyInt_FromLong(self->algorithm->statistics().mLengthIB);
}

PyObject* basis_lengthGB(Basis *self) {
  return PyInt_FromLong(self->algorithm->statistics().mLengthGB);
}

PyObject* basis_numCriterion(Basis *self) {
  return PyInt_FromLong(self->algorithm->statistics().mCriterion);
}

PyObject* basis_numSpoly(Basis *self) {
  return PyInt_FromLong(self->algorithm->statistics().mSpoly);
}

PyObject* basis_numReduction(Basis *self) {
  return PyInt_FromLong(self->algorithm->statistics().mReduction);
}

PyObject* basis_userTime(Basis *self) {
  return PyFloat_FromDouble(self->algorithm->statistics().mTimer.userTime());
}

PyObject* basis_sysTime(Basis *self) {
  return PyFloat_FromDouble(self->algorithm->statistics().mTimer.sysTime());
}

PyObject* basis_realTime(Basis *self) {
  return PyFloat_FromDouble(self->algorithm->statistics().mTimer.realTime());
}

PyObject* basis_hilbertPolynomial(Basis *self) {
  std::ostringstream out;
  out << self->algorithm->statistics().mHilbertPoly;
  return PyString_FromString(out.str().c_str());
}

PyObject* basis_iterStatistics(Basis *self) {
  StatisticsIterator* iter = PyObject_New(StatisticsIterator, &statisticsIterator_type);
  iter->iter = self->algorithm->statistics().begin();
  return (PyObject*)iter;
}

PyObject* basis_iterIB(Basis *self) {
  IBIterator* iter = PyObject_New(IBIterator, &iBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  return (PyObject*)iter;
}

PyObject* basis_iterGB(Basis *self) {
  GBIterator* iter = PyObject_New(GBIterator, &gBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  while(iter->iter && (*iter->iter)->isProlong())
    ++iter->iter;
  return (PyObject*)iter;
}

PyObject* basis_iterLmIB(Basis *self) {
  LmIBIterator* iter = PyObject_New(LmIBIterator, &lMiBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  return (PyObject*)iter;
}

PyObject* basis_iterLmGB(Basis *self) {
  LmGBIterator* iter = PyObject_New(LmGBIterator, &lMgBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  while(iter->iter && (*iter->iter)->isProlong())
    ++iter->iter;
  return (PyObject*)iter;
}

PyObject* basis_iterLcIB(Basis *self) {
  LcIBIterator* iter = PyObject_New(LcIBIterator, &lCiBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  return (PyObject*)iter;
}

PyObject* basis_iterLcGB(Basis *self) {
  LcGBIterator* iter = PyObject_New(LcGBIterator, &lCgBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  while(iter->iter && (*iter->iter)->isProlong())
    ++iter->iter;
  return (PyObject*)iter;
}

PyObject* basis_iterAncIB(Basis *self) {
  AncIBIterator* iter = PyObject_New(AncIBIterator, &aNCiBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  return (PyObject*)iter;
}

PyObject* basis_iterMultVarIB(Basis *self) {
  MultVarIBIterator* iter = PyObject_New(MultVarIBIterator, &mULTVARiBIterator_type);
  iter->iter = self->algorithm->setT().begin();
  return (PyObject*)iter;
}

PyObject* basis_find(Basis *self, PyObject *args) {
  PyObject *py_monom = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_monom))
      return NULL;

  IMonom *monom = NULL;
  IMonomInterface *monomInterface = self->algorithm->setT().wrapInterface()->monomInterface();
  if (PyObject_TypeCheck(py_monom, &PyString_Type)) {
    IMonom *m = monomInterface->create();
    std::istringstream in(PyString_AsString(py_monom));
    in >> *m >> std::ws;
    if (!in.eof()) {
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.basis_find()");
      delete m;
    }
    else
      monom = m;
  }
  else if (PyObject_TypeCheck(py_monom, &monom_type)) {
    IMonom *m = ((Monom*)py_monom)->monom;
    if (monomInterface != m->monomInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.basis_find() monomial with different MonomInterface");
    else
      monom = m;
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Monom' in 'Basis.basis_find()'");

  PyObject *r = NULL;
  if (monom != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      Wrap* res = PyObject_New(Wrap, &wrap_type);
      res->wrap = self->algorithm->division()->find(*monom);
      r = (PyObject*)res;
      if (PyObject_TypeCheck(py_monom, &PyString_Type))
        delete monom;
    }
  }

  return r;
}

/*
PyObject* basis_find(Basis *self, PyObject *args) {
  PyObject *py_monom = NULL;
  PyObject *py_coeff = NULL;
  if (!PyArg_ParseTuple(args, "OO:Basis",
                        &py_monom,
                        &py_coeff))
      return NULL;

  IMonom *monom = NULL;
  IMonomInterface *monomInterface = self->algorithm->setT().wrapInterface()->monomInterface();
  if (PyObject_TypeCheck(py_monom, &PyString_Type)) {
    IMonom *m = monomInterface->create();
    std::istringstream in(PyString_AsString(py_monom));
    in >> *m >> std::ws;
    if (!in.eof()) {
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.basis_find()");
      delete m;
    }
    else
      monom = m;
  }
  else if (PyObject_TypeCheck(py_monom, &monom_type)) {
    IMonom *m = ((Monom*)py_monom)->monom;
    if (monomInterface != m->monomInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.basis_find() monomial with different MonomInterface");
    else
      monom = m;
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Monom' in 'Basis.basis_find()'");

  ICoeff *coeff = NULL;
  ICoeffInterface *coeffInterface = self->algorithm->setT().wrapInterface()->coeffInterface();
  if (PyObject_TypeCheck(py_coeff, &PyString_Type)) {
    ICoeff *c = coeffInterface->create();
    std::istringstream in(PyString_AsString(py_coeff));
    in >> *c >> std::ws;
    if (!in.eof()) {
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.basis_find()");
      delete c;
    }
    else
      coeff = c;
  }
  else if (PyObject_TypeCheck(py_coeff, &coeff_type)) {
    ICoeff *c = ((Coeff*)py_coeff)->coeff;
    if (coeffInterface != c->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.basis_find() coefficient with different CoeffInterface");
    else
      coeff = c;
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Coeff' in 'Basis.basis_find()'");

  PyObject *r = NULL;
  if (monom != NULL && coeff != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      Wrap* res = PyObject_New(Wrap, &wrap_type);
      res->wrap = self->algorithm->division()->find(*monom, *coeff);
      r = (PyObject*)res;
      if (PyObject_TypeCheck(py_monom, &PyString_Type))
        delete monom;
      if (PyObject_TypeCheck(py_coeff, &PyString_Type))
        delete coeff;
    }
  }

  return r;
}
*/

PyObject* basis_minPoly(Basis *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"element", "var", "degree", NULL};

  PyObject *py_poly = NULL;
  const char *py_var = "t";
  int py_degree = 30;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|si:Basis", kwlist,
                                    &py_poly,
                                    &py_var,
                                    &py_degree))
    return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();

  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.nf()");
    else {
      poly = expr.toPoly();
    }
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.nf() polynomial with different PolyInterface");
    else {
      poly = p->polyInterface()->copy(*p);
    }
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Poly' in 'Basis.nf()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      OneParameterPoly* res = PyObject_New(OneParameterPoly, &oneparameterpoly_type);
      res->poly = new IOneParameterPoly(polyInterface->coeffInterface(), py_var);

      self->algorithm->division()->minPoly(*poly, *res->poly, py_degree);

      r = (PyObject*)res;
    }
  }

  return r;
}

PyObject* basis_nf(Basis *self, PyObject *args) {
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_poly))
      return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();
  ICoeff *multiplier = NULL;

  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.nf()");
    else {
      poly = expr.toPoly();
      multiplier = expr.getDenom();
    }
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.nf() polynomial with different PolyInterface");
    else {
      poly = p->polyInterface()->copy(*p);
      if (((Poly*)py_poly)->denom == NULL) {
	multiplier = p->polyInterface()->coeffInterface()->create();
	multiplier->setOne();
      }
      else
        multiplier = p->polyInterface()->coeffInterface()->copy(*((Poly*)py_poly)->denom);
    }
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Poly' in 'Basis.nf()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      Poly* res = PyObject_New(Poly, &poly_type);
      self->algorithm->division()->nf(*multiplier, *poly);
      if (poly->isZero()) {
	delete multiplier;
	res->denom = NULL;
      }
      else {
        self->algorithm->division()->nfTail(*multiplier, *poly);
	if (multiplier->isOne()) {
	  res->denom = NULL;
	  delete multiplier;
	}
	else
	  res->denom = multiplier;
      }
      res->poly = poly;
      r = (PyObject*)res;
    }
  }

  return r;
}

PyObject* basis_isNf(Basis *self, PyObject *args) {
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_poly))
      return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();
  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.nf()");
    else
      poly = expr.toPoly();
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.nf() polynomial with different PolyInterface");
    else
      poly = p;
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Poly' in 'Basis.nf()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      if (!poly->isZero() &&
          self->algorithm->division()->isNf(*poly) &&
          self->algorithm->division()->isNfTail(*poly))
        r = Py_True;
      else
        r = Py_False;
      Py_INCREF(r);
      if (PyObject_TypeCheck(py_poly, &PyString_Type))
        delete poly;
    }
  }

  return r;
}

PyObject* basis_nfHead(Basis *self, PyObject *args) {
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_poly))
      return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();
  ICoeff *multiplier = NULL;

  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.nfHead()");
    else {
      poly = expr.toPoly();
      multiplier = expr.getDenom();
    }
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.nfHead() polynomial with different PolyInterface");
    else {
      poly = p->polyInterface()->copy(*p);
      if (((Poly*)py_poly)->denom == NULL) {
	multiplier = p->polyInterface()->coeffInterface()->create();
	multiplier->setOne();
      }
      else
        multiplier = p->polyInterface()->coeffInterface()->copy(*((Poly*)py_poly)->denom);
    }
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Poly' in 'Basis.nfHead()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      Poly* res = PyObject_New(Poly, &poly_type);
      self->algorithm->division()->nf(*multiplier, *poly);
      if (poly->isZero()) {
	delete multiplier;
	res->denom = NULL;
      }
      else
	if (multiplier->isOne()) {
	  res->denom = NULL;
	  delete multiplier;
	}
	else
	  res->denom = multiplier;
      res->poly = poly;
      r = (PyObject*)res;
    }
  }

  return r;
}

PyObject* basis_isNfHead(Basis *self, PyObject *args) {
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_poly))
      return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();
  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.isNfHead()");
    else
      poly = expr.toPoly();
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.isNfHead() polynomial with different PolyInterface");
    else
      poly = p;
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Poly' in 'Basis.isNfHead()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      if (!poly->isZero() &&
          self->algorithm->division()->isNf(*poly))
        r = Py_True;
      else
        r = Py_False;
      Py_INCREF(r);
      if (PyObject_TypeCheck(py_poly, &PyString_Type))
        delete poly;
    }
  }

  return r;
}

PyObject* basis_nfTail(Basis *self, PyObject *args) {
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_poly))
      return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();
  ICoeff *multiplier = NULL;

  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.nfTail()");
    else {
      poly = expr.toPoly();
      multiplier = expr.getDenom();
    }
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.nfTail() polynomial with different PolyInterface");
    else {
      poly = p->polyInterface()->copy(*p);
      if (((Poly*)py_poly)->denom == NULL) {
	multiplier = p->polyInterface()->coeffInterface()->create();
	multiplier->setOne();
      }
      else
        multiplier = p->polyInterface()->coeffInterface()->copy(*((Poly*)py_poly)->denom);
    }
  }
  else
    PyErr_SetString(PyExc_TypeError, "nfTail element type string or 'Poly' in 'Basis.nfTail()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      Poly* res = PyObject_New(Poly, &poly_type);
      if (!poly->isZero())
        self->algorithm->division()->nfTail(*multiplier, *poly);
      if (poly->isZero()) {
	delete multiplier;
	res->denom = NULL;
      }
      else
	if (multiplier->isOne()) {
	  res->denom = NULL;
	  delete multiplier;
	}
	else
	  res->denom = multiplier;
      res->poly = poly;
      r = (PyObject*)res;
    }
  }

  return r;
}

PyObject* basis_isNfTail(Basis *self, PyObject *args) {
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTuple(args, "O:Basis",
                        &py_poly))
      return NULL;

  IPoly *poly = NULL;
  IPolyInterface *polyInterface = self->algorithm->setT().wrapInterface()->polyInterface();
  if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
    std::istringstream in(PyString_AsString(py_poly));
    IExpression expr(polyInterface);
    in >> expr >> std::ws;
    if (!in.eof())
      PyErr_SetString(PyExc_TypeError, "parsing string in Basis.isNfTail()");
    else
      poly = expr.toPoly();
  }
  else if (PyObject_TypeCheck(py_poly, &poly_type)) {
    IPoly *p = ((Poly*)py_poly)->poly;
    if (polyInterface != p->polyInterface())
      PyErr_SetString(PyExc_TypeError, "in Basis.isNfTail() polynomial with different PolyInterface");
    else
      poly = p;
  }
  else
    PyErr_SetString(PyExc_TypeError, "nf element type string or 'Poly' in 'Basis.isNfTail()'");

  PyObject *r = NULL;
  if (poly != NULL) {
    if (self->algorithm->division() == NULL) {
    }
    else {
      if (self->algorithm->division()->isNfTail(*poly))
        r = Py_True;
      else
        r = Py_False;
      Py_INCREF(r);
      if (PyObject_TypeCheck(py_poly, &PyString_Type))
        delete poly;
    }
  }

  return r;
}

PyObject* basis_outOfTime(Basis *self) {
  PyObject *r = NULL;
  if (self->algorithm->statistics().mOutOfTime)
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);
  return r;
}

PyObject* basis_computationInterrupted(Basis *self) {
  PyObject *r = NULL;
  if (self->algorithm->statistics().mComputationInterrupted)
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);
  return r;
}

int basis_length(Basis *self) {
  return self->algorithm->setT().length();
}

PyObject* basis_item(Basis *self, Py_ssize_t i) {
  PyObject* r = NULL;
  if (0 > i || i >= self->algorithm->setT().length())
    PyErr_SetString(PyExc_ValueError, "Index out of range.");
  else {
    ISetT::ConstIterator k = self->algorithm->setT().begin();
    for(; i > 0; i--)
     ++k;

    Wrap* wrap = PyObject_New(Wrap, &wrap_type);
    wrap->wrap = *k;
    r = (PyObject*)wrap;
  }
  return r;
}

static PySequenceMethods basis_sequence = {
  (lenfunc)basis_length,		/*sq_length*/
  0,					/*sq_concat*/
  0,					/*sq_repeat*/
  (ssizeargfunc)basis_item,		/*sq_item*/
  0,					/*sq_slice*/
  0,					/*sq_ass_item*/
  0,					/*sq_ass_slice*/
  0,					/*sq_contains */
  0,					/*sq_inplace_concat*/
  0,					/*sq_inplace_repeat*/
};

PyTypeObject basis_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "Basis",					/*tp_name*/
  sizeof(Basis),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)basis_dealloc,			/*tp_dealloc*/
  0,						/*tp_print*/
  (getattrfunc)basis_getattr,			/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  &basis_sequence,				/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "Basis object",				/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)basis_getiter,			/*tp_iter */
  0,						/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  (newfunc)basis_new,				/*tp_new */
};

void basisIterator_dealloc(BasisIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* basisIterator_next(BasisIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Wrap* wrap = PyObject_New(Wrap, &wrap_type);
    wrap->wrap = *self->iter;
    ++self->iter;
    r = (PyObject*)wrap;
  }
  return r;
}

PyTypeObject basisIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "BasisIterator",				/*tp_name*/
  sizeof(BasisIterator),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)basisIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "BasisIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  0,						/*tp_iter */
  (iternextfunc)basisIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void statisticsIterator_dealloc(StatisticsIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* statisticsIterator_getiter(StatisticsIterator *self) {
  StatisticsIterator* iter = PyObject_New(StatisticsIterator, &statisticsIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* statisticsIterator_next(StatisticsIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    r = PyTuple_New(4);
    PyTuple_SetItem(r, 0, PyFloat_FromDouble((*self->iter).mUserTime));
    PyTuple_SetItem(r, 1, PyInt_FromLong((*self->iter).mTlength));
    PyTuple_SetItem(r, 2, PyInt_FromLong((*self->iter).mQlength));
    PyTuple_SetItem(r, 3, PyInt_FromLong((*self->iter).mEqLmLength));
    ++self->iter;
  }
  return r;
}

PyTypeObject statisticsIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "StatisticsIterator",				/*tp_name*/
  sizeof(StatisticsIterator),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)statisticsIterator_dealloc,	/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "StatisticsIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)statisticsIterator_getiter,	/*tp_iter */
  (iternextfunc)statisticsIterator_next,	/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void iBIterator_dealloc(IBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* iBIterator_getiter(IBIterator *self) {
  IBIterator* iter = PyObject_New(IBIterator, &iBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* iBIterator_next(IBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Poly* poly = PyObject_New(Poly, &poly_type);
    poly->poly = (*self->iter)->poly().polyInterface()->copy((*self->iter)->poly());
    poly->denom = NULL;
    ++self->iter;
    r = (PyObject*)poly;
  }
  return r;
}

PyTypeObject iBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "IBIterator",					/*tp_name*/
  sizeof(IBIterator),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)iBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "IBIterator object",				/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)iBIterator_getiter,		/*tp_iter */
  (iternextfunc)iBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void gBIterator_dealloc(GBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* gBIterator_getiter(GBIterator *self) {
  GBIterator* iter = PyObject_New(GBIterator, &gBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* gBIterator_next(GBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Poly* poly = PyObject_New(Poly, &poly_type);
    poly->poly = (*self->iter)->poly().polyInterface()->copy((*self->iter)->poly());
    poly->denom = NULL;
    do {
      ++self->iter;
    } while (self->iter && (*self->iter)->isProlong());
    r = (PyObject*)poly;
  }
  return r;
}

PyTypeObject gBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "GBIterator",					/*tp_name*/
  sizeof(GBIterator),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)gBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "GBIterator object",				/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)gBIterator_getiter,		/*tp_iter */
  (iternextfunc)gBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void lMiBIterator_dealloc(LmIBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* lMiBIterator_getiter(LmIBIterator *self) {
  LmIBIterator* iter = PyObject_New(LmIBIterator, &lMiBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* lMiBIterator_next(LmIBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Monom* monom = PyObject_New(Monom, &monom_type);
    monom->monom = (*self->iter)->lm().monomInterface()->copy((*self->iter)->lm());
    ++self->iter;
    r = (PyObject*)monom;
  }
  return r;
}

PyTypeObject lMiBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "LmIBIterator",				/*tp_name*/
  sizeof(LmIBIterator),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)lMiBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "LmIBIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)lMiBIterator_getiter,		/*tp_iter */
  (iternextfunc)lMiBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void lMgBIterator_dealloc(LmGBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* lMgBIterator_getiter(LmGBIterator *self) {
  LmGBIterator* iter = PyObject_New(LmGBIterator, &lMgBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* lMgBIterator_next(LmGBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Monom* monom = PyObject_New(Monom, &monom_type);
    monom->monom = (*self->iter)->lm().monomInterface()->copy((*self->iter)->lm());
    do {
      ++self->iter;
    } while(self->iter && (*self->iter)->isProlong());
    r = (PyObject*)monom;
  }
  return r;
}

PyTypeObject lMgBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "LmGBIterator",				/*tp_name*/
  sizeof(LmGBIterator),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)lMgBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "LmGBIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)lMgBIterator_getiter,		/*tp_iter */
  (iternextfunc)lMgBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void lCiBIterator_dealloc(LcIBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* lCiBIterator_getiter(LcIBIterator *self) {
  LcIBIterator* iter = PyObject_New(LcIBIterator, &lCiBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* lCiBIterator_next(LcIBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Coeff* coeff = PyObject_New(Coeff, &coeff_type);
    coeff->coeff = (*self->iter)->poly().lc().coeffInterface()->copy((*self->iter)->poly().lc());
    ++self->iter;
    r = (PyObject*)coeff;
  }
  return r;
}

PyTypeObject lCiBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "LcIBIterator",				/*tp_name*/
  sizeof(LcIBIterator),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)lCiBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "LcIBIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)lCiBIterator_getiter,		/*tp_iter */
  (iternextfunc)lCiBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void lCgBIterator_dealloc(LcGBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* lCgBIterator_getiter(LcGBIterator *self) {
  LcGBIterator* iter = PyObject_New(LcGBIterator, &lCgBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* lCgBIterator_next(LcGBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Coeff* coeff = PyObject_New(Coeff, &coeff_type);
    coeff->coeff = (*self->iter)->poly().lc().coeffInterface()->copy((*self->iter)->poly().lc());
    do {
      ++self->iter;
    } while(self->iter && (*self->iter)->isProlong());
    r = (PyObject*)coeff;
  }
  return r;
}

PyTypeObject lCgBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "LcGBIterator",				/*tp_name*/
  sizeof(LcGBIterator),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)lCgBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "LcGBIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)lCgBIterator_getiter,		/*tp_iter */
  (iternextfunc)lCgBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void aNCiBIterator_dealloc(AncIBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* aNCiBIterator_getiter(AncIBIterator *self) {
  AncIBIterator* iter = PyObject_New(AncIBIterator, &aNCiBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* aNCiBIterator_next(AncIBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Monom* monom = PyObject_New(Monom, &monom_type);
    monom->monom = (*self->iter)->ancestor().monomInterface()->copy((*self->iter)->ancestor());
    ++self->iter;
    r = (PyObject*)monom;
  }
  return r;
}

PyTypeObject aNCiBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "AncIBIterator",				/*tp_name*/
  sizeof(AncIBIterator),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)aNCiBIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "AncIBIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)aNCiBIterator_getiter,		/*tp_iter */
  (iternextfunc)aNCiBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void mULTVARiBIterator_dealloc(MultVarIBIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* mULTVARiBIterator_getiter(MultVarIBIterator *self) {
  MultVarIBIterator* iter = PyObject_New(MultVarIBIterator, &mULTVARiBIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* mULTVARiBIterator_next(MultVarIBIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    MultVarIterator* iter = PyObject_New(MultVarIterator, &mULTVARIterator_type);
    iter->wrap = *self->iter;
    iter->v = 0;
    ++self->iter;
    r = (PyObject*)iter;
  }
  return r;
}

PyTypeObject mULTVARiBIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "MultVarIBIterator",				/*tp_name*/
  sizeof(MultVarIBIterator),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)mULTVARiBIterator_dealloc,	/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "MultVarIBIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)mULTVARiBIterator_getiter,      	/*tp_iter */
  (iternextfunc)mULTVARiBIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};

void mULTVARIterator_dealloc(MultVarIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* mULTVARIterator_getiter(MultVarIterator *self) {
  MultVarIterator* iter = PyObject_New(MultVarIterator, &mULTVARIterator_type);
  iter->wrap = self->wrap;
  iter->v = self->v;
  return (PyObject*)iter;
}

PyObject* mULTVARIterator_next(MultVarIterator *self) {
  PyObject* r = NULL;
  if (self->v < self->wrap->monomInterface()->dimIndepend()) {
    r = PyInt_FromLong(self->wrap->build(self->v));
    ++self->v;
  }
  return r;
}

PyTypeObject mULTVARIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "MultVarIterator",				/*tp_name*/
  sizeof(MultVarIterator),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)mULTVARIterator_dealloc,          /*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "MultVarIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)mULTVARIterator_getiter,      	/*tp_iter */
  (iternextfunc)mULTVARIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};
