/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ALGORITHM_H
#define ALGORITHM_H

#include <Python.h>
#include <structmember.h>

#include "../ginv/algorithm/ialgorithm.h"

struct  Basis {
  PyObject_HEAD
  IAlgorithm *algorithm;
};

void basis_dealloc(Basis *self);
PyObject* basis_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
PyObject* basis_getattr(Basis *self, char *name);
PyObject* basis_getiter(Basis *self);
PyObject* basis_lengthIB(Basis *self);
PyObject* basis_lengthGB(Basis *self);
PyObject* basis_numCriterion(Basis *self);
PyObject* basis_numSpoly(Basis *self);
PyObject* basis_numReduction(Basis *self);
PyObject* basis_userTime(Basis *self);
PyObject* basis_sysTime(Basis *self);
PyObject* basis_realTime(Basis *self);
PyObject* basis_hilbertPolynomial(Basis *self);
PyObject* basis_iterStatistics(Basis *self);
PyObject* basis_iterIB(Basis *self);
PyObject* basis_iterGB(Basis *self);
PyObject* basis_iterLmIB(Basis *self);
PyObject* basis_iterLmGB(Basis *self);
PyObject* basis_iterLcIB(Basis *self);
PyObject* basis_iterLcGB(Basis *self);
PyObject* basis_iterAncIB(Basis *self);
PyObject* basis_iterMultVarIB(Basis *self);
PyObject* basis_find(Basis *self, PyObject *args);
PyObject* basis_minPoly(Basis *self, PyObject *args, PyObject *kwds);
PyObject* basis_nf(Basis *self, PyObject *args);
PyObject* basis_isNf(Basis *self, PyObject *args);
PyObject* basis_nfHead(Basis *self, PyObject *args);
PyObject* basis_isNfHead(Basis *self, PyObject *args);
PyObject* basis_nfTail(Basis *self, PyObject *args);
PyObject* basis_isNfTail(Basis *self, PyObject *args);
PyObject* basis_outOfTime(Basis *self);
PyObject* basis_computationInterrupted(Basis *self);

extern PyTypeObject basis_type;

struct  BasisIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void basisIterator_dealloc(BasisIterator *self);
PyObject* basisIterator_next(BasisIterator *self);

extern PyTypeObject basisIterator_type;

struct  StatisticsIterator {
  PyObject_HEAD
  IStatistics::ConstIterator iter;
};

void statisticsIterator_dealloc(StatisticsIterator *self);
PyObject* statisticsIterator_getiter(StatisticsIterator *self);
PyObject* statisticsIterator_next(StatisticsIterator *self);

extern PyTypeObject statisticsIterator_type;

struct  IBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void iBIterator_dealloc(IBIterator *self);
PyObject* iBIterator_getiter(IBIterator *self);
PyObject* iBIterator_next(IBIterator *self);

extern PyTypeObject iBIterator_type;

struct  GBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void gBIterator_dealloc(GBIterator *self);
PyObject* gBIterator_getiter(GBIterator *self);
PyObject* gBIterator_next(GBIterator *self);

extern PyTypeObject gBIterator_type;

struct  LmIBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void lMiBIterator_dealloc(LmIBIterator *self);
PyObject* lMiBIterator_getiter(LmIBIterator *self);
PyObject* lMiBIterator_next(LmIBIterator *self);

extern PyTypeObject lMiBIterator_type;

struct  LmGBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void lMgBIterator_dealloc(LmGBIterator *self);
PyObject* lMgBIterator_getiter(LmGBIterator *self);
PyObject* lMgBIterator_next(LmGBIterator *self);

extern PyTypeObject lMgBIterator_type;

struct  LcIBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void lCiBIterator_dealloc(LcIBIterator *self);
PyObject* lCiBIterator_getiter(LcIBIterator *self);
PyObject* lCiBIterator_next(LcIBIterator *self);

extern PyTypeObject lCiBIterator_type;

struct  LcGBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void lCgBIterator_dealloc(LcGBIterator *self);
PyObject* lCgBIterator_getiter(LcGBIterator *self);
PyObject* lCgBIterator_next(LcGBIterator *self);

extern PyTypeObject lCgBIterator_type;

struct  AncIBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void aNCiBIterator_dealloc(AncIBIterator *self);
PyObject* aNCiBIterator_getiter(AncIBIterator *self);
PyObject* aNCiBIterator_next(AncIBIterator *self);

extern PyTypeObject aNCiBIterator_type;

struct  MultVarIBIterator {
  PyObject_HEAD
  ISetT::ConstIterator iter;
};

void mULTVARiBIterator_dealloc(MultVarIBIterator *self);
PyObject* mULTVARiBIterator_getiter(MultVarIBIterator *self);
PyObject* mULTVARiBIterator_next(MultVarIBIterator *self);

extern PyTypeObject mULTVARiBIterator_type;

struct  MultVarIterator {
  PyObject_HEAD
  IWrap *wrap;
  int v;
};

void mULTVARIterator_dealloc(MultVarIterator *self);
PyObject* mULTVARIterator_getiter(MultVarIterator *self);
PyObject* mULTVARIterator_next(MultVarIterator *self);

extern PyTypeObject mULTVARIterator_type;

#endif // ALGORITHM_H

