/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./systemtype.h"
#include "./coeff.h"

#include <string.h>
#include <sstream>

void coeffInterface_dealloc(CoeffInterface *self) {
//   delete self->coeffInterface;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* coeffInterface_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  CoeffInterface *self = (CoeffInterface*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->coeffInterface = NULL;
  }

  return (PyObject *)self;
}

int coeffInterface_init(CoeffInterface *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"type", "systemType", "parameter", "modularShort", "extension", "fieldParameter", NULL};

  const char *py_type = NULL;
  PyObject *py_systemType = NULL;
  PyObject *py_parameter = NULL;
  int py_modularShort = 0;
  PyObject *py_extension = NULL;
  PyObject *py_fieldParameter = NULL;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "sO!|O!iO!O!:CoeffInterface", kwlist,
                                    &py_type,
                                    &systemType_type, &py_systemType,
                                    &PyList_Type, &py_parameter,
                                    &py_modularShort,
                                    &PyList_Type, &py_extension,
                                    &PyList_Type, &py_fieldParameter))
      return -1;

  ISystemType *systemType = ((SystemType *)py_systemType)->systemType;

  IVariables *parameter = NULL;
  if (py_parameter) {
    parameter = new IVariables;
    int n = PyList_Size(py_parameter);
    for(int i=0; i < n; i++) {
      parameter->add(PyString_AsString(PyList_GetItem(py_parameter, i)));
    }
  }

  IVariables *fieldParameter = NULL;
  if (py_fieldParameter) {
    fieldParameter = new IVariables;
    int n = PyList_Size(py_fieldParameter);
    for(int i=0; i < n; i++) {
      fieldParameter->add(PyString_AsString(PyList_GetItem(py_fieldParameter, i)));
    }
  }

  char **extension = NULL;
  if (py_extension) {
    int n = PyList_Size(py_extension);
    extension = new char*[n];
    for(int i=0; i < n; i++) {
      extension[i] = PyString_AsString(PyList_GetItem(py_extension, i));
    }
  }

  ICoeffInterface::Type coeffType = ICoeffInterface::Indefinitely;
  if (strcmp(py_type, "GmpQ") == 0)
    coeffType = ICoeffInterface::GmpQ;
  else if (strcmp(py_type, "GmpZ") == 0)
    coeffType = ICoeffInterface::GmpZ;
  else if (strcmp(py_type, "GmpZZ") == 0)
    coeffType = ICoeffInterface::GmpZZ;
  else if (strcmp(py_type, "ModularShort") == 0)
    coeffType = ICoeffInterface::ModularShort;
  else if (strcmp(py_type, "OneParameterModularShort") == 0)
    coeffType = ICoeffInterface::OneParameterModularShort;
  else if (strcmp(py_type, "OneParameterGmpZ") == 0)
    coeffType = ICoeffInterface::OneParameterGmpZ;
  else if (strcmp(py_type, "AlgebraicFieldExtensionGmpQ") == 0)
    coeffType = ICoeffInterface::AlgebraicFieldExtensionGmpQ;
  else if (strcmp(py_type, "AlgebraicFieldExtensionModularShort") == 0)
    coeffType = ICoeffInterface::AlgebraicFieldExtensionModularShort;
  else if (strcmp(py_type, "AlgebraicFieldExtensionNParameter") == 0)
    coeffType = ICoeffInterface::AlgebraicFieldExtensionNParameter;
  else if (strcmp(py_type, "TwoParameterModularShort") == 0)
    coeffType = ICoeffInterface::TwoParameterModularShort;
  else if (strcmp(py_type, "TwoParameterGmpZ") == 0)
    coeffType = ICoeffInterface::TwoParameterGmpZ;
  else if (strcmp(py_type, "NParameterGmpZ") == 0)
    coeffType = ICoeffInterface::NParameterGmpZ;
  else if (strcmp(py_type, "NParameterField") == 0)
    coeffType = ICoeffInterface::NParameterField;
  else {
    PyErr_SetString(PyExc_TypeError, "unknown type in 'CoeffInterface'");
    return -1;
  }

  if ((coeffType == ICoeffInterface::ModularShort ||
       coeffType == ICoeffInterface::OneParameterModularShort ||
       coeffType == ICoeffInterface::TwoParameterModularShort ||
       coeffType == ICoeffInterface::AlgebraicFieldExtensionModularShort) &&
      py_modularShort < 2) {
    PyErr_SetString(PyExc_TypeError, "error value 'modularShort' in 'CoeffInterface");
    return -1;
  }
  if ((coeffType == ICoeffInterface::OneParameterGmpZ ||
       coeffType == ICoeffInterface::OneParameterModularShort ||
       coeffType == ICoeffInterface::AlgebraicFieldExtensionGmpQ ||
       coeffType == ICoeffInterface::AlgebraicFieldExtensionNParameter ||
       coeffType == ICoeffInterface::AlgebraicFieldExtensionModularShort ||
       coeffType == ICoeffInterface::TwoParameterGmpZ ||
       coeffType == ICoeffInterface::TwoParameterModularShort ||
       coeffType == ICoeffInterface::NParameterField ||
       coeffType == ICoeffInterface::NParameterGmpZ) &&
      parameter == NULL)  {
    PyErr_SetString(PyExc_TypeError, "unknown value 'parameter' in 'CoeffInterface");
    return -1;
  }
  if ((coeffType == ICoeffInterface::AlgebraicFieldExtensionNParameter) &&
      fieldParameter == NULL)  {
    PyErr_SetString(PyExc_TypeError, "unknown value 'fieldParameter' in 'CoeffInterface");
    return -1;
  }
  if ((coeffType == ICoeffInterface::OneParameterGmpZ ||
       coeffType == ICoeffInterface::OneParameterModularShort) &&
      parameter->dim() != 1)  {
    PyErr_SetString(PyExc_TypeError, "according to 'type' len('parameter') == 1 in 'CoeffInterface");
    return -1;
  }

  if ((coeffType == ICoeffInterface::AlgebraicFieldExtensionGmpQ ||
       coeffType == ICoeffInterface::AlgebraicFieldExtensionNParameter || 
       coeffType == ICoeffInterface::AlgebraicFieldExtensionModularShort) && 
      parameter->dim() == 0) {
    PyErr_SetString(PyExc_TypeError, "according to 'type' len('parameter') > 1 in 'CoeffInterface");
    return -1;
  }

  if ((coeffType == ICoeffInterface::TwoParameterGmpZ ||
       coeffType == ICoeffInterface::TwoParameterModularShort) &&
      parameter->dim() != 2)  {
    PyErr_SetString(PyExc_TypeError, "according to 'type' len('parameter') == 2 in 'CoeffInterface");
    return -1;
  }
  if ((coeffType == ICoeffInterface::NParameterGmpZ 
      || coeffType == ICoeffInterface::NParameterField) &&
      parameter->dim() == 0)  {
    PyErr_SetString(PyExc_TypeError, "according to 'type' len('parameter') > 0 in 'CoeffInterface");
    return -1;
  }
  if ((coeffType == ICoeffInterface::AlgebraicFieldExtensionNParameter) &&
      fieldParameter->dim() == 0)  {
    PyErr_SetString(PyExc_TypeError, "according to 'type' len('fieldParameter') > 0 in 'CoeffInterface");
    return -1;
  }
  if ((coeffType == ICoeffInterface::AlgebraicFieldExtensionModularShort ||
      coeffType == ICoeffInterface::AlgebraicFieldExtensionNParameter ||
      coeffType == ICoeffInterface::AlgebraicFieldExtensionGmpQ) &&
      py_extension == NULL) {
    PyErr_SetString(PyExc_TypeError, "unknown value 'extension' in 'CoeffInterface");
    return -1;
  }

  if (coeffType == ICoeffInterface::GmpQ ||
      coeffType == ICoeffInterface::GmpZ ||
      coeffType == ICoeffInterface::GmpZZ)
    self->coeffInterface = ICoeffInterface::create(coeffType, systemType);
  if (coeffType == ICoeffInterface::ModularShort)
    self->coeffInterface = ICoeffInterface::create(coeffType, systemType, py_modularShort);
  if (coeffType == ICoeffInterface::OneParameterGmpZ ||
      coeffType == ICoeffInterface::NParameterGmpZ ||
      coeffType == ICoeffInterface::NParameterField ||
      coeffType == ICoeffInterface::TwoParameterGmpZ)
    self->coeffInterface = ICoeffInterface::create(coeffType, systemType, parameter);
  if (coeffType == ICoeffInterface::OneParameterModularShort ||
      coeffType == ICoeffInterface::TwoParameterModularShort)
    self->coeffInterface = ICoeffInterface::create(coeffType, systemType, parameter, py_modularShort);
  if (coeffType == ICoeffInterface::AlgebraicFieldExtensionGmpQ)
      self->coeffInterface = ICoeffInterface::create(coeffType, systemType, parameter, extension);
  if (coeffType == ICoeffInterface::AlgebraicFieldExtensionModularShort)
    self->coeffInterface = ICoeffInterface::create(coeffType, systemType, parameter, extension, py_modularShort);
  if (coeffType == ICoeffInterface::AlgebraicFieldExtensionNParameter)
    self->coeffInterface = ICoeffInterface::create(coeffType, systemType, fieldParameter, parameter, extension, py_modularShort);

  IASSERT(self->coeffInterface);

  return 0;
}

PyObject *coeffInterface_getattr(CoeffInterface *self, char *name) {
    PyObject *r = NULL;

  static PyMethodDef methods[] = {
    {"type", (PyCFunction)coeffInterface_getType, METH_NOARGS, ""},
    {"isField", (PyCFunction)coeffInterface_isField, METH_NOARGS, ""},
    {"isPseudo", (PyCFunction)coeffInterface_isPseudo, METH_NOARGS, ""},
    {"parameter", (PyCFunction)coeffInterface_parameter, METH_NOARGS, ""},
    {"iterDenom", (PyCFunction)coeffInterface_iterDenom, METH_NOARGS , ""},
    {"lengthDenom", (PyCFunction)coeffInterface_lengthDenom, METH_NOARGS , ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);

  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'CoeffInterface'");

  return r;
}

PyObject* coeffInterface_getType(CoeffInterface *self) {
  PyObject *r = NULL;
  switch(self->coeffInterface->type()) {
    case ICoeffInterface::GmpZ:
      r = PyString_FromString("GmpZ");
      break;
    case ICoeffInterface::GmpQ:
      r = PyString_FromString("GmpQ");
      break;
    case ICoeffInterface::ModularShort:
      r = PyString_FromString("ModularShort");
      break;
    case ICoeffInterface::OneParameterModularShort:
      r = PyString_FromString("OneParameterModularShort");
      break;
    case ICoeffInterface::AlgebraicFieldExtensionGmpQ:
      r = PyString_FromString("AlgebraicFieldExtensionGmpQ");
      break;
    case ICoeffInterface::AlgebraicFieldExtensionModularShort:
      r = PyString_FromString("AlgebraicFieldExtensionModularShort");
      break;
    case ICoeffInterface::AlgebraicFieldExtensionNParameter:
      r = PyString_FromString("AlgebraicFieldExtensionNParameter");
      break;
    case ICoeffInterface::TwoParameterModularShort:
      r = PyString_FromString("TwoParameterModularShort");
      break;
    case ICoeffInterface::OneParameterGmpZ:
      r = PyString_FromString("OneParameterGmpZ");
      break;
    case ICoeffInterface::TwoParameterGmpZ:
      r = PyString_FromString("TwoParameterGmpZ");
      break;
    case ICoeffInterface::NParameterGmpZ:
      r = PyString_FromString("NParameterGmpZ");
      break;
    case ICoeffInterface::NParameterField:
      r = PyString_FromString("NParameterField");
      break;
  }
  return r;
}

PyObject* coeffInterface_isField(CoeffInterface *self) {
  PyObject *r = NULL;
  if (self->coeffInterface->isField())
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);
  return r;
}

PyObject* coeffInterface_isPseudo(CoeffInterface *self) {
  PyObject *r = NULL;
  if (self->coeffInterface->isPseudo())
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);
  return r;
}

PyObject* coeffInterface_parameter(CoeffInterface *self) {
  PyObject *r = NULL;
  int n = 0;
  if (self->coeffInterface->parameters())
    n = self->coeffInterface->parameters()->dim();
  r = PyList_New(n);
  if (n) {
    IVariables::ConstIterator i = self->coeffInterface->parameters()->begin();
    int k = 0;
    while (k < n) {
      PyObject *var = PyString_FromString(*i);
      PyList_SetItem(r, k, var);
      Py_INCREF(r);
      ++i;
      ++k;
    }
  }
  Py_INCREF(r);
  return r;
}

PyObject* coeffInterface_iterDenom(CoeffInterface *self) {
  DenomIterator* iter = PyObject_New(DenomIterator, &denomIterator_type);
  iter->iter = self->coeffInterface->denomListbegin();
  return (PyObject*)iter;
}

PyObject* coeffInterface_lengthDenom(CoeffInterface *self) {
  return PyInt_FromLong(self->coeffInterface->lengthDenom());
}


PyTypeObject coeffInterface_type = {
  PyObject_HEAD_INIT(NULL)
  0,                                            /*ob_size*/
  "CoeffInterface",                             /*tp_name*/
  sizeof(CoeffInterface),                       /*tp_size*/
  0,                                            /*tp_itemsize*/
  (destructor)coeffInterface_dealloc,           /*tp_dealloc*/
  0,                                            /*tp_print*/
  (getattrfunc)coeffInterface_getattr,          /*tp_getattr*/
  0,                                            /*tp_setattr*/
  0,                                            /*tp_compare*/
  0,                                            /*tp_repr*/
  0,                                            /*tp_as_number*/
  0,                                            /*tp_as_sequence*/
  0,                                            /*tp_as_mapping*/
  0,                                            /*tp_hash*/
  0,                                            /*tp_call*/
  0,                                            /*tp_str*/
  0,                                            /*tp_getattro*/
  0,                                            /*tp_setattro*/
  0,                                            /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,     /*tp_flags*/
  "CoeffInterface object",                      /*tp_doc */
  0,                                            /*tp_traverse */
  0,                                            /*tp_clear */
  0,                                            /*tp_richcompare */
  0,                                            /*tp_weaklistoffset */
  0,                                            /*tp_iter */
  0,                                            /*tp_iternext */
  0,                                            /*tp_methods */
  0,                                            /*tp_members */
  0,                                            /*tp_getset */
  0,                                            /*tp_base */
  0,                                            /*tp_dict */
  0,                                            /*tp_descr_get */
  0,                                            /*tp_descr_set */
  0,                                            /*tp_dictoffset */
  (initproc)coeffInterface_init,                /*tp_init */
  0,                                            /*tp_alloc */
  (newfunc)coeffInterface_new,                  /*tp_new */
};

void coeff_dealloc(Coeff *self) {
//   delete self->coeff;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* coeff_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  Coeff *self = (Coeff*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->coeff = NULL;
  }

  return (PyObject *)self;
}

int coeff_init(Coeff *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"coeffInterface", "coeff", NULL};

  PyObject *py_coeffInterface = NULL;
  const char *py_in = NULL;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|s:Coeff", kwlist,
                                    &coeffInterface_type, &py_coeffInterface,
                                    &py_in))
      return -1;

  self->coeff = ((CoeffInterface*)py_coeffInterface)->coeffInterface->create();
  if (py_in) {
    std::istringstream in(py_in);
    self->coeff->input(in, true);
    in >> std::ws;
    if (!in.eof()) {
      std::cout << ">>>" << (char) in.get() << "<<<" << std::endl;
      PyErr_SetString(PyExc_TypeError, "error parsing second argument");
      return -1;
    }
  }

  return 0;
}

int coeff_print(Coeff *c, FILE *fp, int flags) {
  std::ostringstream out;
  out << *c->coeff;
  fputs(out.str().c_str(), fp);
  return 0;
}

PyObject* coeff_str(Coeff *c) {
  std::ostringstream out;
  out << *c->coeff;
  return PyString_FromString(out.str().c_str());
}

PyObject* coeff_getattr(Coeff *self, char *name) {
  PyObject *r = NULL;
  static PyMethodDef methods[] = {
    {"isZero", (PyCFunction)coeff_isZero, METH_NOARGS, ""},
    {"isOne", (PyCFunction)coeff_isOne, METH_NOARGS, ""},
    {"setZero", (PyCFunction)coeff_setZero, METH_NOARGS, ""},
    {"setOne", (PyCFunction)coeff_setOne, METH_NOARGS, ""},
    {"inverse", (PyCFunction)coeff_inverse, METH_NOARGS, ""},
    {"gcd", (PyCFunction)coeff_gcd, METH_VARARGS, ""},
    {"diff", (PyCFunction)coeff_diff, METH_VARARGS, ""},
    {"shift", (PyCFunction)coeff_shift, METH_VARARGS, ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);

  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'Coeff'");

  return r;
}

PyObject* coeff_isZero(Coeff *self) {
  PyObject *r = NULL;
  if (self->coeff->isZero())
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);

  return r;
}

PyObject* coeff_isOne(Coeff *self) {
  PyObject *r = NULL;
  if (self->coeff->isOne())
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);

  return r;
}

PyObject* coeff_setZero(Coeff *self) {
  PyObject *r = NULL;
  self->coeff->setZero();
  Py_INCREF(self);
  r = (PyObject*)self;

  return r;
}

PyObject* coeff_setOne(Coeff *self) {
  PyObject *r = NULL;
  self->coeff->setOne();
  Py_INCREF(self);
  r = (PyObject*)self;

  return r;
}

PyObject* coeff_inverse(Coeff *self) {
  PyObject *r = NULL;
  if (!self->coeff->coeffInterface()->isField())
    PyErr_SetString(PyExc_TypeError, "CoeffInterface not field");
  else {
    ICoeff *c = self->coeff->coeffInterface()->copy(*self->coeff);
    c->inverse();
    Coeff* res = PyObject_New(Coeff, &coeff_type);
    res->coeff = c;
    r = (PyObject*)res;
  }
  return r;
}

PyObject* coeff_gcd(Coeff *self, PyObject *args) {
  PyObject *py_coeff = NULL;
  if (!PyArg_ParseTuple(args, "O!:Coeff",
                        &coeff_type, &py_coeff))
      return NULL;

  PyObject *r = NULL;
  Coeff *coeff = (Coeff*)py_coeff;
  if (self->coeff->coeffInterface() != coeff->coeff->coeffInterface())
    PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
  else {
    ICoeff *c = self->coeff->coeffInterface()->create();
    c->gcd(*self->coeff, *coeff->coeff);
    Coeff* res = PyObject_New(Coeff, &coeff_type);
    res->coeff = c;
    r = (PyObject*)res;
  }

  return r;
}

PyObject* coeff_diff(Coeff *self, PyObject *args) {
  PyObject *r = NULL;
  PyErr_SetString(PyExc_TypeError, "not implemented");
  return r;
}

PyObject* coeff_shift(Coeff *self, PyObject *args) {
  PyObject *r = NULL;
  PyErr_SetString(PyExc_TypeError, "not implemented");
  return r;
}

PyObject* coeff_add(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      ICoeff *c = o1->coeff->coeffInterface()->copy(*o1->coeff);
      c->add(*coeff->coeff);
      Coeff* res = PyObject_New(Coeff, &coeff_type);
      res->coeff = c;
      r = (PyObject*)res;
    }
  }
  return r;
}

PyObject* coeff_sub(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      ICoeff *c = o1->coeff->coeffInterface()->copy(*o1->coeff);
      c->sub(*coeff->coeff);
      Coeff* res = PyObject_New(Coeff, &coeff_type);
      res->coeff = c;
      r = (PyObject*)res;
    }
  }
  return r;
}

PyObject* coeff_multiply(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      ICoeff *c = o1->coeff->coeffInterface()->copy(*o1->coeff);
      c->mult(*coeff->coeff);
      Coeff* res = PyObject_New(Coeff, &coeff_type);
      res->coeff = c;
      r = (PyObject*)res;
    }
  }
  return r;
}

PyObject* coeff_divide(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      ICoeff *c = o1->coeff->coeffInterface()->create();
      c->div(*o1->coeff, *coeff->coeff);
      Coeff* res = PyObject_New(Coeff, &coeff_type);
      res->coeff = c;
      r = (PyObject*)res;
    }
  }
  return r;
}

PyObject* coeff_negative(Coeff *o1) {
  PyObject *r = NULL;
  ICoeff *c = o1->coeff->coeffInterface()->copy(*o1->coeff);
  c->minus();
  Coeff* res = PyObject_New(Coeff, &coeff_type);
  res->coeff = c;
  r = (PyObject*)res;
  return r;
}

int coeff_nonzero(Coeff *o1) {
  return !o1->coeff->isZero();
}

PyObject* coeff_inplace_add(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      o1->coeff->add(*coeff->coeff);
      r = (PyObject*)o1;
      Py_INCREF(r);
    }
  }
  return r;
}

PyObject* coeff_inplace_sub(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      o1->coeff->sub(*coeff->coeff);
      r = (PyObject*)o1;
      Py_INCREF(r);
    }
  }
  return r;
}

PyObject* coeff_inplace_multiply(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      o1->coeff->mult(*coeff->coeff);
      r = (PyObject*)o1;
      Py_INCREF(r);
    }
  }
  return r;
}

PyObject* coeff_inplace_divide(Coeff *o1, PyObject* o2) {
  PyObject *r = NULL;
  if (!PyObject_TypeCheck(o2, &coeff_type))
    PyErr_SetString(PyExc_TypeError, "second argument not Coeff");
  else {
    Coeff *coeff = (Coeff*)o2;
    if (o1->coeff->coeffInterface() != coeff->coeff->coeffInterface())
      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
    else {
      o1->coeff->div(*coeff->coeff);
      r = (PyObject*)o1;
      Py_INCREF(r);
    }
  }
  return r;
}

static PyNumberMethods coeff_as_number = {
  (binaryfunc)coeff_add,                /*nb_add*/
  (binaryfunc)coeff_sub,                /*nb_subtract*/
  (binaryfunc)coeff_multiply,           /*nb_multiply*/
  (binaryfunc)coeff_divide,             /*nb_divide*/
  0,                                    /*nb_remainder*/
  0,                                    /*nb_divmod*/
  0,                                    /*nb_power*/
  (unaryfunc)coeff_negative,            /*nb_negative*/
  0,                                    /*nb_positive*/
  0,                                    /*nb_absolute*/
  (inquiry)coeff_nonzero,               /*nb_nonzero*/
  0,                                    /*nb_invert*/
  0,                                    /*nb_lshift*/
  0,                                    /*nb_rshift*/
  0,                                    /*nb_and*/
  0,                                    /*nb_xor*/
  0,                                    /*nb_or*/
  0,                                    /*nb_coerce*/
  0,                                    /*nb_int*/
  0,                                    /*nb_long*/
  0,                                    /*nb_float*/
  0,                                    /*nb_oct*/
  0,                                    /*nb_hex*/
  (binaryfunc)coeff_inplace_add,        /*nb_inplace_add*/
  (binaryfunc)coeff_inplace_sub,        /*nb_inplace_subtract*/
  (binaryfunc)coeff_inplace_multiply,   /*nb_inplace_multiply*/
  (binaryfunc)coeff_inplace_divide,     /*nb_inplace_divide*/
  0,                                    /*nb_inplace_remainder*/
  0,                                    /*nb_inplace_power*/
  0,                                    /*nb_inplace_lshift*/
  0,                                    /*nb_inplace_rshift*/
  0,                                    /*nb_inplace_and*/
  0,                                    /*nb_inplace_xor*/
  0,                                    /*nb_inplace_or*/
  0,                                    /*nb_floor_divide*/
  0,                                    /*nb_true_divide*/
  0,                                    /*nb_inplace_floor_divide*/
  0,                                    /*nb_inplace_true_divide*/
};

PyTypeObject coeff_type = {
  PyObject_HEAD_INIT(NULL)
  0,                                            /*ob_size*/
  "Coeff",                                      /*tp_name*/
  sizeof(Coeff),                                /*tp_size*/
  0,                                            /*tp_itemsize*/
  (destructor)coeff_dealloc,                    /*tp_dealloc*/
  (printfunc)coeff_print,                       /*tp_print*/
  (getattrfunc)coeff_getattr,                   /*tp_getattr*/
  0,                                            /*tp_setattr*/
  0,                                            /*tp_compare*/
  (reprfunc)coeff_str,                          /*tp_repr*/
  &coeff_as_number,                             /*tp_as_number*/
  0,                                            /*tp_as_sequence*/
  0,                                            /*tp_as_mapping*/
  0,                                            /*tp_hash*/
  0,                                            /*tp_call*/
  (reprfunc)coeff_str,                          /*tp_str*/
  0,                                            /*tp_getattro*/
  0,                                            /*tp_setattro*/
  0,                                            /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,     /*tp_flags*/
  "Coeff object",                               /*tp_doc */
  0,                                            /*tp_traverse */
  0,                                            /*tp_clear */
  0,                                            /*tp_richcompare */
  0,                                            /*tp_weaklistoffset */
  0,                                            /*tp_iter */
  0,                                            /*tp_iternext */
  0,                                            /*tp_methods */
  0,                                            /*tp_members */
  0,                                            /*tp_getset */
  0,                                            /*tp_base */
  0,                                            /*tp_dict */
  0,                                            /*tp_descr_get */
  0,                                            /*tp_descr_set */
  0,                                            /*tp_dictoffset */
  (initproc)coeff_init,                         /*tp_init */
  0,                                            /*tp_alloc */
  (newfunc)coeff_new,                           /*tp_new */
};

void denomIterator_dealloc(DenomIterator *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* denomIterator_getiter(DenomIterator *self) {
  DenomIterator* iter = PyObject_New(DenomIterator, &denomIterator_type);
  iter->iter = self->iter;
  return (PyObject*)iter;
}

PyObject* denomIterator_next(DenomIterator *self) {
  PyObject* r = NULL;
  if (self->iter) {
    Coeff* coeff = PyObject_New(Coeff, &coeff_type);
    coeff->coeff = (*self->iter)->coeffInterface()->copy(*(*self->iter));
    ++self->iter;
    r = (PyObject*)coeff;
  }
  return r;
}

PyTypeObject denomIterator_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "DenomIterator",			       	/*tp_name*/
  sizeof(DenomIterator),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)denomIterator_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  0,						/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "DenomIterator object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  (getiterfunc)denomIterator_getiter,		/*tp_iter */
  (iternextfunc)denomIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  0,						/*tp_init */
  0,						/*tp_alloc */
  0,						/*tp_new */
};
