/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./wrap.h"
#include "./division.h"

#include <string.h>

void divisionInterface_dealloc(DivisionInterface *self) {
//   delete self->divisionInterface;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* divisionInterface_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  DivisionInterface *self = (DivisionInterface*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->divisionInterface = NULL;
  }

  return (PyObject *)self;
}

int divisionInterface_init(DivisionInterface *self, PyObject *args, PyObject *kwds) {
    static char *kwlist[] = {"type", "wrapInterface", NULL};

    const char *py_type = NULL;
    PyObject *py_wrapInterface = NULL;
    if (!PyArg_ParseTupleAndKeywords(args, kwds, "sO!:DivisionInterface", kwlist,
                                     &py_type,
                                     &wrapInterface_type, &py_wrapInterface))
        return -1;

    IDivisionInterface::Type type = IDivisionInterface::Indefinitely;
    if (strcmp("Janet", py_type) == 0)
      type = IDivisionInterface::Janet;
    else if (strcmp("JanetLike", py_type) == 0)
      type = IDivisionInterface::JanetLike;
    else {
      PyErr_SetString(PyExc_TypeError, "unknown type in 'DivisionInterface'");
      return -1;
    }

    IWrapInterface *wrapInterface = ((WrapInterface *)py_wrapInterface)->wrapInterface;

    self->divisionInterface = IDivisionInterface::create(type, wrapInterface);

    return 0;
}

PyObject *divisionInterface_getattr(DivisionInterface *self, char *name) {
  PyObject *r = NULL;
  if (strcmp("type", name) == 0) {
    switch(self->divisionInterface->type()) {
      case IDivisionInterface::Janet:
        r = PyString_FromString("Janet");
        break;
      case IDivisionInterface::JanetLike:
        r = PyString_FromString("JanetLike");
        break;
    }
  }
  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define attribute");

  return r;
}

PyTypeObject divisionInterface_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "DivisionInterface",				/*tp_name*/
  sizeof(DivisionInterface),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)divisionInterface_dealloc,	/*tp_dealloc*/
  0,						/*tp_print*/
  (getattrfunc)divisionInterface_getattr,	/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "DivisionInterface object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  0,						/*tp_iter */
  0,						/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  (initproc)divisionInterface_init,		/*tp_init */
  0,						/*tp_alloc */
  (newfunc)divisionInterface_new,		/*tp_new */
};

/*
void division_dealloc(Division *self) {
//   delete self->division;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* division_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  Division *self = (Division*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->division = NULL;
  }

  return (PyObject *)self;
}

int division_init(Division *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"divisionInterface", NULL};

  PyObject *py_divisionInterface = NULL;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!:Division", kwlist,
                                    &divisionInterface_type, &py_divisionInterface))
      return -1;

  self->division = ((DivisionInterface*)py_divisionInterface)->divisionInterface->create();

  return 0;
}

PyObject* division_getattr(Division *self, char *name) {
  PyObject *r = NULL;
//   if (strcmp("order", name) == 0) {
//     switch(self->monomInterface->order()) {
//       case IMonomInterface::DegRevLex:
//         r = PyString_FromString("DegRevLex");
//         break;
//       case IMonomInterface::Lex:
//         r = PyString_FromString("Lex");
//         break;
//       case IMonomInterface::Elim:
//         r = PyString_FromString("Elim");
//         break;
//     }
//   }
//   else if (strcmp("systemType", name) == 0) {
//     r = (PyObject *)self->systemType;
//     Py_INCREF(r);
//   }
//   else if (strcmp("dimIndepend", name) == 0) {
//     r = PyInt_FromLong(self->monomInterface->dimIndepend());
//   }
//   else if (strcmp("independ", name) == 0) {
//     int n = self->monomInterface->dimIndepend();
//     r = PyList_New(n);
//     IVariables::ConstIterator i = self->monomInterface->independ()->begin();
//     int k = 0;
//     while (k < n) {
//       PyObject *var = PyString_FromString((*i).c_str());
//       PyList_SetItem(r, k, var);
//       Py_INCREF(r);
//       ++i;
//       ++k;
//     }
//     Py_INCREF(r);
//   }
//   else {
//     PyErr_SetString(PyExc_TypeError, "Cannot defenition attribute type");
//   }

  return r;
}

// PyTypeObject division_type = {
//   PyObject_HEAD_INIT(NULL)
//   0,						/*ob_size*/
//   "Division",					/*tp_name*/
//   sizeof(Division),				/*tp_size*/
//   0,						/*tp_itemsize*/
//   (destructor)division_dealloc,			/*tp_dealloc*/
//   0,						/*tp_print*/
//   (getattrfunc)division_getattr,		/*tp_getattr*/
//   0,						/*tp_setattr*/
//   0,						/*tp_compare*/
//   0,						/*tp_repr*/
//   0,						/*tp_as_number*/
//   0,						/*tp_as_sequence*/
//   0,						/*tp_as_mapping*/
//   0,						/*tp_hash*/
//   0,						/*tp_call*/
//   0,						/*tp_str*/
//   0,						/*tp_getattro*/
//   0,						/*tp_setattro*/
//   0,						/*tp_as_buffer*/
//   Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
//   "Division object",				/*tp_doc */
//   0,						/*tp_traverse */
//   0,						/*tp_clear */
//   0,						/*tp_richcompare */
//   0,						/*tp_weaklistoffset */
//   0,						/*tp_iter */
//   0,						/*tp_iternext */
//   0,						/*tp_methods */
//   0,						/*tp_members */
//   0,						/*tp_getset */
//   0,						/*tp_base */
//   0,						/*tp_dict */
//   0,						/*tp_descr_get */
//   0,						/*tp_descr_set */
//   0,						/*tp_dictoffset */
//   (initproc)division_init,			/*tp_init */
//   0,						/*tp_alloc */
//   (newfunc)division_new,			/*tp_new */
// };*/
