/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <Python.h>
#include <structmember.h>

#include <string.h>
#include <sstream>
#include <fstream>

#include "./systemtype.h"
#include "./monom.h"
#include "./coeff.h"
#include "./poly.h"
#include "./oneparameterpoly.h"
#include "./wrap.h"
#include "./division.h"
#include "./basis.h"

#include "../ginv/poly/iexpression.h"

PyObject* basisBuild(PyObject *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"algorithm", "divisionInterface", "system", "percent", "expanded", "time", "quiet", "QlengthBound", "MovedBound", "DegreeBound", NULL};

  const char *py_algorithm = NULL;
  PyObject *py_divisionInterface = NULL;
  PyObject *py_system = NULL;
  int py_percent = 0;
  bool py_expanded = false;
  long py_time = 0;
  bool py_quiet = false;
  int py_QlengthBound = 0;
  int py_MovedBound = 0;
  int py_DegreeBound = 0;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "sO!O!|iiliiii:build", kwlist,
                                   &py_algorithm,
                                   &divisionInterface_type, &py_divisionInterface,
                                   &PyList_Type, &py_system,
				   &py_percent,
                                   &py_expanded,
                                   &py_time,
                                   &py_quiet,
                                   &py_QlengthBound,
                                   &py_MovedBound,
                                   &py_DegreeBound))
      return NULL;

  IDivisionInterface *divisionInterface = ((DivisionInterface*)py_divisionInterface)->divisionInterface;
  IWrapInterface *wrapInterface = divisionInterface->wrapInterface();
  IPolyInterface *polyInterface = wrapInterface->polyInterface();

  ISetQ *setQ = new ISetQ(wrapInterface, py_DegreeBound);
  int n = PyList_Size(py_system);
  for(int i=0; i < n; i++) {
    PyObject *py_poly = PyList_GetItem(py_system, i);
    if (PyObject_TypeCheck(py_poly, &PyString_Type)) {
      std::istringstream in(PyString_AsString(py_poly));
      IExpression expr(polyInterface, py_expanded);
      in >> expr >> std::ws;
      if (!in.eof()) {
        PyErr_SetString(PyExc_TypeError, "error parsing string in system");
        return NULL;
      }
      IPoly *p = expr.toPrimitivePoly();
      if (!p->isZero()) setQ->insert(p);
    }
    else if (PyObject_TypeCheck(py_poly, &poly_type)) {
      setQ->insert(((Poly*)py_poly)->poly);
    }
    else {
      PyErr_SetString(PyExc_TypeError, "system element type string or 'Poly' in 'build'");
      return NULL;
    }
  }

  IAlgorithm *algorithm = NULL;
  if (strcmp("TQ", py_algorithm) == 0)
    algorithm = new IAlgorithmTQ(setQ, divisionInterface->create(), py_time, py_quiet, py_QlengthBound, py_MovedBound, py_DegreeBound);
  else if (strcmp("TQDegree", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex) &&
        !(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLexDeg)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQDegree(setQ, divisionInterface->create(), py_time, py_quiet, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else if (strcmp("TQBlockHigh", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQBlockHigh(setQ, divisionInterface->create(), py_time, py_quiet, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else if (strcmp("TQBlockLow", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQBlockLow(setQ, divisionInterface->create(), py_time, py_quiet, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else if (strcmp("TQBlockNorm", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQBlockNorm(setQ, divisionInterface->create(), py_time, py_quiet, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else {
    PyErr_SetString(PyExc_TypeError, "unknown algorithm in 'basis'");
    return NULL;
  }

  algorithm->build();
  Basis* basis = PyObject_New(Basis, &basis_type);
  basis->algorithm = algorithm;

  return (PyObject*)basis;
}

PyObject* basisLoad(PyObject *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"filename", "algorithm", "divisionInterface", NULL};

  const char *py_filename = NULL;
  const char *py_algorithm = NULL;
  PyObject *py_divisionInterface = NULL;
  long py_time = 0;
  int py_QlengthBound = 0;
  int py_MovedBound = 0;
  int py_DegreeBound = 0;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "ssO!:load", kwlist,
                                   &py_filename,
                                   &py_algorithm,
                                   &divisionInterface_type, &py_divisionInterface))
      return NULL;

  IDivisionInterface *divisionInterface = ((DivisionInterface*)py_divisionInterface)->divisionInterface;
  IWrapInterface *wrapInterface = divisionInterface->wrapInterface();

  ISetQ *setQ = new ISetQ(wrapInterface, py_DegreeBound);

  IAlgorithm *algorithm = NULL;
  if (strcmp("TQ", py_algorithm) == 0)
    algorithm = new IAlgorithmTQ(setQ, divisionInterface->create(), py_time, 0, py_QlengthBound, py_MovedBound, py_DegreeBound);
  else if (strcmp("TQDegree", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQDegree(setQ, divisionInterface->create(), py_time, 0, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else if (strcmp("TQBlockHigh", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQBlockHigh(setQ, divisionInterface->create(), py_time, 0, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else if (strcmp("TQBlockLow", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQBlockLow(setQ, divisionInterface->create(), py_time, 0, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else if (strcmp("TQBlockNorm", py_algorithm) == 0) {
    if (!(wrapInterface->monomInterface()->order() & IMonomInterface::DegRevLex)) {
      PyErr_SetString(PyExc_TypeError, "orderTerm not support DegRevLex in 'basis'");
      return NULL;
    }
    algorithm = new IAlgorithmTQBlockNorm(setQ, divisionInterface->create(), py_time, 0, py_QlengthBound, py_MovedBound, py_DegreeBound);
  }
  else {
    PyErr_SetString(PyExc_TypeError, "unknown algorithm in 'basis'");
    return NULL;
  }

  std::ifstream in;
  in.open(py_filename, std::ifstream::in);
  if (!in) {
    PyErr_SetString(PyExc_TypeError, "cannot read involutive basis from binary file");
    return NULL;
  }
  algorithm->read(in);
  in.close();

  Basis* basis = PyObject_New(Basis, &basis_type);
  basis->algorithm = algorithm;

  return (PyObject*)basis;
}

PyObject* basisSave(PyObject *self, PyObject *args) {
  const char *py_filename = NULL;
  PyObject *py_basis = NULL;
  if (!PyArg_ParseTuple(args, "sO!:save", &py_filename, &basis_type, &py_basis))
      return NULL;

  std::ofstream out;
  out.open(py_filename, std::ofstream::out);
  if (!out) {
    PyErr_SetString(PyExc_TypeError, "cannot write involutive basis to binary file");
    return NULL;
  }
  ((Basis *)py_basis)->algorithm->write(out);
  PyObject *r = NULL;
  if (out.good())
    r = Py_True;
  else
    r = Py_False;
  out.close();
  Py_INCREF(r);
  return r;
}

PyDoc_STRVAR(basisBuild_doc,
            "basisBuild(algorithm, divisionInterface, system, percent) -> basis\n\n");

static PyMethodDef ginv_methods[] = {
    {"basisBuild", (PyCFunction)basisBuild, METH_VARARGS | METH_KEYWORDS, basisBuild_doc},
    {"basisLoad", (PyCFunction)basisLoad, METH_VARARGS | METH_KEYWORDS, ""},
    {"basisSave", (PyCFunction)basisSave, METH_VARARGS, ""},
    {NULL, NULL}
};

PyMODINIT_FUNC initginv() {
  if (PyType_Ready(&systemType_type) < 0)
    return;

  if (PyType_Ready(&monomInterface_type) < 0)
    return;
  if (PyType_Ready(&monom_type) < 0)
    return;
  if (PyType_Ready(&monomIterator_type) < 0)
    return;

  if (PyType_Ready(&coeffInterface_type) < 0)
    return;
  if (PyType_Ready(&coeff_type) < 0)
    return;
  if (PyType_Ready(&denomIterator_type) < 0)
    return;

  if (PyType_Ready(&polyInterface_type) < 0)
    return;
  if (PyType_Ready(&poly_type) < 0)
    return;
  if (PyType_Ready(&polyIterator_type) < 0)
    return;

  if (PyType_Ready(&oneparameterpoly_type) < 0)
    return;

  if (PyType_Ready(&wrapInterface_type) < 0)
    return;
  if (PyType_Ready(&wrap_type) < 0)
    return;

  if (PyType_Ready(&divisionInterface_type) < 0)
    return;

  if (PyType_Ready(&basis_type) < 0)
    return;
  if (PyType_Ready(&basisIterator_type) < 0)
    return;
  if (PyType_Ready(&statisticsIterator_type) < 0)
    return;
  if (PyType_Ready(&iBIterator_type) < 0)
    return;
  if (PyType_Ready(&gBIterator_type) < 0)
    return;
  if (PyType_Ready(&lMiBIterator_type) < 0)
    return;
  if (PyType_Ready(&lMgBIterator_type) < 0)
    return;
  if (PyType_Ready(&lCiBIterator_type) < 0)
    return;
  if (PyType_Ready(&lCgBIterator_type) < 0)
    return;
  if (PyType_Ready(&aNCiBIterator_type) < 0)
    return;
  if (PyType_Ready(&mULTVARiBIterator_type) < 0)
    return;
  if (PyType_Ready(&mULTVARIterator_type) < 0)
    return;

  PyObject *module = Py_InitModule("ginv", ginv_methods);
  if (module == NULL)
    return;

  Py_INCREF(&systemType_type);
  PyModule_AddObject(module, "SystemType", (PyObject *)&systemType_type);

  Py_INCREF(&monomInterface_type);
  PyModule_AddObject(module, "MonomInterface", (PyObject *)&monomInterface_type);
  Py_INCREF(&monom_type);
  PyModule_AddObject(module, "Monom", (PyObject *)&monom_type);
  Py_INCREF(&monomIterator_type);
  PyModule_AddObject(module, "MonomIterator", (PyObject *)&monomIterator_type);

  Py_INCREF(&coeffInterface_type);
  PyModule_AddObject(module, "CoeffInterface", (PyObject *)&coeffInterface_type);
  Py_INCREF(&coeff_type);
  PyModule_AddObject(module, "Coeff", (PyObject *)&coeff_type);
  Py_INCREF(&denomIterator_type);
  PyModule_AddObject(module, "DenomIterator", (PyObject *)&denomIterator_type);

  Py_INCREF(&polyInterface_type);
  PyModule_AddObject(module, "PolyInterface", (PyObject *)&polyInterface_type);
  Py_INCREF(&poly_type);
  PyModule_AddObject(module, "Poly", (PyObject *)&poly_type);
  Py_INCREF(&polyIterator_type);
  PyModule_AddObject(module, "PolyIterator", (PyObject *)&polyIterator_type);

  Py_INCREF(&oneparameterpoly_type);
  PyModule_AddObject(module, "OneParameterPoly", (PyObject *)&oneparameterpoly_type);

  Py_INCREF(&wrapInterface_type);
  PyModule_AddObject(module, "WrapInterface", (PyObject *)&wrapInterface_type);
  Py_INCREF(&wrap_type);
  PyModule_AddObject(module, "Wrap", (PyObject *)&wrap_type);

  Py_INCREF(&divisionInterface_type);
  PyModule_AddObject(module, "DivisionInterface", (PyObject *)&divisionInterface_type);

  Py_INCREF(&basis_type);
  PyModule_AddObject(module, "Basis", (PyObject *)&basis_type);
  Py_INCREF(&basisIterator_type);
  PyModule_AddObject(module, "BasisIterator", (PyObject *)&basisIterator_type);
  Py_INCREF(&statisticsIterator_type);
  PyModule_AddObject(module, "StatisticsIterator", (PyObject *)&statisticsIterator_type);
  Py_INCREF(&iBIterator_type);
  PyModule_AddObject(module, "IBIterator", (PyObject *)&iBIterator_type);
  Py_INCREF(&gBIterator_type);
  PyModule_AddObject(module, "GBIterator", (PyObject *)&gBIterator_type);
  Py_INCREF(&lMiBIterator_type);
  PyModule_AddObject(module, "LMiBIterator", (PyObject *)&lMiBIterator_type);
  Py_INCREF(&lMgBIterator_type);
  PyModule_AddObject(module, "LMgBIterator", (PyObject *)&lMgBIterator_type);
  Py_INCREF(&lCiBIterator_type);
  PyModule_AddObject(module, "LCiBIterator", (PyObject *)&lCiBIterator_type);
  Py_INCREF(&lCgBIterator_type);
  PyModule_AddObject(module, "LCgBIterator", (PyObject *)&lCgBIterator_type);
  Py_INCREF(&aNCiBIterator_type);
  PyModule_AddObject(module, "ANCiBIterator", (PyObject *)&aNCiBIterator_type);
  Py_INCREF(&mULTVARiBIterator_type);
  PyModule_AddObject(module, "MULTVARiBIterator", (PyObject *)&mULTVARiBIterator_type);
  Py_INCREF(&mULTVARIterator_type);
  PyModule_AddObject(module, "MULTVARIterator", (PyObject *)&mULTVARIterator_type);
}

void iAssert(bool a, const char* file, int line) {
  if (!a) {
    int* p = 0;
    *p = 5;
    PyErr_Format(PyExc_TypeError, "assert from file: '%s'  line: %d\n", file, line);
    Py_Exit(EXIT_SUCCESS);
  }
}

void iError(const char* msg, const char* file, int line) {
  PyErr_Format(PyExc_TypeError, "error %s from file: '%s'  line: %d\n", msg, file, line);
  Py_Exit(EXIT_SUCCESS);
}

void iMessage(const char* msg, const char* file, int line) {
  // "warning %s from file: '%s'  line: %d\n", msg, file, line
  //PyErr_WarnEx(PyExc_Warning, msg, 1);
//   PyErr_Warn(PyExc_Warning, msg);
}
