/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./coeff.h"
#include "./oneparameterpoly.h"

#include <string.h>
#include <sstream>

#include "../ginv/poly/ioneparameterpoly.h"

void oneparameterpoly_dealloc(OneParameterPoly *self) {
//   delete self->poly;
//   delete self->denom;
  self->ob_type->tp_free((PyObject*)self);
}

int oneparameterpoly_init(OneParameterPoly *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"coeffInterface", "oneparameterpoly", NULL};

  PyObject *py_coeffInterface = NULL;
  const char *py_in = NULL;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|s:Coeff", kwlist,
                                    &coeffInterface_type, &py_coeffInterface,
                                    &py_in))
      return -1;

  self->poly = new IOneParameterPoly(((CoeffInterface*)py_coeffInterface)->coeffInterface);
  if (py_in) {
    std::istringstream in(py_in);
    in >> *self->poly >> std::ws;
    if (!in.eof()) {
      PyErr_SetString(PyExc_TypeError, "error parsing second argument");
      return -1;
    }
  }

  return 0;
}

int oneparameterpoly_print(OneParameterPoly *p, FILE *fp, int flags) {
  std::ostringstream out;
  out << *p->poly;
  fputs(out.str().c_str(), fp);
  return 0;
}

PyObject* oneparameterpoly_str(OneParameterPoly *p) {
  std::ostringstream out;
  out << *p->poly;
  return PyString_FromString(out.str().c_str());
}

PyObject* oneparameterpoly_getattr(OneParameterPoly *self, char *name) {
  PyObject *r = NULL;
  static PyMethodDef methods[] = {
//    {"length", (PyCFunction)oneparameterpoly_getLength, METH_NOARGS, ""},
//    {"degree", (PyCFunction)oneparameterpoly_degree, METH_NOARGS, ""},
//    {"norm", (PyCFunction)oneparameterpoly_norm, METH_NOARGS, ""},
//    {"lm", (PyCFunction)oneparameterpoly_lm, METH_NOARGS, ""},
//    {"lc", (PyCFunction)oneparameterpoly_lc, METH_NOARGS, ""},
//    {"setZero", (PyCFunction)oneparameterpoly_setZero, METH_NOARGS, ""},
//    {"pp", (PyCFunction)oneparameterpoly_pp, METH_NOARGS, ""},
//    {"isPp", (PyCFunction)oneparameterpoly_isPp, METH_NOARGS, ""},
//    {"prolong", (PyCFunction)oneparameterpoly_prolong, METH_VARARGS, ""},
//    {"mult", (PyCFunction)oneparameterpoly_mult, METH_VARARGS, ""},
//    {"diff", (PyCFunction)oneparameterpoly_diff, METH_VARARGS, ""},
//    {"shift", (PyCFunction)oneparameterpoly_shift, METH_VARARGS, ""},
//    {"reduction", (PyCFunction)oneparameterpoly_reduction, METH_VARARGS, ""},
//    {"soneparameterpoly", (PyCFunction)oneparameterpoly_soneparameterpoly, METH_VARARGS, ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);
  if (r == NULL) {
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'OneParameterPoly'");
  }

  return r;
}

//int oneparameterpoly_compare(OneParameterPoly *o1, PyObject *o2) {
//  int r = -1;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPolynomial");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPolynomial with different OneParameterPolyInterface");
//    else {
//      if (o1->oneparameterpoly->isZero())
//        if (oneparameterpoly->oneparameterpoly->isZero())
//          r = 0;
//        else
//          r = -1;
//      else if (oneparameterpoly->oneparameterpoly->isZero())
//        r = 1;
//      else
//        r = o1->oneparameterpoly->lm().compare(oneparameterpoly->oneparameterpoly->lm());
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_richCompare(OneParameterPoly *o1, PyObject *o2, int opid) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPolynomial");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPolynomial with different OneParameterPolyInterface");
//    else {
//      int cmp = -1;
//      if (o1->oneparameterpoly->isZero())
//        if (oneparameterpoly->oneparameterpoly->isZero())
//          cmp = 0;
//        else
//          cmp = -1;
//      else if (oneparameterpoly->oneparameterpoly->isZero())
//        cmp = 1;
//      else
//        cmp = o1->oneparameterpoly->lm().compare(oneparameterpoly->oneparameterpoly->lm());
//
//      bool res = -1;
//      switch (opid) {
//        case Py_LT:
//          res = (cmp == -1);
//          break;
//        case Py_LE:
//          res = (cmp == -1) || (cmp == 0);
//          break;
//        case Py_EQ:
//          res = (cmp == 0);
//          break;
//        case Py_NE:
//          res = (cmp != 0);
//          break;
//        case Py_GT:
//          res = (cmp == 1);
//          break;
//        case Py_GE:
//          res = (cmp == 1) || (cmp == 0);
//          break;
//      }
//      if (res)
//        r = Py_True;
//      else
//        r = Py_False;
//      Py_INCREF(r);
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_isZero(OneParameterPoly *self) {
//  PyObject *r = NULL;
//  if (self->oneparameterpoly->isZero())
//    r = Py_True;
//  else
//    r = Py_False;
//  Py_INCREF(r);
//  return r;
//}
//
//PyObject* oneparameterpoly_getLength(OneParameterPoly *self) {
//  return PyInt_FromLong(self->oneparameterpoly->length());
//}
//
//PyObject* oneparameterpoly_degree(OneParameterPoly *self) {
//  return PyInt_FromLong(self->oneparameterpoly->degree());
//}
//
//PyObject* oneparameterpoly_norm(OneParameterPoly *self) {
//  return PyInt_FromLong(self->oneparameterpoly->norm());
//}
//
//PyObject* oneparameterpoly_lm(OneParameterPoly *self) {
//  PyObject *r = NULL;
//  if (self->oneparameterpoly->isZero())
//    PyErr_SetString(PyExc_TypeError, "OneParameterPolynomial is zero");
//  else {
//    IMonom *m = self->oneparameterpoly->monomInterface()->copy(self->oneparameterpoly->lm());
//    Monom* res = PyObject_New(Monom, &monom_type);
//    res->monom = m;
//    r = (PyObject*)res;
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_lc(OneParameterPoly *self) {
//  PyObject *r = NULL;
//  if (self->oneparameterpoly->isZero())
//    PyErr_SetString(PyExc_TypeError, "OneParameterPolynomial is zero");
//  else {
//    ICoeff *c = self->oneparameterpoly->coeffInterface()->copy(self->oneparameterpoly->lc());
//    Coeff* res = PyObject_New(Coeff, &coeff_type);
//    res->coeff = c;
//    r = (PyObject*)res;
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_setZero(OneParameterPoly *self) {
//  PyObject *r = NULL;
//  self->oneparameterpoly->setZero();
//  if (self->denom != NULL) {
//    delete self->denom;
//    self->denom = NULL;
//  }
//  Py_INCREF(self);
//  r = (PyObject*)self;
//
//  return r;
//}
//
//PyObject* oneparameterpoly_pp(OneParameterPoly *self) {
//  PyObject *r = NULL;
//  if (!self->oneparameterpoly->isZero()) {
//    self->oneparameterpoly->pp();
//    if (self->denom != NULL) {
//      delete self->denom;
//      self->denom = NULL;
//    }
//  }
//  Py_INCREF(self);
//  r = (PyObject*)self;
//
//  return r;
//}
//
//PyObject* oneparameterpoly_isPp(OneParameterPoly *self) {
//  PyObject *r = NULL;
//  if (self->oneparameterpoly->isZero())
//    r = Py_True;
//  else if (self->oneparameterpoly->isPp())
//    if (self->denom == NULL)
//      r = Py_True;
//    else
//      r = Py_False;
//  else
//    r = Py_False;
//  Py_INCREF(r);
//
//  return r;
//}
//
//PyObject* oneparameterpoly_prolong(OneParameterPoly *self, PyObject *args) {
//  int var = 0;
//  unsigned deg = 1;
//  if (!PyArg_ParseTuple(args, "i|I:OneParameterPoly",
//                        &var, &deg))
//      return NULL;
//
//  PyObject *r = NULL;
//  if (0 > var || var >= self->oneparameterpoly->monomInterface()->dimIndepend())
//    PyErr_SetString(PyExc_TypeError, "'0 <= var < len(independ)' in MonomInterface");
//  else {
//    self->oneparameterpoly->mult(var, deg);
//    Py_INCREF(self);
//    r = (PyObject*)self;
//  }
//
//  return r;
//}
//
//PyObject* oneparameterpoly_mult(OneParameterPoly *self, PyObject *args) {
//  PyObject *py_mult = NULL;
//  if (!PyArg_ParseTuple(args, "O:OneParameterPoly",
//                        &py_mult))
//      return NULL;
//
//  PyObject *r = NULL;
//  if (PyObject_TypeCheck(py_mult, &monom_type)) {
//    Monom *monom = (Monom*)py_mult;
//    if (self->oneparameterpoly->monomInterface() != monom->monom->monomInterface())
//      PyErr_SetString(PyExc_TypeError, "Monomial with different MonomInterface");
//    else {
//      self->oneparameterpoly->mult(*monom->monom);
//      Py_INCREF(self);
//      r = (PyObject*)self;
//    }
//  }
//  else if (PyObject_TypeCheck(py_mult, &coeff_type)) {
//    Coeff *coeff = (Coeff*)py_mult;
//    if (self->oneparameterpoly->coeffInterface() != coeff->coeff->coeffInterface())
//      PyErr_SetString(PyExc_TypeError, "Coeff with different CoeffInterface");
//    else {
//      if (coeff->coeff->isZero()) {
//	self->oneparameterpoly->setZero();
//	if (self->denom != NULL) {
//	  delete self->denom;
//	  self->denom = NULL;
//	}
//      }
//      else if (self->denom != NULL) {
//	ICoeff *g = coeff->coeff->coeffInterface()->create();
//	g->gcd(*self->denom, *coeff->coeff);
//	if (g->isOne()) {
//	  self->oneparameterpoly->mult(*coeff->coeff);
//	}
//	else {
//	  ICoeff *h = coeff->coeff->coeffInterface()->create();
//	  h->div(*coeff->coeff, *g);
//	  self->oneparameterpoly->mult(*h);
//	  delete h;
//	  self->denom->div(*g);
//	}
//	delete g;
//      }
//      else
//	self->oneparameterpoly->mult(*coeff->coeff);
//
//      Py_INCREF(self);
//      r = (PyObject*)self;
//    }
//  }
//
//  return r;
//}
//
//PyObject* oneparameterpoly_diff(OneParameterPoly *self, PyObject *args) {
//  PyObject *r = NULL;
//  PyErr_SetString(PyExc_TypeError, "not implemented");
//  return r;
//}
//
//PyObject* oneparameterpoly_shift(OneParameterPoly *self, PyObject *args) {
//  PyObject *r = NULL;
//  PyErr_SetString(PyExc_TypeError, "not implemented");
//  return r;
//}
//
//PyObject* oneparameterpoly_reduction(OneParameterPoly *self, PyObject *args) {
//  PyObject *py_oneparameterpoly = NULL;
//  if (!PyArg_ParseTuple(args, "O!:OneParameterPoly",
//                        &oneparameterpoly_type, &py_oneparameterpoly))
//      return NULL;
//
//  PyObject *r = NULL;
//  OneParameterPoly *oneparameterpoly = (OneParameterPoly*)py_oneparameterpoly;
//  if (self->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//    PyErr_SetString(PyExc_TypeError, "OneParameterPolynomial with different OneParameterPolyInterface");
//  else if (self->oneparameterpoly->isZero() || oneparameterpoly->oneparameterpoly->isZero()) {
//    Py_INCREF(self);
//    r = (PyObject*)self;
//  }
//  else if (self->oneparameterpoly->lm().divisibility(oneparameterpoly->oneparameterpoly->lm())) {
//    ICoeff *multiplier;
//    if (self->denom != NULL)
//      multiplier = self->denom;
//    else {
//      multiplier = self->oneparameterpoly->coeffInterface()->create();
//      multiplier->setOne();
//    }
//    self->oneparameterpoly->reduction(*multiplier, *oneparameterpoly->oneparameterpoly);
//    if (self->oneparameterpoly->isZero() || multiplier->isOne()) {
//      if (self->denom != NULL) {
//	delete self->denom;
//	self->denom = NULL;
//      }
//      else
//        delete multiplier;
//    }
//    else {
//      ICoeff *c = self->oneparameterpoly->coeffInterface()->create();
//      self->oneparameterpoly->content(*c);
//      c->gcd(*multiplier);
//      if (!c->isOne()) {
//	self->oneparameterpoly->div(*c);
//	multiplier->div(*c);
//	if (multiplier->isOne()) {
//	  if (self->denom != NULL) {
//	    delete self->denom;
//	    self->denom = NULL;
//	  }
//	  else
//	    delete multiplier;
//	}
//      }
//      else
//	if (self->denom == NULL)
//	  self->denom = multiplier;
//      delete c;
//    }
//
//    Py_INCREF(self);
//    r = (PyObject*)self;
//  }
//  else
//    PyErr_SetString(PyExc_TypeError, "reduction impossible");
//
//  return r;
//}
//
//PyObject* oneparameterpoly_soneparameterpoly(OneParameterPoly *self, PyObject *args) {
//  PyObject *r = NULL;
//  PyErr_SetString(PyExc_TypeError, "not implemented");
//  return r;
//}
//
//PyObject *oneparameterpoly_getiter(OneParameterPoly *self) {
//  OneParameterPolyIterator* iter = PyObject_New(OneParameterPolyIterator, &oneparameterpolyIterator_type);
//  iter->iter = ((const IOneParameterPoly*)self->oneparameterpoly)->begin();
//  return (PyObject*)iter;
//}
//
//PyObject* oneparameterpoly_add(OneParameterPoly *o1, PyObject* o2) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPoly");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPoly with different OneParameterPolyInterface");
//    else {
//      IOneParameterPoly *p1 = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*o1->oneparameterpoly);
//      IOneParameterPoly *p2 = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*oneparameterpoly->oneparameterpoly);
//      ICoeff *d = NULL;
//      if (o1->denom != NULL) {
//        d = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	if (oneparameterpoly->denom != NULL) {
//	  d->gcd(*oneparameterpoly->denom);
//          ICoeff *a = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  a->div(*d);
//          ICoeff *b = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	  b->div(*d);
//	  p1->mult(*b);
//	  p2->mult(*a);
//	  delete a;
//	  d->set(*o1->denom);
//	  d->mult(*b);
//	  delete b;
//	}
//	else
//	  p2->mult(*o1->denom);
//      }
//      else if (oneparameterpoly->denom != NULL) {
//        d = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	p1->mult(*oneparameterpoly->denom);
//      }
//      p1->add(*p2);
//      if (p1->isZero()) {
//        if (d != NULL) {
//	  delete d;
//	  d = NULL;
//	}
//      }
//      else if (d != NULL) {
//	ICoeff *c = o1->oneparameterpoly->coeffInterface()->create();
//	p1->content(*c);
//	c->gcd(*d);
//	if (!c->isOne()) {
//	  p1->div(*c);
//	  d->div(*c);
//	  if (d->isOne()) {
//	    delete d;
//	    d = NULL;
//	  }
//	}
//	delete c;
//      }
//
//      OneParameterPoly* res = PyObject_New(OneParameterPoly, &oneparameterpoly_type);
//      res->oneparameterpoly = p1;
//      res->denom = d;
//      r = (PyObject*)res;
//      delete p2;
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_sub(OneParameterPoly *o1, PyObject* o2) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPoly");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPoly with different OneParameterPolyInterface");
//    else {
//      IOneParameterPoly *p1 = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*o1->oneparameterpoly);
//      IOneParameterPoly *p2 = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*oneparameterpoly->oneparameterpoly);
//      ICoeff *d = NULL;
//      if (o1->denom != NULL) {
//        d = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	if (oneparameterpoly->denom != NULL) {
//	  d->gcd(*oneparameterpoly->denom);
//          ICoeff *a = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  a->div(*d);
//          ICoeff *b = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	  b->div(*d);
//	  p1->mult(*b);
//	  p2->mult(*a);
//	  delete a;
//	  d->set(*o1->denom);
//	  d->mult(*b);
//	  delete b;
//	}
//	else
//	  p2->mult(*o1->denom);
//      }
//      else if (oneparameterpoly->denom != NULL) {
//        d = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	p1->mult(*oneparameterpoly->denom);
//      }
//      p1->sub(*p2);
//      if (p1->isZero()) {
//        if (d != NULL) {
//	  delete d;
//	  d = NULL;
//	}
//      }
//      else if (d != NULL) {
//	ICoeff *c = o1->oneparameterpoly->coeffInterface()->create();
//	p1->content(*c);
//	c->gcd(*d);
//	if (!c->isOne()) {
//	  p1->div(*c);
//	  d->div(*c);
//	  if (d->isOne()) {
//	    delete d;
//	    d = NULL;
//	  }
//	}
//	delete c;
//      }
//
//      OneParameterPoly* res = PyObject_New(OneParameterPoly, &oneparameterpoly_type);
//      res->oneparameterpoly = p1;
//      res->denom = d;
//      r = (PyObject*)res;
//      delete p2;
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_multiply(OneParameterPoly *o1, PyObject* o2) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPoly");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPoly with different OneParameterPolyInterface");
//    else {
//      IOneParameterPoly *p = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*o1->oneparameterpoly);
//      ICoeff *d = NULL;
//      p->mult(*oneparameterpoly->oneparameterpoly);
//      if (!p->isZero())
//	if (o1->denom != NULL) {
//	  d = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  if (oneparameterpoly->denom != NULL)
//	    d->mult(*oneparameterpoly->denom);
//	}
//	else if (oneparameterpoly->denom != NULL)
//	  d = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//
//      OneParameterPoly* res = PyObject_New(OneParameterPoly, &oneparameterpoly_type);
//      res->oneparameterpoly = p;
//      res->denom = d;
//      r = (PyObject*)res;
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_negative(OneParameterPoly *o1) {
//  PyObject *r = NULL;
//  IOneParameterPoly *p = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*o1->oneparameterpoly);
//  ICoeff *d = NULL;
//  if (o1->denom != NULL)
//    d = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//  p->minus();
//
//  OneParameterPoly* res = PyObject_New(OneParameterPoly, &oneparameterpoly_type);
//  res->oneparameterpoly = p;
//  res->denom = d;
//  r = (PyObject*)res;
//  return r;
//}
//
//int oneparameterpoly_nonzero(OneParameterPoly *o1) {
//  return !o1->oneparameterpoly->isZero();
//}
//
//PyObject* oneparameterpoly_inplace_add(OneParameterPoly *o1, PyObject* o2) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPoly");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPoly with different OneParameterPolyInterface");
//    else {
//      IOneParameterPoly *p = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*oneparameterpoly->oneparameterpoly);
//      if (o1->denom != NULL)
//	if (oneparameterpoly->denom != NULL) {
//          ICoeff *d = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  d->gcd(*oneparameterpoly->denom);
//          ICoeff *a = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  a->div(*d);
//          ICoeff *b = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	  b->div(*d);
//	  delete d;
//	  o1->oneparameterpoly->mult(*b);
//	  p->mult(*a);
//	  delete a;
//	  o1->denom->mult(*b);
//          delete b;
//	}
//	else
//	  p->mult(*o1->denom);
//      else if (oneparameterpoly->denom != NULL) {
//        o1->denom = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	o1->oneparameterpoly->mult(*oneparameterpoly->denom);
//      }
//      o1->oneparameterpoly->add(*p);
//      if (o1->oneparameterpoly->isZero()) {
//        if (o1->denom != NULL) {
//	  delete o1->denom;
//	  o1->denom = NULL;
//	}
//      }
//      else if (o1->denom != NULL) {
//	ICoeff *c = o1->oneparameterpoly->coeffInterface()->create();
//	o1->oneparameterpoly->content(*c);
//	c->gcd(*o1->denom);
//	if (!c->isOne()) {
//	  o1->oneparameterpoly->div(*c);
//	  o1->denom->div(*c);
//	  if (o1->denom->isOne()) {
//	    delete o1->denom;
//	    o1->denom = NULL;
//	  }
//	}
//	delete c;
//      }
//
//      r = (PyObject*)o1;
//      Py_INCREF(r);
//      delete p;
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_inplace_sub(OneParameterPoly *o1, PyObject* o2) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPoly");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPoly with different OneParameterPolyInterface");
//    else {
//      IOneParameterPoly *p = o1->oneparameterpoly->oneparameterpolyInterface()->copy(*oneparameterpoly->oneparameterpoly);
//      if (o1->denom != NULL)
//	if (oneparameterpoly->denom != NULL) {
//          ICoeff *d = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  d->gcd(*oneparameterpoly->denom);
//          ICoeff *a = o1->oneparameterpoly->coeffInterface()->copy(*o1->denom);
//	  a->div(*d);
//          ICoeff *b = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	  b->div(*d);
//	  delete d;
//	  o1->oneparameterpoly->mult(*b);
//	  p->mult(*a);
//	  delete a;
//	  o1->denom->mult(*b);
//          delete b;
//	}
//	else
//	  p->mult(*o1->denom);
//      else if (oneparameterpoly->denom != NULL) {
//        o1->denom = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//	o1->oneparameterpoly->mult(*oneparameterpoly->denom);
//      }
//      o1->oneparameterpoly->sub(*p);
//      if (o1->oneparameterpoly->isZero()) {
//        if (o1->denom != NULL) {
//	  delete o1->denom;
//	  o1->denom = NULL;
//	}
//      }
//      else if (o1->denom != NULL) {
//	ICoeff *c = o1->oneparameterpoly->coeffInterface()->create();
//	o1->oneparameterpoly->content(*c);
//	c->gcd(*o1->denom);
//	if (!c->isOne()) {
//	  o1->oneparameterpoly->div(*c);
//	  o1->denom->div(*c);
//	  if (o1->denom->isOne()) {
//	    delete o1->denom;
//	    o1->denom = NULL;
//	  }
//	}
//	delete c;
//      }
//
//      r = (PyObject*)o1;
//      Py_INCREF(r);
//      delete p;
//    }
//  }
//  return r;
//}
//
//PyObject* oneparameterpoly_inplace_multiply(OneParameterPoly *o1, PyObject* o2) {
//  PyObject *r = NULL;
//  if (!PyObject_TypeCheck(o2, &oneparameterpoly_type))
//    PyErr_SetString(PyExc_TypeError, "second argument not OneParameterPoly");
//  else {
//    OneParameterPoly *oneparameterpoly = (OneParameterPoly*)o2;
//    if (o1->oneparameterpoly->oneparameterpolyInterface() != oneparameterpoly->oneparameterpoly->oneparameterpolyInterface())
//      PyErr_SetString(PyExc_TypeError, "OneParameterPoly with different OneParameterPolyInterface");
//    else {
//      o1->oneparameterpoly->mult(*oneparameterpoly->oneparameterpoly);
//      if (!o1->oneparameterpoly->isZero())
//	if (o1->denom != NULL) {
//	  if (oneparameterpoly->denom != NULL)
//	    o1->denom->mult(*oneparameterpoly->denom);
//	}
//	else if (oneparameterpoly->denom != NULL)
//	  o1->denom = o1->oneparameterpoly->coeffInterface()->copy(*oneparameterpoly->denom);
//      else if (o1->denom != NULL) {
//	delete o1->denom;
//	o1->denom = NULL;
//      }
//
//      r = (PyObject*)o1;
//      Py_INCREF(r);
//    }
//  }
//  return r;
//}
//
//int oneparameterpoly_length(OneParameterPoly *self) {
//  return self->oneparameterpoly->length();
//}
//
//PyObject* oneparameterpoly_item(OneParameterPoly *self, Py_ssize_t i) {
//  PyObject* r = NULL;
//  if (0 > i || i >= self->oneparameterpoly->length())
//    PyErr_SetString(PyExc_ValueError, "Index out of range.");
//  else {
//    IOneParameterPoly::ConstIterator *k = ((const IOneParameterPoly*)self->oneparameterpoly)->begin();
//    for(; i > 0; i--)
//     ++(*k);
//
//    Monom* m = PyObject_New(Monom, &monom_type);
//    m->monom = (*k).monom().monomInterface()->copy((*k).monom());
//    Coeff* c = PyObject_New(Coeff, &coeff_type);
//    c->coeff = (*k).coeff().coeffInterface()->copy((*k).coeff());
//    r = PyTuple_New(2);
//    PyTuple_SetItem(r, 0, (PyObject*)m);
//    PyTuple_SetItem(r, 1, (PyObject*)c);
//  }
//  return r;
//}

//static PyNumberMethods oneparameterpoly_as_number = {
//  (binaryfunc)oneparameterpoly_add,                 /*nb_add*/
//  (binaryfunc)oneparameterpoly_sub,                 /*nb_subtract*/
//  (binaryfunc)oneparameterpoly_multiply,            /*nb_multiply*/
//  0,                                    /*nb_divide*/
//  0,                                    /*nb_remainder*/
//  0,                                    /*nb_divmod*/
//  0,                                    /*nb_power*/
//  (unaryfunc)oneparameterpoly_negative,             /*nb_negative*/
//  0,                                    /*nb_positive*/
//  0,                                    /*nb_absolute*/
//  (inquiry)oneparameterpoly_nonzero,                /*nb_nonzero*/
//  0,                                    /*nb_invert*/
//  0,                                    /*nb_lshift*/
//  0,                                    /*nb_rshift*/
//  0,                                    /*nb_and*/
//  0,                                    /*nb_xor*/
//  0,                                    /*nb_or*/
//  0,                                    /*nb_coerce*/
//  0,                                    /*nb_int*/
//  0,                                    /*nb_long*/
//  0,                                    /*nb_float*/
//  0,                                    /*nb_oct*/
//  0,                                    /*nb_hex*/
//  (binaryfunc)oneparameterpoly_inplace_add,         /*nb_inplace_add*/
//  (binaryfunc)oneparameterpoly_inplace_sub,         /*nb_inplace_subtract*/
//  (binaryfunc)oneparameterpoly_inplace_multiply,    /*nb_inplace_multiply*/
//  0,                                    /*nb_inplace_divide*/
//  0,                                    /*nb_inplace_remainder*/
//  0,                                    /*nb_inplace_power*/
//  0,                                    /*nb_inplace_lshift*/
//  0,                                    /*nb_inplace_rshift*/
//  0,                                    /*nb_inplace_and*/
//  0,                                    /*nb_inplace_xor*/
//  0,                                    /*nb_inplace_or*/
//  0,                                    /*nb_floor_divide*/
//  0,                                    /*nb_true_divide*/
//  0,                                    /*nb_inplace_floor_divide*/
//  0,                                    /*nb_inplace_true_divide*/
//};
//
//static PySequenceMethods oneparameterpoly_sequence = {
//  (lenfunc)oneparameterpoly_length,                 /*sq_length*/
//  0,                                    /*sq_concat*/
//  0,                                    /*sq_repeat*/
//  (ssizeargfunc)oneparameterpoly_item,              /*sq_item*/
//  0,                                    /*sq_slice*/
//  0,                                    /*sq_ass_item*/
//  0,                                    /*sq_ass_slice*/
//  0,                                    /*sq_contains */
//  0,                                    /*sq_inplace_concat*/
//  0,                                    /*sq_inplace_repeat*/
//};

PyTypeObject oneparameterpoly_type = {
  PyObject_HEAD_INIT(NULL)
  0,                                            /*ob_size*/
  "OneParameterPoly",                           /*tp_name*/
  sizeof(OneParameterPoly),                     /*tp_size*/
  0,                                            /*tp_itemsize*/
  (destructor)oneparameterpoly_dealloc,         /*tp_dealloc*/
  (printfunc)oneparameterpoly_print,            /*tp_print*/
  (getattrfunc)oneparameterpoly_getattr,        /*tp_getattr*/
  0,                                            /*tp_setattr*/
//  (cmpfunc)oneparameterpoly_compare,            /*tp_compare*/
  0,                                            /*tp_compare*/
  (reprfunc)oneparameterpoly_str,               /*tp_repr*/
//  &oneparameterpoly_as_number,                  /*tp_as_number*/
//  &oneparameterpoly_sequence,                   /*tp_as_sequence*/
  0,                                            /*tp_as_number*/
  0,                                            /*tp_as_sequence*/
  0,                                            /*tp_as_mapping*/
  0,                                            /*tp_hash*/
  0,                                            /*tp_call*/
  (reprfunc)oneparameterpoly_str,               /*tp_str*/
  0,                                            /*tp_getattro*/
  0,                                            /*tp_setattro*/
  0,                                            /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,     /*tp_flags*/
  "OneParameterPoly object",                    /*tp_doc */
  0,                                            /*tp_traverse */
  0,                                            /*tp_clear */
//  (richcmpfunc)oneparameterpoly_richCompare,    /*tp_richcompare */
  0,                                            /*tp_richcompare */
  0,                                            /*tp_weaklistoffset */
//  (getiterfunc)oneparameterpoly_getiter,        /*tp_iter */
  0,                                            /*tp_iter */
  0,                                            /*tp_iternext */
  0,                                            /*tp_methods */
  0,                                            /*tp_members */
  0,                                            /*tp_getset */
  0,                                            /*tp_base */
  0,                                            /*tp_dict */
  0,                                            /*tp_descr_get */
  0,                                            /*tp_descr_set */
  0,                                            /*tp_dictoffset */
  (initproc)oneparameterpoly_init,              /*tp_init */
  0,                                            /*tp_alloc */
//  (newfunc)oneparameterpoly_new,                /*tp_new */
  PyType_GenericNew,                            /*tp_new */
};

//void oneparameterpolyIterator_dealloc(OneParameterPolyIterator *self) {
//  self->ob_type->tp_free((PyObject*)self);
//}
//
//PyObject *oneparameterpolyIterator_getiter(OneParameterPoly *self) {
//  OneParameterPolyIterator* iter = PyObject_New(OneParameterPolyIterator, &oneparameterpolyIterator_type);
//  iter->iter = ((const IOneParameterPoly*)(self->oneparameterpoly))->begin();
//  return (PyObject*)iter;
//}
//
//PyObject* oneparameterpolyIterator_next(OneParameterPolyIterator *self) {
//  PyObject* r = NULL;
//  if (*self->iter) {
//    Monom* m = PyObject_New(Monom, &monom_type);
//    m->monom = (*self->iter).monom().monomInterface()->copy((*self->iter).monom());
//    Coeff* c = PyObject_New(Coeff, &coeff_type);
//    c->coeff = (*self->iter).coeff().coeffInterface()->copy((*self->iter).coeff());
//    ++(*self->iter);
//    r = PyTuple_New(2);
//    PyTuple_SetItem(r, 0, (PyObject*)m);
//    PyTuple_SetItem(r, 1, (PyObject*)c);
//  }
//  return r;
//}
//
//PyTypeObject oneparameterpolyIterator_type = {
//  PyObject_HEAD_INIT(NULL)
//  0,                                            /*ob_size*/
//  "OneParameterPolyIterator",                               /*tp_name*/
//  sizeof(OneParameterPolyIterator),                         /*tp_size*/
//  0,                                            /*tp_itemsize*/
//  (destructor)oneparameterpolyIterator_dealloc,             /*tp_dealloc*/
//  0,                                            /*tp_print*/
//  0,                                            /*tp_getattr*/
//  0,                                            /*tp_setattr*/
//  0,                                            /*tp_compare*/
//  0,                                            /*tp_repr*/
//  0,                                            /*tp_as_number*/
//  0,                                            /*tp_as_sequence*/
//  0,                                            /*tp_as_mapping*/
//  0,                                            /*tp_hash*/
//  0,                                            /*tp_call*/
//  0,                                            /*tp_str*/
//  0,                                            /*tp_getattro*/
//  0,                                            /*tp_setattro*/
//  0,                                            /*tp_as_buffer*/
//  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,     /*tp_flags*/
//  "OneParameterPolyIterator object",                        /*tp_doc */
//  0,                                            /*tp_traverse */
//  0,                                            /*tp_clear */
//  0,                                            /*tp_richcompare */
//  0,                                            /*tp_weaklistoffset */
//  (getiterfunc)oneparameterpolyIterator_getiter,            /*tp_iter */
//  (iternextfunc)oneparameterpolyIterator_next,              /*tp_iternext */
//  0,                                            /*tp_methods */
//  0,                                            /*tp_members */
//  0,                                            /*tp_getset */
//  0,                                            /*tp_base */
//  0,                                            /*tp_dict */
//  0,                                            /*tp_descr_get */
//  0,                                            /*tp_descr_set */
//  0,                                            /*tp_dictoffset */
//  0,                                            /*tp_init */
//  0,                                            /*tp_alloc */
//  0,                                            /*tp_new */
//};
