/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef POLY_H
#define POLY_H

#include <Python.h>
#include <structmember.h>

#include "../ginv/poly/ipoly.h"
#include "../ginv/coeff/icoeff.h"

struct  PolyInterface {
  PyObject_HEAD
  IPolyInterface *polyInterface;
};

void polyInterface_dealloc(PolyInterface *self);
PyObject* polyInterface_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
int polyInterface_init(PolyInterface *self, PyObject *args, PyObject *kwds);
PyObject *polyInterface_getattr(PolyInterface *self, char *name);
PyObject* polyInterface_getType(PolyInterface *self);

extern PyTypeObject polyInterface_type;

struct  Poly {
  PyObject_HEAD
  IPoly *poly;
  ICoeff *denom;  // for all cases where ICoeff represents non-invertible elements
};

void poly_dealloc(Poly *self);
PyObject* poly_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
int poly_init(Poly *self, PyObject *args, PyObject *kwds);
int poly_print(Poly *m, FILE *fp, int flags);
PyObject* poly_str(Poly *m);
PyObject* poly_getattr(Poly *self, char *name);
int poly_compare(Poly *o1, PyObject *o2);
PyObject* poly_richCompare(Poly *o1, PyObject *o2, int opid);
PyObject* poly_isZero(Poly *self);
PyObject* poly_getLength(Poly *self);
PyObject* poly_degree(Poly *self);
PyObject* poly_norm(Poly *self);
PyObject* poly_lm(Poly *self);
PyObject* poly_lc(Poly *self);
PyObject* poly_setZero(Poly *self);
PyObject* poly_pp(Poly *self);
PyObject* poly_isPp(Poly *self);
PyObject* poly_prolong(Poly *self, PyObject *args);
PyObject* poly_mult(Poly *self, PyObject *args);
PyObject* poly_diff(Poly *self, PyObject *args);
PyObject* poly_shift(Poly *self, PyObject *args);
PyObject* poly_reduction(Poly *self, PyObject *args);
PyObject* poly_spoly(Poly *self, PyObject *args);
PyObject* poly_getiter(Poly *self);

extern PyTypeObject poly_type;

struct  PolyIterator {
  PyObject_HEAD
  IPoly::ConstIterator *iter;
};

void polyIterator_dealloc(PolyIterator *self);
PyObject* polyIterator_getiter(Poly *self);
PyObject* polyIterator_next(PolyIterator *self);

extern PyTypeObject polyIterator_type;

#endif // POLY_H
