/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./systemtype.h"

#include <string.h>

void systemType_dealloc(SystemType *self) {
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* systemType_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  SystemType *self = (SystemType *)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->systemType = NULL;
  }

  return (PyObject *)self;
}

int systemType_init(SystemType *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"type", "module", "rightPart", NULL};

  const char *py_type = NULL;
  int py_module = 0,
      py_rightPart = 0;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "s|ii:SystemType", kwlist,
                                    &py_type, &py_module, &py_rightPart))
      return -1;

  if (strcmp("Polynomial", py_type) == 0)
    self->systemType = new ISystemType(ISystemType::Polynomial, py_module, py_rightPart);
  else if (strcmp("DifferentialEquation", py_type) == 0)
    self->systemType = new ISystemType(ISystemType::DifferentialEquation, py_module, py_rightPart);
  else if (strcmp("FiniteDifferenceScheme", py_type) == 0)
    self->systemType = new ISystemType(ISystemType::FiniteDifferenceScheme, py_module, py_rightPart);
  else {
    PyErr_SetString(PyExc_TypeError, "unknown value 'type' in 'SystemType'");
    return -1;
  }

  return 0;
}

PyObject *systemType_getattr(SystemType *self, char *name) {
  PyObject *r = NULL;
  static PyMethodDef methods[] = {
    {"type", (PyCFunction)systemType_Type, METH_NOARGS , ""},
    {"module", (PyCFunction)systemType_module, METH_NOARGS , ""},
    {"rightPart", (PyCFunction)systemType_rightPart, METH_NOARGS , ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);

  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'SystemType'");

  return r;
}

PyObject* systemType_Type(SystemType *self) {
  PyObject *r = NULL;
  switch(self->systemType->type()) {
    case ISystemType::Polynomial:
      r = PyString_FromString("Polynomial");
      break;
    case ISystemType::DifferentialEquation:
      r = PyString_FromString("DifferentialEquation");
      break;
    case ISystemType::FiniteDifferenceScheme:
      r = PyString_FromString("FiniteDifferenceScheme");
      break;
  }
  return r;
}

PyObject* systemType_module(SystemType *self) {
  return PyInt_FromLong(self->systemType->module());
}

PyObject* systemType_rightPart(SystemType *self) {
  return PyInt_FromLong(self->systemType->rightPart());
}

PyTypeObject systemType_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "SystemType",					/*tp_name*/
  sizeof(SystemType),				/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)systemType_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  (getattrfunc)systemType_getattr,		/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "SystemType object",				/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  0,						/*tp_iter */
  0,						/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  (initproc)systemType_init,			/*tp_init */
  0,						/*tp_alloc */
  (newfunc)systemType_new,			/*tp_new */
};
