/***************************************************************************
 *   Copyright (C) 2004 by Blinkov Yu.A.                                   *
 *   BlinkovUA@info.sgu.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "./monom.h"
#include "./poly.h"
#include "./wrap.h"

#include <string.h>

void wrapInterface_dealloc(WrapInterface *self) {
//   delete self->wrapInterface;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* wrapInterface_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  WrapInterface *self = (WrapInterface*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->wrapInterface = NULL;
  }

  return (PyObject *)self;
}

int wrapInterface_init(WrapInterface *self, PyObject *args, PyObject *kwds) {
    static char *kwlist[] = {"type", "polyInterface", NULL};

    const char *py_type = NULL;
    PyObject *py_polyInterface = NULL;
    if (!PyArg_ParseTupleAndKeywords(args, kwds, "sO!:WrapInterface", kwlist,
                                     &py_type,
                                     &polyInterface_type, &py_polyInterface))
        return -1;

    IWrapInterface::Type type = IWrapInterface::Indefinitely;
    if (strcmp("Without", py_type) == 0)
      type = IWrapInterface::Without;
    else if (strcmp("CritPartially", py_type) == 0 || strcmp("C1C2", py_type) == 0)
      type = IWrapInterface::CritPartially;
    else if (strcmp("C1", py_type) == 0)
      type = IWrapInterface::C1;
    else if (strcmp("C1C2C3", py_type) == 0)
      type = IWrapInterface::C1C2C3;
    else if (strcmp("C1C2C3C4", py_type) == 0)
      type = IWrapInterface::C1C2C3C4;
    else {
      PyErr_SetString(PyExc_TypeError, "unknown type in 'WrapInterface'");
      return -1;
    }

    IPolyInterface *polyInterface = ((PolyInterface *)py_polyInterface)->polyInterface;

    self->wrapInterface = IWrapInterface::create(type, polyInterface);

    return 0;
}

PyObject *wrapInterface_getattr(WrapInterface *self, char *name) {
 PyObject *r = NULL;
  static PyMethodDef methods[] = {
    {"type", (PyCFunction)wrapInterface_getType, METH_NOARGS , ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);

  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'WrapInterface'");

  return r;
}

PyObject* wrapInterface_getType(WrapInterface *self) {
  PyObject *r = NULL;
  switch(self->wrapInterface->type()) {
    case IWrapInterface::Without:
      r = PyString_FromString("Without");
      break;
    case IWrapInterface::CritPartially:
      r = PyString_FromString("CritPartially");
      break;
    case IWrapInterface::C1:
      r = PyString_FromString("C1");
      break;
    case IWrapInterface::C1C2C3:
      r = PyString_FromString("C1C2C3");
      break;
    case IWrapInterface::C1C2C3C4:
      r = PyString_FromString("C1C2C3C4");
      break;
  }
  return r;
}

PyTypeObject wrapInterface_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "WrapInterface",				/*tp_name*/
  sizeof(WrapInterface),			/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)wrapInterface_dealloc,		/*tp_dealloc*/
  0,						/*tp_print*/
  (getattrfunc)wrapInterface_getattr,		/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "WrapInterface object",			/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  0,						/*tp_iter */
  0,						/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  (initproc)wrapInterface_init,			/*tp_init */
  0,						/*tp_alloc */
  (newfunc)wrapInterface_new,			/*tp_new */
};

void wrap_dealloc(Wrap *self) {
//   delete self->wrap;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* wrap_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
  Wrap *self = (Wrap*)type->tp_alloc(type, 0);
  if (self != NULL) {
    self->wrap = NULL;
  }

  return (PyObject *)self;
}

int wrap_init(Wrap *self, PyObject *args, PyObject *kwds) {
  static char *kwlist[] = {"wrapInterface", "poly", NULL};

  PyObject *py_wrapInterface = NULL;
  PyObject *py_poly = NULL;
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O!:Wrap", kwlist,
                                    &wrapInterface_type, &py_wrapInterface,
                                    &poly_type, &py_poly))
      return -1;

  self->wrap = ((WrapInterface*)py_wrapInterface)->wrapInterface->create(((Poly*)py_poly)->poly);

  return 0;
}

PyObject* wrap_getattr(Wrap *self, char *name) {
 PyObject *r = NULL;
  static PyMethodDef methods[] = {
    {"lm", (PyCFunction)wrap_lm, METH_NOARGS , ""},
    {"ancestor", (PyCFunction)wrap_ancestor, METH_NOARGS , ""},
    {"poly", (PyCFunction)wrap_poly, METH_NOARGS , ""},
    {"multi", (PyCFunction)wrap_multi, METH_NOARGS , ""},
    {"degProlong", (PyCFunction)wrap_degProlong, METH_NOARGS , ""},
    {"isProlong", (PyCFunction)wrap_isProlong, METH_NOARGS , ""},
    {"buildProlong", (PyCFunction)wrap_buildProlong, METH_NOARGS , ""},
    {NULL}
  };
  r = Py_FindMethod(methods, (PyObject*)self, name);

  if (r == NULL)
    PyErr_SetString(PyExc_TypeError, "Cannot define operation in 'Basis'");

  return r;
}

PyObject* wrap_lm(Wrap *self) {
  Monom* res = PyObject_New(Monom, &monom_type);
  res->monom = self->wrap->monomInterface()->copy(self->wrap->lm());
  return (PyObject*)res;
}

PyObject* wrap_ancestor(Wrap *self) {
  Monom* res = PyObject_New(Monom, &monom_type);
  res->monom = self->wrap->monomInterface()->copy(self->wrap->ancestor());
  return (PyObject*)res;
}

PyObject* wrap_poly(Wrap *self) {
  Poly* res = PyObject_New(Poly, &poly_type);
  res->poly = self->wrap->polyInterface()->copy(self->wrap->poly());
  return (PyObject*)res;
}

PyObject* wrap_multi(Wrap *self) {
  return PyInt_FromLong(self->wrap->multi());
}

PyObject* wrap_degProlong(Wrap *self) {
  return PyInt_FromLong(self->wrap->degProlong());
}

PyObject* wrap_isProlong(Wrap *self) {
  PyObject *r = NULL;
  if (self->wrap->isProlong())
    r = Py_True;
  else
    r = Py_False;
  Py_INCREF(r);
  return r;
}

PyObject* wrap_buildProlong(Wrap *self) {
  PyObject *r = NULL;
  int n = self->wrap->monomInterface()->dimIndepend();
  r = PyList_New(n);
  int var = 0;
  while (var < n) {
    PyList_SetItem(r, var, PyInt_FromLong(self->wrap->build(var)));
    ++var;
  }
  return r;
}

PyTypeObject wrap_type = {
  PyObject_HEAD_INIT(NULL)
  0,						/*ob_size*/
  "Wrap",					/*tp_name*/
  sizeof(Wrap),					/*tp_size*/
  0,						/*tp_itemsize*/
  (destructor)wrap_dealloc,			/*tp_dealloc*/
  0,						/*tp_print*/
  (getattrfunc)wrap_getattr,			/*tp_getattr*/
  0,						/*tp_setattr*/
  0,						/*tp_compare*/
  0,						/*tp_repr*/
  0,						/*tp_as_number*/
  0,						/*tp_as_sequence*/
  0,						/*tp_as_mapping*/
  0,						/*tp_hash*/
  0,						/*tp_call*/
  0,						/*tp_str*/
  0,						/*tp_getattro*/
  0,						/*tp_setattro*/
  0,						/*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/*tp_flags*/
  "Wrap object",				/*tp_doc */
  0,						/*tp_traverse */
  0,						/*tp_clear */
  0,						/*tp_richcompare */
  0,						/*tp_weaklistoffset */
  0,						/*tp_iter */
  (iternextfunc)polyIterator_next,		/*tp_iternext */
  0,						/*tp_methods */
  0,						/*tp_members */
  0,						/*tp_getset */
  0,						/*tp_base */
  0,						/*tp_dict */
  0,						/*tp_descr_get */
  0,						/*tp_descr_set */
  0,						/*tp_dictoffset */
  (initproc)wrap_init,				/*tp_init */
  0,						/*tp_alloc */
  (newfunc)poly_new,				/*tp_new */
};
